Package: PhyloProfile
Version: 2.2.0
Date: 2025-10-20
Title: PhyloProfile
Authors@R: c(
    person("Vinh", "Tran", role = c("aut", "cre"), email = "tran@bio.uni-frankfurt.de", comment=c(ORCID="0000-0001-6772-7595")),
    person("Bastian", "Greshake Tzovaras", role = "aut", email = "bgreshake@googlemail.com"),
    person("Ingo", "Ebersberger", role = "aut", email = "ebersberger@bio.uni-frankfurt.de"),
    person("Carla", "Mölbert", role = "ctb", email = "carla.moelbert@gmx.de"))
Description: PhyloProfile is a tool for exploring complex phylogenetic profiles. Phylogenetic profiles, presence/absence patterns of genes over a set of species, are commonly used to trace the functional and evolutionary history of genes across species and time. With PhyloProfile we can enrich regular phylogenetic profiles with further data like sequence/structure similarity, to make phylogenetic profiling more meaningful. Besides the interactive visualisation powered by R-Shiny, the package offers a set of further analysis features to gain insights like the gene age estimation or core gene identification.
URL: https://github.com/BIONF/PhyloProfile/
BugReports: https://github.com/BIONF/PhyloProfile/issues
License: MIT + file LICENSE
Depends: R (>= 4.5.0)
Encoding: UTF-8
biocViews: Software, Visualization, DataRepresentation,
        MultipleComparison, FunctionalPrediction, DimensionReduction
Imports: ape, bioDist, BiocStyle, Biostrings, bsplus, colourpicker,
        data.table, dplyr, DT, energy, fastcluster, ggplot2, gridExtra,
        htmlwidgets, pbapply, plotly, RColorBrewer, RCurl, Rfast,
        scattermore, shiny, shinycssloaders, shinyFiles, shinyjs,
        stringr, tsne, svglite, umap, xml2, zoo, yaml
RoxygenNote: 7.3.3
Suggests: knitr, rmarkdown, testthat, OmaDB
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/PhyloProfile
git_branch: RELEASE_3_22
git_last_commit: aa891b5
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 05:42:51 UTC; biocbuild
Author: Vinh Tran [aut, cre] (ORCID: <https://orcid.org/0000-0001-6772-7595>),
  Bastian Greshake Tzovaras [aut],
  Ingo Ebersberger [aut],
  Carla Mölbert [ctb]
Maintainer: Vinh Tran <tran@bio.uni-frankfurt.de>

# Generated by roxygen2: do not edit by hand

export(addRankDivisionPlot)
export(calcPresSpec)
export(checkColorPalette)
export(checkInputValidity)
export(checkNewick)
export(checkOmaID)
export(clusterDataDend)
export(compareMedianTaxonGroups)
export(compareTaxonGroups)
export(createArchiPlot)
export(createDimRedPlotData)
export(createGeneAgePlot)
export(createLongMatrix)
export(createPercentageDistributionData)
export(createProfileFromOma)
export(createUnrootedTree)
export(createVarDistPlot)
export(createVariableDistributionData)
export(createVariableDistributionDataSubset)
export(dataCustomizedPlot)
export(dataFeatureTaxGroup)
export(dataMainPlot)
export(dataVarDistTaxGroup)
export(dimReduction)
export(estimateGeneAge)
export(fastaParser)
export(featureDistTaxPlot)
export(filterProfileData)
export(fromInputToProfile)
export(geneAgePlotDf)
export(generateSinglePlot)
export(getAllDomainsOma)
export(getAllFastaOma)
export(getCommonAncestor)
export(getCoreGene)
export(getDataClustering)
export(getDataForOneOma)
export(getDendrogram)
export(getDistanceMatrix)
export(getFastaFromFasInput)
export(getFastaFromFile)
export(getFastaFromFolder)
export(getIDsRank)
export(getInputTaxaID)
export(getInputTaxaName)
export(getNameList)
export(getOmaDataForOneOrtholog)
export(getOmaDomainFromURL)
export(getOmaMembers)
export(getQualColForVector)
export(getSelectedFastaOma)
export(getSelectedTaxonNames)
export(getTaxonomyInfo)
export(getTaxonomyMatrix)
export(getTaxonomyRanks)
export(gridArrangeSharedLegend)
export(groupLabelDimRedData)
export(heatmapPlotting)
export(heatmapPlottingFast)
export(highlightProfilePlot)
export(id2name)
export(linearizeArchitecture)
export(mainTaxonomyRank)
export(parseDomainInput)
export(parseInfoProfile)
export(plotDimRed)
export(plotDimRed3D)
export(prepareDimRedData)
export(processNcbiTaxonomy)
export(processOrthoID)
export(reduceProfile)
export(runPhyloProfile)
export(sortInputTaxa)
export(sortTaxaFromTree)
export(taxonomyTableCreator)
export(varDistTaxPlot)
export(wideToLong)
export(xmlParser)
import(BiocStyle)
import(bsplus)
import(data.table, except = c(first, last, between))
import(dplyr)
import(ggplot2)
import(plotly, except = c(last_plot, select))
import(scattermore)
import(svglite)
import(tsne)
import(umap)
importFrom(Biostrings,readAAStringSet)
importFrom(DT,dataTableOutput)
importFrom(DT,renderDataTable)
importFrom(RColorBrewer,brewer.pal)
importFrom(RColorBrewer,brewer.pal.info)
importFrom(RCurl,url.exists)
importFrom(Rfast,Dist)
importFrom(ape,as.phylo)
importFrom(ape,plot.phylo)
importFrom(ape,read.tree)
importFrom(ape,root)
importFrom(bioDist,cor.dist)
importFrom(bioDist,mutualInfo)
importFrom(colourpicker,colourInput)
importFrom(data.table,dcast)
importFrom(data.table,fread)
importFrom(data.table,melt)
importFrom(data.table,rbindlist)
importFrom(data.table,setDT)
importFrom(data.table,setnames)
importFrom(data.table,transpose)
importFrom(dplyr,add_count)
importFrom(dplyr,arrange)
importFrom(dplyr,count)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,tibble)
importFrom(energy,dcor)
importFrom(fastcluster,hclust)
importFrom(grDevices,colorRampPalette)
importFrom(grid,convertHeight)
importFrom(grid,grobHeight)
importFrom(grid,unit)
importFrom(gridExtra,arrangeGrob)
importFrom(gridExtra,grid.arrange)
importFrom(htmlwidgets,saveWidget)
importFrom(pbapply,pblapply)
importFrom(shinyFiles,parseDirPath)
importFrom(shinyFiles,shinyDirButton)
importFrom(shinyFiles,shinyDirChoose)
importFrom(shinycssloaders,withSpinner)
importFrom(shinyjs,disable)
importFrom(shinyjs,enable)
importFrom(shinyjs,html)
importFrom(shinyjs,reset)
importFrom(shinyjs,toggleState)
importFrom(stats,aggregate)
importFrom(stats,as.dist)
importFrom(stats,complete.cases)
importFrom(stats,dist)
importFrom(stats,na.omit)
importFrom(stringr,str_count)
importFrom(stringr,str_split_fixed)
importFrom(stringr,str_wrap)
importFrom(utils,head)
importFrom(utils,tail)
importFrom(xml2,read_xml)
importFrom(xml2,xml_attr)
importFrom(xml2,xml_find_all)
importFrom(yaml,read_yaml)
importFrom(yaml,write_yaml)
importFrom(zoo,na.locf)

# PhyloProfile
[![Bioconductor](https://img.shields.io/badge/available%20at-Bioconductor-orange)](https://bioconductor.org/packages/PhyloProfile)
[![install with bioconda](https://img.shields.io/badge/install%20with-bioconda-brightgreen.svg?style=flat)](http://bioconda.github.io/recipes/bioconductor-phyloprofile/README.html)
[![published in: Bioinformatics](https://img.shields.io/badge/published%20in-Bioinformatics-ff69b4.svg?style=flat)](https://doi.org/10.1093/bioinformatics/bty225)
[![presented at: GCB2018](https://img.shields.io/badge/presented%20at-GCB2018-green.svg?style=flat)](http://gcb2018.de)
[![poster at: SMBE2019](https://img.shields.io/badge/poster%20at-SMBE2019-green)](https://f1000research.com/posters/6-1782)
[![BioC status](http://www.bioconductor.org/shields/build/devel/bioc/PhyloProfile.svg)](https://bioconductor.org/checkResults/release/bioc-LATEST/PhyloProfile)

![](https://github.com/BIONF/phyloprofile-data/blob/gh-pages/www/posterSub.png)
[Click here for the full PDF version of the BOSC2017 poster](https://f1000research.com/posters/6-1782)

*PhyloProfile* is an R package that comes together with a *Shiny*-based tool for integrating, visualizing and exploring multi-layered [phylogenetic profiles](https://en.wikipedia.org/wiki/Phylogenetic_profiling).

Alongside the presence/absence patterns of [orthologs](https://en.wikipedia.org/wiki/Homology_(biology)) across large [taxon](https://en.wikipedia.org/wiki/Taxon) collections, *PhyloProfile* allows the integration of any two additional information layers. These complementary data, like [sequence similarity](https://en.wikipedia.org/wiki/Sequence_alignment) between orthologs, similarities in their [domain architecture](https://www.ncbi.nlm.nih.gov/pubmed/20221914), or differences in [functional annotations](https://en.wikipedia.org/wiki/Protein_function_prediction) enable a more informed interpretation of phylogenetic profiles.

By utilizing the [NCBI taxonomy](https://www.ncbi.nlm.nih.gov/taxonomy), *PhyloProfile* can dynamically collapse taxa into higher [systematic groups](https://en.wikipedia.org/wiki/Taxonomy_(biology)). This enables rapidly changing the resolution from the comparative analyses of proteins in individual species to that of entire kingdoms or even domains without changes to the input data.

*PhyloProfile* furthermore allows for a dynamic filtering of profiles – taking the taxonomic distribution and the additional information layers into account. This, along with functions to estimate [the age of genes](http://www.cell.com/trends/genetics/fulltext/S0168-9525(13)00111-X) and [core gene](https://en.wikipedia.org/wiki/Pan-genome) sets facilitates the exploration and analysis of large phylogenetic profiles.

Take a look at [the functionality](https://github.com/BIONF/PhyloProfile/wiki/Functionality) of *PhyloProfile* and [explore the installation-free online version](http://applbio.biologie.uni-frankfurt.de/phyloprofile/) to learn more.

# Table of Contents
* [Installation &amp; Usage](#installation--usage)
* [Input Data](#input-data)
* [Walkthrough &amp; Examples](#walkthrough--examples)
* [Bugs](#bugs)
* [Acknowledgements](#acknowledgements)
* [License](#license)
* [How-To Cite](#how-to-cite)
* [Contact](#contact)

# Installation & Usage
*PhyloProfile* requires the latest version of [R](https://cran.r-project.org) (check for required R version [here](https://bioconductor.org/packages/PhyloProfile)). Please install or update R on your computer before continue.

* [R for Linux](https://cran.r-project.org/bin/linux/)
* [R for Mac OS](https://cran.r-project.org/bin/macosx/)
* [R for Windows](https://cran.r-project.org/bin/windows/base/)

Then start R to install and use *PhyloProfile*.

## Using BiocManager
*PhyloProfile* is available at [Bioconductor](https://bioconductor.org/packages/PhyloProfile) (require Bioc version ≥ 3.14). To install *PhyloProfile*, start R and enter:

```r
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("PhyloProfile")
```

To install the development version of PhyloProfile, please use the [devel version](https://bioconductor.org/developers/how-to/useDevel/) of Bioconductor:
```r
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install(version='devel')
BiocManager::install("PhyloProfile")
```

or
*install the dev version of *PhyloProfile* from our github repository using __[remotes](https://cran.r-project.org/web/packages/remotes/index.html)__.*

## Using remotes package
The dev version of *PhyloProfile* can be installed from this github repository using `remotes`:

```r
if (!requireNamespace("remotes"))
    install.packages("remotes")
remotes::install_github("BIONF/PhyloProfile")
```

## Using Conda
*PhyloProfile* can also be installed within a [conda](https://docs.conda.io/projects/conda/en/latest/user-guide/install/) environment. First, add [bioconda](https://bioconda.github.io/user/install.html#set-up-channels) to the list of your conda channels:
```
conda config --add channels defaults
conda config --add channels bioconda
conda config --add channels conda-forge
```
then install *PhyloProfile* using the standard `conda install` command:
```
conda install bioconductor-phyloprofile
```

This installation step can take a while regardless of the method used, as all necessary dependencies will be downloaded and installed automatically. *(Note: Depending on your system this sometimes fails, please check the console log for error messages concerning the dependency installation)*

## Start PhyloProfile's Shiny app

From the R terminal, enter:
```r
library(PhyloProfile)
runPhyloProfile()
```
Check your web browser, *PhyloProfile* will be displayed there ;-) For the first time running, the tool will download a [pre-caculated taxonomy data](https://github.com/BIONF/phyloprofile-data). Please be patient until you see a message for uploading input files.

_**Please check our [detailed instructions](https://github.com/BIONF/PhyloProfile/wiki/Installation) if you encounter any problems while installing and starting the program.**_

# Input Data
*PhyloProfile* can read a number of different input files, including multi-FASTA files, regular tab-separated files, OMA ID lists or *OrthoXML* files. The additional information layers can be embedded in the OrthoXML or be provided separately as tab-separated files.

We described all suppported input formats in section [Input Data](https://github.com/BIONF/PhyloProfile/wiki/Input-Data) in our [PhyloProfile's Wiki](https://github.com/BIONF/PhyloProfile/wiki).

# Walkthrough & Examples
Read the [walkthrough slides](https://github.com/BIONF/PhyloProfile/wiki/Walkthrough) to explore the functionality of the *PhyloProfile* GUI.

Check the vignette for learning how to use PhyloProfile's functions in some specific use-cases:
```r
browseVignettes("PhyloProfile")
```

# Bugs
Any [bug reports or comments, suggestions](https://github.com/BIONF/PhyloProfile/blob/master/CONTRIBUTING.md) are highly appreciated. Please [open an issue on GitHub](https://github.com/BIONF/PhyloProfile/issues/new) or be in touch via email.

# Acknowledgements
We would like to thank
1) [Bastian](https://github.com/gedankenstuecke) for the great initial idea and his kind support,
2) Members of [Ebersberger group](http://www.bio.uni-frankfurt.de/43045195/ak-ebersberger) for many valuable suggestions and ...bug reports :)

# Contributors
* [Vinh](https://github.com/trvinh)
* [Bastian](https://github.com/gedankenstuecke)
* [Carla](https://github.com/CarlaMoelbert)

# License
This tool is released under [MIT license](https://github.com/BIONF/PhyloProfile/blob/master/LICENSE).

# How-To Cite
Vinh Tran, Ingo Ebersberger, PhyloProfile v2: Scalable Exploration of Multilayered Phylogenetic Profiles via Dimensionality Reduction, 2025, Preprint available at arXiv: https://doi.org/10.48550/arXiv.2504.19710.
  
Ngoc-Vinh Tran, Bastian Greshake Tzovaras, Ingo Ebersberger, PhyloProfile: dynamic visualization and exploration of multi-layered phylogenetic profiles, Bioinformatics, Volume 34, Issue 17, 01 September 2018, Pages 3041–3043, https://doi.org/10.1093/bioinformatics/bty225

or use the citation function in R CMD to have it directly in BibTex or LaTeX format
```r
citation("PhyloProfile")
```
# Contact
Vinh Tran
tran@bio.uni-frankfurt.de

# Parse main input functions
# These functions check the validity of the main input file
# and convert different input format into long format.

#' Check the validity of the input phylogenetic profile file
#' @description Check if input file has one of the following format: orthoXML,
#' multiple FASTA, tab-delimited matrix (wide or long), or list of OMA IDs.
#' @export
#' @param filein input file
#' @return The format of the input file format, or type of error
#' @importFrom data.table fread
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @seealso \code{\link{checkOmaID}}
#' @examples
#' filein <- system.file(
#'     "extdata", "test.main.wide", package = "PhyloProfile", mustWork = TRUE
#' )
#' checkInputValidity(filein)

checkInputValidity <- function(filein) {
    if (missing(filein) | is.null(filein)) return("No input file given!")
    inputDt <- data.frame(fread(
        file = filein, sep = "\t", header = FALSE, check.names = FALSE,
        fill = TRUE, stringsAsFactors = FALSE))

    if (is.na(inputDt[1, ncol(inputDt)])) {
        return("moreCol")
    } else {
        names(inputDt) <- as.character(unlist(inputDt[1, ]))
        # XML format (starts with <?xml)
        if (grepl("<?xml", colnames(inputDt)[1])) {
            return("xml")
        }
        # FASTA format (starts with ">" )
        else if (grepl(">", colnames(inputDt)[1]) == TRUE) {
            return("fasta")
        }
        # LONG or WIDE format (starts with "geneID")
        else {
            if (grepl("geneID", colnames(inputDt)[1])) {
                # LONG format
                if (is.na(pmatch("ncbi", colnames(inputDt)[3])) ||
                    is.na(pmatch("ncbi", colnames(inputDt)[4])) ||
                    is.na(pmatch("ncbi", colnames(inputDt)[5]))) {
                    return("long")
                }
                # WIDE format
                else {
                    tmp <- inputDt[inputDt == ""][1]
                    if (!is.na(tmp) & tmp == "") {
                        return("emptyCell")
                    } else return("wide")
                }
            }
            else {
                if (grepl("\\s", inputDt[1, 1])) return("invalidFormat")
                # OMA ids
                ids <- unique(inputDt[,1])
                if (length(ids) < 2) ids <- rep(ids, 2)
                invalidOma <- checkOmaID(ids)
                if (length(invalidOma) == 0) {
                    return("oma")
                } else return(invalidOma)
            }
        }
    }
}

#' Parse orthoXML input file
#' @export
#' @param inputFile input file in xml format
#' @return A data frame of input data in long-format containing seed gene IDs (
#' or orthologous group IDs), their orthologous proteins together with the
#' corresponding taxonomy IDs and values of (up to) two additional variables.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom xml2 read_xml
#' @importFrom xml2 xml_find_all
#' @importFrom xml2 xml_attr
#' @importFrom data.table dcast setDT
#' @examples
#' inputFile <- system.file(
#'     "extdata", "test.main.xml", package = "PhyloProfile", mustWork = TRUE
#' )
#' xmlParser(inputFile)

xmlParser <- function(inputFile = NULL){
    if (is.null(inputFile)) stop("Input file is NULL!")
    scoreType <- scoreValue <- NULL
    data <- read_xml(inputFile)
    # get all genes for each taxon
    species <- xml_find_all(data, ".//species")
    geneSpec <- xml_find_all(species, ".//gene")
    refGeneID <- xml_attr(geneSpec, "id")
    if (length(geneSpec) == 0)
        stop("Unsupported XML format! Please check the file header!")
    if (grepl("protId", geneSpec[1])) {
        orthoID <- as.data.frame(unlist(strsplit(
            xml_attr(geneSpec, "protId"), split = '|', fixed = TRUE
        )), stringsAsFactors = FALSE)[,1]
    } else if (grepl("geneId", geneSpec[1])) orthoID <- xml_attr(geneSpec, "id")
    speciesID <- xml_attr(species, "NCBITaxId")
    speciesIDrep <- unlist(
        lapply(species, function (x) length(xml_find_all(x, ".//gene"))))
    speciesDf <- data.frame(
        ncbiID = rep(paste0("ncbi", speciesID), speciesIDrep),
        refGeneID = refGeneID, orthoID = orthoID, stringsAsFactors=FALSE)
    # get orthologs and their scores
    orthoGroup <- xml_find_all(data, ".//orthologGroup")
    groupID <- xml_attr(orthoGroup, "id")
    genes <- xml_find_all(orthoGroup, ".//geneRef")
    refGeneID <- xml_attr(genes, "id")
    score <- xml_find_all(genes, ".//score")
    if (length(score) > 0) {
        scorePair <- lapply(
            score, function (x) c(xml_attr(x, "id"), xml_attr(x, "value")))
        scorePair <- as.data.frame(do.call(rbind, scorePair))
        groupIDrep <- unlist(lapply(
            orthoGroup,
            function (x) ncol(scorePair)*length(xml_find_all(x, ".//geneRef")))
        )
        orthoDf <- data.frame(
            geneID = rep(groupID, groupIDrep),
            refGeneID = rep(refGeneID, each = ncol(scorePair)),
            scoreType = scorePair$V1, scoreValue = scorePair$V2,
            stringsAsFactors = FALSE)
        orthoDf <- data.table::dcast(
            setDT(orthoDf), geneID+refGeneID ~ scoreType, value.var="scoreValue"
        )
    } else {
        groupIDrep <- unlist(lapply(
            orthoGroup,
            function (x) length(xml_find_all(x, ".//geneRef")))
        )
        orthoDf <- data.frame(
            geneID = rep(groupID, groupIDrep),
            refGeneID = refGeneID,
            stringsAsFactors = FALSE)
    }
    # merge into final dataframe
    finalDf <- merge(speciesDf, orthoDf, all.y = TRUE, by = "refGeneID")
    # remove refGeneID column and reorder columns
    finalDf <- finalDf[, -1]
    refcols <- c("geneID", "ncbiID", "orthoID")
    finalDf <- finalDf[, c(refcols, setdiff(names(finalDf), refcols))]
    # remove columns that contains only NA
    finalDf <- finalDf[, colSums(is.na(finalDf)) < nrow(finalDf)]
    # convert geneID, ncbiID and orthoID to factors
    finalDf[seq_len(3)] <- lapply(finalDf[seq_len(3)], as.factor)
    # convert var1, var2 to numeric, suppressing warnings
    if (ncol(finalDf) > 3) {
        finalDf[4:min(5, ncol(finalDf))] <- lapply(
            finalDf[4:min(5, ncol(finalDf))],
            function(col) suppressWarnings(as.numeric(as.character(col)))
        )
    }
    return(finalDf)
}

#' Parse multi-fasta input file
#' @export
#' @param inputFile input multiple fasta file. Check extdata/test.main.fasta or
#' https://github.com/BIONF/PhyloProfile/wiki/Input-Data#multi-fasta-format for
#' the supported FASTA header.
#' @return A data frame of input data in long-format containing seed gene IDs (
#' or orthologous group IDs), their orthologous proteins together with the
#' corresponding taxonomy IDs and values of (up to) two additional variables.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom Biostrings readAAStringSet
#' @examples
#' inputFile <- system.file(
#'     "extdata", "test.main.fasta", package = "PhyloProfile", mustWork = TRUE
#' )
#' fastaParser(inputFile)

fastaParser <- function(inputFile = NULL){
    if (is.null(inputFile)) stop("Input file is NULL!")
    # split sequence IDs into columns
    fastaFile <- Biostrings::readAAStringSet(inputFile)
    seqID <- names(fastaFile)
    tmpDf <- data.frame(
        do.call(rbind, strsplit(seqID, "\\|")), stringsAsFactors = FALSE
    )
    orthoID <- tmpDf[,3]
    if (!is.null(nrow(tmpDf[,-c(1,2, ncol(tmpDf) - 1, ncol(tmpDf))])))
        orthoID <- do.call(paste, tmpDf[,-c(1,2, ncol(tmpDf) - 1, ncol(tmpDf))])
    faDf <- data.frame(
        geneID = tmpDf[,1],
        ncbiID = tmpDf[,2],
        orthoID = orthoID,
        var1 = tmpDf[, ncol(tmpDf) - 1],
        var2 = tmpDf[, ncol(tmpDf)],
        stringsAsFactors = FALSE
    )
    faDf$orthoID <- gsub(" ", "|", faDf$orthoID)

    # remove columns that contains only NA
    faDf <- faDf[, colSums(is.na(faDf)) < nrow(faDf)]
    return(faDf)
}

#' Transform input file in wide matrix into long matrix format
#' @export
#' @param inputFile input file in wide matrix format
#' @return A data frame of input data in long-format containing seed gene IDs (
#' or orthologous group IDs), their orthologous proteins together with the
#' corresponding taxonomy IDs and values of (up to) two additional variables.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom data.table fread
#' @examples
#' inputFile <- system.file(
#'     "extdata", "test.main.wide", package = "PhyloProfile", mustWork = TRUE
#' )
#' wideToLong(inputFile)

wideToLong <- function(inputFile = NULL){
    if (is.null(inputFile)) stop("Input file is NULL!")
    wideDataframe <- data.frame(fread(
        file = inputFile, sep = "\t", header = TRUE, check.names = FALSE,
        stringsAsFactors = FALSE
    ))
    ncbiIDs <- colnames(wideDataframe[, c(-1)])
    orthoInfo <- data.frame(
        do.call(
            rbind, strsplit(as.character(unlist(wideDataframe[, c(-1)])), "#")
        ),
        stringsAsFactors = FALSE
    )
    longDataframe <- data.frame(
        geneID = rep(wideDataframe$geneID, time = ncol(wideDataframe) - 1),
        ncbiID = rep(ncbiIDs, time = 1, each = nrow(wideDataframe)),
        orthoID = orthoInfo$X1,
        var1 = suppressWarnings(as.numeric(orthoInfo$X2)),
        var2 = suppressWarnings(as.numeric(orthoInfo$X3)),
        stringsAsFactors = FALSE
    )
    longDataframe[seq_len(3)] <- lapply(longDataframe[seq_len(3)], as.factor)
    return(longDataframe)
}

#' Create a long matrix format for all kinds of input phylogenetic profiles
#' @export
#' @param inputFile input profile file in orthoXML, multiple FASTA,
#' tab-delimited matrix format (wide or long).
#' @return A data frame of input data in long-format containing seed gene IDs (
#' or orthologous group IDs), their orthologous proteins together with the
#' corresponding taxonomy IDs and values of (up to) two additional variables.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom data.table fread
#' @seealso \code{\link{xmlParser}}, \code{\link{fastaParser}},
#' \code{\link{wideToLong}}
#' @examples
#' inputFile <- system.file(
#'     "extdata", "test.main.wide", package = "PhyloProfile", mustWork = TRUE
#' )
#' createLongMatrix(inputFile)

createLongMatrix <- function(inputFile = NULL){
    if (is.null(inputFile)) stop("Input file is NULL!")

    inputType <- checkInputValidity(inputFile)
    # XML
    if (inputType == "xml") longDataframe <- xmlParser(inputFile)
    # FASTA
    else if (inputType == "fasta") longDataframe <- fastaParser(inputFile)
    # LONG
    else if (inputType == "long") {
        longDataframe <- data.frame(fread(
            file = inputFile, sep = "\t", header = TRUE, check.names = FALSE, 
            stringsAsFactors = TRUE
        ))
    }
    # WIDE
    else if (inputType == "wide") longDataframe <- wideToLong(inputFile)
    else return(NULL)
    # Remove duplicated rows
    longDataframe <- dplyr::distinct(longDataframe)
    return(longDataframe)
}

context("test parsing main input file into long dataframe")

test_that("test fasta input", {
    a <- createLongMatrix("mainFastaTest.txt")
    expect_true(ncol(a) == 5)
})

test_that("test wide input", {
    a <- createLongMatrix("mainWideTest.txt")
    expect_true(nrow(a) == 20)
})

test_that("test xml input", {
    a <- createLongMatrix("mainOrthoxmlTest.xml")
    expect_true(ncol(a) == 5)
})

#' Protein domain architecture plot
#' @param pointInfo() info of clicked point
#' (from reactive fn "pointInfoDetail")
#' @param domainInfo() domain information
#' (from reactive fn "getDomainInformation")
#' @param labelArchiSize lable size (from input$labelArchiSize)
#' @param titleArchiSize title size (from input$titleArchiSize)
#' @param archiHeight plot height (from input$archiHeight)
#' @param archiWidth plot width (from input$archiWidth)
#' @param currentNCBIinfo dataframe of the pre-processed NCBI taxonomy data
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}

source("R/functions.R")

createArchitecturePlotUI <- function(id) {
    ns <- NS(id)
    tagList(
        fluidRow(
            column(
                3,
                radioButtons(
                    ns("resolveOverlap"),
                    "Merge non-overlapped features",
                    choices = c("Yes","No"), selected = "Yes",
                    inline = TRUE
                ),
                checkboxGroupInput(
                    ns("namePostion"),
                    "Display feature names",
                    choices = c(
                        "On the plot" = "plot",
                        "As a legend" = "legend",
                        "On the y-axis" = "axis"
                    ),
                    selected = c("plot","axis")
                )
            ),
            column(
                3,
                selectInput(
                    ns("feature"),
                    "Exclude features",
                    choices = c(
                        "flps","seg","coils","signalp","tmhmm",
                        "smart","pfam",
                        "without E-value" = "noEvalue",
                        "without Bit-score" = "noBitscore"
                    ),
                    multiple = TRUE
                ),
                checkboxInput(
                    ns("featureOpt"), "Other feature options", value = FALSE
                ),
                checkboxInput(
                    ns("plotConfig"), "Plot configuration", value = FALSE
                )
            ),
            column(
                3,
                strong("Show information"),
                checkboxGroupInput(
                    ns("showWeight"),
                    "",
                    choices = "Weight"
                ),
                checkboxGroupInput(
                    ns("showScore"),
                    "",
                    choices = c(
                        "E-value", "Bit-score"
                    )
                ),
                uiOutput(ns("filterEvalue.ui")),
                uiOutput(ns("filterBitscore.ui"))
            ),
            column(
                3,
                selectInput(
                    ns("linearizationBy"),
                    "Linearizing architecture using",
                    choices = c(
                        "None" = "none",
                        "Best E-value" = "evalue",
                        "Best Bit-score" = "bitscore",
                        "Paths" = "path"
                    )
                )
            )
        ),
        br(),
        fluidRow(
            conditionalPanel(
                condition = {sprintf("input['%s'] == 1", ns("featureOpt"))},
                column(
                    3,
                    radioButtons(
                        ns("nameType"),"Type of feature names", inline = TRUE,
                        choices = c("Labels","Texts"), selected = "Labels"
                    )

                ),
                column(
                    4,
                    conditionalPanel(
                        condition = {
                            sprintf("input['%s'] == 'Labels'", ns("nameType"))
                        },
                        radioButtons(
                            ns("labelPos"),"Label position", inline = TRUE,
                            choices = c("Above","Inside","Below"),
                            selected = "Above"
                        )
                    ),
                    conditionalPanel(
                        condition = {
                            sprintf("input['%s'] == 'Texts'", ns("nameType"))
                        },
                        colourpicker::colourInput(
                            ns("nameColor"),
                            "Feature name color",
                            value = "#000000"
                        )
                    )
                ),
                column(
                    5,
                    selectInput(
                        ns("excludeNames"),
                        "Exclude feature names of",
                        choices = c(
                            "flps","seg","coils","signalp","tmhmm",
                            "smart","pfam"
                        ),
                        selected = c("tmhmm","signalp","seg","coils"),
                        multiple = TRUE
                    )
                ),
                column(
                    3,
                    radioButtons(
                        ns("featureClassSort"),
                        "Sort feature classes by shared features",
                        choices = c("Yes","No"), selected = "Yes",
                        inline = TRUE
                    )
                ),
                column(
                    5,
                    conditionalPanel(
                        condition = {
                            sprintf("input['%s']=='No'",ns("featureClassSort"))
                        },
                        selectInput(
                            ns("featureClassOrder"),
                            "Feature class order",
                            choices = c(
                                "pfam", "smart", "tmhmm", "coils", "signalp",
                                "seg", "flps"
                            ),
                            selected = c(
                                "pfam", "smart", "tmhmm", "coils", "signalp",
                                "seg", "flps"
                            ),
                            multiple = TRUE
                        )
                    )
                )
            )
        ),
        br(),
        fluidRow(
            conditionalPanel(
                condition = {sprintf("input['%s'] == 1", ns("plotConfig"))},
                column(
                    3,
                    createPlotSize(ns("archiHeight"),"Plot height(px)",400,200),
                    createPlotSize(ns("archiWidth"),"Plot width (px)", 800, 200)
                ),
                column(
                    3,
                    createTextSize(
                        ns("titleArchiSize"), "Title/Seq ID size (px)", 14, 200
                    ),
                    createTextSize(
                        ns("labelArchiSize"), "Axis label size(px)", 12, 200
                    ),
                    createTextSize(
                        ns("legendArchiSize"), "Legend font size(px)", 12, 200
                    )
                ),
                column(
                    6,
                    column(
                        6,
                        createTextSize(
                            ns("segmentSize"),"Feature segment size (mm)",5,200
                        )
                    ),
                    column(
                        6,
                        createTextSize(
                            ns("nameSize"), "Feature ID size (mm)", 3, 200
                        )
                    ),
                    column(
                        12,
                        sliderInput(
                            ns("firstDist"),
                            "Distance between plot title and the 1st feature",
                            min = 0, max = 5, value = 0.5, step = 0.1, width=400
                        )
                    )
                ),
                column(
                    6,
                    radioButtons(
                        ns("colorType"),"Color feature instances", inline=TRUE,
                        choices = c("Shared","Unique","All","Feature type"),
                        selected = "All"
                    ),
                    checkboxInput(
                        ns("ignoreInstanceNo"), "Ignore number of instances",
                        value = FALSE
                    ),
                ),
                column(
                    6,
                    selectInput(
                        ns("colorPallete"),
                        "Color pallete",
                        choices = c(
                            "Paired", "Set1", "Set2", "Set3", "Accent", "Dark2"
                        ),
                        selected = "Paired"
                    )
                )
            )
        ),
        hr(),
        uiOutput(ns("archiPlot.ui")),
        verbatimTextOutput(ns("hover_info")),
        br(),
        downloadButton(ns("archiDownload"), "Download plot", class = "butDL"),
        hr(),
        tableOutput(ns("linkTable")),
        checkboxInput(
            ns("showDomainTable"), "Show detailed feature table", value = FALSE
        ),
        conditionalPanel(
            condition = {sprintf("input['%s'] == 1", ns("showDomainTable"))},
            DT::dataTableOutput(ns("domainTable"))
        )
    )
}

createArchitecturePlot <- function(
    input, output, session, pointInfo, domainInfo, currentNCBIinfo, font="sans"
){
    # * update excludeNames if no feature type on the y-axis ===================
    observe({
        req(input$namePostion)
        if (
            !("axis" %in% input$namePostion) & !("legend" %in% input$namePostion)
        ) {
            updateSelectInput(
                session, "excludeNames",
                "Exclude feature names of",
                choices = c("seg","coils","signalp","tmhmm","smart","pfam")
            )
        } else if (
            "axis" %in% input$namePostion | "legend" %in% input$namePostion
        ) {
            updateSelectInput(
                session, "excludeNames",
                "Exclude feature names of",
                choices = c(
                    "flps","seg","coils","signalp","tmhmm","smart","pfam"
                ),
                selected = c("tmhmm","signalp","seg","coils")
            )
        }
    })

    # * render e-value / bitscore filter =======================================
    output$filterEvalue.ui <- renderUI({
        req(domainInfo())
        df <- domainInfo()
        maxEvalue = 1
        if ("evalue" %in% colnames(df))
            maxEvalue <- format(
                max(df$evalue[!is.na(df$evalue)]), scientific = TRUE, digits = 2
            )
        if ("E-value" %in% input$showScore) {
            numericInput(
                session$ns("minEvalue"), "Filter E-value:",
                min = 0,
                max = maxEvalue,
                value = format(0.00001, scientific = TRUE, digits = 2)
            )
        }
    })

    output$filterBitscore.ui <- renderUI({
        req(domainInfo())
        df <- domainInfo()
        if ("Bit-score" %in% input$showScore) {
            numericInput(
                session$ns("minBitscore"), "Filter Bit-score:",
                min = min(df$bitscore[!is.na(df$bitscore)]),
                max = 9999,
                value = min(df$bitscore[!is.na(df$bitscore)])
            )
        }
    })

    # * filter domain features =================================================
    filterDomainDf <- reactive({
        outDf <- domainInfo()
        if (is.null(outDf)) stop("Domain info is NULL!")
        if (is.null(nrow(outDf))) stop("Domain info is NULL!")

        # get subset domainDf containing only domains for seed and ortholog
        group <- as.character(pointInfo()[1])
        ortho <- as.character(pointInfo()[2])
        # get sub dataframe based on selected groupID and orthoID
        group <- gsub("\\|", ":", group)
        ortho <- gsub("\\|", ":", ortho)
        grepID <- paste(group, "#", ortho, sep = "")
        outDf <- outDf[outDf$seedID == grepID, ]

        # filter domain df by features
        if (nrow(outDf) == 0) stop("Domain info is NULL!")

        outDf[c("feature_type","feature_id")] <-
            stringr::str_split_fixed(outDf$feature, '_', 2)
        outDf <- outDf[!(outDf$feature_type %in% input$feature),]

        # filter filters without e-value and/or bitscore
        if ("evalue" %in% colnames(outDf)) {
            if ("noEvalue" %in% input$feature)
                outDf <- outDf[!is.na(outDf$evalue),]
            if ("noBitscore" %in% input$feature)
                outDf <- outDf[!is.na(outDf$bitscore),]
        }

        # modify feature IDs
        outDf$feature_id_mod <- outDf$feature_id
        outDf$feature_id_mod <- gsub("SINGLE", "LCR", outDf$feature_id_mod)
        outDf$feature_id_mod[outDf$feature_type == "coils"] <- "Coils"
        outDf$feature_id_mod[outDf$feature_type == "seg"] <- "LCR"
        outDf$feature_id_mod[outDf$feature_type == "tmhmm"] <- "TM"
        # exclude features IDs
        if (!is.null(input$excludeNames)) {
            outDf$feature_id_mod[outDf$feature_type %in% input$excludeNames]<-NA
        }

        # enable/disable option for showing evalue/bitscore
        if ("evalue" %in% colnames(outDf)) {
            shinyjs::enable("showScore")
        } else {
            shinyjs::disable("showScore")
        }

        # enable/disable option for showing weight
        if ("weight" %in% colnames(outDf)) {
            shinyjs::enable("showWeight")
        } else {
            shinyjs::disable("showWeight")
        }

        # Filter data by e-value, bit-score and feature path
        if ("evalue" %in% colnames(outDf)) {
            # filter by e-value and/or bit-score
            if ("E-value" %in% input$showScore) {
                minEvalue <- format(input$minEvalue, scientific = FALSE)
                naOutDf <- outDf[is.na(outDf$evalue),]
                outDf <- outDf[!is.na(outDf$evalue) & outDf$evalue <= minEvalue,]
                outDf <- rbind(outDf,naOutDf)
            }
            if ("Bit-score" %in% input$showScore) {
                naOutDf <- outDf[is.na(outDf$bitscore),]
                outDf <- outDf[
                    !is.na(outDf$bitscore) & outDf$bitscore >= input$minBitscore,
                ]
                outDf <- rbind(outDf,naOutDf)
            }
            # get only best instances
            if ("evalue" %in% input$linearizationBy) {
                linearizedDfs <- lapply(
                    levels(as.factor(outDf$orthoID)),
                    function(orthoID) {
                        return(linearizeArchitecture(outDf, orthoID, "evalue"))
                    }
                )
                outDf <- do.call(rbind, linearizedDfs)
            }
            if ("bitscore" %in% input$linearizationBy) {
                linearizedDfs <- lapply(
                    levels(as.factor(outDf$orthoID)),
                    function(orthoID) {
                        return(linearizeArchitecture(outDf, orthoID,"bitscore"))
                    }
                )
                outDf <- do.call(rbind, linearizedDfs)
            }
            if ("path" %in% input$linearizationBy) {
                outDf <- outDf %>% dplyr::group_by(feature) %>%
                    dplyr::filter(path == "Y")
            }
            # Format e-values
            outDf$evalue[!is.na(outDf$evalue)] <-
                format(
                    outDf$evalue[!is.na(outDf$evalue)], scientific = TRUE,
                    digits = 2
                )
        }
        return(outDf[!is.na(outDf$seedID),])
    })

    # * render plot ============================================================
    output$archiPlot <- renderPlot({
        if (is.null(nrow(filterDomainDf()))) stop("Domain info is NULL!")
        # remove user specified features (from input$featureList)
        df <- filterDomainDf()
        df <- df[!(df$feature_id %in% input$featureList),]
        g <- createArchiPlot(
            pointInfo(), df,
            input$labelArchiSize, input$titleArchiSize, input$legendArchiSize,
            input$showScore, input$showWeight, input$namePostion, 
            input$firstDist,input$nameType, input$nameSize, input$segmentSize, 
            input$nameColor, input$labelPos, input$colorType, 
            input$ignoreInstanceNo, currentNCBIinfo(), input$featureClassSort, 
            input$featureClassOrder, input$colorPallete, input$resolveOverlap, 
            font()
        )
        if (is.character(g) && any(g == "No domain info available!")) {
            msgPlot()
        } else {
            suppressWarnings(grid::grid.draw(g))
        }
    })

    output$archiPlot.ui <- renderUI({
        ns <- session$ns
        if (is.null(nrow(filterDomainDf()))) {
            msg <- paste0(
                "<p><em>Wrong domain file has been uploaded!
        Please check the correct format in
        <a href=\"https://github.com/BIONF/PhyloProfile/wiki/",
                "Input-Data#ortholog-annotations-eg-domains\"
        target=\"_blank\" rel=\"noopener\">our PhyloProfile wiki</a>.</em></p>"
            )
            HTML(msg)
        } else {
            shinycssloaders::withSpinner(
                plotOutput(
                    ns("archiPlot"),
                    height = input$archiHeight,
                    width = input$archiWidth,
                    click = ns("archiClick")
                )
            )
        }
    })

    output$archiDownload <- downloadHandler(
        filename = function() {
            c("domains.svg")
        },
        content = function(file) {
            # remove user specified features (from input$featureList)
            df <- filterDomainDf()
            df <- df[!(df$feature_id %in% input$featureList),]
            g <- createArchiPlot(
                pointInfo(), df,
                input$labelArchiSize,input$titleArchiSize,input$legendArchiSize, 
                input$showScore, input$showWeight, input$namePostion, 
                input$firstDist, input$nameType, input$nameSize, 
                input$segmentSize, input$nameColor, input$labelPos, 
                input$colorType, input$ignoreInstanceNo, currentNCBIinfo(),
                input$featureClassSort, input$featureClassOrder,
                input$colorPallete, input$resolveOverlap, font()
            )
            # save plot to file
            suppressWarnings(ggsave(
                file, plot = g,
                width = input$archiWidth * 0.035,
                height = input$archiHeight * 0.035,
                units = "cm", dpi = 300, device = "svg", limitsize = FALSE
            ))
        }
    )

    # output$selectedDomain <- renderText({
    #     if (is.null(input$archiClick$y)) return("No domain selected!")
    #     y <- input$archiClick$y
    #     # paste(y, round(y), convertY(unit(y, "npc"), "px"))
    #
    # })

    output$featureList.ui <- renderUI({
        ns <- session$ns
        allFeats <- getAllFeatures(pointInfo(), filterDomainDf())
        selectizeInput(
            ns("featureList"), "Exclude individual feature(s)", multiple = TRUE,
            choices = allFeats,
            options=list(placeholder = 'None')
        )
    })

    output$linkTable <- renderTable({
        if (is.null(nrow(filterDomainDf()))) return("No domain info available!")
        features <- getDomainLink(pointInfo(), filterDomainDf())
        features <- features[!duplicated(features),]
    }, sanitize.text.function = function(x) x)

    output$domainTable <- DT::renderDataTable({
        req(filterDomainDf())
        createDomainInfoTable(filterDomainDf())
    }, rownames= FALSE)
}

#' plot error message
#' @return error message in a ggplot object
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}
msgPlot <- function() {
    msg <- paste(
        "No information about domain architecture!",
        "Please check:","if you uploaded the correct domain file/folder; or ",
        "if the selected genes (seed & ortholog) do exist in the uploaded file",
        "(please search for the corresponding seedID and hitID)",
        sep = "\n"
    )
    x <- c(1,2,3,4,5)
    y <- c(1,2,3,4,5)
    g <- ggplot(data.frame(x, y), aes(x,y)) +
        geom_point(color = "white") +
        annotate(
            "text", label = msg, x = 3.5, y = 0.5, size = 5, colour = "red"
        ) +
        theme(axis.line = element_blank(), axis.text = element_blank(),
              axis.ticks = element_blank(), axis.title = element_blank(),
              panel.background = element_blank(),
              panel.border = element_blank(),
              panel.grid = element_blank(),
              plot.background = element_blank()) +
        ylim(0,1)
    return(g)
}

#' Get list of all features
#' @return A list of all features
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}
getAllFeatures <- function(info, domainDf) {
    group <- as.character(info[1])
    ortho <- as.character(info[2])
    # get sub dataframe based on selected groupID and orthoID
    group <- gsub("\\|", ":", group)
    ortho <- gsub("\\|", ":", ortho)
    grepID <- paste(group, "#", ortho, sep = "")
    subdomainDf <- domainDf[grep(grepID, domainDf$seedID), ]
    orthoID <- feature <- NULL
    if (nrow(subdomainDf) < 1) return(paste0("No domain info available!"))
    else {
        # ortho & seed domains df
        orthoDf <- subdomainDf[subdomainDf$orthoID == ortho,]
        seedDf <- subdomainDf[subdomainDf$orthoID != ortho,]
        feature <- c(
            levels(as.factor(orthoDf$feature_id)),
            levels(as.factor(seedDf$feature_id))
        )
    }
    return(levels(as.factor(feature)))
}

#' get pfam and smart domain info (domain name, acc, profile HMM,...)
#' @return dataframe for each type of database
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}
getDomainInfo <- function(info, domainDf, type) {
    group <- as.character(info[1])
    ortho <- as.character(info[2])
    # get sub dataframe based on selected groupID and orthoID
    group <- gsub("\\|", ":", group)
    ortho <- gsub("\\|", ":", ortho)
    grepID <- paste(group, "#", ortho, sep = "")
    domainDf <- domainDf[grep(grepID, domainDf$seedID),]
    domainDf <- domainDf[domainDf$feature_type == type,]
    domainInfoDf <- data.frame()
    if (nrow(domainDf) > 0) {
        # get all features of for this pair proteins
        feature <- getAllFeatures(info, domainDf)
        # filter domain info
        if ("acc" %in% colnames(domainDf)) {
            domainInfoDf <- domainDf[
                , c("orthoID", "feature_id", "acc", "evalue", "bitscore")
            ]
            domainInfoDf$evalue <- format(domainInfoDf$evalue, scientific=TRUE)
        } else {
            domainInfoDf <- domainDf[, c("orthoID", "feature_id")]
            domainInfoDf <- domainInfoDf[!(duplicated(domainInfoDf)),]
            domainInfoDf$acc <- rep(NA, nrow(domainInfoDf))
            domainInfoDf$evalue <- rep(NA, nrow(domainInfoDf))
            domainInfoDf$bitscore <- rep(NA, nrow(domainInfoDf))
        }
    }
    return(domainInfoDf)
}

#' get pfam and smart domain links
#' @return dataframe with domain IDs and their database links
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}
getDomainLink <- function(info, domainDf) {
    featurePfam <- getDomainInfo(info, domainDf, "pfam")
    pfamDf <- createLinkTable(featurePfam, "pfam")
    if (nrow(pfamDf) > 0) pfamDf$db <- "PFAM"
    featureSmart <- getDomainInfo(info, domainDf, "smart")
    smartDf <- createLinkTable(featureSmart, "smart")
    if (length(smartDf) > 0) smartDf$db <- "SMART"
    featDf <- rbind(pfamDf, smartDf)
    if (nrow(featDf) < 1) stop("No PFAM/SMART domain found!")
    featDf <- subset(featDf, select = c(feature_id, db, link))
    colnames(featDf) <- c("Feature ID", "Source", "URL")
    return(featDf)
}

#' plot error message
#' @param featureDf dataframe contains 2 columns feature_id and acc
#' @param featureType pfam or smart
#' @return dataframe contains 2 columns feature_id and link
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}
createLinkTable <- function(featureDf, featureType) {
    featureDf <- featureDf[!(duplicated(featureDf)),]
    if (nrow(featureDf) > 0) {
        if (featureType == "pfam") {
            featureDf$link[is.na(featureDf$acc)] <- paste0(
                "<a href='http://pfam-legacy.xfam.org/family/",
                featureDf$feature_id,
                "' target='_blank'>", "PFAM", "</a>"
            )
            featureDf$link[!(is.na(featureDf$acc))] <- paste0(
                "<a href='https://www.ebi.ac.uk/interpro/entry/pfam/",
                featureDf$acc,
                "' target='_blank'>", "INTERPRO", "</a>"
            )
        } else {
            featureDf$link <- paste0(
                "<a href='http://smart.embl-heidelberg.de/smart/",
                "do_annotation.pl?BLAST=DUMMY&DOMAIN=",
                # featureDf$feature_id, "' target='_blank'>",
                gsub("_smart", "",featureDf$feature_id), "' target='_blank'>",
                "SMART", "</a>"
            )
        }
    }
    return(featureDf)
}

#' plot error message
#' @param domainDf dataframe contains feature information
#' @return dataframe as subset of domainDf
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}

createDomainInfoTable <- function(domainDf) {
    if (is.null(domainDf)) stop("No information!")

    selectedCols <- c("orthoID", "length", "feature", "start", "end")
    if (ncol(domainDf) <= 11) {
        if (!("length" %in% colnames(domainDf)))
            selectedCols <- selectedCols[!selectedCols == "length"]
        outDf <- subset(domainDf, select = selectedCols)
        if ("length" %in% colnames(domainDf)) {
            colnames(outDf) <- c(
                "orthoID", "Length", "Feature", "Start", "End"
            )
        } else {
            colnames(outDf) <- c(
                "orthoID", "Feature", "Start", "End"
            )
        }
    } else if (ncol(domainDf) == 17 & "evalue" %in% colnames(domainDf)) {
        selectedCols <- c(
            selectedCols, "evalue", "bitscore", "pStart", "pEnd", "pLen"
        )
        outDf <- subset(domainDf, select = selectedCols)
        colnames(outDf) <- c(
            "orthoID", "Length", "Feature", "Start", "End", "E-value",
            "Bit-score", "pHMM start", "pHMM end", "pHMM length"
        )
    } else {
        return(paste("Wrong number of columns:", ncol(domainDf)))
    }
    outDf$Feature <- sub("_", " ", outDf$Feature)
    outDf$`Gene ID` <- unlist(lapply(
        strsplit(outDf$orthoID, split = ":"), function (x) return(x[3])
    ))
    outDf <- subset(outDf, select = -c(orthoID))
    outDf <- outDf %>% dplyr::select(`Gene ID`, everything())
    return(outDf)
}

#' Get fasta sequences for AMPK-TOR and BUSCO Arthropoda online demo data
#' @param seqIDs list of sequences IDs. Set seqIDs = "all" if you want to get
#' all fasta sequences from the data set.
#' @param demoData name of demo data set (either "ampk-tor" or "arthropoda").
#' Default = "arthropoda".
#' @return A dataframe with one column contains sequences in fasta format.
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}
#' @seealso \code{\link{mainLongRaw}}
#' @examples
#' data("mainLongRaw", package="PhyloProfile")
#' seqIDs <- unique(as.character(mainLongRaw$orthoID))
#' getFastaDemo(seqIDs, "arthropoda")

getFastaDemo <- function(seqIDs = NULL, demoData = "arthropoda") {
    if (is.null(seqIDs)) stop("No sequence ID given!")
    demoBaseUrl<-"https://github.com/BIONF/phyloprofile-data/raw/master/rdata/"
    if (demoData == "ampk-tor" | demoData == "arthropoda") {
        if (demoData == "ampk-tor") {
            load(url(paste0(demoBaseUrl, "ampkTorFasta.RData")))
            faFile <- ampkTorFasta
        } else {
            load(url(paste0(demoBaseUrl, "arthropodaFasta.RData")))
            faFile <- arthropodaFasta
        }
        
        # get sequences
        if (length(seqIDs) == 1 & seqIDs[1] == "all")
            seqIDs <- names(faFile)
        return(data.frame(
            fasta = paste(
                paste0(">", seqIDs),
                lapply(
                    pmatch(seqIDs, names(faFile)),
                    function (x) as.character(faFile[[x]])
                ),
                sep = "\n"
            ), stringsAsFactors = FALSE
        ))
    }
}

#' Calculate the phylogenetic gene age from the phylogenetic profiles
#' @export
#' @usage estimateGeneAge(processedProfileData, taxaCount, rankName, refTaxon,
#'     var1CO, var2CO, percentCO, taxDB = NULL)
#' @param processedProfileData dataframe contains the full processed
#' phylogenetic profiles (see ?fullProcessedProfile or ?parseInfoProfile)
#' @param taxaCount dataframe counting present taxa in each supertaxon
#' @param rankName working taxonomy rank (e.g. "species", "genus", "family")
#' @param refTaxon reference taxon name (e.g. "Homo sapiens", "Homo" or
#' "Hominidae")
#' @param var1CO cutoff for var1. Default: c(0, 1)
#' @param var2CO cutoff for var2. Default: c(0, 1)
#' @param percentCO cutoff for percentage of species present in each
#' supertaxon. Default: c(0, 1)
#' @param taxDB Path to the taxonomy DB files
#' @return A dataframe contains estimated gene ages for the seed proteins.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom data.table setDT setnames
#' @importFrom dplyr count
#' @seealso \code{\link{parseInfoProfile}} for creating a full processed
#' profile dataframe; \code{\link{getNameList}} and
#' \code{\link{getTaxonomyMatrix}} for getting taxonomy info,
#' \code{\link{fullProcessedProfile}} for a demo input dataframe
#' @examples
#' library(dplyr)
#' data("fullProcessedProfile", package="PhyloProfile")
#' rankName <- "class"
#' refTaxon <- "Mammalia"
#' processedProfileData <- fullProcessedProfile
#' taxonIDs <- levels(as.factor(processedProfileData$ncbiID))
#' sortedInputTaxa <- sortInputTaxa(
#'     taxonIDs, rankName, refTaxon, NULL, NULL
#' )
#' taxaCount <- sortedInputTaxa %>% dplyr::count(supertaxon)
#' var1Cutoff <- c(0, 1)
#' var2Cutoff <- c(0, 1)
#' percentCutoff <- c(0, 1)
#' estimateGeneAge(
#'     processedProfileData,
#'     taxaCount,
#'     rankName,
#'     refTaxon,
#'     var1Cutoff, var2Cutoff, percentCutoff
#' )

estimateGeneAge <- function(
    processedProfileData, taxaCount, rankName, refTaxon,
    var1CO = c(0, 1), var2CO = c(0, 1), percentCO = c(0, 1), taxDB = NULL
){
    infoMsg <- paste(
        "This function supports only Species, Genus, Family, Class, Phylum,",
        "Kingdom and Superkingdom taxonomy ranks."
    )
    if (rankName == "strain" || rankName == "order") {
        stop("Invalid taxonomy rank!\n", infoMsg)
    }
    rankList <- c(
        "genus", "family", "class", "phylum", "kingdom", "norank_33154",
        "superkingdom", "root"
    )
    # get selected (super)taxon ID
    taxList <- getNameList(taxDB)
    superID <- taxList[
        taxList$fullName == refTaxon & taxList$rank == rankName,]$ncbiID
    # full non-duplicated taxonomy data
    Dt <- getTaxonomyMatrix(taxDB)
    # subset of taxonomy data, containing only ranks from rankList
    subDt <- Dt[, c("abbrName", rankList)]
    # get (super)taxa IDs for one of representative species
    firstLine <- Dt[Dt[, rankName] == superID, ][1, ]
    supFirstLine <- firstLine[, c("abbrName", rankList)]
    # compare each taxon IDs with selected taxon & create a "category" DF
    subDtTmp <- subDt
    if (supFirstLine$norank_33154 %in% c(554915, 33154)) {
        subDtTmp$norank_33154[
            subDtTmp$norank_33154 %in% c(554915, 33154) &
                subDtTmp$superkingdom == 2759
            ] <- supFirstLine$norank_33154
    } else {
        subDtTmp$norank_33154[
            !(subDtTmp$norank_33154 %in% c(554915, 33154)) &
                subDtTmp$superkingdom == 2759
            ] <- supFirstLine$norank_33154
    }
    catList <- lapply(
        seq(nrow(subDtTmp)), function (x) {
            cath <- subDtTmp[x, ] %in% supFirstLine
            cath <- paste0(cath, collapse = "")})
    catDf <- data.frame(ncbiID = as.character(subDtTmp$abbrName),
                        cath = do.call(rbind, catList), stringsAsFactors=FALSE)
    catDf$cath <- gsub("TRUE", "1", catDf$cath)
    catDf$cath <- gsub("FALSE", "0", catDf$cath)
    # get main input data
    mdData <- droplevels(processedProfileData)
    # filter by var1, var2 ..
    mdData <- subset(
        mdData, mdData$var1 >= var1CO[1] & mdData$var1 <= var1CO[2]
        & mdData$var2 >= var2CO[1] & mdData$var2 <= var2CO[2])
    # calculate % present taxa
    finalPresSpecDt <- calcPresSpec(mdData, taxaCount)
    mdData <- mdData[
        , c("geneID", "supertaxon","ncbiID","orthoID","var1","var2")
        ]
    mdDataFull <- Reduce(
        function(x, y) merge(x, y, by = c("geneID", "supertaxon"), all.x=TRUE),
        list(mdData, finalPresSpecDt))
    # add "category" into mdData
    mdDtExt <- merge(mdDataFull, catDf, by = "ncbiID", all.x = TRUE)
    mdDtExt$var1[mdDtExt$var1 == "NA" | is.na(mdDtExt$var1)] <- 0
    mdDtExt$var2[mdDtExt$var2 == "NA" | is.na(mdDtExt$var2)] <- 0
    # remove cat for "NA" orthologs and also for orthologs that dont fit cutoffs
    if (nrow(mdDtExt[mdDtExt$orthoID == "NA" | is.na(mdDtExt$orthoID), ]) > 0)
        mdDtExt[mdDtExt$orthoID == "NA" | is.na(mdDtExt$orthoID),]$cath <- NA
    mdDtExt <- mdDtExt[stats::complete.cases(mdDtExt), ]
    # filter by %specpres
    mdDtExt <- mdDtExt[
        mdDtExt$presSpec >=percentCO[1] & mdDtExt$presSpec<= percentCO[2],
        ]

    # get the furthest common taxon with selected taxon for each gene
    geneAgeDf <- as.data.frame(tapply(mdDtExt$cath, mdDtExt$geneID, min))
    data.table::setDT(geneAgeDf, keep.rownames = TRUE)[]
    data.table::setnames(geneAgeDf, seq_len(2), c("geneID", "cath"))  #col names
    row.names(geneAgeDf) <- NULL   # remove row names

    ### move NA cat to working taxonomy rank
    if (rankName == "genus")
        geneAgeDf$cath[is.na(geneAgeDf$cath)] <- "011111111"
    else if (rankName == "family")
        geneAgeDf$cath[is.na(geneAgeDf$cath)] <- "001111111"
    else if (rankName == "class")
        geneAgeDf$cath[is.na(geneAgeDf$cath)] <- "000111111"
    else if (rankName == "phylum")
        geneAgeDf$cath[is.na(geneAgeDf$cath)] <- "000011111"
    else if (rankName == "kingdom")
        geneAgeDf$cath[is.na(geneAgeDf$cath)] <- "000001111"
    else if (rankName == "superkingdom")
        geneAgeDf$cath[is.na(geneAgeDf$cath)] <- "000000011"
    else
        geneAgeDf$cath[is.na(geneAgeDf$cath)] <- "111111111"
    ### merge ranks that are lower than ref rank to ref rank
    index <- 0
    if (rankName != "species")
        index <- match(rankName, rankList)
    maxCat <- paste0(
        paste(rep(0,index), collapse = ""), paste(rep(1,9-index), collapse = "")
    )
    geneAgeDf$cath[geneAgeDf$cath > maxCat] <- maxCat
    ### convert cat into geneAge
    domainDfList <- lapply(
        geneAgeDf[geneAgeDf$cath == "000000001",]$geneID,
        function (x) {
            orthoList <- mdDtExt[
                mdDtExt$geneID == x & mdDtExt$cath == "000000001",]$ncbiID
            orthoDomainID <- levels(as.factor(
                subDt[subDt$abbrName %in% orthoList,]$superkingdom))
            orthoDomainName <- levels(as.factor(c(
                taxList$fullName[taxList$ncbiID == supFirstLine$superkingdom],
                taxList$fullName[taxList$ncbiID %in% orthoDomainID]
            )))
            if (length(orthoDomainName) == 3) {
                age <- paste0("10_", paste(orthoDomainName, collapse = "-"))
            } else {
                age <- paste0("09_", paste(orthoDomainName, collapse = "-"))
            }
            return(data.frame(geneID = x, age, stringsAsFactors = FALSE))
        }
    )
    konList <- lapply(
        geneAgeDf[geneAgeDf$cath == "000000111",]$geneID,
        function (x) {
            orthoList <- mdDtExt[
                mdDtExt$geneID == x & mdDtExt$cath == "000000111",]$ncbiID
            orthoKonID <- levels(as.factor(
                subDt[subDt$abbrName %in% orthoList,]$norank_33154))
            superKingdom <- taxList$ncbiID[
                taxList$ncbiID == supFirstLine$superkingdom
                & taxList$rank == "superkingdom"]
            if (superKingdom == 2759) {
                orthoKonID <- unique(c(orthoKonID, supFirstLine$norank_33154))
                if (length(orthoKonID) == 1) {
                    age <- paste0(
                        "06_", taxList$fullName[taxList$ncbiID == orthoKonID])
                } else {
                    ukon <- levels(as.factor(orthoKonID%in%c(33154, 554915)))
                    if (length(ukon) == 1) {
                        if (ukon == TRUE) {
                            age <- "07_Unikonta"
                        } else {
                            age <- "07_Bikonta"
                        }
                    } else {
                        age <- "08_Eukaryote"
                    }
                }
            } else {
                age <- paste0(
                    "08_", taxList$fullName[
                        taxList$ncbiID == supFirstLine$superkingdom])
            }
            return(data.frame(geneID = x, age, stringsAsFactors = FALSE))
        }
    )
    sarList <- lapply(
        geneAgeDf[geneAgeDf$cath == "000001111",]$geneID,
        function (x) {
            if (
                taxList$fullName[taxList$ncbiID == supFirstLine$kingdom] =="Sar"
            ) {
                orthoList <- mdDtExt[
                    mdDtExt$geneID == x & mdDtExt$cath == "000001111",]$ncbiID
                sarTaxDt <- Dt[
                    Dt$abbrName %in% orthoList,
                    colnames(Dt)[
                        colnames(Dt) %in%
                            c("norank_33630", "norank_543769", "norank_33634")]]
                check <- lapply(
                    sarTaxDt,
                    function(x) {
                        if (length(unique(x)) == 1) return(unique(x))
                        else return(NULL)
                    }
                )
                if (is.null(unlist(check))) {
                    age <- paste0(
                        "05_",
                        taxList$fullName[taxList$ncbiID == supFirstLine$kingdom]
                    )
                } else {
                    age <- paste0(
                        "05_",
                        taxList$fullName[taxList$ncbiID == unlist(check)])
                }
            } else {
                age <- paste0(
                    "05_",
                    taxList$fullName[taxList$ncbiID == supFirstLine$kingdom])
            }
            return(data.frame(geneID = x, age, stringsAsFactors = FALSE))
        }
    )
    geneAgeDfPre <- do.call(rbind, c(domainDfList, konList, sarList))
    if (!(is.null(geneAgeDfPre)))
        geneAgeDf <- merge(geneAgeDf, geneAgeDfPre, by = "geneID", all.x = TRUE)
    geneAgeDf$age[geneAgeDf$cath == "000000011"] <- paste0(
        "08_", taxList$fullName[taxList$ncbiID == supFirstLine$superkingdom])
    geneAgeDf$age[geneAgeDf$cath == "000011111"] <- paste0(
        "04_", taxList$fullName[taxList$ncbiID == supFirstLine$phylum])
    geneAgeDf$age[geneAgeDf$cath == "000111111"] <- paste0(
        "03_", taxList$fullName[taxList$ncbiID == supFirstLine$class])
    geneAgeDf$age[geneAgeDf$cath == "001111111"] <- paste0(
        "02_", taxList$fullName[taxList$ncbiID == supFirstLine$family])
    geneAgeDf$age[geneAgeDf$cath == "011111111"] <- paste0(
        "01_", taxList$fullName[taxList$ncbiID == supFirstLine$genus])
    geneAgeDf$age[geneAgeDf$cath == "111111111"] <- paste0(
        "00_", taxList$fullName[
            taxList$fullName == refTaxon & taxList$rank == rankName])
    # return geneAge data frame
    geneAgeDf <- geneAgeDf[, c("geneID", "cath", "age")]
    return(geneAgeDf)
}

#' Create data for plotting gene ages
#' @param geneAgeDf data frame containing estimated gene ages for seed proteins
#' @return A dataframe for plotting gene age plot containing the absolute number
#' and percentage of genes for each calculated evolutionary ages and the
#' corresponding position for writting those number on the plot.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom dplyr count
#' @export
#' @seealso \code{\link{estimateGeneAge}}
#' @examples
#' geneAgeDf <- data.frame(
#' geneID = c("100136at6656", "100265at6656", "101621at6656", "103479at6656"),
#' cath = c("0000001", "0000011", "0000001", "0000011"),
#' age = c("07_LUCA", "06_Eukaryota", "07_LUCA", "06_Eukaryota")
#' )
#' geneAgePlotDf(geneAgeDf)

geneAgePlotDf <- function(geneAgeDf){
    if (is.null(geneAgeDf)) stop("Gene age data is NULL!")
    age <- NULL
    plotDf <- geneAgeDf %>% dplyr::count(age)
    plotDf$level <- substring(plotDf$age, 1,2)
    levelDf <- data.frame(
        level = c("00", "01", "02", "03", "04", "05", "08", "10"),
        rank = c(
            " (Species)", " (Genus)", " (Family)", " (Class)", " (Phylum)",
            " (Kingdom)", " (Superkingdom)"," (LUCA)"
        ),
        stringsAsFactors = FALSE
    )
    plotDf <- merge(plotDf, levelDf, by = "level", all.x = TRUE)
    plotDf[is.na(plotDf)] <- ""
    plotDf$name <- paste0(substring(plotDf$age, 4), plotDf$rank)
    plotDf$name <- factor(plotDf$name, levels = plotDf$name)
    plotDf$percentage <- round(plotDf$n / sum(plotDf$n) * 100, 1)
    outDf <- plotDf[, c("name", "n", "percentage")]
    colnames(outDf) <- c("name", "count", "percentage")
    return(outDf)
}

#' Create gene age plot
#' @param geneAgePlotDf data frame required for plotting gene age (see
#' ?geneAgePlotDf)
#' @param textFactor increase factor of text size
#' @param font font of text. Default = Arial"
#' @return A gene age distribution plot as a ggplot2 object
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @import ggplot2
#' @importFrom RColorBrewer brewer.pal
#' @export
#' @seealso \code{\link{estimateGeneAge}} and \code{\link{geneAgePlotDf}}
#' @examples
#' geneAgePlotDf <- data.frame(
#'     name = c("Streptophyta (Phylum)", "Bikonta", "Eukaryota (Superkingdom)"),
#'     count = c(7, 1, 30),
#'     percentage = c(18, 3, 79)
#' )
#' createGeneAgePlot(geneAgePlotDf, 1, "sans")

createGeneAgePlot <- function (geneAgePlotDf, textFactor = 1, font = "Arial"){
    name <- count <- percentage <- x <- y <- NULL
    arrowDf <- data.frame(time = c(0, 0), y = c(0, nrow(geneAgePlotDf) + 1))
    n <- nlevels(as.factor(geneAgePlotDf$name))
    mycolors <- grDevices::colorRampPalette(
        RColorBrewer::brewer.pal(11, "Spectral"))(n)
    p <- ggplot(data = geneAgePlotDf, aes(x = name, y = count, fill = name)) +
        geom_bar(stat = "identity", width = 0.5) +
        geom_text(
            aes(label = paste0(percentage, "%")),
            position = position_dodge(0.9),
            size = 3.5 + 1 * textFactor, hjust = 0
        ) +
        geom_line(
            data = data.frame(x = c(0, 0), y = c(0, nrow(geneAgePlotDf) + 1)),
            aes(y = x, x = y),
            arrow = arrow(
                length = unit(0.30,"cm"), ends = "last", type = "closed"
            )
        ) +
        geom_text(
            data = NULL,
            aes(y = 0, x = (nrow(geneAgePlotDf) + 1) * 0.92, label = "time"),
            vjust = -0.5,
            angle = 90, alpha = 0.2
        ) +
        coord_flip() +
        theme_minimal() +
        theme(
            legend.position = "none",
            axis.title.y = element_blank(),
            axis.title.x = element_text(size = 12 * textFactor),
            axis.text = element_text(size = 12 * textFactor),
            text = element_text(family = font)
        ) +
        scale_fill_manual(values = mycolors) +
        scale_x_discrete(limits = rev(levels(geneAgePlotDf$name))) +
        labs(y = "Number of genes")
    return(p)
}

context("test estimation of gene ages")

test_that("test estimation of gene ages", {
    library(dplyr)
    data("fullProcessedProfile", package="PhyloProfile")
    rankName <- "class"
    refTaxon <- "Mammalia"
    var1Cutoff <- c(0,1)
    var2Cutoff <- c(0,1)
    percentCutoff <- c(0,1)
    taxonIDs <- levels(as.factor(fullProcessedProfile$ncbiID))
    sortedInputTaxa <- sortInputTaxa(
        taxonIDs, rankName, refTaxon, NULL
    )
    taxaCount <- sortedInputTaxa %>% dplyr::count(supertaxon)
    geneAge <- estimateGeneAge(
        fullProcessedProfile, taxaCount,
        rankName, refTaxon,
        var1Cutoff, var2Cutoff, percentCutoff
    )
    expect_true(
        geneAge$age[geneAge$geneID == "103479at6656"] == "08_Eukaryota"
    )
})

test_that("test plotting gene age plot", {
    geneAgeDf <- data.frame(
        geneID = c("OG_1017", "OG_1019"),
        cat = c("0000001", "0000001"),
        age = c("07_LUCA", "07_LUCA"),
        stringsAsFactors = FALSE
    )
    plotDf <- geneAgePlotDf(geneAgeDf)
    geneAgeText <- 1
    p <- createGeneAgePlot(plotDf, geneAgeText)
    expect_true(nrow(p$data) == 1)
})

#' Create data for percentage present taxa distribution
#' @usage createPercentageDistributionData(inputData = NULL, rankName = NULL,
#'     taxDB = NULL)
#' @param inputData dataframe contains raw input data in long format
#' (see ?mainLongRaw)
#' @param rankName name of the working taxonomy rank (e.g. "species", "family")
#' @param taxDB Path to the taxonomy DB files
#' @return A dataframe for analysing the distribution of the percentage of
#' species in the selected supertaxa, containing the seed protein IDs,
#' percentage of their orthologs in each supertaxon and the corresponding
#' supertaxon names.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom dplyr count
#' @export
#' @seealso \code{\link{mainLongRaw}}
#' @examples
#' data("mainLongRaw", package="PhyloProfile")
#' createPercentageDistributionData(mainLongRaw, "class")

createPercentageDistributionData <- function(
    inputData = NULL, rankName = NULL, taxDB = NULL
) {
    if (is.null(inputData) | is.null(rankName))
        stop("Input data or rank name cannot be NULL!")
    geneID <- ncbiID <- supertaxon <- NULL
    allMainRanks <- getTaxonomyRanks()
    if (!(rankName[1] %in% allMainRanks)) stop("Invalid taxonomy rank given!")
    # if (ncol(inputData) < 4) {
    #     colnames(inputData) <- c("geneID", "ncbiID", "orthoID")
    # } else if (ncol(inputData) < 5) {
    #     colnames(inputData) <- c("geneID", "ncbiID", "orthoID", "var1")
    # } else {
    #     colnames(inputData) <- c("geneID","ncbiID","orthoID","var1","var2")
    # }
    inputData <- inputData[, seq_len(3)]
    colnames(inputData) <- c("geneID", "ncbiID", "orthoID")
    # count number of inparalogs
    paralogCount <- inputData %>% dplyr::count(geneID, ncbiID)
    inputData <- merge(inputData, paralogCount, by = c("geneID", "ncbiID"))
    colnames(inputData)[ncol(inputData)] <- "paralog"
    # get sorted taxonomy list
    inputTaxonID <- getInputTaxaID(inputData)
    inputTaxonName <- getInputTaxaName(rankName, inputTaxonID, taxDB)
    refTaxon <- inputTaxonName$fullName[1]
    taxaTree <- NULL
    taxaList <- sortInputTaxa(inputTaxonID, rankName, refTaxon, taxaTree, taxDB)
    # calculate frequency of all supertaxa
    taxaCount <- taxaList %>% dplyr::count(supertaxon)
    # merge inputData, inputDatavar2 and taxaList to get taxonomy info
    taxaMdData <- merge(inputData, taxaList, by = "ncbiID")
    # calculate % present species
    finalPresSpecDt <- calcPresSpec(taxaMdData, taxaCount)
    finalPresSpecDt[!is.na(finalPresSpecDt$geneID),]
    return(finalPresSpecDt)
}

#' Create data for additional variable distribution
#' @usage createVariableDistributionData(inputData, var1Cutoff = c(0 ,1),
#'     var2Cutoff = c(0, 1))
#' @param inputData dataframe contains raw input data in long format
#' (see ?mainLongRaw)
#' @param var1Cutoff min and max cutoff for var1. Default = c(0, 1).
#' @param var2Cutoff min and max cutoff for var2. Default = c(0, 1).
#' @return A dataframe for analysing the distribution of the additional
#' variable(s) containing the protein (ortholog) IDs and the values of their
#' variables (var1 and var2).
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @export
#' @seealso \code{\link{mainLongRaw}}
#' @examples
#' data("mainLongRaw", package="PhyloProfile")
#' createVariableDistributionData(
#'     mainLongRaw, c(0, 1), c(0.5, 1)
#' )

createVariableDistributionData <- function(
    inputData, var1Cutoff = c(0, 1), var2Cutoff = c(0, 1)
) {
    if (ncol(inputData) < 4) {
        colnames(inputData) <- c("geneID", "ncbiID", "orthoID")
        splitDt <- inputData[, c("orthoID")]
    } else if (ncol(inputData) < 5) {
        colnames(inputData) <- c("geneID", "ncbiID", "orthoID", "var1")
        splitDt <- inputData[, c("orthoID", "var1")]
    } else {
        colnames(inputData) <- c("geneID", "ncbiID", "orthoID", "var1", "var2")
        splitDt <- inputData[, c("orthoID", "var1", "var2")]
    }
    splitDt$orthoID[splitDt$orthoID == "NA" | is.na(splitDt$orthoID)] <- NA
    splitDt <- splitDt[stats::complete.cases(splitDt), ]

    if (length(levels(as.factor(splitDt$var2))) == 1)
        if (levels(as.factor(splitDt$var2)) == "") splitDt$var2 <- 0

    # Filter based on variable cutoffs
    if ("var1" %in% colnames(splitDt))
        splitDt <- splitDt[
            splitDt$var1 >= var1Cutoff[1] & splitDt$var1 <= var1Cutoff[2],]
    if ("var2" %in% colnames(splitDt))
        splitDt <- splitDt[
            splitDt$var2 >= var2Cutoff[1] & splitDt$var2 <= var2Cutoff[2],]
    return(splitDt)
}

#' Create data for additional variable distribution (for a subset data)
#' @usage createVariableDistributionDataSubset(fullProfileData,
#'     distributionData, selectedGenes, selectedTaxa)
#' @param fullProfileData dataframe contains the full processed profiles (see
#' ?fullProcessedProfile, ?filterProfileData or ?fromInputToProfile)
#' @param distributionData dataframe contains the full distribution data (see
#' ?createVariableDistributionData)
#' @param selectedGenes list of genes of interest. Default = "all".
#' @param selectedTaxa list of taxa of interest Default = "all".
#' @return A dataframe for analysing the distribution of the additional
#' variable(s) for a subset of genes and/or taxa containing the protein
#' (ortholog) IDs and the values of their variables (var1 and var2).
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @export
#' @seealso \code{\link{parseInfoProfile}},
#' \code{\link{createVariableDistributionData}},
#' \code{\link{fullProcessedProfile}}, \code{\link{mainLongRaw}}
#' @examples
#' data("fullProcessedProfile", package="PhyloProfile")
#' data("mainLongRaw", package="PhyloProfile")
#' distributionData <- createVariableDistributionData(
#'     mainLongRaw, c(0, 1), c(0.5, 1)
#' )
#' selectedGenes <- "100136at6656"
#' selectedTaxa <- c("Mammalia", "Saccharomycetes", "Insecta")
#' createVariableDistributionDataSubset(
#'     fullProcessedProfile,
#'     distributionData,
#'     selectedGenes,
#'     selectedTaxa
#' )

createVariableDistributionDataSubset <- function(
    fullProfileData, distributionData,
    selectedGenes = "all", selectedTaxa = "all"
) {
    geneID <- orthoID <- var1.x <- var2.y <- supertaxonMod <- NULL
    # check parameters
    if (is.null(fullProfileData) | is.null(distributionData))
        stop("Full processed profiles or distribution data cannot be NULL!")
    # get geneID and supertaxon name for distributionData
    distributionDataName <- merge(
        distributionData, fullProfileData, by = "orthoID", all.x = TRUE
    )
    distributionDataName$supertaxonMod <- substr(
        distributionDataName$supertaxon, 6,
        nchar(as.character(distributionDataName$supertaxon))
    )
    distributionDataName <- subset(
        distributionDataName,
        select = c(orthoID, var1.x, var2.y, supertaxonMod, geneID)
    )
    colnames(distributionDataName) <- c(
        "orthoID", "var1", "var2", "supertaxonMod", "geneID"
    )
    # filter data
    if (selectedTaxa[1] == "all" & selectedGenes[1] != "all") {
        # select data from dataHeat for selected sequences only
        distributionData <- subset(
            distributionDataName, geneID %in% selectedGenes
        )
    } else if (selectedGenes[1] == "all" & selectedTaxa[1] != "all") {
        # select data from dataHeat for selected taxa only
        distributionData <- subset(
            distributionDataName, supertaxonMod %in% selectedTaxa
        )
    } else if (selectedGenes[1] == "all" & selectedTaxa[1] == "all") {
        return(distributionData)
    } else {
        # select data from dataHeat for selected sequences and taxa
        distributionData <- subset(
            distributionDataName,
            geneID %in% selectedGenes & supertaxonMod %in% selectedTaxa
        )
    }
    return(distributionData)
}

#' Create distribution plot
#' @description Create distribution plot for one of the additional variable or
#' the percentage of the species present in the supertaxa.
#' @usage createVarDistPlot(data, varName = "var", varType = "var1",
#'     percent = c(0, 1), textSize = 12)
#' @param data dataframe contains data for plotting (see
#' ?createVariableDistributionData, ?createVariableDistributionDataSubset or
#' ?createPercentageDistributionData)
#' @param varName name of the variable that need to be analyzed (either name of
#' variable 1 or variable 2 or "percentage of present taxa"). Default = "var".
#' @param varType type of variable (either "var1", "var2" or "presSpec").
#' Default = "var1".
#' @param percent range of percentage cutoff (between 0 and 1). Default = c(0,1)
#' @param textSize text size of the distribution plot (in px). Default = 12.
#' @return A distribution plot for the selected variable as a ggplot object
#' @import ggplot2
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @export
#' @seealso \code{\link{mainLongRaw}},
#' \code{\link{createVariableDistributionData}},
#' \code{\link{createVariableDistributionDataSubset}},
#' \code{\link{createPercentageDistributionData}}
#' @examples
#' data("mainLongRaw", package="PhyloProfile")
#' data <- createVariableDistributionData(
#'     mainLongRaw, c(0, 1), c(0.5, 1)
#' )
#' varName <- "Variable abc"
#' varType <- "var1"
#' percent <- c(0,1)
#' textSize <- 12
#' createVarDistPlot(
#'     data,
#'     varName,
#'     varType,
#'     percent,
#'     textSize
#' )

createVarDistPlot <- function(
    data, varName = "var", varType = "var1", percent = c(0, 1), textSize = 12
) {
    if (is.null(data)) stop("Input data cannot be NULL!")
    if (varType == "presSpec") {
        # remove presSpec < cutoffMin or > cutoffMax
        if (percent[1] > 0) {
            data <- data[
                data$presSpec >= percent[1] & data$presSpec <= percent[2],
            ]
        } else {
            if (percent[2] > 0) {
                data <- data[data$presSpec > 0 & data$presSpec <= percent[2], ]
            } else {
                data <- data[data$presSpec > 0, ]
            }
        }
    } else data <- data[!is.na(data[,varType]), ]

    data.mean <- mean(data[,varType])
    p <- ggplot(data, aes(x = data[,varType])) +
        geom_histogram(binwidth = .01, alpha = .5, position = "identity") +
        geom_vline(
            data = data, aes(xintercept = data.mean, colour = "red"),
            linetype = "dashed", linewidth = 1
        ) + theme_minimal()
    p <- p + theme(
            legend.position = "none",
            axis.title = element_text(size = textSize),
            axis.text = element_text(size = textSize)
        ) + labs(
            x = paste0(varName," (mean = ", round(mean(data[,varType]), 3),")"),
            y = "Frequency"
        )
    return(p)
}

context("test creating data and plots for distritbution analysis function")

test_that("test distritbution analyze functions", {
    library(dplyr)
    # raw input
    inputData <- createLongMatrix("mainWideTest.txt")
    # selected rank name
    rankName <- "species"
    # distribution data for percentage of species present in supertaxa
    percentDistributionData <- createPercentageDistributionData(
        inputData, rankName
    )
    expect_true(nrow(percentDistributionData) == 20)

    # distribution data for 2 additional variables
    distributionData <- createVariableDistributionData(
        inputData, c(0, 1), c(0.5, 1)
    )
    expect_true(nrow(distributionData) == 17)

    # distribution data for 2 additional variables of a subset of taxa
    inputTaxonID <- getInputTaxaID(inputData)
    inputTaxonName <- getInputTaxaName(rankName, inputTaxonID)
    refTaxon <- inputTaxonName$fullName[1]
    taxaTree <- NULL
    sortedTaxa <- sortInputTaxa(
        inputTaxonID, rankName, refTaxon, taxaTree
    )
    taxaCount <- sortedTaxa %>% dplyr::count(supertaxon)
    fullProfileData <- parseInfoProfile(
        inputData,
        sortedTaxa,
        taxaCount,
        999
    )
    selectedGenes <- c("100136at6656", "103479at6656")
    selectedTaxa <- c("Homo sapiens", "Branchiostoma floridae")
    subsetDistributionData <- createVariableDistributionDataSubset(
        fullProfileData,
        distributionData,
        selectedGenes,
        selectedTaxa
    )
    expect_true(ncol(subsetDistributionData) == 5)

    # plot distribution of var1
    p <- createVarDistPlot(
        distributionData, "variable 1 name", "var1", NULL, 12
    )
    expect_true(nrow(p$data) == 17)
})

#' Startup script for PhyloProfile
#' 1) install and load packages
#' 2) start the PhyloProfile app
library(PhyloProfile)
source("R/functions.R")

# List of dependent packages --------------------------------------------------
packages <- c(
    "ape", "BiocStyle", "bsplus", "data.table", "dplyr", "ggplot2", "gridExtra",
    "htmlwidgets", "shiny", "shinyFiles", "shinyjs", "scattermore", 
    "svglite", "plotly")

# Load packages
lapply(packages, library, character.only = TRUE)

#' Get fasta sequences from main input file in multi-fasta format
#' @export
#' @param seqIDs list of sequences IDs. Set seqIDs = "all" if you want to get
#' all fasta sequences from the input file.
#' @param file raw phylogenetic profile input file in multi-fasta format.
#' @return A dataframe with one column contains sequences in fasta format.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom Biostrings readAAStringSet
#' @examples
#' file <- system.file(
#'     "extdata", "test.main.fasta",
#'     package = "PhyloProfile", mustWork = TRUE
#' )
#' getFastaFromFasInput("all", file)

getFastaFromFasInput <- function(seqIDs = NULL, file = NULL) {
    if (is.null(seqIDs)) stop("No sequence ID given!")
    if (is.null(seqIDs) | is.null(file))
        stop("Sequence ID and input fasta file cannot be NULL!")
    faFile <- Biostrings::readAAStringSet(file)

    if (length(seqIDs) == 1 & seqIDs[1] == "all") seqIDs <- names(faFile)
    return(data.frame(
        fasta = paste(
            paste0(">", seqIDs),
            lapply(
                pmatch(seqIDs, names(faFile)),
                function (x) as.character(faFile[[x]])
            ),
            sep = "\n"
        ), stringsAsFactors = FALSE
    ))
}

#' Get fasta sequences from main input file in multi-fasta format
#' @export
#' @param seqIDs list of sequences IDs. Set seqIDs = "all" if you want to get
#' all fasta sequences from the concatenated input fasta file.
#' @param concatFasta input concatenated fasta file.
#' @return A dataframe with one column contains sequences in fasta format.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom Biostrings readAAStringSet
#' @examples
#' concatFasta <- system.file(
#'     "extdata", "fastaFiles/concatenatedFile.fa",
#'     package = "PhyloProfile", mustWork = TRUE
#' )
#' getFastaFromFasInput("all", concatFasta)

getFastaFromFile <- function(seqIDs = NULL, concatFasta = NULL) {
    if (is.null(seqIDs)) stop("No sequence ID given!")
    if (!is.null(concatFasta)) {
        # read fasta file and save sequences into dataframe
        faFile <- Biostrings::readAAStringSet(toString(concatFasta))

        if (length(seqIDs) == 1 & seqIDs[1] == "all") seqIDs <- names(faFile)
        if (
            length(unique(pmatch(seqIDs, names(faFile)))) == 1 &
            is.na(unique(pmatch(seqIDs, names(faFile)))[1])
        ) {
            return(data.frame(
                fasta = paste0("Please check FASTA header format!"),
                stringsAsFactors = FALSE
            ))
        } else {
            return(data.frame(
                fasta = paste(
                    paste0(">", seqIDs),
                    lapply(
                        pmatch(seqIDs, names(faFile)),
                        function (x) as.character(faFile[[x]])
                    ),
                    sep = "\n"
                ), stringsAsFactors = FALSE
            ))
        }
    } else {
        return(data.frame(
            fasta = paste0("Please provide FASTA file(s)!"),
            stringsAsFactors = FALSE
        ))
    }
}

#' Get fasta sequences
#' @description Get fasta sequences for the input phylogenetic profiles.
#' @export
#' @usage getFastaFromFolder(seqIDs = NULL, path = NULL, dirFormat = NULL,
#'     fileExt = NULL, idFormat = NULL)
#' @param seqIDs list of sequences IDs.
#' @param path path to fasta folder.
#' @param dirFormat directory format (either 1 for "path/speciesID.fa*" or 2 for
#' "path/speciesID/speciesID.fa*")
#' @param fileExt fasta file extension ("fa", "fasta", "fas" or "txt")
#' @param idFormat fasta header format (1 for ">speciesID:seqID", 2 for
#' ">speciesID@seqID", 3 for ">speciesID|seqID" or 4 for "seqID")
#' @return A dataframe with one column contains sequences in fasta format.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom Biostrings readAAStringSet
#' @seealso \code{\link{mainLongRaw}}
#' @examples
#' seqIDs <- "RAT@10116@1|D3ZUE4"
#' path <- system.file(
#'     "extdata", "fastaFiles", package = "PhyloProfile", mustWork = TRUE
#' )
#' dirFormat <- 1
#' fileExt <- "fa"
#' idFormat <- 3
#' getFastaFromFolder(seqIDs, path, dirFormat, fileExt, idFormat)

getFastaFromFolder <- function(
    seqIDs = NULL, path = NULL, dirFormat = NULL, fileExt = NULL, idFormat=NULL
) {
    if (is.null(seqIDs)) stop("No sequence ID given!")
    if (is.null(path)) return(
        data.frame(
            fasta = paste0("Please provide path to FASTA file(s)!"),
            stringsAsFactors = FALSE))
    if (path != "") {
        # get list of species IDs
        if (idFormat == 1) {
            specID <- unlist(strsplit(seqIDs, "\\:"))[1]
        } else if (idFormat == 2) {
            specID <- unlist(strsplit(seqIDs, "@"))[1]
        } else if (idFormat == 3) {
            specID <- unlist(strsplit(seqIDs, "\\|"))[1]
        }
        # read all specices FASTA files at once
        if (idFormat == 4) {
            fileList <- list.files(path, pattern = fileExt)
            if(length(fileList) == 0) {
                return(data.frame(
                    fasta = paste0(
                        "No FASTA files with ", fileExt, "found in ", path
                    ), stringsAsFactors = FALSE))
            } else {
                fileWithPath <- paste0(path, "/", fileList)
                faFile <- Biostrings::readAAStringSet(fileWithPath)
            }
        } else {
            specID <- as.character(levels(as.factor(specID)))
            specID <- specID[specID != ""]
            file <- paste0(path, "/", specID, ".", fileExt)
            if (dirFormat == 2)
                file <- paste0(path, "/", specID, "/", specID, ".", fileExt)
            file <- file[file.exists(file)]
            if (length(file) == 0) {
                return(data.frame(
                    fasta = paste0(
                        "No FASTA files found in ", path, "! Please check ",
                        "settings for file extension and ID format again."
                    ), stringsAsFactors = FALSE
                ))
            } else faFile <- Biostrings::readAAStringSet(file)
        }
        # now get selected sequences
        if (length(unique(pmatch(seqIDs, names(faFile)))) == 1
            & is.na(unique(pmatch(seqIDs, names(faFile)))[1])) {
            return(data.frame(
                fasta = paste0("Please check FASTA header format!"),
                stringsAsFactors = FALSE))
        } else {
            return(data.frame(
                fasta = paste(
                    paste0(">", seqIDs),
                    lapply(
                        pmatch(seqIDs, names(faFile)),
                        function (x) as.character(faFile[[x]])
                    ), sep = "\n"), stringsAsFactors = FALSE))
        }
    } else {
        return(data.frame(
            fasta = paste0("Please provide path to FASTA file(s)!"),
            stringsAsFactors = FALSE))
    }
}

#' Prepare data for dimension reduction
#' @export
#' @usage prepareDimRedData(longDf = NULL, taxonRank = NULL, type = "taxa",
#'     taxDB = NULL, filterVar = "both", cutoff = 0, groupLabelsBy = "taxa")
#' @param longDf input phyloprofile file in long format
#' @param taxonRank taxonomy rank for labels (e.g. "phylum")
#' @param type type of clustering, either "taxa" (default) or "genes"
#' @param taxDB path to taxonomy database
#' @param filterVar choose variable (either "var1", "var2" or "both") to filter
#' the data. Default: "both"
#' @param cutoff cutoff to filter data values. Default: 0
#' @param groupLabelsBy group labels by the number of "taxa" (default) or
#' "genes"
#' @return A dataframe in wide format
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @examples
#' rawInput <- system.file(
#'    "extdata", "test.main.long", package = "PhyloProfile", mustWork = TRUE
#' )
#' longDf <- createLongMatrix(rawInput)
#' prepareDimRedData(longDf, "phylum")

prepareDimRedData <- function(
    longDf = NULL, taxonRank = NULL, type = "taxa", taxDB = NULL,
    filterVar = "both", cutoff = 0, groupLabelsBy = "taxa"
) {
    if (is.null(longDf)) stop("Input data cannot be NULL")
    if (is.null(taxonRank)) stop("Taxon rank must be specified!")
    FAS_F <- FAS_B <- geneID <- ncbiID <- abbrName <- fullName <- NULL
    var1 <- var2 <- Freq <- Label <- NULL
    if (type == "genes") groupLabelsBy <- "genes"
    # Handle column renaming and filtering based on input dimensions
    colNames <- c("geneID", "ncbiID", "orthoID", "var1", "var2", "geneName")
    colNames <- colNames[seq_len(ncol(longDf))]
    colnames(longDf) <- colNames
    # Add default values for missing columns
    if (!"var1" %in% colnames(longDf)) longDf$var1 <- 1
    # Apply filters based on cutoff values
    filterFlag <- ifelse("var1" %in% colnames(longDf), 1, 0)
    if (filterFlag) {
        longDf <- longDf %>%
            filter(
                (filterVar == "both" & var1 >= cutoff & var2 >= cutoff) |
                    (filterVar == "var1" & var1 >= cutoff) |
                    (filterVar == "var2" & var2 >= cutoff)
            )
    }
    # calculate mean of var1 and var2 (if var2 available)
    if (all(c("var1", "var2") %in% colnames(longDf)))
        longDfSub <- longDf %>% mutate(var1 = (var1 + var2) / 2)
    longDfSub <- longDfSub %>% select(geneID, ncbiID, var1)

    # get taxon names for input taxa based on a selected supertaxon rank
    taxMatrix <- getTaxonomyMatrix(taxDB)
    nameList <- getNameList(taxDB)
    superTaxonDf <- taxMatrix %>%
        filter(abbrName %in% longDfSub$ncbiID) %>%
        select(abbrName, one_of(taxonRank))
    colnames(superTaxonDf) <- c("ncbiID", "superID")
    superTaxonDf <- dplyr::left_join(
        superTaxonDf, nameList, by = c("superID" = "ncbiID")
    )
    # transform to wide format, add taxon names and ortholog count
    # if co-orthologs present, get max value (e.g. FAS) as the representative
    if (type == "taxa") {
        wideDf <- data.table::dcast(
            setDT(longDfSub), ncbiID ~ geneID, value.var = c("var1"),
            fun.aggregate = max, fill = -1
        )
        wideDf <- dplyr::left_join(
            wideDf, superTaxonDf %>% select(ncbiID, Label = fullName),
            by = "ncbiID"
        )
        # add count (how many seed genes each taxon has, excluding co-orthologs)
        seedWithTax <- longDfSub %>% select(geneID, ncbiID)
        seedWithTax <- seedWithTax[!duplicated(seedWithTax),]
        countDf <- data.frame(table(seedWithTax$ncbiID))
        colnames(countDf) <- c("ncbiID", "Freq")
        wideDf <- dplyr::left_join(
            wideDf, countDf, by = c("ncbiID")
        )
    } else {
        wideDf <- data.table::dcast(
            setDT(longDfSub), geneID ~ ncbiID, value.var = c("var1"),
            fun.aggregate = max, fill = -1
        )
        wideDf$Label <- wideDf$geneID
        # add count (how many taxa has orthologs for each seed gene)
        seedWithTax <- longDfSub %>% select(geneID, ncbiID)
        seedWithTax <- seedWithTax[!duplicated(seedWithTax),]
        countDf <- data.frame(table(seedWithTax$geneID))
        colnames(countDf) <- c("geneID", "Freq")
        wideDf <- dplyr::left_join(
            wideDf, countDf, by = c("geneID")
        )
    }
    # calculate genes / taxa frequency for each label
    if (groupLabelsBy == "genes") {
        countFreqDf <- data.frame(
            wideDf %>% group_by(Label) %>% summarise(n = max(Freq))
        )
    } else {
        countFreqDf <- data.frame(wideDf %>% count(Label))
    }
    wideDf <- left_join(wideDf, countFreqDf, by = "Label")
    wideDf$n <- as.numeric(wideDf$n)
    wideDf$Label <- as.character(wideDf$Label)
    return(data.frame(wideDf, check.names = FALSE))
}

#' Perform dimension reduction 2D
#' @export
#' @usage dimReduction(data4dimRed = NULL, by = "taxa", type = "binary",
#'     randomSeed = 123, reductionTechnique = "umap", dimension = "2d",
#'     tsneIter = 1000)
#' @param data4dimRed data for dimension reduction (from prepareDimRedData)
#' @param by cluster data by "taxa" (default) or "genes"
#' @param type type of data, either "binary" (default) or "non-binary"
#' @param randomSeed random seed. Default: 123
#' @param reductionTechnique dimensionality reduction technique, either "umap"
#' (default) or "tsne"
#' @param dimension either "2d" (default) or "3d"
#' @param tsneIter number of iterations for t-SNE. Default: 1000
#' @import umap tsne
#' @return A table contains coordinates of the 2D dimension reduction
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @seealso \code{\link{prepareDimRedData}}
#' @examples
#' rawInput <- system.file(
#'    "extdata", "test.main.long", package = "PhyloProfile", mustWork = TRUE
#' )
#' longDf <- createLongMatrix(rawInput)
#' data4dimRed <- prepareDimRedData(longDf, "phylum")
#' dimReduction(data4dimRed)

dimReduction <- function(
    data4dimRed = NULL, by = "taxa", type = "binary", randomSeed = 123,
    reductionTechnique = "umap", dimension = "2d", tsneIter = 1000
) {
    if (is.null(data4dimRed)) stop("Input data cannot be NULL!")
    # Determine subset columns based on `by`
    subsetCols <- if (by == "taxa") {
        setdiff(colnames(data4dimRed), c("ncbiID", "Label", "Freq", "n"))
    } else {
        setdiff(colnames(data4dimRed), c("geneID", "Label", "Freq", "n"))
    }
    subsetDt <- data4dimRed[, subsetCols, drop = FALSE]
    # Convert to binary if required
    if (type == "binary") {
        subsetDt <- ifelse(subsetDt >= 0, 1, 0)
    }
    # Define the dimensionality
    dim <- ifelse(dimension == "2d", 2, 3)
    # Perform dimensionality reduction
    outDf <- switch(
        reductionTechnique,
        umap = {performUmap(subsetDt, randomSeed, dim)},
        tsne = {
            tsne::tsne(subsetDt, initial_dims=dim, k=dim, max_iter = tsneIter)
        },
        pca = {performPCA(subsetDt)},
        stop("Unsupported reduction technique: ", reductionTechnique)
    )
    return(outDf)
}

#' Helper function to handle UMAP logic
#' @param umapDt data matrix for UMAP
#' @param randomSeed random seed. Default: 123
#' @param dim dimension, either 2 for 2D (default) or 3 for 3D
#' @import umap
#' @return A table contains coordinates UMAP reduction
#' @author Vinh Tran tran@bio.uni-frankfurt.de

performUmap <- function(umapDt, randomSeed = 123, dim = 2) {
    # Attempt UMAP reduction with fallback for sample size issues
    tryCatch({
        suppressWarnings(
            umap::umap(
                umapDt, random_state = randomSeed, preserve.seed = TRUE,
                n_components = dim
            )$layout
        )
    }, error = function(cond) {
        message("Error with UMAP: ", conditionMessage(cond))
        fallbackUmap(umapDt, randomSeed, dim)
    }, warning = function(cond) {
        message("Warning with UMAP: ", conditionMessage(cond))
        fallbackUmap(umapDt, randomSeed, dim)
    })
}

#' Fallback for UMAP in case of insufficient samples
#' @param umapDt data matrix for UMAP
#' @param randomSeed random seed. Default: 123
#' @param dim dimension, either 2 for 2D (default) or 3 for 3D
#' @import umap
#' @return A table contains coordinates UMAP reduction
#' @author Vinh Tran tran@bio.uni-frankfurt.de

fallbackUmap <- function(umapDt, randomSeed, dim) {
    warning("PROBLEM: Too few samples for UMAP! Adjusting n_neighbors.")
    umap::umap(
        umapDt, random_state = randomSeed, preserve.seed = TRUE,
        n_neighbors = max(1, nrow(umapDt) - 1), n_components = dim
    )$layout
}


#' Helper function to perform PCA
#' @param pcaDt data matrix for PCA
#' @return A table contains coordinates of the first 3 PCs
#' @author Vinh Tran tran@bio.uni-frankfurt.de

performPCA <- function(pcaDt) {
    if (!is.data.frame(pcaDt)) as.data.frame(pcaDt)
    # Perform PCA
    prin_comp <- stats::prcomp(pcaDt)
    
    # Extract the importance (proportion of variance explained by each PC)
    importancePCs <- as.data.frame(prin_comp$sdev^2) / sum(prin_comp$sdev^2)
    colnames(importancePCs) <- "Proportion of Variance"
    # Print the Proportion of Variance for the first 3 PCs
    importancePCs <- importancePCs[seq_len(3), , drop = FALSE]
    # Check if the first 3 PCs explain at least 80% of the variance
    sumVariance <- sum(importancePCs$`Proportion of Variance`)
    if (sumVariance < 0.8) {
        msg <- paste0(
            "The first 3 PCs can only explain ", round(sumVariance*100),
            "% of the data variance (<80%)!"
        )
        warning(msg)
    }
    
    # Extract the principal components scores
    components <- as.data.frame(prin_comp$x)
    # Return only the first 3 principal components
    return(components[, seq_len(3), drop = FALSE])
}


#' Reduce the number of labels for DIM reduction plot based on the
#' gene/taxon frequency
#' @export
#' @usage groupLabelDimRedData(data4dimRed = NULL, freqCutoff = c(0,200))
#' @param data4dimRed data for dimension reduction (from prepareDimRedData)
#' @param freqCutoff gene/taxon frequency cutoff range. Any labels that are
#' outside of this range will be assigned as [Other]
#' @return A dataframe similar to input data4dimRed, but with modified Label
#' column, where less frequent labels are grouped together as "Other"
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @seealso \code{\link{prepareDimRedData}}
#' @examples
#' rawInput <- system.file(
#'    "extdata", "test.main.long", package = "PhyloProfile", mustWork = TRUE
#' )
#' longDf <- createLongMatrix(rawInput)
#' data4dimRed <- prepareDimRedData(longDf, "phylum")
#' groupLabelDimRedData(data4dimRed, freqCutoff = c(3,5))

groupLabelDimRedData <- function(data4dimRed = NULL, freqCutoff = c(0,200)) {
    if (is.null(data4dimRed) || length(data4dimRed) == 0)
        stop("Input data cannot be NULL or EMPTY!")
    data4dimRed$Label[
        data4dimRed$n < freqCutoff[1] | data4dimRed$n > freqCutoff[2]
    ] <- "[Other]"
    return(data4dimRed)
}

#' Generate data for dimension reduction plot
#' @export
#' @usage createDimRedPlotData(dimRedCoord = NULL, data4dimRed = NULL,
#'     freqCutoff = c(0,200), excludeTaxa = "None", currentNCBIinfo = NULL)
#' @param dimRedCoord data contains DIM reduction coordinates (from
#' dimReduction)
#' @param data4dimRed data for dimension reduction (from prepareDimRedData())
#' @param freqCutoff gene/taxon frequency cutoff range. Any labels that are
#' outside of this range will be assigned as [Other]
#' @param excludeTaxa hide taxa from plot. Default: "None"
#' @param currentNCBIinfo table/dataframe of the pre-processed NCBI taxonomy
#' data (/PhyloProfile/data/preProcessedTaxonomy.txt)
#' @importFrom utils tail
#' @return A plot as ggplot object
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @seealso \code{\link{prepareDimRedData}}, \code{\link{dimReduction}}
#' @examples
#' rawInput <- system.file(
#'    "extdata", "test.main.long", package = "PhyloProfile", mustWork = TRUE
#' )
#' longDf <- createLongMatrix(rawInput)
#' data4dimRed <- prepareDimRedData(longDf, "phylum")
#' dimRedCoord <- dimReduction(data4dimRed)
#' createDimRedPlotData(dimRedCoord, data4dimRed)

createDimRedPlotData <- function(
    dimRedCoord = NULL, data4dimRed = NULL, freqCutoff = c(0, 200),
    excludeTaxa = "None", currentNCBIinfo = NULL
) {
    if (is.null(dimRedCoord) || is.null(data4dimRed) ||
        length(dimRedCoord) == 0 || length(data4dimRed) == 0) {
        stop("Input data cannot be NULL or EMPTY!")
    }
    Label <- Freq <- NULL
    data4dimRed$X <- dimRedCoord[,1]
    data4dimRed$Y <- dimRedCoord[,2]
    if (ncol(dimRedCoord) == 3) data4dimRed$Z <- dimRedCoord[,3]
    # join less freq items into "other"
    data4dimRed <- groupLabelDimRedData(data4dimRed, freqCutoff)
    # exclude taxa
    if (length(excludeTaxa) > 0) {
        data4dimRed$X[data4dimRed$Label %in% excludeTaxa] <- NA
        data4dimRed$Y[data4dimRed$Label %in% excludeTaxa] <- NA
    }
    # convert tax IDs into names
    if ("ncbiID" %in% colnames(data4dimRed)) {
        if (is.null(currentNCBIinfo)) {
            dataPath <- system.file(
                "PhyloProfile", "data/",
                package = "PhyloProfile", mustWork = TRUE
            )
            nameFullFile <- paste0(dataPath, "/preProcessedTaxonomy.txt")
            if (file.exists(nameFullFile)) {
                currentNCBIinfo <-as.data.frame(data.table::fread(nameFullFile))
            }
        }
        if (!is.null(currentNCBIinfo)) {
            id2nameDf <- PhyloProfile::id2name(
                gsub("ncbi","",unique(data4dimRed$ncbiID)), currentNCBIinfo
            )
            id2nameDf$ncbiID <- paste0("ncbi", id2nameDf$ncbiID)
            data4dimRed <- merge(
                data4dimRed, id2nameDf, by = "ncbiID", all.x = TRUE
            )
        } else {
            data4dimRed$fullName <- "NA"
        }
    }
    return(data4dimRed)
}


#' Create dimension reduction plot
#' @export
#' @usage plotDimRed(plotDf = NULL, legendPos = "bottom",
#'     colorPalette = "Set2", transparent = 0, textSize = 12, font = "Arial",
#'     highlightTaxa = NULL, dotZoom = 0)
#' @param plotDf data for dimension reduction 2D plot
#' @param legendPos position of legend. Default: "right"
#' @param colorPalette color palette. Default: "Set2"
#' @param transparent transparent level (from 0 to 1). Default: 0
#' @param textSize size of axis and legend text. Default: 12
#' @param font font of text. Default = Arial"
#' @param highlightTaxa list of taxa to be highlighted
#' @param dotZoom dot size zooming factor. Default: 0
#' @return A plot as ggplot object
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @seealso \code{\link{prepareDimRedData}}, \code{\link{dimReduction}},
#' \code{\link{createDimRedPlotData}}
#' @examples
#' rawInput <- system.file(
#'    "extdata", "test.main.long", package = "PhyloProfile", mustWork = TRUE
#' )
#' longDf <- createLongMatrix(rawInput)
#' data4dimRed <- prepareDimRedData(longDf, "phylum")
#' dimRedCoord <- dimReduction(data4dimRed)
#' plotDf <- createDimRedPlotData(dimRedCoord, data4dimRed)
#' plotDimRed(plotDf, font = "sans")

plotDimRed <- function(
    plotDf = NULL, legendPos = "bottom", colorPalette = "Set2",
    transparent = 0, textSize = 12, font = "Arial", highlightTaxa = NULL,
    dotZoom = 0
) {
    if (is.null(plotDf)) stop("Input data cannot be NULL!")
    X <- Y <- Label <- Freq <- NULL
    # add colors
    plotDf <- addDimRedTaxaColors(plotDf, colorPalette, highlightTaxa)
    if ("color" %in% colnames(plotDf)) {
        colorScheme <- structure(
            plotDf$color, .Names = plotDf$Label
        )
    } else {
        colorScheme <- structure(
            head(
                suppressWarnings(
                    RColorBrewer::brewer.pal(
                        nlevels(as.factor(plotDf$Label)), colorPalette
                    )
                ), levels(as.factor(plotDf$Label))
            ), .Names = levels(as.factor(plotDf$Label))
        )
    }
    # generate plot
    plot <- ggplot(plotDf, aes(x = X, y = Y, color = Label)) +
        geom_point(aes(size = Freq), alpha = 1 - transparent) +
        geom_rug(alpha = 1) +
        theme_minimal() +
        labs(x = "", y = "")+
        scale_radius(range = c(1 + dotZoom, 6 + dotZoom))
    # change legend title
    if ("ncbiID" %in% colnames(plotDf)) {
        plot <- plot + guides(
            color = guide_legend(override.aes = list(alpha = 1), ncol = 5),
            size = guide_legend(title = "Number of genes", ncol = 2)
        )
    } else
        plot <- plot + guides(
            color = guide_legend(override.aes = list(alpha = 1), ncol = 5),
            size = guide_legend(title = "Number of taxa", ncol = 1)
        )
    plot <- plot + theme(
            legend.position = legendPos,
            legend.text = element_text(size = textSize),
            legend.title = element_text(size = textSize),
            axis.text = element_text(size = textSize),
            axis.title = element_text(size = textSize),
        )
    plot <- plot + scale_color_manual(values = colorScheme)
    plot <- plot + theme(text = element_text(family = font))
    return(plot)
}


#' Create dimension reduction 3D plot
#' @export
#' @usage plotDimRed3D(plotDf = NULL, legendPos = "bottom",
#'     colorPalette = "Set2", transparent = 0,highlightTaxa = NULL,
#'     dotZoom = 0)
#' @param plotDf data for dimension reduction 3D plot
#' @param legendPos position of legend. Default: "right"
#' @param colorPalette color palette. Default: "Set2"
#' @param transparent transparent level (from 0 to 1). Default: 0
#' @param highlightTaxa list of taxa to be highlighted
#' @param dotZoom dot size zooming factor. Default: 0
#' @return A plot as ggplot object
#' @rawNamespace import(plotly, except = c(last_plot, select))
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @seealso \code{\link{prepareDimRedData}}, \code{\link{dimReduction}},
#' \code{\link{createDimRedPlotData}}
#' @examples
#' rawInput <- system.file(
#'    "extdata", "test.main.long", package = "PhyloProfile", mustWork = TRUE
#' )
#' longDf <- createLongMatrix(rawInput)
#' data4dimRed <- prepareDimRedData(longDf, "phylum")
#' dimRedCoord3d <- dimReduction(data4dimRed, dimension = "3d")
#' plotDf <- createDimRedPlotData(dimRedCoord3d, data4dimRed)
#' plotDimRed3D(plotDf)

plotDimRed3D <- function(
    plotDf = NULL, legendPos = "bottom", colorPalette = "Set2",
    transparent = 0, highlightTaxa = NULL, dotZoom = 0
) {
    if (is.null(plotDf)) stop("Input data cannot be NULL!")
    X <- Y <- Z <- Label <- Freq <- color <- NULL
    if (!("Z" %in% colnames(plotDf))) stop("PlotDf seems to be 2D plot!")
    # add colors
    plotDf <- addDimRedTaxaColors(plotDf, colorPalette, highlightTaxa)
    colorDf <- unique(plotDf %>% select(Label, color))
    # calculate min and max dot size
    minSize <- dotZoom
    maxSize <- (max(plotDf$Freq)*dotZoom/min(plotDf$Freq))
    # generate 3D scatter plot
    if ("ncbiID" %in% colnames(plotDf)) {
        plot <- plot_ly(
            plotDf, x = ~X, y = ~Y, z = ~Z,
            hovertemplate = ~paste0(
                "Taxon ID: ", gsub("ncbi","",ncbiID), "<br>Name: ", fullName,
                "<br>Label: ", Label, "<br>Freq: ", Freq
            )
        )
    } else {
        plot <- plot_ly(
            plotDf, x = ~X, y = ~Y, z = ~Z,
            hovertemplate = ~paste0(
                "Gene ID: ", geneID, "<br>Freq: ", Freq
            )
        )
    }

    if (dotZoom == 0) {
        plot <- plot %>% add_markers(
            color = ~Label, colors = colorDf$color, size = ~Freq,
            alpha = 1 - transparent, span = I(0)
        )
    } else {
        plot <- plot %>% add_markers(
            color = ~Label, colors = colorDf$color, size = ~Freq,
            sizes = c(minSize, maxSize), alpha = 1 - transparent, span = I(0)
        )
    }
    if (legendPos == "none") plot <- plot %>% hide_legend()
    return(plot |> layout(dragmode = "lasso"))
}


#' Add colors for taxa in dimension reduction plot
#' @usage addDimRedTaxaColors(plotDf = NULL, colorPalette = "Set2",
#'     highlightTaxa = NULL)
#' @param plotDf data for dimension reduction plot
#' @param colorPalette color palette. Default: "Set2"
#' @param highlightTaxa list of taxa to be highlighted
#' @return A dataframe for dimension reduction plot with an additional column
#' for the assigned color to each taxon
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @seealso \code{\link{prepareDimRedData}}, \code{\link{dimReduction}},
#' \code{\link{createDimRedPlotData}}
#' @examples
#' rawInput <- system.file(
#'    "extdata", "test.main.long", package = "PhyloProfile", mustWork = TRUE
#' )
#' longDf <- createLongMatrix(rawInput)
#' data4dimRed <- prepareDimRedData(longDf, "phylum")
#' dimRedCoord <- dimReduction(data4dimRed)
#' plotDf <- createDimRedPlotData(dimRedCoord, data4dimRed)
#' PhyloProfile:::addDimRedTaxaColors(plotDf, colorPalette = "Set2")

addDimRedTaxaColors <- function(
    plotDf = NULL, colorPalette = "Set2", highlightTaxa = NULL
) {
    if (is.null(plotDf)) stop("plotDf cannot be null!")
    allTaxa <- levels(as.factor(plotDf$Label))
    allColors <- getQualColForVector(allTaxa)
    if (checkColorPalette(allTaxa, colorPalette)) {
        allColors <-
            suppressWarnings(head(
                RColorBrewer::brewer.pal(length(allTaxa), colorPalette),
                length(allTaxa)
            ))
    }
    colorScheme <- data.frame(color = allColors, Label = allTaxa)
    if (!(is.null(highlightTaxa)))
        colorScheme$color[!(colorScheme$Label %in% highlightTaxa)] <- "#d4d6d9"
    plotDf <- merge(plotDf, colorScheme, by = "Label", all.x = TRUE)
    return(plotDf)
}

#' Gene age estimation plot
#' @param data for gene age plot (from reactive fn geneAgeDf)
#' @param geneAgeWidth plot width (from input$geneAgeWidth)
#' @param geneAgeHeight plot width (from input$geneAgeHeight)
#' @param geneAgeText text size (from input$geneAgeText)
#' @return list of genes of a selected age
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}

source("R/functions.R")

plotGeneAgeUI <- function(id) {
    ns <- NS(id)
    tagList(
        column(
            8,
            downloadButton(ns("geneAgePlotDownload"),
                           "Download plot", class = "butDL"),
            tags$head(
                tags$style(HTML(
                    ".butDL{background-color:#476ba3;} .butDL{color: white;}"))
            ),
            uiOutput(ns("geneAge.ui"))
        ),
        column(1),
        column(
            3,
            downloadButton(ns("geneAgeTableDownload"), "Download gene list"),
            br(), br(),
            DT::dataTableOutput(ns("geneAge.table"))
        )
    )
}

plotGeneAge <- function(input, output, session,
                        data, geneAgeWidth, geneAgeHeight, geneAgeText, font) {
    # render gene age plot -----------------------------------------------------
    output$geneAgePlot <- renderPlot({
        if (is.null(data())) stop("No input data available!")
        createGeneAgePlot(geneAgePlotDf(data()), geneAgeText(), font())
    })

    output$geneAge.ui <- renderUI({
        ns <- session$ns
        # shinycssloaders::withSpinner(
            plotOutput(
                ns("geneAgePlot"),
                width = 800 * geneAgeWidth(),
                height = 300 * geneAgeHeight(),
                click = ns("plotClickGeneAge")
            )
        # )
    })

    output$geneAgePlotDownload <- downloadHandler(
        filename = function() {
            "geneAgePlot.svg"
        },
        content = function(file) {
            ggsave(
                file,
                plot = createGeneAgePlot(
                    geneAgePlotDf(data()), geneAgeText(), font()
                ),
                width = 800 * geneAgeWidth() * 0.035,
                height = 300 * geneAgeHeight() * 0.035,
                units = "cm", dpi = 300, device = "svg"
            )
        }
    )

    # render genAge.table based on clicked point on geneAgePlot ---------------
    selectedgeneAge <- reactive({
        if (is.null(data())) stop("No input data available!")
        selectedGene <- getSelectedGeneAge(data(), input$plotClickGeneAge$y)
        return(selectedGene)
    })

    output$geneAge.table <- DT::renderDataTable(
        options = list(searching = FALSE, pageLength = 10
    ), {
        if (is.null(data())) stop("No input data available!")
        if (is.null(input$plotClickGeneAge$x)) return()
        data <- as.data.frame(selectedgeneAge())
        data
    })

    output$geneAgeTableDownload <- downloadHandler(
        filename = function(){
            c("selectedGeneList.out")
        },
        content = function(file){
            dataOut <- selectedgeneAge()
            write.table(dataOut, file,
                        sep = "\t",
                        row.names = FALSE,
                        quote = FALSE)
        }
    )

    return(selectedgeneAge)
}

#' get list of gene for a selected age
#' @param geneAgeDf data of estimated gene age (from fn "estimateGeneAge")
#' @param clickedX x coordinate of selected age

getSelectedGeneAge <- function(geneAgeDf, clickedY){
    data <- geneAgeDf
    # calculate the coordinate range for each age group
    rangeDf <- data %>% dplyr::count(age)
    rangeDf <- rangeDf[seq(dim(rangeDf)[1],1),]
    if (is.null(clickedY)) return()
    else {
        corY <- round(clickedY)
        selectAge <- as.character(rangeDf$age[corY])
        subData <- subset(data, age == selectAge)
        data <- data[data$age == selectAge, ]
        geneList <- list(levels(as.factor(subData$geneID)))
        names(geneList) <- substring(selectAge, 4)
        return(geneList)
    }
}

#' Core gene identification
#' @param filteredData full processed main data
#' (from reactive fn "getDataFiltered")
#' @param rankSelect selected taxonomy rank (input$rankSelect)
#' @param taxaCore selected list of taxa (input$taxaCore)
#' @param percentCore cutoff of percentage taxa present in a supertaxon
#' @param var1Cutoff variable 1 cutoff
#' @param var2Cutoff variable 2 cutoff
#' @param coreCoverage the least percentage of selected taxa should be
#' considered
#' @param taxDB path to the taxonomy DB files
#' (input$percentCore)
#' @return list of core genes
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}

# source("R/functions.R")

identifyCoreGeneUI <- function(id){
    ns <- NS(id)
    DT::dataTableOutput(ns("coreGene.table"))
}

identifyCoreGene <- function(
    input, output, session,
    filteredData, taxaCount, rankSelect, taxaCore, percentCore,
    var1Cutoff, var2Cutoff, coreCoverage, taxDB
){

    output$coreGene.table <- DT::renderDataTable({
        data <- coreGeneDf()
        if (is.null(data)) stop("Input data is NULL!")
        else {
            data <- as.data.frame(data)
            data
        }
    })

    coreGeneDf <- reactive({
        coreGeneDf <- getCoreGene(
            rankSelect(), taxaCore(), filteredData(), taxaCount(),
            var1Cutoff(), var2Cutoff(), percentCore(), coreCoverage(), taxDB()
        )
        return(coreGeneDf)
    })

    if (!is.null(coreGeneDf)) return(coreGeneDf)
}

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
    library(PhyloProfile),
    collapse = TRUE,
    comment = "#>",
    dev = 'png',
    crop = NULL
)

## ----example_input, echo = FALSE, results = 'asis'----------------------------
data("mainLongRaw", package="PhyloProfile")
knitr::kable(head(mainLongRaw, 10), row.names = FALSE)

## ----poster, echo=FALSE, fig.cap="PhyloProfile's GUI", out.width = '100%'-----
knitr::include_graphics("./posterSub.png")

## ----profile-clustering, fig.show='hold', dev='png'---------------------------
### An example for plotting the clustered profiles tree.
### See ?getDendrogram for more details.

#' Load built-in data
data("finalProcessedProfile", package="PhyloProfile")
data <- finalProcessedProfile

#' Calculate distance matrix
#' Check ?getDistanceMatrix
profileType <- "binary"
profiles <- getDataClustering(
    data, profileType, var1AggregateBy, var2AggregateBy)
method <- "mutualInformation"
distanceMatrix <- getDistanceMatrix(profiles, method)

#' Create clustered profile tree
clusterMethod <- "complete"
dd <- clusterDataDend(as.dist(distanceMatrix), clusterMethod)
getDendrogram(dd)

## ----gene-age-----------------------------------------------------------------
### An example for calculating gene age for the built-in data set.
### See ?estimateGeneAge for more details.
#' Load built-in data
data("fullProcessedProfile", package="PhyloProfile")

#' Choose the working rank and the reference taxon
rankName <- "class"
refTaxon <- "Mammalia"

#' Count taxa within each supertaxon
taxonIDs <- levels(as.factor(fullProcessedProfile$ncbiID))
sortedInputTaxa <- sortInputTaxa(
    taxonIDs, rankName, refTaxon, NULL, NULL, NULL
)
library(dplyr)
taxaCount <- sortedInputTaxa %>% dplyr::count(supertaxon)

#' Set cutoff for 2 additional variables and the percentage of present species
#' in each supertaxon
var1Cutoff <- c(0,1)
var2Cutoff <- c(0,1)
percentCutoff <- c(0,1)

#' Estimate gene age
estimateGeneAge(
    fullProcessedProfile, taxaCount, rankName, refTaxon,
    var1Cutoff, var2Cutoff, percentCutoff
)

## ----core-gene----------------------------------------------------------------
### An example for calculating core gene set for the built-in data set.
### See ?getCoreGene for more details.

#' Load built-in data
data("fullProcessedProfile", package="PhyloProfile")

#' Choose the working rank and a set of taxa of interest
rankName <- "class"
refTaxon <- "Mammalia"
taxaCore <- c("Mammalia", "Saccharomycetes", "Insecta")

#' Set cutoff for 2 additional variables and the percentage of present species
#' in each supertaxon
var1Cutoff <- c(0.75, 1.0)
var2Cutoff <- c(0.75, 1.0)
percentCutoff <- c(0.0, 1.0)

#' Set core coverage, the % of taxa that must be present in the selected set
coreCoverage <- 1

#' Count taxa within each supertaxon
taxonIDs <- levels(as.factor(fullProcessedProfile$ncbiID))
sortedInputTaxa <- sortInputTaxa(
    taxonIDs, rankName, refTaxon, NULL, NULL, NULL
)
taxaCount <- sortedInputTaxa %>% dplyr::count(supertaxon)

#' Identify core genes
getCoreGene(
    rankName,
    taxaCore,
    fullProcessedProfile,
    taxaCount,
    var1Cutoff, var2Cutoff,
    percentCutoff, coreCoverage
)

## ----group-comparison---------------------------------------------------------
#' Load built-in data
data("mainLongRaw", package="PhyloProfile")
data <- mainLongRaw
#' choose the in-group taxa
inGroup <- c("ncbi9606", "ncbi10116")
#' choose variable to be compared
variable <- colnames(data)[4]
#' compare the selected variable between the in-group and out-group taxa
compareTaxonGroups(data, inGroup, TRUE, variable, 0.05)

## ----distribution-analysis, fig.show='hold', dev='png'------------------------
### An example for plotting the distribution of the 1st additional variable.
### See ?createVarDistPlot for more details.

#' Load built-in data
data("mainLongRaw", package="PhyloProfile")

#' Process data for distribution analysis
#' See ?createVariableDistributionData
data <- createVariableDistributionData(
    mainLongRaw, c(0, 1), c(0.5, 1)
)
head(data, 6)

#' Choose a variable for plotting and set the variable name
varType <- "var1"
varName <- "Variable 1"

#' Set cutoff for the percentage of present species in each supertaxon
percentCutoff <- c(0,1)

#' Set text size
distTextSize <- 12

#' Create distribution plot
createVarDistPlot(
    data,
    varName,
    varType,
    percentCutoff,
    distTextSize
)

## ----process-input------------------------------------------------------------
#' Load built-in data
#' If input data is in other format (e.g. fasta, OrthoXML, or wide matrix),
#' see ?createLongMatrix
rawInput <- system.file(
    "extdata", "test.main.long", package = "PhyloProfile", mustWork = TRUE
)

#' Set working rank and the reference taxon
rankName <- "class"
refTaxon <- "Mammalia"

#' Input a user-defined taxonomy tree to replace NCBI taxonomy tree (optional)
taxaTree <- NULL
#' Or a sorted taxon list (optional)
sortedTaxonList <- NULL

#' Choose how to aggregate the additional variables when pocessing the data
#' into supertaxon
var1AggregateBy <- "max"
var2AggregateBy <- "mean"

#' Set cutoffs for for percentage of species present in a supertaxon,
#' allowed number of co orthologs, and cutoffs for the additional variables
percentCutoff <- c(0.0, 1.0)
coorthologCutoffMax <- 10
var1Cutoff <- c(0.75, 1.0)
var2Cutoff <- c(0.5, 1.0)

#' Choose the relationship of the additional variables, if they are related to
#' the orthologous proteins or to the species
var1Relation <- "protein"
var2Relation <- "species"

#' Identify categories for input genes (by a mapping tab-delimited file)
groupByCat <- FALSE
catDt <- NULL

#' Process the input file into a phylogenetic profile data that contains
#' taxonomy information and the aggregated values for the 2 additional variables
profileData <- fromInputToProfile(
    rawInput,
    rankName,
    refTaxon,
    taxaTree,
    sortedTaxonList,
    var1AggregateBy,
    var2AggregateBy,
    percentCutoff,
    coorthologCutoffMax,
    var1Cutoff,
    var2Cutoff,
    var1Relation,
    var2Relation,
    groupByCat,
    catDt
)

head(profileData)

## ----profile-plot, fig.show='hold', dev='png'---------------------------------
#' Load built-in processed data
data("finalProcessedProfile", package="PhyloProfile")

#' Create data for plotting
plotDf <- dataMainPlot(finalProcessedProfile)

#' You can also choose a subset of genes and/or taxa for plotting with:
#' selectedTaxa <- c("Mammalia", "Echinoidea", "Gunneridae")
#' selectedSeq <- "all"
#' plotDf <- dataCustomizedPlot(
#'     finalProcessedProfile, selectedTaxa, selectedSeq
#' )

#' Identify plot's parameters
plotParameter <- list(
    "xAxis" = "taxa",
    "var1ID" = "FAS_FW",
    "var2ID"  = "FAS_BW",
    "midVar1" = 0.5,
    "midColorVar1" =  "#FFFFFF",
    "lowColorVar1" =  "#FF8C00",
    "highColorVar1" = "#4682B4",
    "midVar2" = 1,
    "midColorVar2" =  "#FFFFFF",
    "lowColorVar2" = "#CB4C4E",
    "highColorVar2" = "#3E436F",
    "paraColor" = "#07D000",
    "xSize" = 8,
    "ySize" = 8,
    "legendSize" = 8,
    "mainLegend" = "top",
    "dotZoom" = 0,
    "xAngle" = 60,
    "guideline" = 0,
    "colorByGroup" = FALSE,
    "colorByOrthoID" = FALSE
)

#' Generate profile plot
p <- heatmapPlotting(plotDf, plotParameter)
p
#' To highlight a gene and/or taxon of interest
taxonHighlight <- "Mammalia"
workingRank <- "class"
geneHighlight <- "none"
#' Then use ?highlightProfilePlot function
# highlightProfilePlot(
#    p, plotDf, taxonHighlight, workingRank, geneHighlight, plotParameter$xAxis
# )

## ----domain-plot, fig.show='hold', dev='png'----------------------------------
#' Load protein domain architecture file
domainFile <- system.file(
    "extdata", "domainFiles/101621at6656.domains",
    package = "PhyloProfile", mustWork = TRUE
)

#' Identify IDs of gene of interest and its ortholog partner
seedID <- "101621at6656"
orthoID <- "101621at6656|AGRPL@224129@0|224129_0:001955|1"
info <- c(seedID, orthoID)

#' Get data for 2 selected genes from input file
domainDf <- parseDomainInput(seedID, domainFile, "file")

#' Modify feature IDs
domainDf$feature_id_mod <- domainDf$feature_id
domainDf$feature_id_mod <- gsub("SINGLE", "LCR", domainDf$feature_id_mod)
domainDf$feature_id_mod[domainDf$feature_type == "coils"] <- "Coils"
domainDf$feature_id_mod[domainDf$feature_type == "seg"] <- "LCR"
domainDf$feature_id_mod[domainDf$feature_type == "tmhmm"] <- "TM"

#' Generate plot
plot <- createArchiPlot(info, domainDf, 9, 9)
grid::grid.draw(plot)

## ----get-tax-id-name----------------------------------------------------------
#' Load raw input
data("mainLongRaw", package="PhyloProfile")
inputDf <- mainLongRaw

#' Set working rank and the reference taxon
rankName <- "phylum"

#' Get taxonomy IDs and names for the input data
inputTaxonID <- getInputTaxaID(inputDf)
inputTaxonName <- getInputTaxaName(rankName, inputTaxonID)

head(inputTaxonName)

## ----sort-taxa----------------------------------------------------------------
#' Get list of taxon IDs and names for input profile
data("mainLongRaw", package="PhyloProfile")
inputDf <- mainLongRaw
rankName <- "phylum"
inputTaxonID <- getInputTaxaID(inputDf)

#' Input a user-defined taxonomy tree to replace NCBI taxonomy tree (optional)
inputTaxaTree <- NULL

#' Sort taxonomy list based on a selected refTaxon
refTaxon <- "Chordata"
sortedTaxonomy <- sortInputTaxa(
    taxonIDs = inputTaxonID,
    rankName = rankName,
    refTaxon = refTaxon,
    taxaTree = inputTaxaTree,
    NULL,
    NULL
)

head(
    sortedTaxonomy[
        , c("ncbiID", "fullName", "supertaxon", "supertaxonID", "rank")
    ]
)

## ----citation-----------------------------------------------------------------
citation("PhyloProfile")

## ----sessionInfo, echo = FALSE------------------------------------------------
sessionInfo(package = "PhyloProfile")

#' Download filtered data from customized profile
#' @param data MAIN data for downloading (from module "downloadFilteredMain.R"
#' @param fasta fasta sequences (from reactive fn "customizedFastaDownload")
#' @param inSeq selected sequences in customized profile (from input$inSeq)
#' @param inTaxa selected taxa in customized profile (from input$inTaxa)
#' @return data of customized profile for downloading
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}

downloadFilteredCustomizedUI <- function(id) {
    ns <- NS(id)

    tabPanel(
        "Customized data",
        column(
            12,
            strong(
                em(
                    "NOTE: Depending on your choice in Download filtered data
                    -> Main data, either all or only representative sequences
                    will be downloaded!"
                ),
                style = "color:red"
            ),
            hr()
        ),
        column(
            12,
            DT::dataTableOutput(ns("filteredCustomData"))
        ),
        column(
            4,
            downloadButton(ns("downloadCustomData"),
                           "Download customized data")
        ),
        column(
            4,
            downloadButton(ns("downloadCustomFasta"),
                           "Download FASTA sequences"),
            uiOutput(ns("downloadCustomFasta.ui"))
        ),
        column(
            4,
            downloadButton(ns("downloadCustomLong"),
                           "Download data as PhyloProfile input format")
        )
    )
}

downloadFilteredCustomized <- function(
    input, output, session, data, fasta, inSeq, inTaxa
){

    # filtered data for downloading (Customized Profile) -----------------------
    downloadCustomData <- reactive({
        data <- as.data.frame(data())

        # get subset of data according to selected genes/taxa
        if (!is.null(inSeq()) | !is.null(inTaxa())) {
            if (inSeq()[1] != "all" & inTaxa()[1] == "all") {
                # select data for selected sequences only
                customData <- subset(data, geneID %in% inSeq())
            } else if (inSeq()[1] == "all" & inTaxa()[1] != "all") {
                # select data for selected taxa only
                customData <- subset(data, supertaxon %in% inTaxa())
            } else if (inSeq()[1] != "all" & inTaxa()[1] != "all") {
                # select data for selected sequences and taxa
                customData <- subset(data, geneID %in% inSeq()
                                      & supertaxon %in% inTaxa())
            } else {
                customData <- data
            }
        } else {
            customData <- data
        }
        # return data
        customData <- as.matrix(customData)
        return(customData)
    })

    # download data ------------------------------------------------------------
    output$downloadCustomData <- downloadHandler(
        filename = function(){
            c("customFilteredData.out")
        },
        content = function(file){
            dataOut <- downloadCustomData()
            write.table(dataOut, file,
                        sep = "\t",
                        row.names = FALSE,
                        quote = FALSE)
        }
    )

    # render download data table -----------------------------------------------
    output$filteredCustomData <- DT::renderDataTable({
        data <- downloadCustomData()
        data
    })

    # download FASTA -----------------------------------------------------------
    output$downloadCustomFasta <- downloadHandler(
        filename = function(){
            c("customFilteredSeq.fa")
        },
        content = function(file){
            fastaOutDf <- fasta()
            write.table(fastaOutDf,
                        file,
                        sep = "\t",
                        col.names = FALSE,
                        row.names = FALSE,
                        quote = FALSE)
        }
    )

    # download data as long format ---------------------------------------------
    downloadCustomDataLong <- reactive({
        downloadCustomData <- downloadCustomData()
        if ("geneName" %in% colnames(downloadCustomData)) {
            downloadCustomDataLong <- downloadCustomData[,c(1,5,3,7,8,2)]
        } else {
            if (ncol(downloadCustomData) == 6) {
                downloadCustomDataLong <- downloadCustomData[,c(1,4,2)]
            } else if (ncol(downloadCustomData) == 7) {
                downloadCustomDataLong <- downloadCustomData[,c(1,4,2,6)]
            } else if (ncol(downloadCustomData) == 8) {
                downloadCustomDataLong <- downloadCustomData[,c(1,4,2,6,7)]
            }
        }
        return(downloadCustomDataLong)
    })

    output$downloadCustomLong <- downloadHandler(
        filename = function(){
            c("customFilteredData.phyloprofile")
        },
        content = function(file){
            dataOut <- downloadCustomDataLong()
            write.table(dataOut, file,
                        sep = "\t",
                        row.names = FALSE,
                        quote = FALSE)
        }
    )

    return(downloadCustomData)
}

#' Popup windows for (sub-)selecting rank & (super)taxon of interest
#' @param rankSelect initial selected taxonomy rank (input$rankSelect)
#' @param inputTaxonID list of all input taxon IDs
#' @taxDB Path to the taxonomy DB files
#' (from reactive fn inputTaxonID)
#' @return list of all taxa in the initial taxonomy rank based on selected
#' supertaxon from input$taxaSelectCus
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}

source("R/functions.R")

selectTaxonRankUI <- function(id) {
    ns <- NS(id)
    tagList(
        uiOutput(ns("rankSelectCus.ui")),
        uiOutput(ns("taxaSelectCus.ui"))
    )
}

selectTaxonRank <- function(
    input, output, session, rankSelect, inputTaxonID, taxDB
){

    # render list of available taxonomy ranks
    # (where the lowest rank is the same as the chosen main rank)
    output$rankSelectCus.ui <- renderUI({
        ns <- session$ns
        mainRank <- rankSelect()
        mainChoices <- list("Strain " = "strain",
                            "Species" = "species",
                            "Genus" = "genus",
                            "Family" = "family",
                            "Order" = "order",
                            "Class" = "class",
                            "Phylum" = "phylum",
                            "Kingdom" = "kingdom",
                            "Superkingdom" = "superkingdom",
                            "unselected" = "")
        cusChoices <-
            mainChoices[match(mainRank, mainChoices):(length(mainChoices)-1)]
        selectInput(
            ns("rankSelectCus"),
            label = h5("Select taxonomy rank:"),
            choices = as.list(cusChoices),
            selected = mainRank
        )
    })

    # get list of possible taxon names for each selected rank from
    # rankSelectCus
    getTaxaCus <- reactive({
        rankSelectCus <- input$rankSelectCus

        if (length(rankSelectCus) == 0) return()
        else {
            # load list of unsorted taxa
            Dt <- getTaxonomyMatrix(taxDB(), TRUE, inputTaxonID())
            # load list of taxon name
            nameList <- getNameList(taxDB())
            # get rank name from rankSelect
            rankName <- rankSelectCus

            choice <- data.frame(
                ncbiID = unlist(Dt[rankName]), stringsAsFactors = FALSE
            )
            choice <- merge(choice, nameList, by = "ncbiID", all = FALSE)
            return(choice)
        }
    })

    # render list of possible taxon names from getTaxaCus()
    output$taxaSelectCus.ui <- renderUI({
        ns <- session$ns
        choice <- getTaxaCus()
        choice$fullName <- as.factor(choice$fullName)
        selectInput(
            ns("taxaSelectCus"),
            h5("Choose (super)taxon of interest:"),
            as.list(levels(choice$fullName)),
            levels(choice$fullName)[1]
        )
    })

    # get list of all taxa of the current taxonomy rank (from input$rankSelect)
    # based on selected supertaxon from input$taxaSelectCus
    cusTaxaName <- reactive({
        taxaSelectCus <- input$taxaSelectCus
        rankName <- input$rankSelectCus
        if (taxaSelectCus == "") return()

        # load list of unsorted taxa
        Dt <- getTaxonomyMatrix(taxDB(), TRUE, inputTaxonID())

        # get ID of selected (super)taxon from input$taxaSelectCus
        taxaList <- getNameList(taxDB())
        superID <- taxaList$ncbiID[taxaList$fullName == taxaSelectCus
                                    & taxaList$rank %in% c(rankName, "norank")]

        # from that ID, get list of all taxa for main selected taxon
        mainRankName <- rankSelect()
        customizedtaxaID <-
            levels(as.factor(Dt[mainRankName][Dt[rankName] == superID, ]))

        cusTaxaName <-
            taxaList$fullName[taxaList$ncbiID %in% customizedtaxaID]
        return(cusTaxaName)
    })
    return(cusTaxaName)
}

#' Download filtered data from main profile
#' @param full processed main data (from reactive fn "getDataFiltered")
#' @param fasta fasta sequences (from reactive fn "mainFastaDownload")
#' @param var1ID name of 1st variable (from input$var1ID)
#' @param var2ID name of 2nd variable (from input$var2ID)
#' @param var1 cutoff value of 1st variable (from input$var1)
#' @param var2 cutoff value of 2nd variable (from input$var2)
#' @param percent cutoff value of percentage species in each supertaxon
#' (from input$percent)
#' @return data of main profile for downloading
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}

downloadFilteredMainUI <- function(id) {
    ns <- NS(id)

    tabPanel(
        "Main data",
        column(
            4,
            checkboxInput(
                ns("getRepresentativeMain"),
                strong(em("Download representative sequences")),
                value = FALSE,
                width = NULL
            )
        ),
        column(
            4,
            conditionalPanel(
                condition = {
                    sprintf("input['%s'] == true",
                            ns("getRepresentativeMain"))
                },
                uiOutput(ns("refVarMain.ui"))
            )
        ),
        column(
            4,
            conditionalPanel(
                condition = {
                    sprintf("input['%s'] == true",
                            ns("getRepresentativeMain"))
                },
                radioButtons(
                    inputId = ns("refTypeMain"),
                    label = {
                        "Select representative by"
                    },
                    choices = list("max", "min"),
                    selected = "max",
                    inline = TRUE
                )
            )
        ),
        column(
            12,
            DT::dataTableOutput(ns("filteredMainData"))
        ),
        column(
            4,
            downloadButton(ns("downloadData"),
                           "Download filtered data")
        ),
        column(
            4,
            downloadButton(ns("downloadFasta"),
                           "Download FASTA sequences")
        ),
        column(
            4,
            downloadButton(ns("downloadLong"),
                           "Download data as PhyloProfile input format")
        )
    )
}

downloadFilteredMain <- function(
    input, output, session, data, taxaCount, fasta, var1ID, var2ID, var1, var2, 
    percent
){

    # render options for downloading -------------------------------------------
    output$refVarMain.ui <- renderUI({
        ns <- session$ns
        if (nchar(var2ID()) < 1 & nchar(var1ID()) < 1) {
            radioButtons(
                inputId = ns("refVarMain"), label = "Reference variable",
                choices = list(var1ID(), var2ID()),
                selected = var1ID()
            )
        } else if (nchar(var2ID()) < 1) {
            radioButtons(
                inputId = ns("refVarMain"),
                label = "Reference variable",
                choices = list(var1ID()),
                selected = var1ID()
            )
        } else {
            radioButtons(
                inputId = ns("refVarMain"),
                label = "Reference variable",
                choices = list(var1ID(), var2ID()),
                selected = var1ID()
            )
        }
    })

    # filtered data for downloading (Main Profile ) ----------------------------
    downloadData <- reactive({
        if (is.null(data())) stop("Data for downloading is NULL!")
        ### filtered data
        dataOut <- data()
        if (length(var1()) == 1) {
            dataOut <- dataOut[dataOut$var1 >= var1()[1], ]
        } else {
            if (max(dataOut$var1) <= var1()[2]) {
                dataOut <- dataOut[
                    dataOut$var1 >= var1()[1] & dataOut$var1 <= var1()[2], 
                ]
            }
        }
        if (!all(is.na(dataOut$var2))) {
            if (length(var2()) == 1) {
                dataOut <- dataOut[dataOut$var2 >= var2()[1], ]
            } else {
                dataOut <- dataOut[
                    dataOut$var2 >= var2()[1] & dataOut$var2 <= var2()[2], 
                ]
            }
        } else {
            dataOut$var2 <- 0
        }
        # calculate presSpec
        finalPresSpecDt <- calcPresSpec(dataOut, taxaCount())
        dataOut <- Reduce(
            function(x, y) merge(x, y, by = c("geneID", "supertaxon"), all.x=TRUE),
            list(dataOut, finalPresSpecDt))
        # dataOut <- as.data.frame(dataOut[dataOut$presSpec > 0, ])
        dataOut <- dataOut[
            dataOut$presSpec >= percent()[1] & dataOut$presSpec <= percent()[2], 
        ]
        dataOut <- dataOut[!is.na(dataOut$geneID), ]
        ### select only representative genes if chosen
        if (input$getRepresentativeMain == TRUE) {
            if (is.null(input$refVarMain)) return()
            else {
                if (input$refVarMain == var1ID()) {
                    dataOutAgg <- aggregate(
                        dataOut$var1,
                        by = list(dataOut$geneID, dataOut$ncbiID),
                        FUN = input$refTypeMain
                    )
                } else if (input$refVarMain == var2ID()) {
                    dataOutAgg <- aggregate(
                        dataOut$var2,
                        by = list(dataOut$geneID, dataOut$ncbiID),
                        FUN = input$refTypeMain
                    )
                } else {
                    dataOutAgg <-
                        dataOut[dataOut, c("geneID", "ncbiID", "var1")]
                }
                colnames(dataOutAgg) <- c("geneID", "ncbiID", "varBest")

                dataOutRepresentative <- merge(dataOut, dataOutAgg,
                                                 by = c("geneID", "ncbiID"),
                                                 all.x = TRUE)

                if (input$refVarMain == var1ID()) {
                    dataOut <-
                        dataOutRepresentative[
                            dataOutRepresentative$var1 ==
                                dataOutRepresentative$varBest,
                            ]
                } else if (input$refVarMain == var2ID()) {
                    dataOut <-
                        dataOutRepresentative[
                            dataOutRepresentative$var2 ==
                                dataOutRepresentative$varBest,
                            ]
                } else {
                    dataOut <- dataOut
                }
                # used to select only one ortholog,
                # if there exist more than one "representative"
                dataOut$dup <- paste0(dataOut$geneID, "#", dataOut$ncbiID)
                dataOut <- dataOut[!duplicated(c(dataOut$dup)), ]
            }
        }

        # sub select columns of dataOut
        dataOut <- dataOut[, c(
            "geneID", "geneName", "orthoID", "fullName", "ncbiID", "supertaxon",
            "var1", "var2", "presSpec"
        )]
        if (all(as.character(dataOut$geneID)==as.character(dataOut$geneName))) {
            dataOut <- subset(dataOut, select = -c(geneName))
        }
        dataOut <- dataOut[order(dataOut$geneID, dataOut$supertaxon), ]
        dataOut <- dataOut[complete.cases(dataOut), ]

        dataOut$geneID <- as.character(dataOut$geneID)
        dataOut$fullName <- as.character(dataOut$fullName)
        dataOut$ncbiID <- as.character(dataOut$ncbiID)
        dataOut$supertaxon <- substr(
            dataOut$supertaxon, 8, nchar(as.character(dataOut$supertaxon))
        )
        dataOut$var1 <- as.character(dataOut$var1)
        dataOut$var2 <- as.character(dataOut$var2)
        dataOut$presSpec <- dataOut$presSpec

        # rename columns
        names(dataOut)[names(dataOut) == "presSpec"] <- "%Spec"
        if (nchar(var1ID()) > 0) {
            names(dataOut)[names(dataOut) == "var1"] <- var1ID()
        } else {
            dataOut <- subset(dataOut, select = -c(var1) )
        }
        if (nchar(var2ID()) > 0) {
            names(dataOut)[names(dataOut) == "var2"] <- var2ID()
        } else {
            dataOut <- subset(dataOut, select = -c(var2) )
        }

        # return data for downloading
        return(dataOut)
    })

    # download data ------------------------------------------------------------
    output$downloadData <- downloadHandler(
        filename = function(){
            c("filteredData.out")
        },
        content = function(file){
            dataOut <- downloadData()
            write.table(dataOut, file,
                        sep = "\t",
                        row.names = FALSE,
                        quote = FALSE)
        }
    )

    # render download data table -----------------------------------------------
    output$filteredMainData <- DT::renderDataTable({
        data <- downloadData()
        as.matrix(data)
    })

    # download FASTA -----------------------------------------------------------
    output$downloadFasta <- downloadHandler(
        filename = function(){
            c("filteredSeq.fa")
        },
        content = function(file){
            fastaOutDf <- fasta()
            write.table(fastaOutDf, file,
                        sep = "\t",
                        col.names = FALSE,
                        row.names = FALSE,
                        quote = FALSE)
        }
    )

    # download data as long format ---------------------------------------------
    downloadDataLong <- reactive({
        downloadData <- downloadData()
        if ("geneName" %in% colnames(downloadData)) {
            downloadDataLong <- downloadData[,c(1,5,3,7,8,2)]
        } else {
            if (ncol(downloadData) == 6) {
                downloadDataLong <- downloadData[,c(1,4,2)]
            } else if (ncol(downloadData) == 7) {
                downloadDataLong <- downloadData[,c(1,4,2,6)]
            } else if (ncol(downloadData) == 8) {
                downloadDataLong <- downloadData[,c(1,4,2,6,7)]
            }
        }
        return(downloadDataLong)
    })

    output$downloadLong <- downloadHandler(
        filename = function(){
            c("filteredData.phyloprofile")
        },
        content = function(file){
            dataOut <- downloadDataLong()
            write.table(dataOut, file,
                        sep = "\t",
                        row.names = FALSE,
                        quote = FALSE)
        }
    )

    return(downloadData)
}

#' Get all taxa that share a common ancestor
#' @description Identify the common ancestor for a selected taxa and return a
#' list of all taxa that have that common ancestor from an large input taxa set.
#' @export
#' @param inputTaxa ID list of all input taxa (e.g. "ncbi12345")
#' @param inGroup ID list of selected taxa used for identify the common ancestor
#' (e.g.: "ncbi55555")
#' @param taxDB Path to the taxonomy DB files
#' @return A list containing the taxonomy rank and name of the common ancestor,
#' together with a dataframe storing the full taxonomy info of all taxa that
#' share that corresponding common ancestor.
#' @author Vinh Tran (tran@bio.uni-frankfurt.de)
#' @examples
#' inputTaxa <- c("ncbi34740", "ncbi9606", "ncbi374847", "ncbi123851",
#'     "ncbi5664", "ncbi189518", "ncbi418459", "ncbi10116", "ncbi284812",
#'     "ncbi35128", "ncbi7070")
#' inGroup <-  c("ncbi9606", "ncbi10116")
#' getCommonAncestor(inputTaxa, inGroup)

getCommonAncestor <- function(inputTaxa = NULL, inGroup = NULL, taxDB = NULL) {
    if (is.null(inputTaxa) | is.null(inGroup))
        stop("Input taxa and in-group ID list cannot be NULL!")
    # get list of pre-calculated taxonomy info
    taxMatrix <- getTaxonomyMatrix(taxDB, TRUE, inputTaxa)
    # get subset taxonomy info for selected in-group taxa
    selectedTaxMatrix <- taxMatrix[
        taxMatrix$abbrName %in% inGroup,
        which(!duplicated(t(taxMatrix)))
        ]
    # identify common ancestor
    V1 <- vapply(
        selectedTaxMatrix[,c(seq(4, ncol(selectedTaxMatrix)))], max,
        FUN.VALUE = numeric(1)
    )
    V2 <- vapply(
        selectedTaxMatrix[,c(seq(4, ncol(selectedTaxMatrix)))],
        function (x) as.integer(sum(x)/length(x)),
        FUN.VALUE = numeric(1)
    )
    checkDf <- t(rbind(V1, V2))
    commonRank <- rownames(checkDf[V1 == V2,])[1]
    commonID <- checkDf[V1 == V2,][1]
    commonTaxa <- taxMatrix[taxMatrix[, commonRank] == commonID, ]
    return(list(commonRank, commonID, commonTaxa))
}

#' Compare the score distributions between 2 taxon groups
#' @description Given the phylogenetic profiles that contains up to 2 additional
#' variables besides the presence/absence information of the orthologous
#' proteins. This function will compare the distribution of those variables
#' between 2 different taxon groups (e.g. parasitic species vs non-parasitic
#' species), which are defined as in-group and out-group. In-group is identified
#' by the user. Out-group contains all taxa in the input phylogenetic profiles
#' that are not part of the in-group.
#' @usage compareTaxonGroups(data, inGroup, useCommonAncestor, variable,
#'     significanceLevel, taxDB)
#' @export
#' @param data input phylogenetic profile in long format (see ?mainLongRaw and
#' ?createLongMatrix)
#' @param inGroup ID list of in-group taxa (e.g. "ncbi1234")
#' @param useCommonAncestor TRUE/FALSE if using all taxa that share the same
#' common ancestor with the pre-selected in-group as the in-group taxa.
#' Default = TRUE.
#' @param variable name of the variable that need to be compared
#' @param significanceLevel significant cutoff for the statistic test (between
#' 0 and 1). Default = 0.05.
#' @param taxDB Path to the taxonomy DB files
#' @return list of genes that have a significant difference in the variable
#' distributions between the in-group and out-group taxa and their corresponding
#' p-values.
#' @author Vinh Tran (tran@bio.uni-frankfurt.de)
#' @examples
#' data("mainLongRaw", package="PhyloProfile")
#' data <- mainLongRaw
#' inGroup <- c("ncbi9606", "ncbi10116")
#' variable <- colnames(data)[4]
#' compareTaxonGroups(data, inGroup, TRUE, variable, 0.05)

compareTaxonGroups <- function(
    data = NULL,
    inGroup = NULL,
    useCommonAncestor = TRUE,
    variable = NULL,
    significanceLevel = 0.05,
    taxDB = NULL
) {
    if (is.null(data) | is.null(inGroup) | is.null(variable))
        stop("Input profiles, in-group IDs and variable name cannot be NULL!")
    if (!(variable %in% colnames(data))) stop("Invalid variable")
    # add other taxa that share a common ancestor with the given in-group
    commonTaxa <- getCommonAncestor(
        levels(as.factor(data$ncbiID)), inGroup, taxDB
    )
    if (useCommonAncestor == TRUE)
        inGroup <- as.character(commonTaxa[[3]]$abbrName)

    # perform distribution comparison test
    data$geneID <- as.character(data$geneID)
    pvalues <- vapply(
        unique(data$geneID),
        function (x) {
            varIn <- data[data$geneID == x & data$ncbiID %in% inGroup, variable]
            varOut <- data[
                data$geneID == x & !(data$ncbiID %in% inGroup), variable
                ]
            pvalue <- try(distributionTest(varIn, varOut, significanceLevel))
            if (is.numeric(pvalue))
                return(distributionTest(varIn, varOut, significanceLevel))
            else return(999)
        }, FUN.VALUE = numeric(1)
    )
    return(sort(unlist(pvalues)))
}

#' Compare the distribution of 2 numeric vectors
#' @description This function tests the difference between the distributions of
#' two input numeric samples using the statistical tess. First the
#' Kolmogorov-Smirnov is used to check if 2 samples have the same distribution.
#' If yes, Wilcoxon-Mann-Whitney will be used to compare the distribution
#' difference.
#' @usage distributionTest(varIn, varOut, significanceLevel)
#' @param varIn first numeric vector
#' @param varOut second numeric vector
#' @param significanceLevel significant cutoff of the Kolmogorov-Smirnov test.
#' Default = 0.05.
#' @return p-value of the comparison test.
#' @author Carla Mölbert (carla.moelbert@gmx.de)

distributionTest <- function(
    varIn = NULL, varOut = NULL, significanceLevel = 0.05
){
    if (is.null(varIn) | is.null(varOut))
        stop("Vector of in-group and out-group cannot be NULL!")
    # remove NA values
    varIn <- varIn[!is.na(varIn)]
    varOut <- varOut[!is.na(varOut)]
    # if there is no data in one of the groups the p-value is NULL
    if (length(varIn) == 0 | length(varOut) == 0)
        stop("No data for In/Out-group taxa!")
    else {
        # * Kolmogorov-Smirnov Test
        # H0 : The two samples have the same distribution
        ks <- suppressWarnings(
            stats::ks.test(unique(varIn), unique(varOut), exact = FALSE)
        )

        if (ks$p.value <= significanceLevel) return(ks$p.value)
        else {
            # * Wilcoxon-Mann-Whitney Test
            # H0: the samples have the same location parameters
            wilcox <- suppressWarnings(
                stats::wilcox.test(
                    varIn, varOut, alternative = "two.sided", paired = FALSE
                )
            )
            return(wilcox$p.value)
        }
    }
}

#' Compare the median values of a variable between 2 taxon groups
#' @description Given the phylogenetic profiles that contains up to 2 additional
#' variables besides the presence/absence information of the orthologous
#' proteins. This function will compare the median scores of those variables
#' between 2 different taxon groups (e.g. parasitic species vs non-parasitic
#' species), which are defined as in-group and out-group. In-group is identified
#' by the user. Out-group contains all taxa in the input phylogenetic profiles
#' that are not part of the in-group.
#' @usage compareMedianTaxonGroups(data, inGroup, useCommonAncestor, variable,
#'     taxDB)
#' @export
#' @param data input phylogenetic profile in long format (see ?mainLongRaw and
#' ?createLongMatrix)
#' @param inGroup ID list of in-group taxa (e.g. "ncbi1234")
#' @param useCommonAncestor TRUE/FALSE if using all taxa that share the same
#' common ancestor with the pre-selected in-group as the in-group taxa.
#' Default = TRUE.
#' @param variable name of the variable that need to be compared
#' @param taxDB Path to the taxonomy DB files
#' @return List of genes that have a difference in the variable's median scores
#' between the in-group and out-group taxa and their corresponding delta-median.
#' @author Vinh Tran (tran@bio.uni-frankfurt.de)
#' @examples
#' data("mainLongRaw", package="PhyloProfile")
#' data <- mainLongRaw
#' inGroup <- c("ncbi9606", "ncbi10116")
#' variable <- colnames(data)[4]
#' compareMedianTaxonGroups(data, inGroup, TRUE, variable)

compareMedianTaxonGroups <- function(
    data = NULL, inGroup = NULL, useCommonAncestor = TRUE, variable = NULL,
    taxDB = NULL
) {
    if (is.null(data) | is.null(inGroup) | is.null(variable))
        stop("Input profiles, in-group IDs and variable name cannot be NULL!")
    if (!(variable %in% colnames(data))) stop("Invalid variable")
    # add other taxa that share a common ancestor with the given in-group
    commonTaxa <- getCommonAncestor(
        levels(as.factor(data$ncbiID)), inGroup, taxDB
    )
    if (useCommonAncestor == TRUE)
        inGroup <- as.character(commonTaxa[[3]]$abbrName)

    # return delta-median scores for two taxa groups
    data$geneID <- as.character(data$geneID)
    deltaMedian <- vapply(
        unique(data$geneID),
        function (x) {
            varIn <- data[data$geneID == x & data$ncbiID %in% inGroup, variable]
            varOut <- data[
                data$geneID == x & !(data$ncbiID %in% inGroup), variable]
            return(
                abs(
                    stats::median(varIn[!is.na(varIn)])
                    - stats::median(varOut[!is.na(varOut)])
                )
            )
        },
        FUN.VALUE = numeric(1)
    )
    return(sort(unlist(deltaMedian)))
}


#' Create data for variable distribution comparison plot
#' @description Create data for plotting the distribution comparison between 2
#' groups of taxa for a selected gene.
#' @usage dataVarDistTaxGroup(data, inGroup, gene, variable)
#' @export
#' @param data input phylogenetic profile in long format (see ?mainLongRaw and
#' ?createLongMatrix)
#' @param inGroup ID list of in-group taxa (e.g. "ncbi1234")
#' @param gene ID of gene that need to be plotted the distribution comparison
#' between in- and out-group taxa.
#' @param variable var1 or c(var1, var2)
#' @return Dataframe containing list of values for all available variables for
#' the selected genes in in-group and out-group taxa (max. 3 columns).
#' @author Vinh Tran (tran@bio.uni-frankfurt.de)
#' @seealso \code{\link{createLongMatrix}}
#' @examples
#' data("mainLongRaw", package="PhyloProfile")
#' data <- mainLongRaw
#' inGroup <- c("ncbi9606", "ncbi10116")
#' variable <- colnames(data)[c(4, 5)]
#' dataVarDistTaxGroup(data, inGroup, "101621at6656", variable)

dataVarDistTaxGroup <- function(
    data = NULL, inGroup = NULL, gene = NULL, variable = NULL
) {
    if (is.null(data) | is.null(inGroup) | is.null(gene) | is.null(variable))
        stop("Input profiles, in-group IDs, gene & variable ID cannot be NULL!")
    # remove "empty" variable (char "")
    variable <- variable[unlist(lapply(variable, function (x) x != ""))]
    # get 2 lists of values for in-group and out-group
    varIn <- data.frame(
        data[data$geneID == gene & data$ncbiID %in% inGroup, ][, variable]
    )
    colnames(varIn) <- variable
    varOut <- data.frame(
        data[data$geneID == gene & !(data$ncbiID %in% inGroup), ][, variable]
    )
    colnames(varOut) <- variable
    if (nrow(varIn) == 0 & nrow(varOut) == 0) stop("No data for in-/out-group!")

    varIn$type <- "In-group"
    varOut$type <- "Out-group"
    out <- rbind(
        varIn[stats::complete.cases(varIn),],
        varOut[stats::complete.cases(varOut),]
    )
    return(out[, c(variable, "type")])
}

#' Create a single violin distribution plot
#' @export
#' @param plotDf dataframe for plotting containing values for each variable in
#' in-group and out-group.
#' @param parameters plot parameters, including size of x-axis, y-axis,
#' legend and title; position of legend ("right", "bottom" or "none");
#' mean/median point; names of in-group and out-group; and plot title.
#' NOTE: Leave blank or NULL to use default values.
#' @param variable name of variable that need to be plotted (one of the column
#' names of input dataframe plotDf).
#' @return A violin plot as a ggplot object.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @import ggplot2
#' @examples
#' data("mainLongRaw", package="PhyloProfile")
#' data <- mainLongRaw
#' inGroup <- c("ncbi9606", "ncbi10116")
#' varNames <- colnames(data)[c(4, 5)]
#' plotDf <- dataVarDistTaxGroup(data, inGroup, "101621at6656", varNames)
#' plotParameters <- list(
#'     "xSize" = 12,
#'     "ySize" = 12,
#'     "titleSize" = 15,
#'     "legendSize" = 12,
#'     "legendPosition" = "right",
#'     "mValue" = "mean",
#'     "inGroupName" = "In-group",
#'     "outGroupName" = "Out-group",
#'     "title" = "101621at6656"
#' )
#' generateSinglePlot(plotDf, plotParameters, colnames(plotDf)[1])

generateSinglePlot <- function(plotDf, parameters, variable) {
    type <- .data <- NULL
    xNames <- c(
        paste(
            parameters$inGroupName, " \n n = ",
            nrow(plotDf[plotDf$type == parameters$inGroupName,]), sep = ""
        ),
        paste(
            parameters$outGroupName, " \n n = ",
            nrow(plotDf[plotDf$type == parameters$outGroupName,]), sep = ""
        )
    )

    plot <- ggplot(plotDf, aes(x = factor(type), y = .data[[variable]])) +
        geom_violin(
            aes(fill = factor(type)), position = position_dodge(),
            scale = "width", alpha = .5) +
        geom_boxplot(width = 0.1) +
        scale_x_discrete(labels = xNames) +
        # add mean/median point and color of that point
        stat_summary(
            aes(colour = parameters$mValue), fun = parameters$mValue,
            geom = "point", size = 3, show.legend = TRUE, shape = 8
        ) +
        scale_color_manual("", values = c("red")) +
        # add title and theme
        labs(x = "", y = variable) +
        theme_minimal() +
        theme(
            axis.text.x = element_text(size = parameters$xSize, hjust = 1),
            axis.text.y = element_text(size = parameters$ySize),
            axis.title.y = element_text(size = parameters$ySize),
            legend.position = parameters$legendPosition,
            legend.text = element_text(size = parameters$legendSize),
            legend.title = element_blank()
        )
    return(plot)
}

#' Plot Multiple Graphs with Shared Legend in a Grid
#' @export
#' @usage gridArrangeSharedLegend(...,  ncol = length(list(...)), nrow = 1,
#'     position = c("bottom", "right"), title = NA, titleSize = 12)
#' @param ...  Plots to be arranged in grid
#' @param ncol Number of columns in grid
#' @param nrow Number of rows in grid
#' @param position Gird position (bottom or right)
#' @param title Title of grid
#' @param titleSize Size of grid title
#' @import ggplot2
#' @importFrom gridExtra arrangeGrob
#' @return Grid of plots with common legend
#' @author Phil Boileau, \email{philippe.boileau@rimuhc.ca}
#' @note adapted from https://rdrr.io/github/PhilBoileau/CLSAR/src/R/
#' gridArrangeSharedLegend.R
#' @examples
#' \dontrun{
#' data("mainLongRaw", package="PhyloProfile")
#' data <- mainLongRaw
#' inGroup <- c("ncbi9606", "ncbi10116")
#' varNames <- colnames(data)[c(4, 5)]
#' plotDf <- dataVarDistTaxGroup(data, inGroup, "101621at6656", varNames)
#' plotParameters <- list(
#'     "xSize" = 12,
#'     "ySize" = 12,
#'     "titleSize" = 15,
#'     "legendSize" = 12,
#'     "legendPosition" = "right",
#'     "mValue" = "mean",
#'     "inGroupName" = "In-group",
#'     "outGroupName" = "Out-group",
#'     "title" = "101621at6656"
#' )
#' plotVar1 <- generateSinglePlot(plotDf, plotParameters, colnames(plotDf)[1])
#' plotVar2 <- generateSinglePlot(plotDf, plotParameters, colnames(plotDf)[2])
#' g <- gridArrangeSharedLegend(
#'     plotVar1, plotVar2,
#'     position = plotParameters$legendPosition,
#'     title = plotParameters$title,
#'     size = plotParameters$titleSize
#' )
#' }

gridArrangeSharedLegend <- function(
    ...,  ncol = length(list(...)), nrow = 1, position = c("bottom", "right"),
    title = NA, titleSize = 12
) {
    plots <- list(...)
    position <- match.arg(position)
    g <- ggplotGrob(plots[[1]] + theme(legend.position = position))$grobs
    legend <- g[[
        which(vapply(g, function(x) x$name, FUN.VALUE = "") == "guide-box")
        ]]
    lheight <- sum(legend$height)
    lwidth <- sum(legend$width)
    gl <- lapply(plots, function(x) x + theme(legend.position="none"))
    gl <- c(gl, ncol = ncol, nrow = nrow)

    combined <- switch(
        position,
        "bottom" = gridExtra::arrangeGrob(
            do.call(gridExtra::arrangeGrob, gl),
            legend,
            ncol = 1,
            heights = grid::unit.c(grid::unit(1, "npc") - lheight, lheight),
            top = grid::textGrob(
                title,
                vjust = 1,
                gp = grid::gpar(
                    fontface = "bold", fontsize = titleSize
                )
            )
        ),
        "right" = gridExtra::arrangeGrob(
            do.call(gridExtra::arrangeGrob, gl),
            legend,
            ncol = 2,
            widths = grid::unit.c(grid::unit(1, "npc") - lwidth, lwidth),
            top = grid::textGrob(
                title,
                vjust = 1,
                gp = grid::gpar(
                    fontface = "bold", fontsize = titleSize
                )
            )
        )
    )
    return(combined)
}

#' Create variable distribution comparison plot
#' @description Create variable distribution plots between 2 groups of taxa for
#' a selected gene.
#' @export
#' @param data dataframe for plotting. Last column
#' indicates what type of taxon group (in- or out-group). The first (or first 2)
#' column contains values of the variables. See ?dataVarDistTaxGroup
#' @param plotParameters plot parameters, including size of x-axis, y-axis,
#' legend and title; position of legend ("right", "bottom" or "none");
#' mean/median point; names of in-group and out-group; and plot title.
#' NOTE: Leave blank or NULL to use default values.
#' @return Distribution plots as a grob (gtable) object. Use grid.draw to plot.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @seealso \code{\link{dataVarDistTaxGroup}}
#' @examples
#' data("mainLongRaw", package="PhyloProfile")
#' data <- mainLongRaw
#' inGroup <- c("ncbi9606", "ncbi10116")
#' variable <- colnames(data)[c(4, 5)]
#' plotDf <- dataVarDistTaxGroup(data, inGroup, "101621at6656", variable)
#' plotParameters <- list(
#'     "xSize" = 12,
#'     "ySize" = 12,
#'     "titleSize" = 15,
#'     "legendSize" = 12,
#'     "legendPosition" = "right",
#'     "mValue" = "mean",
#'     "inGroupName" = "In-group",
#'     "outGroupName" = "Out-group",
#'     "title" = "101621at6656"
#' )
#' g <- varDistTaxPlot(plotDf, plotParameters)
#' grid::grid.draw(g)

varDistTaxPlot <- function(data, plotParameters) {
    if (is.null(data)) stop("Input data cannot be NULL!")
    if (missing(plotParameters)) stop("Plot parameters are missing!")
    if (is.null(plotParameters)) {
        plotParameters <- list(
            "xSizeGC" = 12, "ySizeGC" = 12, 
            "titleSizeGC" = 15, "legendSizeGC"  = 12,
            "widthVarGC" = 600, "heightVarGC" =  400,
            "legendGC" =  "right", "mValueGC" = "mean",
            "widthFeatureGC" = 600, "heightFeatureGC" =  400,
            "xAxisGC" = "no", "angleGC" = 60,
            "inGroupName" = "In-group", "outGroupName" = "Out-group")
    }
    
    # rename in-group and out-group
    data$type[data$type == "In-group"] <- plotParameters$inGroupName
    data$type[data$type == "Out-group"] <- plotParameters$outGroupName
    # return plot(s)
    if (ncol(data) == 2) {
        plotVar1 <- generateSinglePlot(data, plotParameters, colnames(data)[1])
        return(
            gridExtra::arrangeGrob(
                plotVar1,
                top = grid::textGrob(
                    plotParameters$title, vjust = 1,
                    gp = grid::gpar(
                        fontface = "bold", fontsize = plotParameters$titleSize
                    )
                )
            )
        )
    } else {
        plotVar1 <- generateSinglePlot(data, plotParameters, colnames(data)[1])
        plotVar2 <- generateSinglePlot(data, plotParameters, colnames(data)[2])
        if (plotParameters$legendPosition == "none") {
            return(
                gridExtra::arrangeGrob(
                    plotVar1, plotVar2,
                    nrow = 1,
                    top = grid::textGrob(
                        plotParameters$title, vjust = 1,
                        gp = grid::gpar(
                            fontface = "bold",
                            fontsize = plotParameters$titleSize
                        )
                    )
                )
            )
        } else {
            return(
                gridArrangeSharedLegend(
                    plotVar1, plotVar2,
                    position = plotParameters$legendPosition,
                    title = plotParameters$title
                )
            )
        }
    }
}

#' Create data for feature distribution comparison plot
#' @description Create data for plotting the distribution of the protein domain
#' features between 2 group of taxa for a selected gene (average number of
#' feature occurrency per protein/ortholog).
#' @usage dataFeatureTaxGroup(mainDf, domainDf, inGroup, gene)
#' @export
#' @param mainDf input phylogenetic profile in long format (see ?mainLongRaw
#' and ?createLongMatrix)
#' @param domainDf dataframe contains domain info for the seed and ortholog.
#' This including the seed ID, orthologs IDs, sequence lengths, feature names,
#' start and end positions, feature weights (optional) and the status to
#' determine if that feature is important for comparison the architecture
#' between 2 proteins* (e.g. seed protein vs ortholog) (optional). (see
#' ?parseDomainInput)
#' @param inGroup ID list of in-group taxa (e.g. "ncbi1234")
#' @param gene ID of gene that need to be plotted the feature distribution
#' comparison between in- and out-group taxa.
#' @return Dataframe containing all feature names, their frequencies (absolute
#' count and the average instances per protein - IPP) in each taxon group and
#' the corresponding taxa group type (in- or out-group).
#' @author Vinh Tran (tran@bio.uni-frankfurt.de)
#' @seealso \code{\link{createLongMatrix}}, \code{\link{parseDomainInput}}
#' @examples
#' data("mainLongRaw", package="PhyloProfile")
#' mainDf <- mainLongRaw
#' gene <- "101621at6656"
#' inputFile <- system.file(
#'     "extdata", "domainFiles/101621at6656.domains",
#'     package = "PhyloProfile", mustWork = TRUE
#' )
#' type <- "file"
#' domainDf <- parseDomainInput(gene, inputFile, type)
#' inGroup <- c("ncbi9606", "ncbi10116")
#' dataFeatureTaxGroup(mainDf, domainDf, inGroup, gene)

dataFeatureTaxGroup <- function(
    mainDf = NULL, domainDf = NULL, inGroup = NULL, gene = NULL
) {
    if (is.null(mainDf) | is.null(inGroup) | is.null(gene) | is.null(domainDf))
        stop("Input profiles, in-group IDs, gene & domain data cannot be NULL!")
    # get ncbiIDs for the domain data
    mainDf$orthoID <- gsub("\\|", ":", mainDf$orthoID)
    domainDfSub <- merge(
        domainDf[grep(paste0(gene, "#"), domainDf$seedID),][
            ,c("orthoID","seedID","feature")
        ],
        mainDf[, c("orthoID", "ncbiID")], by = "orthoID", all.x = TRUE
    )
    domainDfSub <- domainDfSub[stats::complete.cases(domainDfSub),]
    # identify in-group and out-group
    domainDfSub$type[domainDfSub$ncbiID %in% inGroup] <- "In-group"
    domainDfSub$type[!(domainDfSub$ncbiID %in% inGroup)] <- "Out-group"
    # count number of orthologs for each taxon group
    nInGroup <- length(
        unique(domainDfSub$seedID[domainDfSub$type == "In-group"]))
    nOutGroup <- length(
        unique(domainDfSub$seedID[domainDfSub$type == "Out-group"]))
    # get intances and count the number of intances for each taxon group
    domainIn <- domainDfSub$feature[domainDfSub$ncbiID %in% inGroup]
    domainOut <- domainDfSub$feature[!(domainDfSub$ncbiID %in% inGroup)]
    countDomainIn <- data.frame(table(unlist(as.character(domainIn))))
    countDomainIn$type <- "In-group"
    countDomainIn$ipp <- countDomainIn$Freq/nInGroup
    countDomainOut <- data.frame(table(unlist(as.character(domainOut))))
    countDomainOut$type <- "Out-group"
    countDomainOut$ipp <- countDomainOut$Freq/nOutGroup
    # calculate delta IPP
    mergedDf <- merge(countDomainIn, countDomainOut, by = "Var1", all = TRUE)
    mergedDf[is.na(mergedDf)] <- 0
    mergedDf$dIPPtmp <- mergedDf$ipp.x - mergedDf$ipp.y
    mergedDf$dIPP <- mergedDf$dIPPtmp / (mergedDf$ipp.x + mergedDf$ipp.y)
    # return
    outDf <- merge(
        rbind(countDomainIn, countDomainOut), mergedDf[, c("Var1", "dIPP")],
        by = "Var1", all.x = TRUE)
    colnames(outDf) <- c("Feature", "Count", "Taxon_group", "IPP", "dIPP")
    return(outDf)
}

#' Create feature distribution comparison plot
#' @description Create protein feature distribution plots between 2 groups of
#' taxa for a selected gene.
#' @export
#' @param data dataframe for plotting (see ?dataFeatureTaxGroup)
#' @param plotParameters plot parameters, including size of x-axis, y-axis,
#' legend and title; position of legend ("right", "bottom" or "none"); names of
#' in-group and out-group; flip the plot coordinate ("Yes" or "No").
#' NOTE: Leave blank or NULL to use default values.
#' @return Distribution plots as a ggplot2 object.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @seealso \code{\link{dataFeatureTaxGroup}}
#' @examples
#' data("mainLongRaw", package="PhyloProfile")
#' data <- mainLongRaw
#' gene <- "101621at6656"
#' inputFile <- system.file(
#'     "extdata", "domainFiles/101621at6656.domains",
#'     package = "PhyloProfile", mustWork = TRUE
#' )
#' type <- "file"
#' domainDf <- parseDomainInput(gene, inputFile, type)
#' inGroup <- c("ncbi9606", "ncbi10116")
#' plotDf <- dataFeatureTaxGroup(data, domainDf, inGroup, gene)
#' plotParameters <- list(
#'     "xSize" = 12,
#'     "ySize" = 12,
#'     "angle" = 15,
#'     "legendSize" = 12,
#'     "inGroupName" = "In-group",
#'     "outGroupName" = "Out-group",
#'     "flipPlot" = "No"
#' )
#' featureDistTaxPlot(plotDf, plotParameters)

featureDistTaxPlot <- function(data, plotParameters) {
    Feature <- NULL
    IPP <- NULL
    Taxon_group <- NULL
    if (is.null(data)) stop("Input data cannot be NULL!")
    if (missing(plotParameters)) stop("Plot parameters are missing!")
    if (is.null(plotParameters)) {
        plotParameters <- list(
            "xSizeGC" = 12, "ySizeGC" = 12, 
            "titleSizeGC" = 15, "legendSizeGC"  = 12,
            "widthVarGC" = 600, "heightVarGC" =  400,
            "legendGC" =  "right", "mValueGC" = "mean",
            "widthFeatureGC" = 600, "heightFeatureGC" =  400,
            "xAxisGC" = "no", "angleGC" = 60,
            "inGroupName" = "In-group", "outGroupName" = "Out-group")
    }
        
    data$Taxon_group[data$Taxon_group == "In-group"] <-
        plotParameters$inGroupName
    data$Taxon_group[data$Taxon_group == "Out-group"] <-
        plotParameters$outGroupName

    plot <- ggplot(data, aes(x = Feature, y = IPP, fill = Taxon_group)) +
        geom_bar(stat="identity", width=.5, position = "dodge") +
        theme_minimal() +
        theme(
            axis.title.y = element_text(size = plotParameters$ySize),
            axis.text.y = element_text(size = plotParameters$ySize),
            axis.title.x = element_text(size = plotParameters$xSize),
            axis.text.x = element_text(
                size = plotParameters$xSize,
                angle = plotParameters$angle,
                hjust = 1
            ),
            legend.text = element_text(size = plotParameters$legendSize),
            legend.title = element_blank()
        )
    if (plotParameters$flipPlot == "Yes") plot <- plot + coord_flip()
    return(plot)
}

#' Import user defined NCBI taxonomy database
#' @return none (imported files will be saved in PhyloProfile package folder)
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}
exportNcbiTax <- function(outDir, taxDB) {
    taxFiles <- c(
        "newTaxa.txt",
        "taxonNamesReduced.txt",
        "idList.txt",
        "rankList.txt",
        "taxonomyMatrix.txt"
    )
    # Check required files
    message("1) Checking taxonomy files in ", outDir,"...")
    flag = 1
    for (file in taxFiles) {
        if (file.exists(paste0(outDir, "/", file))) flag = 0
    }

    if (flag == 0) {
        message(
            "<p><span style=\"color: #ff0000;\"><strong>WARNING</strong></span>: Some of the taxonomy files already exist in the selected folder! Please rename or move them to another place!</p>"
        )
    } else {
        message("2) Exporting data from ", taxDB, "...")
        if (file.exists(paste0(taxDB, "/preCalcTree.nw")))
            taxFiles <- c(taxFiles, "preCalcTree.nw")
        for (file in taxFiles) {
            system(
                paste(
                    "cp",
                    paste0(taxDB, "/", file),
                    paste0(outDir, "/", file)
                )
            )
        }
        message(
            "3) FINISHED! The current NCBI taxonomy database has been saved in ", outDir, "!<br>"
        )
    }
}

#' Process ortholog IDs
#' @description Process ortholog IDs to identify duplicated IDs
#' @param dataHeat a data frame contains processed profiles (see
#' ?fullProcessedProfile, ?filterProfileData)
#' @return the same dataframe as input, but the ortholog IDs are changed into
#' <taxID:orthoID>. New column "orthoFreq" specifies if the ortholog IDs are
#' single or duplicated
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @export
#' @examples
#' ?processOrthoID
#' data("finalProcessedProfile", package="PhyloProfile")
#' processOrthoID(finalProcessedProfile)

processOrthoID <- function(dataHeat = NULL) {
    if (is.null(dataHeat)) stop("Input data cannot be NULL!")
    orthoID <- geneID <- orthoFreqCount <- orthoFreqNew <- supertaxonID <- NULL
    orthoIDNew <- orthoFreq <- NULL
    # Check if idFormat is "bionf"
    idFormat <- "other"
    firstOrtho <- strsplit(dataHeat$orthoID[1],'\\|')[[1]]
    firstGeneID <- dataHeat$geneID[1]
    firstSupertaxonID <- dataHeat$supertaxonID[1]
    if (
        length(firstOrtho) >= 3 && firstOrtho[1] == firstGeneID &&
        grepl(firstSupertaxonID, firstOrtho[2])
    ) { idFormat <- "bionf" }

    # parse orthoID
    if (idFormat == "bionf") {
        dataHeat <- within(
            dataHeat,
            orthoMod <- data.frame(
                do.call('rbind', strsplit(orthoID,'|',fixed=TRUE))
            )
        )
        dataHeat$orthoIDNew <- paste(
            dataHeat$orthoMod$X2, dataHeat$orthoMod$X3, sep = "#"
        )
    } else {
        dataHeat$orthoIDNew <- paste(
            dataHeat$supertaxonID, dataHeat$orthoID, sep = "#"
        )
    }
    dataHeat <- dataHeat[ , !(names(dataHeat) %in% ("orthoMod"))]
    # Count occurrences of orthoIDNew and label as "Single" or "Multiple"
    countOrthoDf <- dataHeat %>%
        count(orthoIDNew, name = "orthoFreq") %>%
        mutate(orthoFreq = ifelse(orthoFreq > 1, "Multiple", "Single"))
    # Merge the frequency labels back to the main data
    dataHeat <- dataHeat %>%
        left_join(countOrthoDf, by = "orthoIDNew")
    # Check for "Multiple" freq in co-orthologs at geneID and supertaxonID level
    freqDt <- dataHeat %>%
        distinct(geneID, supertaxonID, orthoFreq) %>%
        group_by(geneID, supertaxonID) %>%
        mutate(
            orthoFreq = ifelse("Multiple" %in% orthoFreq, "Multiple", orthoFreq)
        ) %>% ungroup()
    # Merge the updated frequency labels back to the main data
    dataHeat <- dataHeat %>% left_join(
        freqDt, by = c("geneID", "supertaxonID"), suffix = c("", "New"),
        relationship = "many-to-many"
    )
    # Step 3: Update orthoFreq with the new frequency labels and clean up
    dataHeat <- dataHeat %>% mutate(orthoFreq = orthoFreqNew) %>%
        select(-orthoFreqNew)
    return(dataHeat)
}

#' Create data for main profile plot
#' @export
#' @param dataHeat a data frame contains processed profiles (see
#' ?fullProcessedProfile, ?filterProfileData)
#' @return A dataframe for plotting the phylogenetic profile, containing seed
#' protein IDs (geneID), ortholog IDs (orthoID) together with their ncbi
#' taxonomy IDs (ncbiID and abbrName), full names (fullName), indexed supertaxa
#' (supertaxon), values for additional variables (var1, var2) and the aggregated
#' values of those additional variables for each supertaxon (mVar1, mVar2),
#' number of original and filtered co-orthologs in each supertaxon (paralog and
#' paralogNew), number of species in each supertaxon (numberSpec) and the % of
#' species that have orthologs in each supertaxon (presSpec).
#' @importFrom stats na.omit
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @seealso \code{\link{filterProfileData}}
#' @examples
#' data("finalProcessedProfile", package="PhyloProfile")
#' dataMainPlot(finalProcessedProfile)

dataMainPlot <- function(dataHeat = NULL){
    if (is.null(dataHeat)) stop("Input data cannot be NULL!")
    paralogNew <- orthoID <- NULL
    # reduce number of inparalogs based on filtered dataHeat
    dataHeatTb <- data.table(stats::na.omit(dataHeat))
    dataHeatTb[, paralogNew := .N, by = c("geneID", "supertaxon")]
    dataHeatTb <- data.frame(dataHeatTb[, c(
        "geneID", "supertaxon", "paralogNew"
    )])

    dataHeat <- dataHeat %>% left_join(dataHeatTb, by=c("geneID","supertaxon"))
    dataHeat$paralog <- dataHeat$paralogNew
    dataHeat <- dataHeat[!duplicated(dataHeat), ]

    # remove unneeded dots
    dataHeat$presSpec[dataHeat$presSpec == 0] <- NA
    dataHeat$paralog[dataHeat$presSpec < 1] <- NA
    dataHeat$paralog[dataHeat$paralog == 1] <- NA
    # rescale numbers of paralogs
    if (length(unique(stats::na.omit(dataHeat$paralog))) > 0) {
        maxParalog <- max(stats::na.omit(dataHeat$paralog))
        dataHeat$paralogSize <- (dataHeat$paralog / maxParalog) * 3
    }
    # remove prefix number of taxa names but keep the order
    dataHeat$supertaxon <- factor(
        substr(
            as.character(dataHeat$supertaxon), 8 ,
            nchar(as.character(dataHeat$supertaxon))),
        levels = substr(
            levels(as.factor(dataHeat$supertaxon)), 8,
            nchar(levels(as.factor(dataHeat$supertaxon)))))
    # count ortholog IDs for each taxon
    dataHeat <- processOrthoID(dataHeat)
    return(dataHeat)
}

#' Create data for customized profile plot
#' @description Create data for customized profile plot based on a selected
#' list of genes and/or taxa, containing seed protein IDs (geneID), ortholog IDs
#' (orthoID) together with their ncbi taxonomy IDs (ncbiID and abbrName), full
#' names (fullName), indexed supertaxa (supertaxon), values for additional
#' variables (var1, var2) and the aggregated values of those additional
#' variables for each supertaxon (mVar1, mVar2), number of original and filtered
#' co-orthologs in each supertaxon (paralog and paralogNew), number of species
#' in each supertaxon (numberSpec) and the % of species that have orthologs in
#' each supertaxon (presSpec).
#' @export
#' @usage dataCustomizedPlot(dataHeat = NULL, selectedTaxa = "all",
#'     selectedSeq = "all")
#' @param dataHeat a data frame contains processed profiles (see
#' ?fullProcessedProfile, ?filterProfileData)
#' @param selectedTaxa selected subset of taxa. Default = "all".
#' @param selectedSeq selected subset of genes. Default = "all".
#' @return A dataframe contains data for plotting the customized profile.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom stats na.omit
#' @seealso \code{\link{filterProfileData}}
#' @examples
#' data("finalProcessedProfile", package="PhyloProfile")
#' selectedTaxa <- c("Mammalia", "Saccharomycetes", "Insecta")
#' selectedSeq <- "all"
#' dataCustomizedPlot(finalProcessedProfile, selectedTaxa, selectedSeq)

dataCustomizedPlot <- function(
    dataHeat = NULL, selectedTaxa = "all", selectedSeq = "all"
){
    if (is.null(dataHeat)) stop("Input data cannot be NULL!")
    geneID <- supertaxonMod <- paralogNew <- NULL
    dataHeat$supertaxonMod <- {
        substr(
            dataHeat$supertaxon, 8, nchar(as.character(dataHeat$supertaxon))
        )
    }
    if (selectedTaxa[1] == "all" & selectedSeq[1] != "all") {
        dataHeat <- subset(dataHeat, geneID %in% selectedSeq)
    } else if (selectedSeq[1] == "all" & selectedTaxa[1] != "all") {
        dataHeat <- subset(dataHeat, supertaxonMod %in% selectedTaxa)
    } else {
        dataHeat <- subset(
            dataHeat, geneID %in% selectedSeq & supertaxonMod %in% selectedTaxa
        )
    }
    # reduce number of inparalogs based on filtered dataHeat
    dataHeatTb <- data.table(stats::na.omit(dataHeat))
    dataHeatTb[, paralogNew := .N, by = c("geneID", "supertaxon")]
    dataHeatTb <- data.frame(
        dataHeatTb[, c("geneID", "supertaxon", "paralogNew")]
    )
    dataHeat <- merge(
        dataHeat, dataHeatTb, by = c("geneID", "supertaxon"), all.x = TRUE
    )
    dataHeat$paralog <- dataHeat$paralogNew
    dataHeat <- dataHeat[!duplicated(dataHeat), ]
    # remove unneeded dots
    dataHeat$presSpec[dataHeat$presSpec == 0] <- NA
    dataHeat$paralog[dataHeat$presSpec < 1] <- NA
    dataHeat$paralog[dataHeat$paralog == 1] <- NA
    # rescale numbers of paralogs
    if (length(unique(stats::na.omit(dataHeat$paralog))) > 0) {
        maxParalog <- max(stats::na.omit(dataHeat$paralog))
        dataHeat$paralogSize <- (dataHeat$paralog / maxParalog) * 3
    }
    # remove prefix number of taxa names but keep the order
    dataHeat$supertaxon <- factor(
        substr(
            as.character(dataHeat$supertaxon), 8,
            nchar(as.character(dataHeat$supertaxon))),
        levels = substr(
            levels(dataHeat$supertaxon), 8,
            nchar(levels(dataHeat$supertaxon))))
    # count ortholog IDs for each taxon
    if (nrow(dataHeat) == 0) return(dataHeat)
    dataHeat <- processOrthoID(dataHeat)
    return(dataHeat)
}

#' Create profile heatmap plot
#' @export
#' @param data dataframe for plotting the heatmap phylogentic profile (either
#' full or subset profiles)
#' @param parm plot parameters, including (1) type of x-axis "taxa" or
#' "genes" - default = "taxa"; (2) display gene IDs (default) or gene names;
#' (3+4) names of 2 variables var1ID and var2ID - default = "var1" & "var2";
#' (5+6) mid value and color for mid value of var1 -
#' default is 0.5 and #FFFFFF; (7) color for lowest var1 - default = "#FF8C00";
#' (8) color for highest var1 - default = "#4682B4"; (9+10) mid value and color
#' for mid value of var2 - default is 1 and #FFFFFF;(11) color for lowest var2 -
#' default = "#FFFFFF", (12) color for highest var2 - default = "#F0E68C", (13)
#' color of co-orthologs - default = "#07D000"; (14+15+16) text sizes for x, y
#' axis and legend - default = 9 for each; (17) legend position "top", "bottom",
#' "right", "left" or "none" - default = "top"; (18) zoom ratio of the
#' co-ortholog dots from -1 to 3 - default = 0; (19) angle of x-axis from 0 to
#' 90 - default = 60; (20) show/hide separate line for reference taxon 1/0 -
#' default = 0; (21) enable/disable coloring gene categories TRUE/FALSE -
#' default = FALSE; (22) enable/disable coloring duplicated ortholog IDs
#' TRUE/FALSE - default=FALSE). NOTE: Leave blank or NULL to use default values.
#' @return A profile heatmap plot as a ggplot object.
#' @import ggplot2
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @seealso \code{\link{dataMainPlot}}, \code{\link{dataCustomizedPlot}}
#' @examples
#' data("finalProcessedProfile", package="PhyloProfile")
#' plotDf <- dataMainPlot(finalProcessedProfile)
#' plotParameter <- list(
#'     "xAxis" = "taxa",
#'     "geneIdType" = "geneID",
#'     "var1ID" = "FAS_FW",
#'     "var2ID"  = "FAS_BW",
#'     "midVar1" = 0.5,
#'     "midColorVar1" =  "#FFFFFF",
#'     "lowColorVar1" =  "#FF8C00",
#'     "highColorVar1" = "#4682B4",
#'     "midVar2" = 1,
#'     "midColorVar2" =  "#FFFFFF",
#'     "lowColorVar2" = "#CB4C4E",
#'     "highColorVar2" = "#3E436F",
#'     "paraColor" = "#07D000",
#'     "xSize" = 8,
#'     "ySize" = 8,
#'     "legendSize" = 8,
#'     "mainLegend" = "top",
#'     "dotZoom" = 0,
#'     "xAngle" = 60,
#'     "guideline" = 0,
#'     "colorByGroup" = FALSE,
#'     "catColors" = NULL,
#'     "colorByOrthoID" = FALSE
#' )
#'
#' heatmapPlotting(plotDf, plotParameter)

heatmapPlotting <- function(data = NULL, parm = NULL){
    if (is.null(data)) stop("Input data cannot be NULL!")
    if (is.null(parm))
        parm <- list(
            "xAxis" = "taxa", "geneIdType" = "geneName",
            "var1ID" = "var1", "var2ID"  = "var2",
            "midVar1" = 0.5, "midColorVar1" =  "#FFFFFF",
            "lowColorVar1" =  "#FF8C00", "highColorVar1" = "#4682B4",
            "midVar2" = 1, "midColorVar2" =  "#FFFFFF",
            "lowColorVar2" = "#CB4C4E", "highColorVar2" = "#3E436F",
            "paraColor" = "#07D000", "xSize" = 8, "ySize" = 8, "legendSize" = 8,
            "mainLegend" = "top", "dotZoom" = 0, "xAngle" = 60, "guideline" = 0,
            "colorByGroup" = FALSE,"catColors" = NULL,"colorByOrthoID" = FALSE)
    geneID<-geneName<-supertaxon<-category<-var1<-var2<- presSpec<-paralog<-NULL
    orthoFreq <- xmin <- xmax <- ymin <- ymax <- NULL
    ### create heatmap plot
    if (!(is.null(parm$geneIdType))) {
        if (parm$geneIdType == "geneName") data$geneID <- data$geneName
    }
    # create a canvas
    if (parm$xAxis == "genes") {
        p <- ggplot(data,aes(x=geneID, y=supertaxon)) +
            labs(x = "Gene ID", y = "Taxon")
    } else {
        p <- ggplot(data, aes(y = geneID, x = supertaxon)) +
            labs(y = "Gene ID", x = "Taxon")
    }
    # create geom_tile & scale_fill_gradient for var2 OR gene category
    if (parm$colorByGroup == TRUE) {
        p <- p + geom_raster(aes(fill = factor(category)), alpha = 0.3)
        if (!is.null(parm$catColors))
            p <- p + scale_fill_manual(values = parm$catColors)
    } else {
        if (length(unique(stats::na.omit(data$var2))) != 1)
            p <- p + scale_fill_gradient2(
                low = parm$lowColorVar2, high = parm$highColorVar2,
                mid = parm$midColorVar2, midpoint = parm$midVar2,
                na.value = "gray95", limits = c(0, 1)) +
                geom_raster(aes(fill = var2))
    }

    # create geom_point for found ortho; coloring by var1 or
    # orthoIDs (only when working on lowest taxonomy rank)
    # and inparalogs (co-orthologs)
    if (length(unique(stats::na.omit(data$presSpec))) < 2) {
        # working on the lowest taxonomy rank
        if (parm$colorByOrthoID == TRUE) {
            # color by ortho IDs
            p <- p + geom_point(
                aes(colour = factor(orthoFreq)), na.rm = TRUE,
                size = data$presSpec*5*(1+parm$dotZoom), show.legend = TRUE
            )
        } else {
            if (length(unique(stats::na.omit(data$var1))) == 1) {
                p <- p + geom_point(
                    aes(colour = var1), na.rm = TRUE,
                    size = data$presSpec*5*(1+parm$dotZoom), show.legend = FALSE
                )
            } else {
                p <- p +
                    geom_point(
                        aes(colour = var1), na.rm = TRUE,
                        size = data$presSpec * 5 * (1 + parm$dotZoom)) +
                    scale_color_gradient2(
                        low = parm$lowColorVar1, high = parm$highColorVar1,
                        mid = parm$midColorVar1, midpoint = parm$midVar1,
                        limits = c(0,1)
                    )
            }
            # plot inparalogs (if available)
            if (length(unique(stats::na.omit(data$paralog))) > 0) {
                # Define min and max for your desired size range
                min_size <- 1
                max_size <- 3*(1+parm$dotZoom)
                # Manually rescale paralog size
                min_val <- min(data$paralog, na.rm = TRUE)
                max_val <- max(data$paralog, na.rm = TRUE)
                # Avoid division by zero (when min == max)
                if (min_val == max_val) {
                    data$paralogSize <- max_size
                } else {
                    data$paralogSize <-
                        ((data$paralog - min_val) / (max_val - min_val)) *
                        (max_size - min_size) + min_size
                }
                # create breaks for legend
                paralog_vals <- sort(unique(na.omit(data$paralog)))
                n_vals <- length(paralog_vals)
                custom_breaks <- if (n_vals <= 5) {
                    paralog_vals
                } else {
                    paralog_vals[round(seq(2, n_vals, length.out = 5))]
                }
                p <- p +
                    geom_point(
                        data = data, aes(size = paralog), color=parm$paraColor,
                        na.rm = TRUE, show.legend = TRUE) +
                    guides(size = guide_legend(title = "# of co-orthologs")) +
                scale_size_continuous(
                    breaks = custom_breaks, range = c(min_size, max_size))
            }
        }
    } else {
        # scale dot size based on % preSpec in each super taxon
        if (length(unique(stats::na.omit(data$var1))) == 1) {
            p <- p + geom_point(aes(size=presSpec),color="#336a98",na.rm = TRUE)
        } else {
            p <- p + geom_point(aes(colour=var1, size = presSpec),na.rm = TRUE)+
                scale_color_gradient2(
                    low = parm$lowColorVar1, high = parm$highColorVar1,
                    mid = parm$midColorVar1, midpoint = parm$midVar1,
                    limits = c(0,1)
                )
        }
        # remain the scale of point while filtering % presSpec
        presentVl <- data$presSpec[!is.na(data$presSpec)]
        p <- p + scale_size_continuous(range = c(
            (floor(min(presentVl) * 10) / 10 * 5) * (1 + parm$dotZoom),
            (floor(max(presentVl) * 10) / 10 * 5) * (1 + parm$dotZoom)))
    }

    # create legend
    if (parm$colorByGroup == FALSE) {
        if (parm$colorByOrthoID == FALSE) {
            p <- p + guides(fill = guide_colourbar(title = parm$var2ID),
                            color = guide_colourbar(title = parm$var1ID))
        } else {
            p <- p + guides(fill = guide_colourbar(title = parm$var2ID),
                            color = guide_legend(title = 'OrthoID copy number'))
        }
    } else {
        p <- p + guides(fill = guide_legend("Category"),
                        color = guide_colourbar(title = parm$var1ID))
    }

    # text size, legend position
    p <- p + theme_minimal(base_size = 9)
    vjustValue <- 1
    if (parm$xAngle == 90) vjustValue <- 0.5
    p <- p + theme(
        axis.text.x = element_text(
            angle = parm$xAngle, hjust = 1, size = parm$xSize,
            vjust = vjustValue
        ),
        axis.text.y = element_text(size = parm$ySize),
        axis.title.x = element_text(size = parm$xSize),
        axis.title.y = element_text(size = parm$ySize),
        legend.title = element_text(size = parm$legendSize),
        legend.text = element_text(size = parm$legendSize),
        legend.position = parm$mainLegend)
    return(p)
}


#' Create profile heatmap plot using scattermore
#' @export
#' @param data dataframe for plotting the heatmap phylogentic profile (either
#' full or subset profiles)
#' @param parm plot parameters, including (1) type of x-axis "taxa" or
#' "genes" - default = "taxa"; (2) display gene IDs (default) or gene names;
#' (3+4) names of 2 variables var1ID and var2ID - default = "var1" & "var2";
#' (5+6) mid value and color for mid value of var1 -
#' default is 0.5 and #FFFFFF; (7) color for lowest var1 - default = "#FF8C00";
#' (8) color for highest var1 - default = "#4682B4"; (9+10) mid value and color
#' for mid value of var2 - default is 1 and #FFFFFF;(11) color for lowest var2 -
#' default = "#FFFFFF", (12) color for highest var2 - default = "#F0E68C", (13)
#' color of co-orthologs - default = "#07D000"; (14+15+16) text sizes for x, y
#' axis and legend - default = 9 for each; (17) legend position "top", "bottom",
#' "right", "left" or "none" - default = "top"; (18) zoom ratio of the
#' co-ortholog dots from -1 to 3 - default = 0; (19) color dots based on either
#' "var1" or "var2". NOTE: Leave blank or NULL to
#' use default values.
#' @return A profile heatmap plot as a ggplot object.
#' @import ggplot2
#' @import scattermore
#' @importFrom grDevices colorRampPalette
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @seealso \code{\link{dataMainPlot}}, \code{\link{dataCustomizedPlot}}
#' @examples
#' data("finalProcessedProfile", package="PhyloProfile")
#' plotDf <- dataMainPlot(finalProcessedProfile)
#' plotParameter <- list(
#'     "xAxis" = "taxa",
#'     "geneIdType" = "geneID",
#'     "var1ID" = "FAS_FW",
#'     "var2ID"  = "FAS_BW",
#'     "midVar1" = 0.5,
#'     "midColorVar1" =  "#FFFFFF",
#'     "lowColorVar1" =  "#FF8C00",
#'     "highColorVar1" = "#4682B4",
#'     "midVar2" = 1,
#'     "midColorVar2" =  "#FFFFFF",
#'     "lowColorVar2" = "#CB4C4E",
#'     "highColorVar2" = "#3E436F",
#'     "paraColor" = "#07D000",
#'     "xSize" = 8,
#'     "ySize" = 8,
#'     "legendSize" = 8,
#'     "mainLegend" = "top",
#'     "dotZoom" = 0,
#'     "colorVar" = "var1"
#' )
#'
#' heatmapPlottingFast(plotDf, plotParameter)

heatmapPlottingFast <- function(data = NULL, parm = NULL) {
    if (is.null(data)) stop("Input data cannot be NULL!")
    if (is.null(parm))
        parm <- list(
            "xAxis" = "taxa", "geneIdType" = "geneName",
            "var1ID" = "var1", "var2ID"  = "var2",
            "midVar1" = 0.5, "midColorVar1" =  "#FFFFFF",
            "lowColorVar1" =  "#FF8C00", "highColorVar1" = "#4682B4",
            "midVar2" = 1, "midColorVar2" =  "#FFFFFF",
            "lowColorVar2" = "#CB4C4E", "highColorVar2" = "#3E436F",
            "paraColor" = "#07D000", "xSize" = 8, "ySize" = 8, "legendSize" = 8,
            "mainLegend" = "top", "dotZoom" = 0, "colorVar" = "var1")
    geneID<-geneName<-supertaxon<-category<-var1<-var2<- presSpec<-paralog<-NULL
    orthoFreq <- xmin <- xmax <- ymin <- ymax <- x <- y <- NULL
    if (is.null(parm$colorVar)) parm$colorVar <- "var1"
    ### create heatmap plot
    if (!(is.null(parm$geneIdType))) {
        if (parm$geneIdType == "geneName") data$geneID <- data$geneName
    }
    # create a canvas
    if (parm$xAxis == "genes") {
        p <- ggplot(data,aes(x = geneID, y = supertaxon)) +
            labs(x = "Gene ID", y = "Taxon")
    } else {
        p <- ggplot(data, aes(y = geneID, x = supertaxon)) +
            labs(y = "Gene ID", x = "Taxon")
    }
    # format theme
    p <- p + theme_minimal(base_size = 9)
    p <- p + theme(
        axis.text = element_blank(),
        axis.title.x = element_text(size = parm$xSize),
        axis.title.y = element_text(size = parm$ySize),
        legend.title = element_text(size = parm$legendSize),
        legend.text = element_text(size = parm$legendSize),
        legend.position = parm$mainLegend)
    # Extract x, y, and color values
    x_values <- as.numeric(data$supertaxon)
    y_values <- as.numeric(data$geneID)
    # Create color vector: green if paralog > 1, else gradient by var1 or var2
    if (parm$colorVar == "var1") {
        cl <- colorRampPalette(c(parm$lowColorVar1, parm$highColorVar1))(100)[
            cut(data$var1, breaks = 100)
        ]
    } else
        cl <- colorRampPalette(c(parm$lowColorVar2, parm$highColorVar2))(100)[
            cut(data$var2, breaks = 100)
        ]
    cl[data$paralog > 1] <- parm$paraColor
    # Add scatter plot
    p <- p +
        geom_scattermost(
            cbind(x_values, y_values), color = cl,
            pointsize = 2 * (1 + parm$dotZoom), pixels = c(1000, 1000)
        )
    if (parm$colorVar == "var1") {
        p <- p +
            geom_point(data=data.frame(x = double(0)), aes(x, x, color = x)) +
            scale_color_gradientn(
                colors = colorRampPalette(
                    c(parm$lowColorVar1, parm$highColorVar1))(100),
                limits = c(0,1), name = parm$var1ID
            )
    } else {
        p <- p +
            geom_point(data=data.frame(x = double(0)), aes(x, x, color = x)) +
            scale_color_gradientn(
                colors = colorRampPalette(
                    c(parm$lowColorVar2, parm$highColorVar2))(100),
                limits = c(0,1), name = parm$var2ID
            )
    }
    # Add only one real co-ortholog point with fill = green to generate legend
    if(any(data$paralog > 1, na.rm = TRUE)) {
        co_point <- data[data$paralog > 1, ][1, ]
        co_point$x <- as.numeric(co_point$supertaxon)
        co_point$y <- as.numeric(co_point$geneID)
        p <- p + geom_point(
            data = co_point, aes(x = x, y = y, fill = "Co-orthologs"),
            shape = 21, size = 3
        ) + scale_fill_manual(
            values = c("Co-orthologs" = parm$paraColor), name = NULL
        )
    }
    return(p)
}

#' Highlight gene and/or taxon of interest on the phylogenetic profile plot
#' @export
#' @usage highlightProfilePlot(profilePlot = NULL, plotDf = NULL,
#'     taxonHighlight = "none", workingRank = "none", geneHighlight = NULL,
#'     taxDB = NULL, xAxis = "taxa")
#' @param profilePlot initial (highlighted) profile plot
#' @param plotDf dataframe for plotting the heatmap phylogentic profile
#' @param taxonHighlight taxon of interst. Default = "none".
#' @param workingRank working taxonomy rank (needed only for highlight taxon).
#' @param geneHighlight gene of interest. Default = NULL.
#' @param taxDB Path to the taxonomy DB files
#' @param xAxis type of x-axis (either "genes" or "taxa")
#' @return A profile heatmap plot with highlighted gene and/or taxon of interest
#' as ggplot object.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @seealso \code{\link{dataMainPlot}}, \code{\link{dataCustomizedPlot}},
#' \code{\link{heatmapPlotting}}
#' @examples
#' data("finalProcessedProfile", package="PhyloProfile")
#' plotDf <- dataMainPlot(finalProcessedProfile)
#' plotParameter <- list(
#'     "xAxis" = "taxa",
#'     "geneIdType" = "geneID",
#'     "var1ID" = "FAS_FW",
#'     "var2ID"  = "FAS_BW",
#'     "midVar1" = 0.5,
#'     "midColorVar1" =  "#FFFFFF",
#'     "lowColorVar1" =  "#FF8C00",
#'     "highColorVar1" = "#4682B4",
#'     "midVar2" = 1,
#'     "midColorVar2" =  "#FFFFFF",
#'     "lowColorVar2" = "#CB4C4E",
#'     "highColorVar2" = "#3E436F",
#'     "paraColor" = "#07D000",
#'     "xSize" = 8,
#'     "ySize" = 8,
#'     "legendSize" = 8,
#'     "mainLegend" = "top",
#'     "dotZoom" = 0,
#'     "xAngle" = 60,
#'     "guideline" = 0,
#'     "colorByGroup" = FALSE,
#'     "colorByOrthoID" = FALSE
#' )
#' profilePlot <- heatmapPlotting(plotDf, plotParameter)
#' taxonHighlight <- "none"
#' workingRank <- "class"
#' geneHighlight <- "100265at6656"
#' highlightProfilePlot(
#'     profilePlot, plotDf, taxonHighlight, workingRank, geneHighlight,
#'     NULL, plotParameter$xAxis
#' )

highlightProfilePlot <- function(
        profilePlot = NULL, plotDf = NULL, taxonHighlight = "none",
        workingRank = "none", geneHighlight = NULL, taxDB = NULL, xAxis = "taxa"
){
    if (is.null(plotDf)) stop("Input data cannot be NULL!")
    if (is.null(profilePlot)) stop("Profile plot cannot be NULL!")
    xmin <- xmax <- ymin <- ymax <- NULL
    if (is.null(taxonHighlight) && is.null(geneHighlight)) return(profilePlot)
    # highlight taxon
    if (length(taxonHighlight) > 0 && !("none" %in% taxonHighlight)) {
        # get selected highlight taxon ID
        taxName <- getNameList(taxDB)
        taxonHighlightID <- taxName$ncbiID[
            taxName$fullName %in% taxonHighlight & taxName$rank == workingRank]
        if (length(taxonHighlightID) == 0L) {
            taxonHighlightID <- taxName$ncbiID[
                taxName$fullName %in% taxonHighlight
            ]
        } else if(length(taxonHighlightID) < length(taxonHighlight)) {
            missingID <- taxName$ncbiID[
                taxName$fullName %in% taxonHighlight
            ]
            missingID <- missingID[!(missingID %in% taxonHighlightID)]
            taxonHighlightID <- c(taxonHighlightID, missingID)
        }
        # get taxonID together with it sorted index
        taxonHighlightID <- intersect(taxonHighlightID, plotDf$supertaxonID)
        if (length(taxonHighlightID) > 0) {
            selTaxon <- unique(as.character(
                plotDf$supertaxon[plotDf$supertaxonID %in% taxonHighlightID]
            ))
            selIndex <- match(selTaxon, levels(as.factor(plotDf$supertaxon)))
            if (xAxis == "taxa") {
                rect <- data.frame(
                    xmin=selIndex-0.5, xmax=selIndex+0.5, ymin=-Inf, ymax=Inf)
            } else
                rect <- data.frame(
                    ymin=selIndex-0.5, ymax=selIndex+0.5, xmin=-Inf, xmax=Inf)
            profilePlot <- profilePlot + geom_rect(
                data = rect, color = "yellow", alpha = 0.3, inherit.aes = FALSE,
                aes(xmin = xmin, xmax = xmax, ymin = ymin, ymax = ymax))
        }
    }
    # highlight gene
    if (length(geneHighlight) > 0 && !("none" %in% geneHighlight)) {
        geneHighlight <- intersect(geneHighlight, unique(plotDf$geneID))
        if (length(geneHighlight) > 0) {
            selIndex <- match(geneHighlight, levels(as.factor(plotDf$geneID)))
            if (xAxis == "taxa") {
                rect <- data.frame(
                    ymin=selIndex-0.5, ymax=selIndex+0.5, xmin=-Inf, xmax=Inf)
            } else
                rect <- data.frame(
                    xmin=selIndex-0.5, xmax=selIndex+0.5, ymin=-Inf, ymax=Inf)
            profilePlot <- profilePlot + geom_rect(
                data = rect, color = "yellow", alpha = 0.3, inherit.aes = FALSE,
                aes(xmin = xmin, xmax = xmax, ymin = ymin, ymax = ymax))
        }
    }
    return(profilePlot)
}

#' Add taxonomy rank division lines to the heatmap plot
#' @export
#' @usage addRankDivisionPlot(profilePlot = NULL, plotDf = NULL,
#'     taxDB = NULL, workingRank = NULL, superRank = NULL, xAxis = "taxa",
#'     font = "Arial", groupLabelSize = 14, groupLabelDist = 2,
#'     groupLabelAngle = 90, refLine = TRUE)
#' @param profilePlot initial (highlighted) profile plot
#' @param plotDf dataframe for plotting the heatmap phylogentic profile
#' @param taxDB path to taxonomy database (taxonomyMatrix.txt file required!)
#' @param workingRank working taxonomy rank (e.g. species)
#' @param superRank taxonomy rank for division lines (e.g. superkingdom)
#' @param xAxis type of x-axis (either "genes" or "taxa")
#' @param font font of text. Default = Arial"
#' @param groupLabelSize size of rank labels
#' @param groupLabelDist size of the plot area for rank labels
#' @param groupLabelAngle angle of rank labels
#' @param refLine add vertical line to separate reference taxon
#' @return A profile heatmap plot with highlighted gene and/or taxon of interest
#' as ggplot object.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @seealso \code{\link{heatmapPlotting}}, \code{\link{highlightProfilePlot}},
#' \code{\link{getTaxonomyMatrix}}
#' @examples
#' data("finalProcessedProfile", package="PhyloProfile")
#' plotDf <- dataMainPlot(finalProcessedProfile)
#' plotParameter <- list(
#'     "xAxis" = "taxa",
#'     "geneIdType" = "geneID",
#'     "var1ID" = "FAS_FW",
#'     "var2ID"  = "FAS_BW",
#'     "midVar1" = 0.5,
#'     "midColorVar1" =  "#FFFFFF",
#'     "lowColorVar1" =  "#FF8C00",
#'     "highColorVar1" = "#4682B4",
#'     "midVar2" = 1,
#'     "midColorVar2" =  "#FFFFFF",
#'     "lowColorVar2" = "#CB4C4E",
#'     "highColorVar2" = "#3E436F",
#'     "paraColor" = "#07D000",
#'     "xSize" = 8,
#'     "ySize" = 8,
#'     "legendSize" = 8,
#'     "mainLegend" = "top",
#'     "dotZoom" = 0,
#'     "xAngle" = 60,
#'     "guideline" = 0,
#'     "colorByGroup" = FALSE,
#'     "colorByOrthoID" = FALSE
#' )
#' profilePlot <- heatmapPlotting(plotDf, plotParameter)
#' workingRank <- "class"
#' superRank <- "superkingdom"
#' addRankDivisionPlot(
#'     profilePlot, plotDf, NULL, workingRank, superRank, "taxa", font = "sans"
#' )

addRankDivisionPlot <- function(
        profilePlot = NULL, plotDf = NULL, taxDB = NULL,
        workingRank = NULL, superRank = NULL, xAxis = "taxa", font = "Arial",
        groupLabelSize = 14, groupLabelDist = 2, groupLabelAngle = 90,
        refLine = TRUE
) {
    if (is.null(plotDf)) stop("Input data cannot be NULL!")
    if (is.null(profilePlot)) stop("Profile plot cannot be NULL!")
    profilePlot <- profilePlot + theme(text = element_text(family = font))
    if (is.null(workingRank) || is.null(superRank)) {
        if (refLine == TRUE) {
            # guideline for separating ref species
            if (xAxis == "genes") {
                profilePlot <- profilePlot + labs(y = "Taxon") +
                    geom_hline(yintercept = 0.5, colour = "dodgerblue4") +
                    geom_hline(yintercept = 1.5, colour = "dodgerblue4")
            } else
                profilePlot <- profilePlot + labs(x = "Taxon") +
                    geom_vline(xintercept = 0.5, colour = "dodgerblue4") +
                    geom_vline(xintercept = 1.5, colour = "dodgerblue4")
        }
        return(profilePlot)
    } else {
        # sort supertaxonID based on the sorted supertaxon
        plotDf <- plotDf[with(plotDf,order(supertaxon)),]
        plotDf$supertaxonID <- factor(
            plotDf$supertaxonID, levels = unique(plotDf$supertaxonID)
        )
        # get input (super)taxa
        inputTax <- levels(as.factor(plotDf$supertaxonID))
        # subset taxonomy matrix to contain only superrank for division line
        taxMatrix <- getTaxonomyMatrix(taxDB)
        subTaxMatrix <- subset(
            taxMatrix[taxMatrix[[workingRank]] %in% inputTax,],
            select = c(as.character(workingRank), as.character(superRank))
        )
        subTaxMatrix <- subTaxMatrix[!duplicated(subTaxMatrix),]
        # remove IDs that do not have real NCBI superRank
        nameList <- getNameList(taxDB)
        subNameList <- nameList[
            nameList$ncbiID %in% subTaxMatrix[[superRank]],
            c("ncbiID", "rank", "fullName")
        ]
        colnames(subNameList) <- c(superRank, "rank", "name")
        mergedDf <- merge(subTaxMatrix, subNameList, by = superRank, all.x=TRUE)
        mergedDf <- mergedDf[mergedDf$rank == superRank,]
        subTaxMatrix <- mergedDf
        # group input taxa based on superrank and get their index
        groupedList <- lapply(
            levels(as.factor(subTaxMatrix[[superRank]])),
            function(x) {
                tmp <- subTaxMatrix[subTaxMatrix[[superRank]] == x,]
                tmp$index <- which(
                    levels(as.factor(plotDf$supertaxonID))
                    %in% tmp[[workingRank]]
                )
                return(tmp)
            }
        )
        # add vertical line to divide taxon groups
        max_taxa <- length(unique(as.character(plotDf$geneID)))
        for(i in groupedList) {
            min <- min(i$index)
            max <- max(i$index)
            if (xAxis == "taxa") {
                profilePlot <- profilePlot +
                    geom_vline(xintercept = min - 0.5, colour = "dodgerblue4") +
                    geom_vline(xintercept = max + 0.5, colour = "dodgerblue4") +
                    annotate(
                        geom = "text", angle = groupLabelAngle, hjust = 0,
                        size = groupLabelSize,
                        x = min,
                        y = max_taxa + 0.5,
                        label = unique(as.character(i$name))
                    )

            } else {
                profilePlot <- profilePlot +
                    geom_hline(yintercept = min - 0.5, colour = "dodgerblue4") +
                    geom_hline(yintercept = max + 0.5, colour = "dodgerblue4") +
                    annotate(
                        geom = "text", angle = groupLabelAngle - 90, hjust = 0,
                        size = groupLabelSize,
                        y = min,
                        x = max_taxa + 1,
                        label = unique(as.character(i$name))
                    )
            }
        }
        longestLabelLen <- max(nchar(unique(as.character(subTaxMatrix$name))))
        marginSize <- longestLabelLen * groupLabelSize * groupLabelDist
        if (xAxis == "taxa") {
            return(
                profilePlot +
                    coord_cartesian(clip = "off") +
                    theme(
                        plot.margin = margin(
                            t = marginSize, r = 10, b = 10, l = 10
                        )
                    )
            )
        } else
            return(
                profilePlot +
                    coord_cartesian(clip = "off") +
                    theme(
                        plot.margin = margin(
                            t = 10, r = marginSize, b = 10, l = 10
                        )
                    )
            )
    }
}

context("test creating data for plotting profiles")

test_that("test profile plot data generation", {
    data("finalProcessedProfile", package="PhyloProfile")

    plotDf <- dataMainPlot(finalProcessedProfile)
    expect_true(nrow(plotDf) == 88)

    selectedTaxa <- c("Mammalia", "Saccharomycetes", "Insecta")
    selectedSeq <- "all"
    customizedPlotDf <- dataCustomizedPlot(
        finalProcessedProfile, selectedTaxa, selectedSeq
    )
    expect_true(nrow(customizedPlotDf) == 11)
})

#' Group Comparison
#' @param filteredDf phylogenetic profile data after filtering
#' @param inGroup list of taxon IDs for the in-group taxa
#' @param variable selected variable for calculating the differences betwee
#' in- and out-group ("var1" or "var2")
#' @param varName names of the additional variables
#' @param compareType type of comparison (statistical tests or mean comparison)
#' @param significanceLevel confidence level for stat tests or delta mean cutoff
#' @param plotParameters plot parameters including xSize, ySize, angle (x-axis
#' angle), legendPosition and legendSize, titleSize, flipPlot, mValue (mean or
#' median point), widthVar and heightVar (width and height for variable
#' distribution plot), widthFeature and heightFeature (width and height for
#' feature distribution plot), inGroupName and outGroupName.
#' @param domainDf dataframe of domain architectures
#' @param doCompare check if Compare button clicked (input$doCompare)
#' @param doUpdate check if Update button clicked (input$updateGC)
#' @param taxDB Path to the taxonomy DB files
#' @return list of candidate genes and their p-values (or delta means)
#' @authors Carla Mölbert {carla.moelbert@gmx.de}, Vinh Tran
#' {tran@bio.uni-frankfurt.de}

groupComparisonUI <- function(id){
    ns <- NS(id)
    sidebarLayout(
        sidebarPanel(
            width = 3,
            uiOutput(ns("candidateGenes.ui")) %>%
                bsplus::bs_embed_popover(
                  title = "",
                  content = "Select gene to show the plots",
                  placement = "right",
                  trigger = "hover"
                ),

            downloadButton(ns("downloadPlot"),"Download plot",
                           class = "butDL"),
            tags$head(
                tags$style(HTML(
                    ".butDL{background-color:#476ba3;} .butDL{color: white;}"))
            ),
            hr(),
            uiOutput(ns("featureTypeSelect.ui")),
            selectInput(
                ns("filterGainLoss"),
                "Filter gain/loss features",
                choices = c("gain", "loss", "none"),
                selected = "none"
            ),
            numericInput(
                ns("featureThreshold"),
                "Number of intances per protein (IPP) cutoff",
                min = 0, max = 9999, step = 0.1, value = 0.25
            ),
            sliderInput(
                ns("dIPPThreshold"),
                "Delta IPP / Sum IPP cutoff",
                min = 0 , max = 1, value = 0,
                step = 0.01, round = FALSE
            )
        ),
        mainPanel(
            width = 9,
            tags$style(
                HTML("#plotsUI { height:650px; overflow-y:scroll}")
            ),
            uiOutput(ns("downloadALl.ui")),
            uiOutput(ns("varPlots.ui")),
            uiOutput(ns("featurePlots.ui"))
        )
    )
}

groupComparison <- function (
    input, output, session,
    filteredDf,
    inGroup,
    variable,
    varName,
    compareType,
    significanceLevel,
    plotParameters,
    domainDf,
    doCompare,
    doUpdate,
    taxDB
) {
    ### get candidate genes and their p-values
    candidateGenes <- reactive({
        if (is.null(inGroup())) return()
        if (is.null(variable()) | variable()[1] == "none")
            stop("No variable available or selected!")

        if (compareType() == "Statistical tests") {
            pvalues <- compareTaxonGroups(
                filteredDf(),
                inGroup(),
                FALSE,
                variable(),
                significanceLevel(),
                taxDB()
            )
            return(pvalues[pvalues <= significanceLevel()])
        } else {
            deltaMean <- compareMedianTaxonGroups(
                filteredDf(),
                inGroup(),
                FALSE,
                variable(),
                taxDB()
            )
            return(deltaMean[deltaMean <= significanceLevel()])
        }
    })

    ### render list of candidate genes
    output$candidateGenes.ui <- renderUI({
        ns <- session$ns
        doCompare()
        isolate({
            candidateGenes <- names(candidateGenes())

            if (length(candidateGenes()) > 0) {
                candidateGenesList <- c("all", candidateGenes)
                selectInput(
                    ns("candidateGenes"), "Candidate gene(s):",
                    choices = candidateGenesList,
                    selected = candidateGenesList[2],
                    multiple = FALSE
                )
            } else {
                selectInput(ns("candidateGenes"), "Candidate gene(s):", "none")
            }
        })
    })

    ### generate data for variable distribution plotting
    plotDf <- reactive({
        if (is.null(candidateGenes())) return()
        req(input$candidateGenes)
        if (length(input$candidateGenes) == 0 | input$candidateGenes == "none")
            return()

        if (input$candidateGenes[1] == "all") return()
        return(
            getVarDistributionData(
                filteredDf(), inGroup(), input$candidateGenes, varName()
            )
        )
    })

    ### render variable distribution plot(s)
    output$distributionPlot <- renderPlot({
        if (is.null(plotDf())) return()
        g <- plotVarDistribution(
            input$candidateGenes, candidateGenes(), compareType(), plotDf(),
            plotParameters()
        )
        grid::grid.newpage()
        grid::grid.draw(g)
    })

    output$varPlots.ui <- renderUI({
        if (is.null(plotDf())) return()
        ns <- session$ns
        plotOutput(
            ns("distributionPlot"),
            width = plotParameters()$widthVar,
            height = plotParameters()$heightVar
        )
    })

    ### get data for protein features plotting
    featureDf <- reactive({
        doUpdate()
        if (is.null(domainDf()) | is.null(plotDf())) return()
        if (length(input$candidateGenes) == 0 | input$candidateGenes == "none")
            return()

        featureDf <- dataFeatureTaxGroup(
            filteredDf(), domainDf(), inGroup(), input$candidateGenes
        )
        ippCutoff <- isolate(input$featureThreshold)
        dIPPCutoff <- isolate(input$dIPPThreshold)

        if (input$filterGainLoss == "loss") {
            loss <- featureDf$Feature[featureDf$Taxon_group == "Out-group"
                                      & featureDf$IPP >= ippCutoff]
            return(
                featureDf[featureDf$Feature %in% loss
                          & featureDf$dIPP < 0
                          & abs(featureDf$dIPP) >= dIPPCutoff,]
            )
        } else if (input$filterGainLoss == "gain") {
            gain <- featureDf$Feature[featureDf$Taxon_group == "In-group"
                                      & featureDf$IPP >= ippCutoff]
            return(
                featureDf[featureDf$Feature %in% gain
                          & featureDf$dIPP >= dIPPCutoff,]
            )
        } else {
            dfIn <- featureDf[featureDf$Taxon_group == "In-group",]
            dfOut <- featureDf[featureDf$Taxon_group == "Out-group",]
            mergedDf <- merge(dfIn, dfOut, by = "Feature", all = TRUE)
            mergedDf[is.na(mergedDf)] <- 0
            allFeature <- as.character(
                mergedDf$Feature[
                    mergedDf$IPP.x >= ippCutoff | mergedDf$IPP.y >= ippCutoff
                ]
            )
            return(featureDf[featureDf$Feature %in% allFeature
                             & abs(featureDf$dIPP) >= dIPPCutoff,])
        }
    })

    ### List with possible features for the selected gene
    output$featureTypeSelect.ui <- renderUI({
        ns <- session$ns
        if (is.null(featureDf())) {
            selectInput(ns("featureTypeSelect"),
                        "Feature type(s) of interest",
                        choices = "ALL",
                        selected = "ALL",
                        multiple = TRUE,
                        selectize = FALSE)
        } else {
            featureDf <- data.frame(
                do.call(
                    rbind, strsplit(as.character(featureDf()$Feature), "_")
                ), stringsAsFactors = FALSE
            )
            featureList <- unique(featureDf[,1])
            selectInput(ns("featureTypeSelect"),
                        "Feature type(s) of interest:",
                        choices = c("ALL", featureList),
                        selected = "ALL",
                        multiple = TRUE,
                        selectize = FALSE)
        }
    })

    ### filter feature data based on selected type of features
    featureDfSelected <- reactive({
        if (is.null(featureDf())) return()
        if ("ALL" %in% input$featureTypeSelect) return(featureDf())
        else {
            featureDf <- featureDf()
            selectedDf <- lapply(
                input$featureTypeSelect,
                function (x) featureDf[grep(x, featureDf$Feature),]
            )
            return(do.call(rbind, selectedDf))
        }
    })

    ### render feature distribution plot(s)
    output$featurePlot <- renderPlot({
        if (is.null(featureDfSelected())) return()
        featureDistTaxPlot(featureDfSelected(), plotParameters())
    })

    output$featurePlots.ui <- renderUI({
        if (is.null(featureDf())) return()
        ns <- session$ns
        plotOutput(
            ns("featurePlot"),
            width = plotParameters()$widthFeature,
            height = plotParameters()$heightFeature
        )
    })

    ### download plots for single candidate
    output$downloadPlot <- downloadHandler(
        filename = function() {
            paste0("GC_", input$candidateGenes, ".pdf")
        },
        content = function(file) {
            ggsave(
                file,
                plot = arrangeGrob(
                    plotVarDistribution(
                        input$candidateGenes,
                        candidateGenes(),
                        compareType(),
                        plotDf(),
                        plotParameters()
                    ),
                    featureDistTaxPlot(featureDfSelected(), plotParameters()),
                    ncol = 1
                ),
                dpi = 300, device = "pdf"
            )
        }
    )

    ### download plots for all candidates ###########
    output$downloadALl.ui <- renderUI({
        ns <- session$ns
        if (is.null(input$candidateGenes)) return()
        if (input$candidateGenes[1] == "all") {
            tagList(
                h3(em("For analyzing all candidate genes, please download them
                      to your computer! Current plot settings and all thresholds
                      will be applied!")),
                downloadButton(
                    ns("downloadAll"),"Download all plots", class = "butDL"
                )
            )
        }
    })

    output$downloadAll <- downloadHandler(
        filename = function() {
            paste0("GC_allGenes.zip")
        },
        content = function(file) {
            fs <- NULL
            for(gene in names(candidateGenes())) {
                fs <- c(fs, paste0("GC_", gene, ".pdf"))
                pdf(paste0("GC_", gene, ".pdf"))
                plot <- arrangeGrob(
                    plotVarDistribution(
                        gene,
                        candidateGenes(),
                        compareType(),
                        getVarDistributionData(
                            filteredDf(), inGroup(),
                            gene, varName()
                        ),
                        plotParameters()
                    ),
                    featureDistTaxPlot(
                        getSelectedFeatureData(
                            filteredDf(), domainDf(), inGroup(), gene,
                            input$featureThreshold, input$dIPPThreshold,
                            input$filterGainLoss, input$featureTypeSelect
                        ),
                        plotParameters()
                    ),
                    ncol = 1
                )
                grid.draw(plot)
                dev.off()
            }
            zip(zipfile = file, files = fs)
        },
        contentType = "application/zip"
    )

    ### disable/enable download functions
    observeEvent(input$candidateGenes, {
        if (is.null(filteredDf())) shinyjs::disable("downloadPlot")
        else if (doCompare() == FALSE) shinyjs::disable("downloadPlot")
        else if (input$candidateGenes[1] == "all")
            shinyjs::disable("downloadPlot")
        else shinyjs::enable("downloadPlot")
    })

    ### return list of candidate genes
    outGenes <- reactive({
        return(names(candidateGenes()))
    })
    if (!is.null(outGenes)) return(outGenes)
}

################################# FUNCTIONS ####################################

#' Get data for variable distribution plot
#' @param data main (filtered) phylogenetic profile dataframe
#' @param inGroup list of in-group taxa (ncbi ID)
#' @param candidateGene selected candidate gene
#' @param varName names of up to 2 variables
#' @return dataframe contains values of the variables that are used for plotting
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}
getVarDistributionData <- function(
    data, inGroup, candidateGene, varName
) {
    names(data)[names(data)=="var1"] <- varName[1]
    if (length(varName) == 2)
        names(data)[names(data)=="var2"] <- varName[2]
    return(
        dataVarDistTaxGroup(data, inGroup, candidateGene, varName)
    )
}

#' Plot variable distribution
#' @param candidateGene name of candidate gene
#' @param candidateGenesList named candidate gene list (used to get p-value or
#' delta median for selected candidate gene)
#' @param compareType "Statistical tests" or "Median values"
#' @param plotDf Data for plotting
#' @param plotParameters plot parameters
#' @return grid object. Use grid.draw() to plot.
#' @authors Vinh Tran {tran@bio.uni-frankfurt.de}
plotVarDistribution <- function(
    candidateGene, candidateGenesList, compareType, plotDf, plotParameters
) {
    pValue <- candidateGenesList[
        match(candidateGene, names(candidateGenesList))]
    if (compareType == "Statistical tests") {
        title = paste0(candidateGene, "   p-value = ", round(pValue, 3))
    } else {
        title <- paste0(candidateGene, "   delta-mean = ", round(pValue, 3))
    }

    g <- varDistTaxPlot(plotDf, c(plotParameters, title = title))
    return(g)
}

#' Get data for feature comparison plot
#' @param mainData main (filtered) phylogenetic profile dataframe
#' @param domainDf dataframe of domain architectures
#' @param inGroup list of in-group taxa (ncbi ID)
#' @param candidateGene name of candidate gene
#' @param ippCutoff cutoff for number of intances per protein
#' @param dIPPCutoff cutoff for the differences in the IPP of the same feature
#' between in- and out-group
#' @param filterGainLoss get only possible gain/loss features
#' @param featureTypeSelect type of features (pfam/smart/cast/etc.)
#' @return dataframe for feature comparison plot
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}
getSelectedFeatureData <- function(
    mainData, domainDf, inGroup, candidateGene, ippCutoff, dIPPCutoff,
    filterGainLoss, featureTypeSelect
) {
    featureDf <- dataFeatureTaxGroup(
        mainData, domainDf, inGroup, candidateGene
    )

    if (filterGainLoss == "loss") {
        loss <- featureDf$Feature[featureDf$Taxon_group == "Out-group"
                                  & featureDf$IPP >= ippCutoff]
        featureDfOut <- featureDf[
            featureDf$Feature %in% loss & featureDf$dIPP < 0
            & abs(featureDf$dIPP) >= dIPPCutoff,
        ]
    } else if (filterGainLoss == "gain") {
        gain <- featureDf$Feature[featureDf$Taxon_group == "In-group"
                                  & featureDf$IPP >= ippCutoff]
        featureDfOut <- featureDf[
            featureDf$Feature %in% gain & featureDf$dIPP >= dIPPCutoff,
        ]
    } else {
        dfIn <- featureDf[featureDf$Taxon_group == "In-group",]
        dfOut <- featureDf[featureDf$Taxon_group == "Out-group",]
        mergedDf <- merge(dfIn, dfOut, by = "Feature", all = TRUE)
        mergedDf[is.na(mergedDf)] <- 0
        allFeature <- as.character(
            mergedDf$Feature[
                mergedDf$IPP.x >= ippCutoff | mergedDf$IPP.y >= ippCutoff
                ]
        )
        featureDfOut <- featureDf[
            featureDf$Feature %in% allFeature
            & abs(featureDf$dIPP) >= dIPPCutoff,
        ]
    }

    ### filter data based on selected feature types
    if ("ALL" %in% featureTypeSelect) return(featureDfOut)
    selectedDf <- lapply(
        featureTypeSelect,
        function (x) featureDfOut[grep(x, featureDfOut$Feature),]
    )
    return(do.call(rbind, selectedDf))
}

#' Profile plot
#' @param data data for heatmap plot (from reactive fn "dataHeat")
#' @param clusteredDataHeat clustered data (from reactive fn "clusteredDataHeat"
#' @param applyCluster choose clustered data or not (from input$applyCluster)
#' @param parameters plot parameters (colors, size, variable names, ...)
#' @param inSeq subset sequences for customized profile (input$inSeq)
#' @param inTaxa subset taxa for customized profile (input$inTaxa)
#' @param rankSelect selected taxonomy rank (input$rankSelect)
#' @param inSelect selected taxon name (input$inSelect)
#' @param taxonHighlight highlighted taxon (input$taxonHighlight)
#' @param geneHighlight highlighted gene (input$geneHighlight)
#' @param typeProfile either "mainProfile" or "customizedProfile"
#' @param taxDB path to taxonomy database (taxonomyMatrix.txt file required!)
#' @param superRank taxonomy rank for division lines (e.g. superkingdom)
#' @param allTaxa list of all input (super)taxa and their ncbi IDs
#' @param mode plot mode, either "fast" or "normal"
#' @taxDB Path to the taxonomy DB files
#' @return info for selected point on the profile
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}

source("R/functions.R")
rtCheck <- FALSE

createProfilePlotUI <- function(id) {
    ns <- NS(id)
    tagList(
        uiOutput(ns("plot.ui")),
        br(),
        downloadButton(ns("profileDownload"),"Download profile",
                       class = "butDL"),
        br(),br(),hr(),
        uiOutput(ns("brushedInfo.ui")),
        radioButtons(
            ns("showSettings"), "Show subplot settings",
            choices = c("show", "hide"), selected = "hide", inline = TRUE
        ),
        conditionalPanel(
            condition = "input.showSettings == 'show'",
            column(
                2,
                radioButtons(
                    inputId = ns("xAxisSub"),
                    label = "X-axis",
                    choices = list("taxa", "genes"),
                    selected = "taxa",
                    inline = TRUE
                )
            ),
            column(
                2,
                createPlotSize(ns("widthSub"), "Width (px)", 1200)
            ),
            column(
                2,
                createPlotSize(ns("heightSub"), "Height (px)", 1200)
            ),
            column(
                2,
                createTextSize(ns("fontSizeSub"), "Axis size", 14,100)
            ),
            column(
                2,
                createTextSize(ns("legendSizeSub"), "Legend size", 8,100)
            ),
            column(
                2,
                selectInput(
                    ns("legendPosSub"), label = "Legend position:",
                    choices = list(
                        "Right" = "right", "Left" = "left", "Top" = "top",
                        "Bottom" = "bottom", "Hide" = "none"
                    ),
                    selected = "right"
                )
            ),
            br(),
            downloadButton(
                ns("profileSubDownload"),"Download subplot", class = "butDL"
            ),
            strong(""),
            downloadButton(
                ns("dataSubDownload"),"Download selected data", class = "butDL"
            ),
            ns = NS(id)
        ),
        tags$head(
            tags$style(HTML(
                ".butDL{background-color:#476ba3;} .butDL{color: white;}"))
        )
    )
}

createProfilePlot <- function(
    input, output, session, data, clusteredDataHeat, applyCluster, parameters,
    inSeq, inTaxa, rankSelect, inSelect, taxonHighlight, geneHighlight,
    typeProfile, taxDB, superRank, allTaxa, mode = "normal"
) {
    # data for heatmap ---------------------------------------------------------
    dataHeat <- reactive({
        if (is.null(data())) stop("Profile data is NULL!")
        if (rtCheck) {
            checkpointp101 <- Sys.time()
            print(paste("checkpoint-P101 - before data heatmap",checkpointp101))
        }
        if (typeProfile() == "customizedProfile") {
            if (is.null(inTaxa()) | is.null(inSeq())) return()
            if (applyCluster() == TRUE) {
                dataHeat <- dataCustomizedPlot(
                    clusteredDataHeat(), inTaxa(), inSeq()
                )
            } else { dataHeat <- dataCustomizedPlot(data(), inTaxa(), inSeq()) }
        } else {
            if (applyCluster() == TRUE) {
                dataHeat <- dataMainPlot(clusteredDataHeat())
            } else { dataHeat <- dataMainPlot(data()) }
        }
        # add all input taxa (if available)
        if (!(is.null(allTaxa()))) {
            mergedDf <- dataHeat %>% full_join(
                allTaxa(), by = c("supertaxonID", "supertaxon")
            ) %>% mutate(
                geneID = ifelse(is.na(geneID), dataHeat$geneID[1], geneID),
                supertaxon = factor(
                    supertaxon, levels = levels(allTaxa()$supertaxon)
                )
            )
            return(mergedDf)
        }
        if (rtCheck) {
            checkpointp102 <- Sys.time()
            print(paste(
                "checkpoint-P102 - data heatmap done", checkpointp102,
                " --- ",  checkpointp102 - checkpointp101
            ))
        }
        return(dataHeat)
    })

    # basic profile plot -------------------------------------------------------
    basicProfile <- reactive({
        if (is.null(dataHeat())) stop("Profile data is NULL!")
        if (typeProfile() == "customizedProfile") {
            if (length(inSeq()) == 0 || length(inTaxa()) == 0) return()
            if ("all" %in% inSeq() & "all" %in% inTaxa()) return()
        }
        if (rtCheck) {
            checkpointp201 <- Sys.time()
            print(paste("checkpoint-P201 - before basic plot", checkpointp201))
        }
        if (mode() == "fast" & typeProfile() != "customizedProfile") {
            hpf <- heatmapPlottingFast(dataHeat(), parameters())
            if (rtCheck) {
                checkpointp202 <- Sys.time()
                print(paste(
                    "checkpoint-P202 - basic plot done", checkpointp202,
                    " --- ",  checkpointp202 - checkpointp201
                ))
            }
            return(hpf)
        } else {
            hp <- heatmapPlotting(dataHeat(), parameters())
            if (rtCheck) {
                checkpointp202 <- Sys.time()
                print(paste(
                    "checkpoint-P202 - basic plot done", checkpointp202,
                    " --- ",  checkpointp202 - checkpointp201
                ))
            }
            return(hp)
        }
    })

    # get superRank ------------------------------------------------------------
    getSuperRank <- reactive({
        if (!is.null(superRank()) && superRank() == "") return(NULL)
        else return(superRank())
    })

    # render heatmap profile ---------------------------------------------------
    output$plot <- renderPlot({
        if (is.null(basicProfile())) return()
        if (rtCheck) {
            checkpointp301 <- Sys.time()
            print(paste("checkpoint-P301 - before final plot", checkpointp301))
        }
        withProgress(message = 'PLOTTING...', value = 0.5, {
            p <- highlightProfilePlot(
                basicProfile(), dataHeat(), taxonHighlight(), rankSelect(),
                geneHighlight(), taxDB(), parameters()$xAxis
            )
            refLine <- TRUE
            if (mode() == "fast") refLine <- FALSE
            p <- addRankDivisionPlot(
                p, dataHeat(), taxDB(), rankSelect(), getSuperRank(),
                parameters()$xAxis, parameters()$font,
                parameters()$groupLabelSize, parameters()$groupLabelDist,
                parameters()$groupLabelAngle, refLine
            )
            if (rtCheck) {
                checkpointp302 <- Sys.time()
                print(paste(
                    "checkpoint-P302 - final plot done", checkpointp302,
                    " --- ",  checkpointp302 - checkpointp301
                ))
            }
            return(p)
        })
    })

    output$plot.ui <- renderUI({
        ns <- session$ns
        if (typeProfile() == "customizedProfile") {
            if (is.null(inSeq()[1]) | is.null(inTaxa()[1])) return()
            else if (inSeq()[1] == "all" & inTaxa()[1] == "all")  return()
        }

        shinycssloaders::withSpinner(
            if (mode() == "fast" & typeProfile() == "mainProfile") {
                plotOutput(
                    ns("plot"),
                    width = parameters()$width,
                    height = parameters()$height,
                    brush = ns("plotBrush")
                )
            } else
                plotOutput(
                    ns("plot"),
                    width = parameters()$width,
                    height = parameters()$height,
                    click = ns("plotClick")
                )
        )
    })

    output$profileDownload <- downloadHandler(
        filename = function() {
            c("profile.svg")
        },
        content = function(file) {
            ggsave(
                file,
                plot = addRankDivisionPlot(
                    basicProfile(), dataHeat(), taxDB(), rankSelect(),
                    getSuperRank(), parameters()$xAxis, parameters()$font,
                    parameters()$groupLabelSize, parameters()$groupLabelDist,
                    parameters()$groupLabelAngle, FALSE
                ),
                width = parameters()$width * 0.035,
                height = parameters()$height * 0.035,
                units = "cm", dpi = 300, device = "svg", limitsize = FALSE
            )
        }
    )

    # get info of brushed area -------------------------------------------------
    brushedData <- reactive({
        if (is.null(input$plotBrush)) return(data.frame())
        dt <- brushedPoints(
            dataHeat(), input$plotBrush, xvar = "supertaxon", yvar = "geneID"
        )
        dt$geneID <- droplevels(dt$geneID)
        dt$supertaxon <- droplevels(dt$supertaxon)
        return(dt)
    })

    # create subplot -----------------------------------------------------------
    # ** show/hide subplot settings --------------------------------------------
    observe({
        if (mode() == "fast" & typeProfile() == "mainProfile") {
            updateRadioButtons(
                session, "showSettings", "Show subplot settings",
                choices = c("show", "hide"), selected = "show", inline = TRUE
            )
        } else {
            updateRadioButtons(
                session, "showSettings", "Show subplot settings",
                choices = c("show", "hide"), selected = "hide", inline = TRUE
            )
        }
    })

    # ** auto update subplot size ----------------------------------------------
    observe({
        req(nrow(brushedData()) > 0)
        df <- brushedData()
        nrTaxa <- length(unique(as.character(df$supertaxon)))
        nrGene <- length(unique(as.character(df$geneID)))
        adaptedSize <- adaptPlotSize(
            nrTaxa, nrGene, parameters()$xAxis, parameters()$dotZoom
        )
        if (length(adaptedSize) > 0) {
            h <- adaptedSize[1]
            hv <- adaptedSize[2]
            wv <- adaptedSize[3]
            if (hv > 25000) hv <- 25000
            if (wv > 25000) wv <- 25000
            if (h <= 20) {
                if (superRank() == "") {
                    updateSelectInput(
                        session, "legendPosSub", label = "Legend position:",
                        choices = list("Right" = "right",
                                       "Left" = "left",
                                       "Top" = "top",
                                       "Bottom" = "bottom",
                                       "Hide" = "none"),
                        selected = "top"
                    )
                } else {
                    updateSelectInput(
                        session, "legendPosSub", label = "Legend position:",
                        choices = list("Right" = "right",
                                       "Left" = "left",
                                       "Top" = "top",
                                       "Bottom" = "bottom",
                                       "Hide" = "none"),
                        selected = "right"
                    )
                }
                updateNumericInput(session, "widthSub", value = wv  + 50)
            } else if (h <= 30) {
                updateNumericInput(session, "widthSub", value = wv + 50)
            } else {
                updateNumericInput(session, "widthSub", value = wv)
            }
            updateNumericInput(session, "heightSub", value = hv)
        }
    })

    # ** get settings for subplot; replace xAxis, font size and legend pos -----
    subPlotParameters <- reactive({
        plotParameters <- parameters()
        plotParameters$xAxis <- input$xAxisSub
        plotParameters$mainLegend <- input$legendPosSub
        plotParameters$xSize <- input$fontSizeSub
        plotParameters$ySize <- input$fontSizeSub
        plotParameters$legendSize <- input$legendSizeSub
        return(plotParameters)
    })

    # ** render subplot --------------------------------------------------------
    output$brushedInfo.ui <- renderUI({
        ns <- session$ns
        req(nrow(brushedData()) > 0)
        tagList(
            h4("SELECTED DATA"),
            br(),
            plotOutput(
                ns("subPlot"),
                width = input$widthSub,
                height = input$heightSub,
                click = ns("plotClick")
            )
        )
    })

    output$subPlot <- renderPlot({
        req(nrow(brushedData()) > 0)
        p <- heatmapPlotting(brushedData(), subPlotParameters())
        p <- highlightProfilePlot(
            p, brushedData(), taxonHighlight(), rankSelect(),
            geneHighlight(), taxDB(), subPlotParameters()$xAxis
        )
        p <- addRankDivisionPlot(
            p, brushedData(), taxDB(), rankSelect(),
            getSuperRank(), parameters()$xAxis, parameters()$font,
            parameters()$groupLabelSize, parameters()$groupLabelDist,
            parameters()$groupLabelAngle, FALSE
        )
        p
    })

    # ** download subplot ------------------------------------------------------
    output$profileSubDownload <- downloadHandler(
        filename = function() {
            c("subProfile.svg")
        },
        content = function(file) {
            p <- heatmapPlotting(brushedData(), subPlotParameters())
            p <- addRankDivisionPlot(
                p, brushedData(), taxDB(), rankSelect(),
                getSuperRank(), parameters()$xAxis, parameters()$font,
                parameters()$groupLabelSize, parameters()$groupLabelDist,
                parameters()$groupLabelAngle, FALSE
            )
            ggsave(
                file, plot = p,
                width = input$widthSub * 0.035, height = input$heightSub *0.035,
                units = "cm", dpi = 300, device = "svg", limitsize = FALSE
            )
        }
    )

    # ** download data of subplot ----------------------------------------------
    output$dataSubDownload <- downloadHandler(
        filename = function() {
            c("subData.txt")
        },
        content = function(file){
            dataOut <- brushedData() %>% select(
                geneID, supertaxonID, supertaxon, orthoID, var1, var2, geneName
            )
            if (!(identical(dataOut$geneID, dataOut$geneName)))
                dataOut <- dataOut %>% select(-c(geneName))
            contains_id <- mapply(function(id, name) {
                grepl(paste0("@", id, "@"), name)
            }, dataOut$supertaxonID, dataOut$orthoID)
            if (all(contains_id == TRUE))
                dataOut <- dataOut %>% mutate(
                    supertaxonID = paste0("ncbi", supertaxonID)
                ) %>% select(
                    geneID, ncbiID = supertaxonID, orthoID, var1, var2
                )
            write.table(
                dataOut, file, sep = "\t", row.names = FALSE, quote = FALSE
            )
        }
    )

    # get info of clicked point on heatmap plot --------------------------------
    selectedpointInfo <- reactive({
        req(dataHeat())
        if (typeProfile() == "customizedProfile") {
            req(inSeq())
            req(inTaxa())
        }
        # get selected supertaxon name
        taxaList <- getNameList(taxDB())
        rankName <- rankSelect()
        inSelect <- taxaList$ncbiID[taxaList$fullName == inSelect()]

        dataHeat <- dataHeat()
        if (mode() == "fast" & typeProfile() == "mainProfile")
            dataHeat <- brushedData()

        if (is.null(dataHeat) | nrow(dataHeat) == 0) {
            message("WARNING: Data for heatmap is NULL!")
            return()
        }

        if (typeProfile() == "customizedProfile") {
            # get sub-dataframe of selected taxa and sequences
            if (is.null(inSeq()[1]) | is.null(inTaxa()[1])) {
                message("WARNING: Subset taxa or genes is NULL!")
                return()
            }
            if (inTaxa()[1] == "all" & inSeq()[1] != "all") {
                # select data from dataHeat for selected sequences only
                dataHeat <- subset(dataHeat, geneID %in% inSeq())
            } else if (inSeq()[1] == "all" & inTaxa()[1] != "all") {
                # select data from dataHeat for selected taxa only
                dataHeat <- subset(dataHeat, supertaxon %in% inTaxa())
            } else {
                # select data from dataHeat for selected sequences and taxa
                dataHeat <- subset(dataHeat, geneID %in% inSeq()
                                   & supertaxon %in% inTaxa())
            }

            # drop all other supertaxon that are not in sub-dataframe
            dataHeat$supertaxon <- factor(dataHeat$supertaxon)
            dataHeat$geneID <- factor(dataHeat$geneID)
        }

        # get values
        if (is.null(input$plotClick$x)) return()
        else {
            # get cooridiate point
            if (parameters()$xAxis == "genes") {
                corX <- round(input$plotClick$y);
                corY <- round(input$plotClick$x)
            } else {
                corX <- round(input$plotClick$x);
                corY <- round(input$plotClick$y)
            }
            # get geneID
            genes <- levels(dataHeat$geneID)
            geneID <- toString(genes[corY])
            # get supertaxon (spec)
            supertaxa <- levels(dataHeat$supertaxon)
            spec <- toString(supertaxa[corX])
            # get var1 and var2 score
            var1 <- NA
            if (!is.na(dataHeat$var1[dataHeat$geneID == geneID
                                     & dataHeat$supertaxon == spec][1])) {
                var1 <- max(
                    na.omit(dataHeat$var1[dataHeat$geneID == geneID
                                          & dataHeat$supertaxon == spec])
                )
            }
            var2 <- NA
            if (!is.na(dataHeat$var2[dataHeat$geneID == geneID
                                     & dataHeat$supertaxon == spec][1])) {
                var2 <- {
                    max(na.omit(dataHeat$var2[dataHeat$geneID == geneID
                                              & dataHeat$supertaxon == spec]))
                }
            }
            # get percentage of present species and total of taxa
            Percent <- NA
            if (!is.na(dataHeat$presSpec[dataHeat$geneID == geneID
                                         & dataHeat$supertaxon == spec][1])) {
                Percent <- {
                    max(
                        na.omit(
                            dataHeat$presSpec[dataHeat$geneID == geneID
                                              & dataHeat$supertaxon == spec]
                        )
                    )
                }
            }
            presentTaxa <- NA
            if (!is.na(dataHeat$presentTaxa[dataHeat$geneID == geneID
                                            & dataHeat$supertaxon == spec][1])){
                presentTaxa <- max(
                    na.omit(dataHeat$presentTaxa[dataHeat$geneID == geneID
                                                 & dataHeat$supertaxon == spec])
                )
            }
            totalTaxa <- NA
            if (
                !is.na(dataHeat$totalTaxa[dataHeat$geneID == geneID
                                          & dataHeat$supertaxon == spec][1])
            ) {
                totalTaxa <- max(
                    na.omit(
                        dataHeat$totalTaxa[dataHeat$geneID == geneID
                                           & dataHeat$supertaxon == spec]
                    )
                )
            }


            # get ortholog ID
            orthoID <- dataHeat$orthoID[dataHeat$geneID == geneID
                                        & dataHeat$supertaxon == spec]
            totalOrtho <- length(orthoID)

            # get working taxonomy level
            strain <- "Y"
            if (nlevels(as.factor(dataHeat$totalTaxa)) > 1) strain <- "N"

            if (is.na(Percent)) return()
            else {
                info <- c(
                    geneID,
                    list(orthoID),
                    totalOrtho,
                    spec,
                    round(var1, 2),
                    round(Percent, 2),
                    round(var2, 2),
                    presentTaxa,
                    totalTaxa,
                    strain
                )
                return(info)
            }
        }
    })


    return(selectedpointInfo)
}

# * popup for plotting detailed plot -------------------------------------------
detailedPlotModal <- function() {
    modalDialog(
        title = "Detailed plot",
        size = "l",
        easyClose = TRUE,
        footer = modalButton("Close"),
        
        fluidRow(
            column(
                2, createPlotSize("detailedHeight", "Height (px)", 100)
            ),
            column(
                3, createTextSize("detailedText", "Text size (px)", 12, 150)
            ),
            column(
                7,
                checkboxInput(
                    "detailedRemoveNA",
                    strong("Hide taxa that have no ortholog (NAs)",
                           style = "color:red"),
                    value = FALSE
                ),
                checkboxInput(
                    "detailedFilter",
                    strong("Apply filters",
                           style = "color:red"),
                    value = FALSE
                )
            )
        ),
        hr(),
        createDetailedPlotUI("detailedPlot"),
        actionButton("doDomainPlot", "Show domain architecture"),
        uiOutput("checkDomainFiles"),
        br(),
        h4("Sequence:"),
        verbatimTextOutput("fasta"),
        br(),
        h4("Links:"),
        uiOutput("dbLink.ui")
    )
}

# * popup for plotting domain architecture plot --------------------------------
archiPlotModal <- function(id, title = "Domain architecture") {
    modalDialog(
        title = title,
        size = "l",
        easyClose = TRUE,
        footer = modalButton("Close"),
        createArchitecturePlotUI("archiPlot")
    )
}

# * popup for confirming parsing taxa from input file ------------------
parseConfirmModal <- function() {
    modalDialog(
        title = "Get taxonomy info",
        size = "s",
        easyClose = FALSE,
        footer = NULL,
        HTML(
            '<p>Fetching Missing Taxonomy Information and
      Post-processing.</p><p><em>This window will close
      automatically when everything is done. Please wait...</em></p>
      <p><strong><span style="color: #ff0000;">PLEASE RELOAD THIS
      TOOL WHEN FINISHED!!!</span></strong></p>'
        )
    )
}

# * popup for upload gene names ----------------------------------------
uploadGeneNameModal <- function() {
    modalDialog(
        title = "Upload gene names",
        size = "m",
        em(paste(
            "This option is used to map gene IDs from the main input with",
            "their gene names. Please upload the mapping file for gene IDs - ",
            "gene names in tab-delimited format! Check",
            "https://github.com/BIONF/PhyloProfile/wiki/Input-Data#gene-names",
            "for more info."
        )),
        br(),
        fileInput("geneName", "")
    )   
}

# * popup for upload gene category -------------------------------------
uploadGeneNameModal <- function() {
    modalDialog(
        title = "Upload gene categories",
        size = "s",
        em(paste(
            "This option is used to map gene IDs from the main input with",
            "a category (e.g. metabolic pathway, phenotype, lifestyle, etc.). ",
            "Please upload the mapping file for gene ID - ",
            "category in tab-delimited format! Check",
            "https://github.com/BIONF/PhyloProfile/wiki/Input-Data#gene-categories",
            "for more info."
        )),
        br(),
        fileInput("geneCategory", "")
    )   
}

# * popup for FASTA upload ---------------------------------------------
uploadFastaModal <- function() {
    modalDialog(
        title = "FASTA upload",
        size = "s",
        selectInput(
            "inputType", "Choose location for:",
            c("Concatenated fasta file", "Fasta folder")
        ),
        hr(),
        conditionalPanel(
            condition = "input.inputType == 'Concatenated fasta file'",
            fileInput("concatFasta", ""),
            uiOutput("concatFasta.existCheck")
        ),
        conditionalPanel(
            condition = "input.inputType == 'Fasta folder'",
            textInput("path", "Main FULL path:", ""),
            selectInput(
                "dirFormat", "Directory format:",
                choices = list("path/speciesID.fa*" = 1,
                               "path/speciesID/speciesID.fa*" = 2),
                selected = "Path/speciesID.fasta"
            ),
            selectInput(
                "fileExt", "File extension:",
                choices = list("fa" = "fa",
                               "fasta" = "fasta",
                               "fas" = "fas",
                               "txt" = "txt"),
                selected = "fa"
            ),
            selectInput(
                "idFormat",
                "ID format:",
                choices = list(">speciesID:seqID" = 1,
                               ">speciesID@seqID" = 2,
                               ">speciesID|seqID" = 3,
                               ">seqID" = 4),
                selected = 4
            )
        )
    )
}

# * popup for adding new taxa from input file --------------------------
addTaxaModal <- function() {
    modalDialog(
        title = "Add new taxa",
        size = "m",
        HTML(
            "<p><em>Use this form to add taxon that does not exist in NCBI
                taxonomy database (or alternatively you can manually prepare the
                <span style=\"text-decoration: underline;\">
                <span style=\"color: #ff0000; text-decoration: underline;\">
                PhyloProfile/data/newTaxa.txt file with the following
                description for each field).</em></p>
                <p><span style=\"color: #ff0000;\"><em><strong>
                NOTE: ID and name of new taxon must be
                <span style=\"text-decoration: underline;\">different</span>
                from any existing NCBI taxa.</strong></em></span></p>"
        ),
        textInput(
            "newID",
            "ID (must be a number and greater than 999999005,
                e.g. 999999901)",
            999999901,
            width = 500
        ),
        textInput(
            "newName",
            "Name (e.g. Saccharomyces cerevisiae strain ABC)",
            "",
            width = 500
        ),
        textInput(
            "newRank",
            "Rank (e.g. strain, species, order, etc.)",
            "species",
            width = 500
        ),
        textInput(
            "newParent",
            "Parent ID (NCBI taxonomy ID of the next higher rank,
                e.g. 4932 (S.cerevisiae species))",
            4932,
            width = 500
        ),
        actionButton("newAdd", "Add new taxon"),
        hr(),
        fileInput("newTaxaFile",
                  "Or upload file contains IDs for new taxa"),
        HTML(
            "<p><em>Taxonomy file for new taxa has to be a tab-delimited
                text file and has the following header (please follow the rule
                above):</em></p><p>ncbiID &nbsp;fullName &nbsp;rank
                &nbsp;parentID</p>"
        ),
        uiOutput("wrongNewTaxaUI"),
        hr(),
        actionButton("newDone", "Finish adding", class = "btn btn-warning")
    )
}

# * popup for getting taxa from OMA browser ----------------------------
getOmaModal <- function(){
    modalDialog(
        title = "Get OMA data",
        size = "s",
        selectInput(
            "selectedOmaType",
            label = "Select type of OMA orthologs:",
            choices = list("HOG", "OG"),# "PAIR"),
            selected = "HOG"
        ),
        actionButton("getDataOma", "Get data", class = "btn btn-danger"),
        downloadButton("downloadFilesOma", "Save data"),
        br(),
        em("This windows will close automatically when eveything is done!",
           style = "color:red")
    )
}

# * popup for setting ortholog ID format -------------------------------
seqIdFormatModal <- function() {
    modalDialog(
        title = "Sequence ID format",
        size = "s",
        selectInput(
            "seqIdFormat",
            "ID format:",
            choices = list(
                "BIONF format (seed|taxon|ortho)" = 1,
                "seqID" = 2,
                "something<separator>seqID" = 3,
                "seqID<separator>something" = 4
            ),
            selected = 1
        ),
        selectInput(
            "separator",
            "Separator:",
            choices = list(
                "none" = "none",
                "|" = 1,
                "@" = 2,
                "#" = 3,
                ";" = 4
            ),
            selected = 1
        ),
        em("Please note! Only these ID formats are accepted!")
    )
}

# * popup for setting Gene age plot configurations ---------------------
geneAgeConfigModal <- function() {
    modalDialog(
        title = "Plot appearance configuration",
        size = "s",
        sliderInput(
            "geneAgeWidth",
            "Width zoom (*600px)",
            min = 0.1,
            max = 5,
            step = 0.1,
            value = 1,
            width = "100%"
        ),
        sliderInput(
            "geneAgeHeight", "Height zoom (*150px)",
            min = 0.1,
            max = 5,
            step = 0.1,
            value = 1,
            width = "100%"
        ),
        sliderInput(
            "geneAgeText", "Text size zoom",
            min = 0.1,
            max = 5,
            step = 0.1,
            value = 1,
            width = "100%"
        ),
        br(),
        hr(),
        actionButton("resetGeneAgeProtConfig", "Reset", class = "btn btn-danger")    
    )
}

# * popup for setting Group compariosn plot configurations -------------
gcPlotConfigModal <- function() {
    modalDialog(
        title = "Plot appearance configuration",
        size = "l",
        column(
            6,
            createTextSize("xSizeGC", "X-axis label size (px)", 12, '100%')
        ),
        column(
            6,
            createTextSize("ySizeGC", "Y-axis label size (px)", 12, '100%')
        ),
        column(
            6,
            createTextSize("titleSizeGC", "Title size (px)", 15, '100%')
        ),
        column(
            6,
            createTextSize("legendSizeGC", "Legend label size (px)",
                           12, '100%')
        ),
        hr(),
        column(
            12,
            HTML("<p><span style=\"color: #ff0000;\"><strong><em>Options
                     for variable plot only:</em></strong></span></p>")
        ),
        column(
            6, createPlotSize("widthVarGC", "Width (px)", 600)
        ),
        column(
            6, createPlotSize("heightVarGC", "Height (px)", 400)
        ),
        column(
            6,
            selectInput(
                "legendGC", label = "Legend position:",
                choices = list("Right" = "right",
                               "Bottom" = "bottom",
                               "Hide" = "none"),
                selected = "right",
                width = '100%'
            )
        ),
        column(
            6,
            selectInput(
                "mValueGC", label = "Show mean/median point:",
                choices = list("Mean" = "mean",
                               "Median" = "median"),
                selected = "mean",
                width = '100%')
        ),
        hr(),
        column(
            12,
            HTML("<p><span style=\"color: #ff0000;\"><strong><em>Options
                     for feature plot only:</em></strong></span></p>")
        ),
        column(
            6, createPlotSize("widthFeatureGC", "Width (px)", 600)
        ),
        column(
            6, createPlotSize("heightFeatureGC", "Height (px)", 400)
        ),
        column(
            6,
            radioButtons(
                inputId = "xAxisGC",
                label = "Flip coordinates",
                choices = list("Yes", "No"),
                selected = "No",
                inline = TRUE
            )
        ),
        column(
            6,
            sliderInput(
                "angleGC", "Angle of the X-axis label",
                min = 0,
                max = 180,
                step = 1,
                value = 60,
                width = 250
            )
        ),
        hr(),
        column(
            12,
            HTML("<p><span style=\"color: #ff0000;\"><strong><em>Names
                     of in-group and out-group taxa:</em></strong></span></p>")
        ),
        column(
            6,
            textInput("inGroupName",
                      NULL,
                      value = "In-group",
                      width = "100%",
                      placeholder = "Name of in-group taxa")
        ),
        column(
            6,
            textInput("outGroupName",
                      NULL,
                      value = "Out-group",
                      width = "100%",
                      placeholder = "Name of in-group taxa")
        ),
        br(),
        hr(),
        actionButton("resetConfigGC", "Reset", class = "btn btn-danger"),
    )
}

# * popup for input in-group and out-group on Group comparison ---------
gcUploadModal <- function() {
    modalDialog(
        title = "Upload files for group comparison",
        size = "l",
        h4(strong("Sequence(s) that need to be compared")),
        HTML("<p><em>Please use the same sequence IDs as being shown in the
                 phylogenetic profiles!</em></p>"),
        fileInput("gcFile", NULL),
        hr(),
        h4(strong("In-group and out-group taxa")),
        HTML("<p><em>Please upload an <strong>tab-delimited ID list</strong>
                 of in-group and out-group taxa. The ID must follow the format
                 \"<span style=\"text-decoration: underline;\">
                 <strong>ncbi12345</strong></span>\".
                 Example:</em></p>
                     <p><code>
                     ncbi12345 &nbsp; &nbsp;in-group</code></p>
                     <p><code>ncbi24242 &nbsp; &nbsp;in-group</code></p>
                     <p><code>ncbi33333 &nbsp; &nbsp;out-group
                 </code></p>"),
        
        fileInput("taxonGroupGC", NULL),
        conditionalPanel(
            condition = "output.checkTaxonGroupGC == false",
            h5(strong("Invalid taxa were found:")),
            DT::dataTableOutput("invalidTaxonGroupGC")
        )
    )
}


# * popup for selecting a whole taxon clade -------------------
taxaSelectModal <- function(id, title, checkboxId, checkboxLabel) {
    modalDialog(
        title = title,
        size = "s",
        easyClose = TRUE,
        footer = NULL,
        selectTaxonRankUI(id),
        checkboxInput(
            checkboxId,
            strong(checkboxLabel, style = "color:red"),
            value = FALSE
        )
    )
}

# * popup for selecting profile plot colors -------------------
colorSettingsModal <- function() {
    modalDialog(
        title = "Set colors for profile",
        size = "s",
        easyClose = TRUE,
        footer = NULL,
        
        colourpicker::colourInput(
            "lowColorVar1", "Low variable 1 (dot)", value = "#FF8C00"
        ),
        colourpicker::colourInput(
            "midColorVar1", "Mid variable 1 (dot)", value = "#40ABCF"
        ),
        colourpicker::colourInput(
            "highColorVar1", "High variable 1 (dot)", value = "#164294"
        ),
        numericInput(
            "midVar1", "Midpoint variable 1", 
            min = 0, max = 1, step = 0.01, value = 0.5
        ),
        actionButton(
            "defaultColorVar1", "Default", style = "padding:4px; font-size:100%"
        ),
        hr(),
        
        colourpicker::colourInput(
            "lowColorVar2", "Low variable 2 (background)", value = "#CC8D8D"
        ),
        colourpicker::colourInput(
            "midColorVar2", "Mid variable 2 (background)", value = "#FFFFFF"
        ),
        colourpicker::colourInput(
            "highColorVar2", "High variable 2 (background)", value = "#616587"
        ),
        numericInput(
            "midVar2", "Midpoint variable 2", 
            min = 0, max = 1, step = 0.01, value = 1
        ),
        actionButton(
            "defaultColorVar2", "Default", style = "padding:4px; font-size:100%"
        ),
        hr(),
        
        colourpicker::colourInput(
            "paraColor", "Color for inparalogs", value = "#07d000"
        ),
        actionButton(
            "defaultColorPara", "Default", style = "padding:4px; font-size:100%"
        )
    )
}


# * popup for selecting profile plot appearance (except colors) ----------------
plotConfigModal <- function(suffix = "", defaults, resetId, applyId) {
    modalDialog(
        title = "Plot appearance configuration",
        size = "m",
        easyClose = TRUE,
        footer = NULL,
        
        fluidRow(
            column(
                6, 
                createTextSize(
                    idWithSuffix("xSize", suffix), "X-axis label size (px)", 
                    defaults$xSize, 100
                )
            ),
            column(
                6, 
                createTextSize(
                    idWithSuffix("ySize", suffix), "Y-axis label size (px)", 
                    defaults$ySize, 100
                )
            ),
            column(
                6, 
                createTextSize(
                    idWithSuffix("legendSize", suffix), "Legend label size (px)",
                    8, 150
                )
            ),
            column(6, selectInput(
                idWithSuffix(
                    if (suffix=="") "mainLegend" else "selectedLegend", ""
                ),
                "Legend position:",
                choices = list(
                    "Right" = "right", "Left" = "left", "Top" = "top",
                    "Bottom" = "bottom", "Hide" = "none"
                ),
                selected = defaults$legend,
                width = 150
            )),
            
            column(
                6,
                createTextSize(
                    idWithSuffix("groupLabelSize", suffix), 
                    "Group label size (px)", 7, 100
                )
            ),
            column(
                6, 
                numericInput(
                    idWithSuffix("groupLabelDist", suffix), 
                    "Height for group label",
                    min = 1, max = 2, step = 0.1, 
                    value = defaults$groupLabelDist, width = 100
                )
            ),
            column(12,
                   HTML("<strong>Angle for taxonomic group label</strong>:<br>"),
                   sliderInput(
                       idWithSuffix("groupLabelAngle", suffix), "", 
                       min = 0, max = 90, step = 10, 
                       value = defaults$groupLabelAngle, width = 250
                   ),
                   br()
            ),
            column(12,
                   HTML("<strong>Angle for x-axis label</strong>:<br>"),
                   sliderInput(
                       idWithSuffix("xAngle", suffix), "", 
                       min = 0, max = 90, step = 10, 
                       value = defaults$xAngle, width = 250
                   ),
                   br()
            ),
            column(12,
                   HTML("<strong>Zooming factor (α) for dots on profile</strong>:<br>"),
                   sliderInput(
                       idWithSuffix("dotZoom", suffix), "", 
                       min = -1, max = 3, step = 0.1, 
                       value = defaults$dotZoom, width = 250
                   ),
                   HTML("<em>dot size = (1+α)*defaultSize<br>defaultSize=[0:5]</em>"),
                   uiOutput(idWithSuffix("dotSizeInfo", suffix)),
                   br()
            )
        ),
        br(), hr(),
        actionButton(resetId, "Reset", class = "btn btn-danger"),
    )
}

# Get OMA information functions
# using OmaDB package (https://github.com/DessimozLab/OmaDB)

#' Check the validity of input OMA IDs
#' @description Check if input IDs are valid OMA IDs for OMA Browser
#' @export
#' @param ids list of ids needs to be checked
#' @return List of invalid IDs (not readable for OMA)
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @examples
#' ### Uncomment the following line to run the function
#' # checkOmaID("HUMAN29398")

checkOmaID <- function(ids) {
    if (!requireNamespace("OmaDB", quietly = TRUE)) {
        stop(
            "Package \"OmaDB\" must be installed to use this function.",
            call. = FALSE
        )
    }
    ids <- as.character(ids)
    getOma <- OmaDB::getProtein(ids)
    return(setdiff(ids, names(getOma)))
}

#' Get OMA members
#' @description Get OMA ortholog group, OMA HOG or OMA pair's members for a
#' seed protein from OMA Browser.
#' @export
#' @param id ID of the seed protein (OMA or UniProt ID)
#' @param orthoType type of OMA orthologs: either "HOG", "OG"
#' (orthologous group) or "PAIR" (orthologous pair - CURRENTLY NOT WORKING).
#' Default = "OG".
#' @return List of OMA orthologs for an input seed protein.
#' @author Carla Mölbert carla.moelbert@gmx.de
#' @examples
#' ### Uncomment the following line to run the function
#' # getOmaMembers("HUMAN29397", "OG")

getOmaMembers <- function(id = NULL, orthoType = "OG") {
    if (!requireNamespace("OmaDB", quietly = TRUE)) {
        stop(
            "Package \"OmaDB\" must be installed to use this function.",
            call. = FALSE
        )
    }
    if (is.null(id)) stop("No OMA ID given!")
    # get the members of the Hierarchical Orthologous Group
    if (orthoType == "HOG") {
        members <- suppressWarnings(
            OmaDB::getHOG(id = id, level = "root", members = TRUE)$members$omaid
        )
    }
    # get the members of the Ortholoug group
    else if (orthoType == "OG") {
        members <- suppressWarnings(
            OmaDB::getOMAGroup(id = id)$members$omaid
        )
    }
    return(members)
}

#' Get domain annotation from OMA Browser
#' @export
#' @description Get domain annotation from OMA Browser based on a URL or a
#' raw data frame contains annotation info from OMA
#' @param domainURL URL address for domain annotation of ONE OMA id or a
#' raw data frame contains annotation info from OMA
#' @return Data frame contains feature names with their start and end positions
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @examples
#' ### Uncomment the following line to run the function
#' # getOmaDomainFromURL("https://omabrowser.org/api/protein/7916808/domains/")

getOmaDomainFromURL <- function(domainURL = NULL) {
    if (!requireNamespace("OmaDB", quietly = TRUE)) {
        stop(
            "Package \"OmaDB\" must be installed to use this function.",
            call. = FALSE
        )
    }
    if (is.null(domainURL)) stop("OMA domain URL is NULL!")
    if (grepl("https://", domainURL[1])) {
        domains <- OmaDB::resolveURL(domainURL)$regions
    } else {
        domains <- domainURL$regions
    }
    
    if (length(domains) == 0) return(list())
    pos <- strsplit(domains$location, ":")
    allPos <- unlist(pos)
    start <- allPos[c(TRUE, FALSE)]
    end <- allPos[c(FALSE, TRUE)]

    feature <- ifelse(
        nchar(domains$name) > 0,
        paste0(domains$source, "_", domains$domainid, " (",domains$name,")"),
        paste0(domains$source, "_", domains$domainid)
    )
    feature <- gsub("#", "-", feature)
    featureRep <- vapply(pos, FUN.VALUE = numeric(1), function (x) length(x)/2)

    return(data.frame(
        feature = rep(feature, featureRep), start, end, stringsAsFactors = FALSE
    ))
}

#' Get taxonomy ID, sequence and annotation for one OMA protein
#' @export
#' @param id oma ID of one protein
#' @return Data frame contains the input protein ID with its taxonomy ID,
#' sequence, length and domain annotations (tab delimited) for input OMA protein
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @examples
#' ### Uncomment the following line to run the function
#' # getOmaDataForOneOrtholog("HUMAN29397")

getOmaDataForOneOrtholog <- function(id = NULL) {
    if (!requireNamespace("OmaDB", quietly = TRUE)) {
        stop(
            "Package \"OmaDB\" must be installed to use this function.",
            call. = FALSE
        )
    }
    if (is.null(id)) stop("No OMA ID given!")
    # get ncbi taxonomy id
    specName <- substr(id, 1, 5)
    taxonID <- paste0(
        "ncbi", OmaDB::getTaxonomy(members = specName, newick = FALSE)$id
    )
    # get raw data
    raw <- OmaDB::getProtein(id)

    # get sequence
    seq <- as.character(raw$sequence)

    # get sequence length
    length <- raw$sequence_length

    # get annotation
    rawDomains <- suppressWarnings(raw$domains)
    domainDf <- suppressWarnings(getOmaDomainFromURL(rawDomains))
    domainDfJoin <- c(domainDf, sep = "#")
    domains <- paste(unlist(do.call(paste, domainDfJoin)), collapse = "\t")

    # return data frame contains all info
    omaDf <- data.frame(
        orthoID = id,
        taxonID = taxonID,
        seq = seq,
        length = length,
        domains = domains,
        stringsAsFactors = FALSE
    )
    return(omaDf)
}

#' Get OMA info for a query protein and its orthologs
#' @description Get taxonomy IDs, sequences, length and annotations for an OMA
#' orthologous group (or OMA HOG).
#' @export
#' @param seedID OMA protein ID
#' @param orthoType type of OMA orthologs ("OG" or "HOG"). Default = "OG".
#' @return Data frame contains info for all sequences of the input OMA group
#' (or HOG). That info contains the protein IDs, taxonomy IDs, sequences,
#' lengths, domain annotations (tab delimited) and the corresponding seed ID.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @examples
#' ### Uncomment the following line to run the function
#' # getDataForOneOma("HUMAN29397", "OG")

getDataForOneOma <- function(seedID = NULL, orthoType = "OG"){
    if (!requireNamespace("OmaDB", quietly = TRUE)) {
        stop(
            "Package \"OmaDB\" must be installed to use this function.",
            call. = FALSE
        )
    }
    if (is.null(seedID)) stop("No OMA seed ID given!")
    # get members
    idList <- getOmaMembers(seedID, orthoType)
    specName <- substr(idList, 1, 5)

    # get taxonomy IDs
    taxonID <- paste0(
        "ncbi",
        lapply(
            specName,
            function (x) OmaDB::getTaxonomy(members = x, newick = FALSE)$id
        )
    )

    # get sequences, protein lengths and their domain annotations
    raw <- OmaDB::getProtein(idList)
    seq <- lapply(
        seq(length(raw)), function (x) as.character(raw[[x]]$sequence)
    )
    length <- lapply(
        seq(length(raw)), function (x) raw[[x]]$sequence_length
    )
    domains <- lapply(
        seq(length(raw)),
        function (x) {
            rawDomains <- suppressWarnings(raw[[x]]$domains)
            domainDf <- suppressWarnings(getOmaDomainFromURL(rawDomains))
            domainDfJoin <- c(domainDf, sep = "#")
            paste(unlist(do.call(paste, domainDfJoin)), collapse = "\t")
        }
    )
    # return data frame
    return(data.frame(
        orthoID = idList,
        taxonID = taxonID,
        seq = unlist(seq),
        length = unlist(length),
        domains = unlist(domains),
        seed = rep(seedID, length(idList)),
        stringsAsFactors = TRUE
    ))
}

#' Create a phylogenetic profile from a raw OMA dataframe
#' @export
#' @param finalOmaDf raw OMA data for a list of proteins (see ?getDataForOneOma)
#' @return Dataframe of the phylogenetic profiles in long format, which
#' contains the seed protein IDs, their orthologous proteins and the
#' corresponding taxononmy IDs of the orthologs.
#' @seealso \code{\link{getDataForOneOma}}
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @examples
#' ### Uncomment the following lines to run the function
#' # omaData <- getDataForOneOma("HUMAN29397", "OG")
#' # createProfileFromOma(omaData)

createProfileFromOma <- function(finalOmaDf = NULL) {
    if (is.null(finalOmaDf)) stop("Raw OMA data cannot be NULL!")
    profileDf <- finalOmaDf[, c("seed", "taxonID", "orthoID")]
    colnames(profileDf) <- c("geneID", "ncbiID", "orthoID")
    return(profileDf[!duplicated(profileDf), ])
}

#' Create domain annotation dataframe from a raw OMA dataframe
#' @export
#' @param finalOmaDf raw OMA data for a list of proteins (see ?getDataForOneOma)
#' @return Dataframe of the domain annotation used for PhyloProfile, which
#' contains seed IDs, ortholog IDs, ortholog lengths, annotated features, start
#' and end positions of those features.
#' @seealso \code{\link{getDataForOneOma}}
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom data.table rbindlist
#' @examples
#' ### Uncomment the following line to run the function
#' # omaData <- getDataForOneOma("HUMAN29397", "OG")
#' # getAllDomainsOma(omaData)

getAllDomainsOma <- function(finalOmaDf = NULL) {
    if (is.null(finalOmaDf)) stop("Raw OMA data cannot be NULL!")

    seedID <- finalOmaDf$seed
    orthoID <- finalOmaDf$orthoID
    length <- finalOmaDf$length
    feature <- finalOmaDf$domains

    featureCount <- vapply(
        feature,
        function (x) length(unlist(strsplit(x, "\t"))),
        FUN.VALUE = numeric(1)
    )
    domainDf <- data.frame(
        do.call(rbind, strsplit(unlist(strsplit(feature, "\t")), "#")),
        stringsAsFactors = FALSE
    )
    colnames(domainDf) <- c("feature", "start", "end")
    domainDf$seedID <- rep(paste0(seedID, "#", orthoID), featureCount)
    domainDf$orthoID <- rep(orthoID, featureCount)
    domainDf$length <- rep(length, featureCount)

    seedDf <- domainDf[domainDf$orthoID %in% levels(as.factor(seedID)),]
    seedDfFull <- data.frame(
        feature = rep(seedDf$feature, nlevels(as.factor(domainDf$seedID))),
        start = rep(seedDf$start, nlevels(as.factor(domainDf$seedID))),
        end = rep(seedDf$end, nlevels(as.factor(domainDf$seedID))),
        seedID = rep(levels(as.factor(domainDf$seedID)), each = nrow(seedDf)),
        orthoID = rep(seedDf$orthoID, nlevels(as.factor(domainDf$seedID))),
        length = rep(seedDf$length, nlevels(as.factor(domainDf$seedID))),
        stringsAsFactors = FALSE
    )

    outDf <- data.table::rbindlist(list(domainDf, seedDfFull))
    outDf$length <- as.numeric(outDf$length)
    outDf$start <- as.numeric(outDf$start)
    outDf$end <- as.numeric(outDf$end)

    return(
        outDf[, c("seedID", "orthoID", "length", "feature", "start", "end")]
    )
}

#' Get all fasta sequences from a raw OMA dataframe
#' @export
#' @param finalOmaDf raw OMA data for a list of proteins (see ?getDataForOneOma)
#' @return A list contains all protein sequences in fasta format.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @seealso \code{\link{getDataForOneOma}}
#' @examples
#' ### Uncomment the following line to run the function
#' # omaData <- getDataForOneOma("HUMAN29397", "OG")
#' # getAllFastaOma(omaData)

getAllFastaOma <- function(finalOmaDf = NULL) {
    if (is.null(finalOmaDf)) stop("Raw OMA data cannot be NULL!")
    fastaDf <- finalOmaDf[, c("orthoID", "seq")]
    fastaOut <- paste(paste0(">", fastaDf$orthoID), fastaDf$seq, sep = "\n")
    return(unique(fastaOut))
}

#' Get selected fasta sequences from a raw OMA dataframe
#' @export
#' @param finalOmaDf raw OMA data for a list of proteins (see ?getDataForOneOma)
#' @param seqID OMA ID of selected protein
#' @return Required protein sequence in fasta format.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @seealso \code{\link{getDataForOneOma}}
#' @examples
#' ### Uncomment the following line to run the function
#' # omaData <- getDataForOneOma("HUMAN29397", "OG")
#' # getSelectedFastaOma(omaData, "HUMAN29397")

getSelectedFastaOma <- function(finalOmaDf = NULL, seqID = NULL) {
    if (is.null(finalOmaDf)) stop("Raw OMA data cannot be NULL!")
    selectedDf <- subset(
        finalOmaDf[, c("orthoID", "seq")],
        finalOmaDf$orthoID == seqID
    )
    header <- paste0(">", selectedDf$orthoID[1])
    return(paste(header, selectedDf$seq[1], sep = "\n"))
}

context("test functions for getting data from OMA browser")

# test_that("test for a wrong OMA ID", {
#     id <- "RATNOBLABLA"
#     expect_error(checkOmaID(id))
# })
# 
# test_that("test for getting HOG orthologs", {
#     id <- "HUMAN29397"
#     orthologs <- getOmaMembers(id, "HOG")
#     expect_true(length(orthologs) > 1)
# })
# 
# test_that("test for getting protein annotations and fasta for one OMA id", {
#     id <- "HUMAN29397"
#     omaDf <- getOmaDataForOneOrtholog(id)
#     expect_true(ncol(omaDf) == 5)
# })
# 
# test_that("test for parsing annotation", {
#     id <- "HUMAN29397"
#     omaDf <- getDataForOneOma(id, "HOG")
#     domainDf <- getAllDomainsOma(omaDf)
#     expect_true(ncol(domainDf) == 6)
# })

# Functions for working with phylogenetic trees

#' Check the validity of input newick tree
#' @export
#' @param tree input newick tree
#' @param inputTaxonID list of all input taxon IDs for the phylogenetic profiles
#' @return Possible formatting error of input tree. 0 = suitable tree for using
#' with PhyloProfile, 1 = missing parenthesis; 2 = missing comma;
#' 3 = tree has singleton; or a list of taxa that do not exist in the input
#' phylogenetic profile.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @seealso \code{\link{getInputTaxaID}} for getting input taxon IDs,
#' \code{\link{ppTree}} for an example of input tree
#' @examples
#' data("ppTree", package="PhyloProfile")
#' checkNewick(ppTree, c("ncbi3702", "ncbi3711", "ncbi7029"))

checkNewick <- function(tree, inputTaxonID = NULL){
    if (missing(tree)) return("No tree given!")
    # get tree structure
    treeStruc <- gsub("\\w", "", as.character(tree$V1), perl = TRUE)

    open <- lengths(regmatches(treeStruc, gregexpr("\\(", treeStruc)))
    close <- lengths(regmatches(treeStruc, gregexpr("\\)", treeStruc)))
    comma <- lengths(regmatches(treeStruc, gregexpr("\\,", treeStruc)))
    singleton <- lengths(regmatches(treeStruc, gregexpr("\\(\\)", treeStruc)))

    if (singleton > 0) return(3) # tree contains singleton
    if (open != close) return(1) # missing parenthesis
    else {
        if ((comma - open) > 1 | (comma - open) < 0) {
            # return(2) # missing comma
        } else {
            # get list of tips
            nodeString <- gsub("\\W+", "#", as.character(tree$V1))
            nodeList <- unlist(strsplit(nodeString, "#"))
            # list of input taxa
            inputTaxa <- inputTaxonID
            # get missing taxa
            setdiff(inputTaxa, nodeList)
            missingTaxa <- setdiff(nodeList, inputTaxa)
            missingTaxa <- missingTaxa[lapply(missingTaxa, nchar) > 0]
            if (length(missingTaxa) > 0) {
                # contains taxa that not exist in main input
                return(paste(missingTaxa, collapse = "; "))
            } else return(0)
        }
    }
    return(0)
}

#' Create unrooted tree from a taxonomy matrix
#' @export
#' @param df data frame contains taxonomy matrix used for generating tree
#' @importFrom ape as.phylo
#' @return A unrooted taxonomy tree as an object of class "phylo".
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @seealso \code{\link{taxa2dist}} for distance matrix generation from a
#' taxonomy matrix, \code{\link{getTaxonomyMatrix}} for getting taxonomy
#' matrix, \code{\link{ppTaxonomyMatrix}} for a demo taxonomy matrix data
#' @examples
#' data("ppTaxonomyMatrix", package = "PhyloProfile")
#' createUnrootedTree(ppTaxonomyMatrix)

createUnrootedTree <- function(df){
    if (missing(df)) return("No taxonomy matrix given!")
    fullName <- ncbiID <- NULL
    # prepare matrix for calculating distances
    distDf <- subset(df, select = -c(ncbiID, fullName))
    row.names(distDf) <- distDf$abbrName
    distDf <- distDf[, -1]
    # calculate distance matrix
    taxdis <- tryCatch(taxa2dist(distDf), error = function(e) e)
    # create tree
    tree <- ape::as.phylo(stats::hclust(taxdis))
    # return
    return(tree)
}

#' Get sorted supertaxon list based on a rooted taxonomy tree
#' @export
#' @param tree an "phylo" object for a rooted taxonomy tree
#' @return A list of sorted taxa obtained the input taxonomy tree.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @seealso \code{\link{ppTaxonomyMatrix}} for a demo taxonomy matrix data
#' @examples
#' data("ppTaxonomyMatrix", package = "PhyloProfile")
#' # create taxonomy tree rooted by ncbi10090
#' tree <- createUnrootedTree(ppTaxonomyMatrix)
#' rootedTree <- ape::root(tree, outgroup = "ncbi10090", resolve.root = TRUE)
#' # get taxon list sorted from tree
#' sortTaxaFromTree(rootedTree)

sortTaxaFromTree <- function(tree){
    if (missing(tree)) return("No tree given!")
    isTip <- tree$edge[, 2] <= length(tree$tip.label)
    orderedTips <- tree$edge[isTip, 2]
    taxonList <- rev(tree$tip.label[orderedTips])
    return(taxonList)
}

#' taxa2dist
#' @param x taxa matrix
#' @param varstep var-step
#' @param check check
#' @param labels labels
#' @return a distance matrix
#' @author function from taxize library

taxa2dist <- function(x, varstep = FALSE, check = TRUE, labels) {
    rich <- apply(x, 2, function(taxa) length(unique(taxa)))
    S <- nrow(x)
    if (check) {
        keep <- rich < S & rich > 1
        rich <- rich[keep]
        x <- x[, keep]
    }
    i <- rev(order(rich))
    x <- x[, i]
    rich <- rich[i]
    if (varstep) {
        add <- -diff(c(nrow(x), rich, 1))
        add <- add/c(S, rich)
        add <- add/sum(add) * 100
    }
    else {
        add <- rep(100/(ncol(x) + check), ncol(x) + check)
    }
    if (!is.null(names(add)))
        names(add) <- c("Base", names(add)[-length(add)])
    if (!check)
        add <- c(0, add)
    out <- matrix(add[1], nrow(x), nrow(x))
    for (i in seq_len(ncol(x))) {
        out <- out + add[i + 1] * outer(x[, i], x[, i], "!=")
    }
    out <- stats::as.dist(out)
    attr(out, "method") <- "taxa2dist"
    attr(out, "steps") <- add
    if (missing(labels)) {
        attr(out, "Labels") <- rownames(x)
    }
    else {
        if (length(labels) != nrow(x))
            warning("Labels are wrong: needed ", nrow(x), " got ",
                    length(labels))
        attr(out, "Labels") <- as.character(labels)
    }
    if (!check && any(out <= 0))
        warning("you used 'check=FALSE' and some distances are zero
                -- was this intended?")
    out
}

#' Profile clustering
#' @param distanceMatrix
#' @param clusterMethod Method to cluster the distances (input$clusterMethod)
#' @param plotWidth Width of the generated plot (input$clusterPlot.width)
#' @param plotHeight Height of the generated plot (input$clusterPlot.height)

source("R/functions.R")

clusterProfileUI <- function(id) {
    ns <- NS(id)
    tagList(
        column(
            8,
            downloadButton("downloadCluster",
                           "Download plot", class = "butDL"),
            tags$head(tags$style(
                HTML(".butDL{background-color:#476ba3;} .butDL{color: white;}")
            )),
            uiOutput(ns("cluster.ui"))
        ),
        column(
            4,
            downloadButton(ns("downloadDistanceMatrix"),
                           "Download distance matrix"),
            downloadButton(ns("downloadClusterGenes"),
                           "Download gene list"),
            tableOutput(ns("brushedCluster.table"))
        )
    )
}

clusterProfile <- function(input, output, session,
                            distanceMatrix,
                            clusterMethod,
                            plotWidth, plotHeight) {
    # Reactive function holding data for clustering ============================
    clusterData <- reactive({
        if (is.null(distanceMatrix)) stop("Distance matrix is NULL!")
        df <- clusterDataDend(as.dist(distanceMatrix()), clusterMethod())
        return(df)
    })

    # Dendrogram ===============================================================
    output$dendrogram <- renderPlot({
        getDendrogram(clusterData())
    })

    output$cluster.ui <- renderUI({
        ns <- session$ns
        # shinycssloaders::withSpinner(
        plotOutput(
            ns("dendrogram"),
            width = plotWidth(),
            height = plotHeight(),
            brush = brushOpts(
                id = ns("plotBrush"),
                delay = input$brushDelay,
                delayType = input$brushPolicy,
                direction = input$brushDir,
                resetOnNew = input$brushReset
            )
        )
        # )
    })

    # download clustered plot ==================================================
    output$downloadCluster <- downloadHandler(
        filename = function() {
            "clusteredPlot.pdf"
        },
        content = function(file) {
            ggsave(
                file,
                plot = getDendrogram(clusterData()),
                dpi = 300,
                device = "pdf",
                limitsize = FALSE
            )
        }
    )

    # Brushed cluster table ====================================================
    #' render brushedCluster.table based on clicked point on dendrogram plot
    brushedClusterGene <- reactive({
		clusteredTree <- ape::as.phylo(clusterData())
		labels <- rev(sortTaxaFromTree(clusteredTree))

        # get list of selected gene(s)
        if (is.null(input$plotBrush))
            return()
        else {
            top <- round(input$plotBrush$ymin)
            bottom <- round(input$plotBrush$ymax)
            df <- labels[bottom:top]
        }

        # return list of genes
        df <- df[!is.na(df)]
        return(df)
    })

    output$brushedCluster.table <- renderTable({
        if (is.null(input$plotBrush$ymin))
            return()

        data <- as.data.frame(brushedClusterGene())
        data$number <- rownames(data)
        colnames(data) <- c("geneID", "No.")
        data <- data[, c("No.", "geneID")]
        data
    })

    #' download gene list from brushedCluster.table
    output$downloadClusterGenes <- downloadHandler(
        filename = function() {
            c("selectedClusteredGeneList.out")
        },
        content = function(file) {
            dataOut <- brushedClusterGene()
            write.table(
                dataOut,
                file,
                sep = "\t",
                row.names = FALSE,
                quote = FALSE
            )
        }
    )

    #' download distance matrix
    output$downloadDistanceMatrix <- downloadHandler(
        filename = function() {
            c("distanceMatrixClustering.out")
        },
        content = function(file) {
            dataOut <- distanceMatrix()
            dataOut <- as.matrix(dataOut)
            write.table(
                dataOut,
                file,
                col.names = TRUE,
                row.names = TRUE,
                quote = FALSE,
                sep = " \t"
            )
        }
    )

    #' Return the brushed genes
    return(brushedClusterGene)
}

#' Parse domain input file
#' @description Get all domain annotations for one seed protein IDs.
#' @export
#' @param seed seed protein ID
#' @param inputFile name of input file (file name or path to folder contains
#' individual domain files)
#' @param type type of data (file" or "folder"). Default = "file".
#' @return A dataframe for protein domains including seed ID, its orthologs IDs,
#' sequence lengths, feature names, start and end positions, feature weights
#' (optional) and the status to determine if that feature is important for
#' comparison the architecture between 2 proteins* (e.g. seed protein vs
#' ortholog) (optional).
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom stringr str_split_fixed
#' @importFrom data.table fread
#' @seealso \code{\link{getDomainFolder}}
#' @examples
#' seed <- "101621at6656"
#' inputFile <- system.file(
#'     "extdata", "domainFiles/101621at6656.domains",
#'     package = "PhyloProfile", mustWork = TRUE
#' )
#' type <- "file"
#' parseDomainInput(seed, inputFile, type)

parseDomainInput <- function(seed = NULL, inputFile = NULL, type = "file") {
    file <- NULL
    # check parameters
    if (type == "folder" & is.null(seed)) stop("Seed ID cannot be NULL!")
    if (is.null(inputFile)) stop("Input domain file cannot be NULL!")
    # get domain from single file
    if (type == "file") {
        file <- inputFile
    }
    # or from a domain folder
    else {
        file <- getDomainFolder(seed, inputFile)
        if (file == "noSelectHit") {return("noSelectHit")}
        else if (file == "noFileInFolder") {return("noFileInFolder")}
    }
    if (file != FALSE) {
        exeptions <- c("noFileInput", "noSelectHit", "noFileInFolder")
        if (!(file %in% exeptions)) {
            domains <- data.frame(fread(file, sep = "\t", header = FALSE))
        }
    }
    if (ncol(domains) == 5) {
        colnames(domains) <-
            c("seedID", "orthoID", "feature", "start", "end")
    } else if (ncol(domains) == 6) {
        colnames(domains) <-
            c("seedID", "orthoID", "length", "feature", "start", "end")
    } else if (ncol(domains) == 7) {
        colnames(domains) <-
            c("seedID", "orthoID", "length", "feature", "start", "end","weight")
    } else if (ncol(domains) == 8) {
        colnames(domains) <- c(
            "seedID", "orthoID", "length", "feature", "start", "end","weight",
            "path")
    } else if (ncol(domains) == 14) {
        domains <- domains[-1,]
        colnames(domains) <- c(
            "seedID", "orthoID", "length", "feature", "start", "end","weight",
            "path","acc","evalue","bitscore","pStart","pEnd","pLen")
        domains$length <- as.integer(domains$length)
        domains$start <- as.integer(domains$start)
        domains$end <- as.integer(domains$end)
        domains$pStart <- as.integer(domains$pStart)
        domains$pEnd <- as.integer(domains$pEnd)
        domains$pLen <- as.integer(domains$pLen)
        domains$evalue <- as.numeric(domains$evalue)
        domains$bitscore <- as.numeric(domains$bitscore)
    } else {
        return("ERR")
    }
    if (nrow(domains) == 0) return("ERR-0")
    domains$seedID <- as.character(domains$seedID)
    domains$orthoID <- as.character(domains$orthoID)
    domains$seedID <- gsub("\\|",":",domains$seedID)
    domains$orthoID <- gsub("\\|",":",domains$orthoID)
    domains[c("feature_type","feature_id")] <-
        stringr::str_split_fixed(domains$feature, '_', 2)
    domains$feature_id[domains$feature_type == "smart"] <-
        paste0(domains$feature_id[domains$feature_type == "smart"], "_smart")
    return(domains)
}

#' Get domain file from a folder for a seed protein
#' @param seed seed protein ID
#' @param domainPath path to domain folder
#' @return Domain file and its complete directory path for the selected protein.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @examples
#' domainPath <- paste0(
#'     path.package("PhyloProfile", quiet = FALSE), "/extdata/domainFiles"
#' )
#' PhyloProfile:::getDomainFolder("101621at6656", domainPath)

getDomainFolder <- function(seed, domainPath){
    if (is.null(seed)) {
        fileDomain <- "noSelectHit"
    } else {
        # check file extension
        allExtension <- c("txt", "csv", "list", "domains", "architecture")
        fileDomain <- paste0(domainPath, "/", seed, ".", allExtension)

        checkExistance <- lapply(fileDomain, function (x) file.exists(x))
        fileDomain <- fileDomain[match(TRUE, checkExistance)]

        if (is.na(fileDomain)) fileDomain <- "noFileInFolder"
    }
    return(fileDomain)
}

context("test creating domain plot for a pair seed and ortholog proteins")

test_that("test domain plot", {
    info <- c("101621at6656", "101621at6656|AGRPL@224129@0|224129_0:001955|1")
    domainDf <- parseDomainInput(
        "101621at6656","domains/101621at6656.domains","file"
    )
    expect_true(nrow(domainDf) == 3)
})

context("test parsing domain input file(s) into dataframe")

test_that("test with single file", {
    a <- parseDomainInput(NULL, "domains/101621at6656.domains", "file")
    expect_true(ncol(a) == 10)
})

test_that("test with domain folder", {
    a <- parseDomainInput("101621at6656", "domains", "folder")
    expect_true(ncol(a) == 10)

    b <- parseDomainInput("OG_1020", "domains", "folder")
    expect_true(b == "noFileInFolder")
})

#' Import function files
sourceFiles = list.files(path = "R", pattern = "*.R$", full.names = TRUE)
lapply(sourceFiles, source, .GlobalEnv)

#' MAIN UI =====================================================================
shinyUI(
    fluidPage(
        includeCSS("www/custom.css"),
        tags$style(type = "text/css", "body {padding-top: 80px;}"),
        shinyjs::useShinyjs(),
        use_bs_popover(),

        # Application title
        titlePanel("", windowTitle = "PhyloProfile"),

        # TOP WELLPANEL FOR PLOT CONFIGURATION ---------------------------------
        conditionalPanel(
            condition = "input.tabs=='Main profile'",
            wellPanel(
                fluidRow(
                    column(
                        2,
                        radioButtons(
                            inputId = "xAxis",
                            label = "Choose type of x-axis:",
                            choices = list("taxa", "genes"),
                            selected = "taxa",
                            inline = TRUE
                        ),
                        radioButtons(
                            inputId = "geneIdType",
                            label = "Display genes using:",
                            choices = list(
                                "Gene IDs" = "geneID","Gene names" = "geneName"
                            ),
                            selected = "geneID",
                            inline = TRUE
                        ),
                        checkboxInput(
                            "keepOrder",
                            strong(em("Retain gene order")),
                            value = TRUE,
                            width = NULL 
                        ) %>%
                            bs_embed_popover(
                                title = "",
                                content = "Do not change gene order while filtering data",
                                placement = "bottom",
                                trigger = "hover"
                            )
                    ),
                    column(
                        1,
                        createPlotSize("width", "Width (px)", 900),
                        checkboxInput(
                            "autoSizing",
                            strong(em("Auto sizing")),
                            value = TRUE,
                            width = NULL
                        )
                    ),
                    column(
                        1, createPlotSize("height", "Height (px)", 900),
                        actionButton("mainPlotConfig", "Appearance")
                    ),
                    column(
                        2, uiOutput("var1Cutoff.ui")
                    ),
                    column(
                        2, uiOutput("var2Cutoff.ui")
                    ),
                    column(
                        2, uiOutput("percentCutoff.ui"),
                        actionButton(
                            "applyFilter",
                            label = "Apply filter",
                            icon = icon("check"),
                            class = "btn btn-warning"
                        )
                    ),
                    column(
                        2,
                        numericInput(
                            "coortholog",
                            "Max co-orthologs",
                            min = 1,
                            max = 999,
                            step = 1,
                            value = 999,
                            width = 150
                        ),
                        actionButton(
                            "resetMain",
                            label = "Reset cutoffs",
                            icon = icon("backward"),
                            class = "btn btn-danger"
                        )
                    )
                )
            )
        ),

        conditionalPanel(
            condition = "input.tabs=='Customized profile'",
            wellPanel(
                fluidRow(
                    column(
                        2,
                        radioButtons(
                            inputId = "xAxisSelected",
                            label = "Choose type of x-axis:",
                            choices = list("taxa", "genes"),
                            selected = "taxa",
                            inline = TRUE
                        ),
                        hr(),
                    ),
                    column(
                        1,
                        createPlotSize("selectedWidth", "Width (px)", 900),
                        checkboxInput(
                            "selectedAutoSizing",
                            strong(em("Auto sizing")),
                            value = TRUE,
                            width = NULL
                        )
                    ),
                    column(
                        1, createPlotSize("selectedHeight", "Height (px)", 900),
                        actionButton("selectedPlotConfig", "Appearance")
                    ),
                    column(
                        2, uiOutput("var1Filter.ui")
                    ),
                    column(
                        2, uiOutput("var2Filter.ui")
                    ),
                    column(
                        2, uiOutput("percentFilter.ui"),
                        actionButton(
                            "applyFilterCustom",
                            label = "Apply filter",
                            icon = icon("check"),
                            class = "btn btn-warning"
                        )
                        
                    ),
                    column(
                        2,
                        uiOutput("coorthologFilter.ui"),
                        actionButton(
                            "resetSelected",
                            "Reset cutoffs",
                            icon = icon("backward"),
                            class = "btn btn-danger"
                        )
                    )
                )
            )
        ),

        # MAIN NARVARPAGE TABS -------------------------------------------------
        navbarPage(
            em(strong("PhyloProfile v2.2.0")),
            id = "tabs",
            collapsible = TRUE,
            inverse = TRUE,
            fluid = TRUE,
            position = "fixed-top",

            # INPUT TAB --------------------------------------------------------
            tabPanel(
                "Input & settings",
                # * 1st column -------------------------------------------------
                column(
                    4,
                    # ** Main input --------------------------------------------
                    strong(h4("Main input:")),
                    conditionalPanel(
                        condition = "input.do",
                        em(
                            strong(
                                "RELOAD THIS TOOL TO UPLOAD A NEW INPUT
                                FILE!!!", style = "color:red"
                            )
                        )
                    ),

                    selectInput(
                        "demoData", label = h5("Use online demo data:"),
                        choices = list(
                            "None" = "none",
                            "AMPK-TOR" = "ampk-tor",
                            "BUSCO Arthropoda" = "arthropoda"
                        ),
                        selected = "none",
                        width = "80%"
                    ),

                    uiOutput("noInternetMsg"),
                    uiOutput("demoDataDescribe"),
                    uiOutput("mainInputFile.ui"),
                    uiOutput("inputCheck.ui"),

                    fluidRow(
                        column(
                            6,
                            conditionalPanel(
                                condition = "output.checkOmaInput",
                                actionButton(
                                    "openOmaWindows", "Get data from OMA"
                                ),
                                br()
                            )
                        )
                    ),

                    # ** Variable 1 --------------------------------------------
                    fluidRow(
                        column(
                            4, uiOutput("var1ID.ui")
                        ),
                        column(
                            4,
                            selectInput(
                                "var1AggregateBy",
                                label = h5("Aggregate by:"),
                                choices = list(
                                    "Max" = "max",
                                    "Min" = "min",
                                    "Mean" = "mean",
                                    "Median" = "median"
                                ),
                                selected = "max",
                                width = 130
                            )
                        ),
                        column(
                            4,
                            selectInput(
                                "var1Relation", label = h5("Relationship:"),
                                choices = list(
                                    "Prot-Prot" = "protein",
                                    "Prot-Spec" = "species"
                                ),
                                selected = "protein",
                                width = 130
                            ) %>%
                                bs_embed_popover(
                                    title = "",
                                    content = paste(
                                        "select if variable is the value between",
                                        "seed protein - ortholog protein",
                                        "or seed protein - search taxon",
                                        sep = " "
                                    ),
                                    placement = "bottom",
                                    trigger = "hover"
                                )
                        )
                    ),

                    # ** Variable 2 --------------------------------------------
                    fluidRow(
                        column(
                            4, uiOutput("var2ID.ui")
                        ),
                        column(
                            4,
                            selectInput(
                                "var2AggregateBy",
                                label = h5("Aggregate by:"),
                                choices = list(
                                    "Max" = "max",
                                    "Min" = "min",
                                    "Mean" = "mean",
                                    "Median" = "median"
                                ),
                                selected = "max",
                                width = 130
                            )
                        ),
                        column(
                            4,
                            selectInput(
                                "var2Relation", label = h5("Relationship:"),
                                choices = list(
                                    "Prot-Prot" = "protein",
                                    "Prot-Spec" = "species"
                                ),
                                selected = "protein",
                                width = 130
                            )
                        )
                    ),
                    hr(),

                    # ** Domain input ------------------------------------------
                    strong(h4("Additional annotation input:")),
                    radioButtons(
                        inputId = "annoLocation", label = "",
                        choices = list("from file", "from folder"),
                        selected = "from file",
                        inline = TRUE
                    ),

                    uiOutput("domainInputFile.ui")
                ),

                # * 2nd column -------------------------------------------------
                column(
                    3,
                    uiOutput("fileExistMsgUI"),
                    uiOutput("inputMsgUI"),

                    # ** List of new taxa --------------------------------------
                    conditionalPanel(
                        condition = "output.unkTaxaStatus == 'unknown' ||
                        output.unkTaxaStatus == 'ncbi' ||
                        output.unkTaxaStatus == 'invalid'",
                        strong(h4("New taxa were found:")),
                        DT::dataTableOutput("unkTaxaFull"),
                        br(),
                        downloadButton("unkTaxa.download", "Download ID list")
                    ),

                    # ** Other input options -----------------------------------
                    conditionalPanel(
                        condition = "output.unkTaxaStatus == 0",
                        strong(h4("Choose genes of interest:")),
                        radioButtons(
                            inputId = "geneListSelected",
                            label = "",
                            choices = list("all", "from file"),
                            selected = "all",
                            inline = TRUE
                        ),

                        conditionalPanel(
                            condition = "input.geneListSelected == 'from file'",
                            fileInput("geneList", "")
                        ),
                        uiOutput("totalGeneNumber.ui"),
                        column(
                            6,
                            numericInput(
                                "stIndex",
                                h5("Show from:"),
                                min = 1,
                                max = 1600,
                                value = 1,
                                width = 130
                            ) %>%
                                bs_embed_popover(
                                    title = "",
                                    content = "Set start index for sequence range",
                                    placement = "bottom",
                                    trigger = "hover"
                                )
                        ),

                        column(
                            6,
                            numericInput(
                                "endIndex",
                                h5("...to:"),
                                min = 1,
                                max = 1600,
                                value = 1000,
                                width = 130
                            ) %>%
                                bs_embed_popover(
                                    title = "",
                                    content = "Set end index for sequence range",
                                    placement = "bottom",
                                    trigger = "hover"
                                )
                        ),
                        hr(),

                        strong(h4("Sequence source:")),
                        column(
                            6,
                            selectInput(
                                "seedSource",
                                label = h5("Seeds:"),
                                choices = list(
                                    "NCBI" = "ncbi",
                                    "UniProt" = "uniprot",
                                    "OrthoDB" = "orthodb",
                                    "OMA" = "oma",
                                    "User-defined" = "user"
                                ),
                                selected = "uniprot",
                                width = 130
                            ),
                            conditionalPanel(
                                condition = "input.seedSource == 'orthodb'",
                                textInput(
                                    "orthodbSeedVer",
                                    h5("OrthoDB version"),
                                    value = "",
                                    placeholder = "e.g. 10-1"
                                ) %>%
                                    bs_embed_popover(
                                        title = "",
                                        content = "Leave blank for latest version",
                                        placement = "bottom",
                                        trigger = "hover"
                                    )
                            )
                        ),
                        column(
                            6,
                            selectInput(
                                "orthoSource",
                                label = h5("Orthologs:"),
                                choices = list(
                                    "NCBI" = "ncbi",
                                    "UniProt" = "uniprot",
                                    "OrthoDB" = "orthodb",
                                    "OMA" = "oma",
                                    "User-defined" = "user"
                                ),
                                selected = "ncbi",
                                width = 130
                            ),
                            conditionalPanel(
                                condition = "input.orthoSource == 'orthodb'",
                                textInput(
                                    "orthodbOrthoVer",
                                    h5("OrthoDB version"),
                                    value = "",
                                    placeholder = "e.g. 10-1"
                                ) %>%
                                    bs_embed_popover(
                                        title = "",
                                        content = "Leave blank for latest version",
                                        placement = "bottom",
                                        trigger = "hover"
                                    )
                            )
                        ),
                        actionButton(
                            "selectSequenceID", "Ortholog ID format",
                            style = "padding:4px; font-size:100%"
                        ),
                        h5(""),
                        hr(),

                        strong(h4("Other optional input:")),

                        actionButton("fastaUpload", "FASTA file(s)"),
                        h5(""),

                        actionButton("uploadGeneCategory", "Gene categories"),
                        h5(""),

                        actionButton("uploadGeneName", "Gene names"),
                        hr(),

                        strong(h4("General configuration:")),
                        column (
                            6,
                            strong("Colors"),
                            br(), br(),
                            actionButton(
                                "setColor", "Change colors",
                                style = "padding:4px; font-size:100%"
                            )
                        ),
                        column(
                            6,
                            strong("Font"),
                            selectInput(
                                "font","", 
                                choices = sort(unique(systemfonts::system_fonts()$family)),
                                selected = "Arial"
                            ) %>%
                                bs_embed_popover(
                                    title = "",
                                    content = "Note: This will be applied for all plots!",
                                    placement = "bottom",
                                    trigger = "hover"
                                )
                        ),
                        h5(""),
                        hr()
                    )
                ),

                # * 3rd column -------------------------------------------------
                column(
                    4,
                    # ** Location for taxonomy files ---------------------------
                    strong(h4("Taxonomy DB location:")),
                    radioButtons(
                        inputId = "taxDbLoc", label = "",
                        choices = list("Default", "User-defined"),
                        selected = "Default",
                        inline = TRUE
                    ),
                    conditionalPanel(
                        condition = "input.taxDbLoc == 'User-defined'",
                        shinyFiles::shinyDirButton(
                            "taxDbDir", "Select taxonomy DB" ,
                            title = "Please select a folder",
                            buttonType = "default", class = NULL
                        ),
                        br(), br(),
                        uiOutput("userTaxDBwarning")
                    ),
                    verbatimTextOutput("taxDbPath"),
                    hr(),

                    # ** Msg for parsing new taxa ------------------------------
                    conditionalPanel(
                        condition = "output.unkTaxaStatus == 'unknown' ||
                        output.unkTaxaStatus == 'ncbi' ||
                        output.unkTaxaStatus == 'invalid'",

                        conditionalPanel(
                            condition = "output.unkTaxaStatus == 'invalid'",
                            HTML(
                                "<p><em>Some new taxa have
                                <span style=\"color: #ff0000;\">invalid IDs
                                </span> (either in newTaxa.txt or in the main
                                profile input or both). IDs of non-NCBI taxa
                                have to be greater than 999999005.</em></p>
                                <p><em>Please replace those IDs before
                                continuing!</em></p>"
                            )
                        ),

                        conditionalPanel(
                            condition = "output.unkTaxaStatus == 'unknown'",
                            HTML(
                                '<p><em>NCBI taxonomy information of some taxa
                                can neither</em></p><ul><li><em>be retrieved
                                from NCBI (<span style="color: #0000ff;
                                ">Source="ncbi"</span>) nor </em></li>
                                <li><em>be found in
                                <span style="color: #ff0000;">
                                PhyloProfile/data/newTaxa.txt</span>&nbsp;(<span
                                style="color: #0000ff;">Source="new"</span>)
                                file</em></li></ul><p><strong><em>Please add
                                taxonomy information for those unknown taxa and
                                <span style="color: #ff0000;"> reload the tool
                                </span> to continue!</em></strong></p>'
                            ),
                            h5(""),
                            actionButton(
                                "addTaxa",
                                "Add taxonomy info",
                                icon = icon("plus"),
                                class = "btn btn-warning"
                            )
                        ),

                        conditionalPanel(
                            condition = "output.unkTaxaStatus == 'ncbi'",
                            HTML(
                                '<p><em>NCBI taxonomy information of some taxa
                                can either</em></p><ul><li><em>be retrieved
                                from NCBI (<span style="color: #0000ff;
                                ">Source="ncbi"</span>) or </em></li>
                                <li><em>be found in
                                <span style="color: #ff0000;">
                                PhyloProfile/data/newTaxa.txt</span>&nbsp;(<span
                                style="color: #0000ff;">Source="new"</span>)
                                file</em></li></ul><p><strong><em>Click here to
                                get required taxonomy information for those
                                taxa!</em></strong></p>'
                            ),
                            h5(""),
                            actionButton(
                                inputId = "butParse",
                                label = "Get taxonomy info",
                                class = "btn btn-warning",
                                disabled = FALSE
                            ),

                            hr(),
                            uiOutput("endParsingMsg"),
                            tableOutput("invalidID.output"),
                            hr(),
                            conditionalPanel(
                                condition = "output.unkTaxaStatus
                                            == 'invalid'",
                                downloadButton(
                                    "invalidID.download", "Download invalid IDs"
                                )
                            )
                        )
                    ),

                    # ** List of ranks & available taxa ------------------------
                    conditionalPanel(
                        condition = "output.unkTaxaStatus == 0",
                        strong(h4("Seed (super)taxon:")),
                        br(),

                        strong(h5("Select taxonomy rank:")),
                        uiOutput("rankSelect"),

                        strong(h5("Choose (super)taxon of interest:")),
                        selectizeInput(
                            "inSelect", "", choices = NULL, selected = NULL
                        ),

                        hr(),

                        # ** Sort gene IDs options -----------------------------
                        strong(h4("Order seed IDs")),
                        radioButtons(
                            inputId = "orderGenes",
                            label = "",
                            choices = list(
                                "none", "alphabetically",
                                "profile similarity", "user defined"
                            ),
                            selected = "profile similarity",
                            inline = TRUE
                        ),
                        conditionalPanel(
                            condition = "input.orderGenes
                                        == 'user defined'",
                            uiOutput("inputSortedGenes.ui"),
                            uiOutput("checkSortedGenes.ui")
                        ),

                        # ** Sort taxa options ---------------------------------
                        strong(h4("Order taxa")),
                        radioButtons(
                            inputId = "orderTaxa",
                            label = "",
                            choices = list(
                                "automatically", "by user defined tree",
                                "by a sorted list"
                            ),
                            selected = "automatically",
                            inline = TRUE
                        ),
                        conditionalPanel(
                            condition = "input.orderTaxa
                                        == 'by user defined tree'",
                            uiOutput("inputTree.ui") %>%
                                bsplus::bs_embed_popover(
                                    title = "",
                                    content = "in newick format",
                                    placement = "bottom",
                                    trigger = "hover"
                                ),
                            uiOutput("checkNewick.ui")
                        ),
                        conditionalPanel(
                            condition = "input.orderTaxa
                                        == 'by a sorted list'",
                            uiOutput("inputSortedTaxa.ui"),
                            uiOutput("checkSortedTaxa.ui")
                        ),

                        checkboxInput(
                            "showAllTaxa",
                            strong("Display all input taxa"),
                            value = FALSE,
                            width = NULL
                        ) %>%
                            bsplus::bs_embed_popover(
                                title = "",
                                content = "Including taxa with no orthologs",
                                placement = "bottom",
                                trigger = "hover"
                            ),
                        hr(),

                        actionButton(
                            "do", "PLOT",
                            class = "btn btn-danger btn-lg"
                        ),
                        h5("")
                    )
                )
            ),

            # MAIN PROFILE TAB =================================================
            tabPanel(
                "Main profile",
                sidebarLayout(
                    # * sidebar panel for profile highlight --------------------
                    sidebarPanel(
                        width = 4,
                        column(
                            12,
                            style = "padding:0px;",
                            column(
                                4,
                                style = "padding:0px;",
                                radioButtons(
                                    inputId = "plotMode",
                                    label = "Plot mode:",
                                    choices = list(
                                        "Normal" = "normal", "Fast" = "fast"
                                    ),
                                    selected = "normal",
                                    inline = TRUE
                                )  %>%
                                    bsplus::bs_embed_popover(
                                        title = "",
                                        content = "Fast mode is only recommended for large data",
                                        placement = "top",
                                        trigger = "hover"
                                    )
                            ),
                            column(
                                8,
                                uiOutput("colorVar.ui")
                            )
                        ),
                        column(
                            12,
                            style = "padding:0px;",
                            strong("Select gene to highlight:")
                        ),
                        column(
                            12,
                            fluidRow(
                                column(
                                    8,
                                    style = "padding:0px;",
                                    selectizeInput(
                                        "geneHighlight","", NULL, multiple=TRUE,
                                        options = list(placeholder = 'none')
                                    )
                                ),
                                column(
                                    4,
                                    fileInput(
                                        "geneHighlightFile", "", width = "100%"
                                    )
                                )
                            )
                        ),
                        column(
                            12,
                            style = "padding:0px;",
                            strong(
                                "Select (super)taxon to highlight:"
                            )
                        ),
                        column(
                            8,
                            style = "padding:0px;",
                            selectizeInput(
                                "taxonHighlight","", NULL, multiple=TRUE,
                                options = list(placeholder = 'none')
                            )
                        ),
                        column(
                            4,
                            h3(""),
                            actionButton("taxonHighlightBrowse", "Browse...")
                        ),
                        column(
                            12,
                            checkboxInput(
                                "colorByGroup",
                                strong("Highlight genes by categories"),
                                value = FALSE
                            ),
                            checkboxInput(
                                "colorByOrthoID",
                                strong("Highlight duplicated ortholog IDs"),
                                value = FALSE
                            ) %>%
                                bsplus::bs_embed_popover(
                                    title = "",
                                    content = paste(
                                        "Please check in the Clustering profiles",
                                        "function, if the profiles are clustered",
                                        "using ortho IDs"
                                    ),
                                    placement = "bottom",
                                    trigger = "hover"
                                ),
                            hr()
                        ),
                        uiOutput("superRankSelect.ui"),
                        hr(),
                        checkboxInput(
                            "autoUpdate",
                            strong(em("Auto update plot")),
                            value = FALSE,
                            width = NULL
                        ),
                        actionButton(
                            "updateBtn", "Update appearance",
                            icon = icon("sync"), class = "btn btn-warning"
                        )
                    ),
                    # * main panel for profile plot ----------------------------
                    mainPanel(
                        conditionalPanel(
                            condition = "input.do > 0",
                            createProfilePlotUI("mainProfile")
                        )
                    )
                )
            ),

            # CUSTOMIZED PROFILE TAB ===========================================
            tabPanel(
                "Customized profile",
                sidebarLayout(
                    # * sidebar panel for subseting data -----------------------
                    sidebarPanel(
                        width = 4,
                        column(
                            12,
                            style = "padding:0px;",
                            strong("Select sequence(s) of interest:")
                        ),

                        column(
                            12,
                            fluidRow(
                                column(
                                    8,
                                    style = "padding:0px;",
                                    selectizeInput(
                                        "inSeq","", NULL, multiple=TRUE,
                                        choices = c('all'), selected = "all"
                                    )
                                ),
                                column(
                                    4,
                                    fileInput("customFile", "", width = "100%")
                                )
                            )
                        ),

                        column(
                            12,
                            style = "padding:0px;",
                            strong(
                                "Select (super)taxon/(super)taxa of interest:"
                            )
                        ),
                        column(
                            12,
                            fluidRow(
                                column(
                                    8,
                                    style = "padding:0px;",
                                    uiOutput("cusTaxa.ui")
                                ),
                                column(
                                    4,
                                    h3(""),
                                    actionButton("cusTaxa", "Browse...")
                                )
                            )
                        ),
                        uiOutput("cusSuperRankSelect.ui"),

                        h5(""),
                        actionButton(
                            "plotCustom", "Update apperance",
                            icon = icon("sync"), class = "btn btn-warning"
                        )
                    ),

                    # * main panel for customized profile plot -----------------
                    mainPanel(
                        conditionalPanel(
                            condition = "output.sameProfile == true",
                            h4(
                                "Please select subset of genes and/
                                or taxa for customized profile!"
                            )
                        ),

                        conditionalPanel(
                            condition = "input.do > 0 | input.plotCustom > 0",
                            createProfilePlotUI("customizedProfile")
                        )
                    )
                )
            ),

            # DIMENSIONALITY REDUCTION reduction TAB ===========================
            tabPanel(
                "Dimension Reduction",
                # * Top panel for plot configuration ---------------------------
                wellPanel(
                    fluidRow(
                        column(
                            2,
                            column(
                                6,
                                style = "padding:0px;",
                                selectInput(
                                    "reductionTechnique", label = "Technique",
                                    choices = list("UMAP" = "umap",
                                                   "t-SNE" = "tsne",
                                                   "PCA" = "pca"),
                                    selected = "umap"
                                )
                            ),
                            column(
                                6,
                                conditionalPanel(
                                    condition = 'input.reductionTechnique == "tsne"',
                                    numericInput(
                                        "tsneIter", "# iter.",
                                        min = 100, max = 5000, step = 100, 
                                        value = 1000
                                    )
                                )
                            ),
                            column(
                                12,
                                style = "padding:0px;",
                                radioButtons(
                                    "dimRedType", "Apply on",
                                    c("Taxa" = "taxa", "Genes" = "genes"),
                                    inline = TRUE
                                ),
                                selectInput(
                                    "dimRedDataType", label = "using",
                                    choices = list(
                                        "Presence/Absence data" = "binary",
                                        "Numeric scores" = "nonbinary"
                                    ),
                                    selected = "binary"
                                )
                            )
                        ),
                        column(
                            4,
                            column(
                                3,
                                createPlotSize(
                                    "dimRedPlot.width", "Plot width", 900
                                ),
                                createPlotSize(
                                    "dimRedPlot.height", "Plot height", 400
                                )
                            ),
                            column(
                                4,
                                createTextSize(
                                    "dimRedPlot.textsize", "Text size", 12
                                ),
                                selectInput(
                                    "dimRedPlot.legend", 
                                    label = "Legend position:",
                                    choices = list("Right" = "right",
                                                   "Left" = "left",
                                                   "Top" = "top",
                                                   "Bottom" = "bottom",
                                                   "Hide" = "none"),
                                    selected = "bottom"
                                )
                            ),
                            column(
                                5,
                                sliderInput(
                                    "dimRedPlot.dotzoom", "Dot size zooming",
                                    min = -3, max = 10, step = 1, value = 0
                                )
                            )
                        ),
                        column(
                            4,
                            column(
                                6,
                                radioButtons(
                                    "dimRedGroupLabelsBy",
                                    "Summarize as [Other] by",
                                    choices = c("taxa", "genes"),
                                    inline = TRUE
                                ),
                                em(paste("If frequency smaller or higher than",
                                         "Freq cutoff, labels will be grouped",
                                         "as [Other]"))
                            ),
                            column(
                                6,
                                sliderInput(
                                    "dimRedLabelNr", "Freq cutoff", min = 0,
                                    max = 99, step = 1, value = c(5,99),
                                    width = 200
                                ),
                                radioButtons(
                                    "dimRedPlotType", "Plot dimensions", 
                                    choices = list(
                                        "2D" = "ggplot", "3D" = "plotly"
                                    ),
                                    selected = "ggplot", inline = TRUE
                                )
                            )
                        ),
                        column(
                            2,
                            sliderInput(
                                "dimRedDotAlpha", "Transparent level", min = 0,
                                max = 1, step = 0.05, value = 0.0, width = 200
                            ),
                            numericInput(
                                "randomSeed", "Random seed",
                                min = 0, max = 9999, step = 1, value = 123
                            )
                        )
                    )
                ),
                sidebarLayout(
                    # * Sidebar panel for data filter --------------------------
                    sidebarPanel(
                        tableOutput("dimRedHoverInfo"),
                        hr(),
                        selectInput(
                            "dimRedRank", label = "Taxonomy rank for labels",
                            choices = getTaxonomyRanks(),
                            selected = "phylum"
                        ),
                        hr(),
                        textInput(
                            "dimRedGroupHigherRank",
                            "Group labels into higher rank",
                            value = "",
                            placeholder = paste(
                                "Type taxon names in higher rank",
                                "(e.g.: Fungi;Metazoa)"
                            )
                        ),
                        uiOutput("dimRedGroupHigherRank.warning"),
                        uiOutput("dimRedCustomLabel.ui"),
                        actionButton(
                            "dimRedApplyChangeLables",
                            "Change labels",
                            icon = icon("play"),
                            class = "btn btn-success"
                        ),
                        actionButton(
                            "dimRedResetLables",
                            "Reset labels",
                            icon = icon("rotate-left"),
                            class = "btn btn-secondary"
                        ) %>%
                            bsplus::bs_embed_popover(
                                title = "",
                                content = "Click `Change labels` after reset!",
                                placement = "bottom",
                                trigger = "hover"
                            ),
                        hr(),
                        column(
                            12,
                            style = "padding:0px;",
                            strong("Choose labels to hide")
                        ),
                        column(
                            12,
                            fluidRow(
                                column(
                                    8,
                                    style = "padding:0px;",
                                    selectInput(
                                        "excludeDimRedTaxa", label = "",
                                        choices = "", selected = NULL, 
                                        multiple = TRUE
                                    )
                                ),
                                column(
                                    4,
                                    fileInput(
                                        "excludeDimRedTaxaFile", "", width = "100%"
                                    )
                                )
                            )
                        ),
                        column(
                            12,
                            style = "padding:0px;",
                            strong("Choose labels to highlight")
                        ),
                        column(
                            12,
                            fluidRow(
                                column(
                                    8,
                                    style = "padding:0px;",
                                    selectInput(
                                        "highlightDimRedTaxa", label = "",
                                        choices = "", selected = NULL, 
                                        multiple = TRUE
                                    )
                                ),
                                column(
                                    4,
                                    fileInput(
                                        "highlightDimRedTaxaFile", "", width = "100%"
                                    )
                                )
                            )
                        ),
                        hr(),
                        selectInput(
                            "colorPalleteDimRed",
                            "Color pallete",
                            choices = c(
                                "Paired", "Set1", "Set2", "Set3",
                                "Accent", "Dark2"
                            ),
                            selected = "Dark2"
                        ),
                        hr(),
                        selectInput(
                            "dimRedFilterVar",
                            "Choose variable for data filtering",
                            choices = c("Var1" = "var1", "Var2" = "var2",
                                        "Both" = "both"),
                            selected = "both"
                        ),
                        sliderInput(
                            "dimRedCutoff", "Filter cutoff", min = 0, max = 1,
                            step = 0.05, value = 0, width = '100%'
                        ),
                        hr(),
                        strong("Add following data to Customized profile"),
                        checkboxInput("addSpecDimRed", em("Selected taxa")) %>%
                            bsplus::bs_embed_popover(
                                title = "",
                                content = paste("Only available when working with the",
                                                "lowest taxonomy rank!"),
                                placement = "bottom",
                                trigger = "hover"
                            ),
                        checkboxInput("addGeneDimRed", em("Selected genes")),
                        uiOutput("addDimRedCustomProfileCheck.ui")
                    ),
                    # * Main panel for DIM red plot and tables -----------------
                    mainPanel(
                        conditionalPanel(
                            condition = 'input.dimRedPlotType == "ggplot"',
                            column(
                                12,
                                em(
                                    "Drag to select, then double click to zoom in. Double click empty space to zoom out",
                                    style = "color:darkblue"
                                )
                            )
                        ),
                        conditionalPanel(
                            condition = 'input.dimRedPlotType == "plotly"',
                            column(
                                12,
                                em(
                                    "Click on data point to select",
                                    style = "color:darkblue"
                                )
                            )
                        ),      
                        uiOutput("dimRedPlot.ui"),
                        br(),
                        column(
                            10,
                            column(
                                3,
                                actionButton(
                                    inputId = "plotDimRed",
                                    label = "PLOT",
                                    class = "btn btn-danger"
                                )
                            ),
                            conditionalPanel(
                                condition = 'input.dimRedPlotType == "plotly"',
                                column(
                                    3,
                                    actionButton(
                                        "clear", "CLEAR selected points"
                                    )
                                )
                            ),
                            column(
                                3,
                                conditionalPanel(
                                    condition = 'input.dimRedPlotType == "plotly"',
                                    
                                    downloadButton(
                                        "dimRedDownloadPlot3D", "Download 3D plot",
                                        class = "butDL"
                                    )
                                ),
                                conditionalPanel(
                                    condition = 'input.dimRedPlotType == "ggplot"',
                                    
                                    downloadButton(
                                        "dimRedDownloadPlot", "Download 2D plot",
                                        class = "butDL"
                                    )
                                )
                            ),
                            column(
                                3,
                                downloadButton(
                                    "dimRedDownloadData", "Download data",
                                    class = "butDL"
                                ) %>%
                                    bsplus::bs_embed_popover(
                                        title = "",
                                        content = paste("Use plotDimRed() to manually create",
                                                        "dimension reduction plot!"),
                                        placement = "bottom",
                                        trigger = "hover"
                                    )
                            )
                        ),
                        br(),
                        uiOutput("dimRedTable.ui")
                    )
                )
            ),

            # FUNCTION TAB =====================================================
            navbarMenu(
                "Function",
                # * Profile clustering ----------------------------------------
                tabPanel(
                    "Profile clustering", value = "profile_clustering",
                    h4(strong("Profile clustering")),
                    uiOutput("descClusteringUI"),

                    wellPanel(
                        fluidRow(
                            column(
                                2, uiOutput("selectProfileType")
                            ),
                            column(
                                3, uiOutput("selectDistMethod")
                            ),

                            column(
                                3,
                                selectInput(
                                    "clusterMethod",
                                    label = h5("Cluster method:"),
                                    choices = list(
                                        "single" = "single",
                                        "complete" = "complete",
                                        "average (UPGMA)" = "average",
                                        "mcquitty (WPGMA)" = "mcquitty",
                                        "median (WPGMC)" = "median",
                                        "centroid (UPGMC)" = "centroid"
                                    ),
                                    selected = "complete"
                                )
                            ),

                            column(
                                1,
                                createPlotSize(
                                    "clusterPlot.width", "Width (px)", 600
                                )
                            ),
                            column(
                                1,
                                createPlotSize(
                                    "clusterPlot.height", "Height (px)", 600
                                )
                            ),
                            column(
                                2,
                                checkboxInput(
                                    "applyCluster",
                                    em(strong(
                                        "Apply clustering to profile plot",
                                        style = "color:darkblue"
                                    )),
                                    value = TRUE
                                ) %>%
                                    bsplus::bs_embed_popover(
                                        title = "",
                                        content = "Uncheck this to sort genes by alphabet",
                                        placement = "bottom",
                                        trigger = "hover"
                                    ),

                                uiOutput("applyClusterCheck.ui"),

                                checkboxInput(
                                    "addClusterCustomProfile",
                                    strong(em(
                                        "Add selected genes to Customized
                                        profile",
                                        style = "color:red"
                                    )),
                                    value = FALSE,
                                    width = NULL
                                ),
                                uiOutput("addClusterCustomProfileCheck.ui")
                            )
                        )
                    ),

                    clusterProfileUI("profileClustering")
                ),

                # * Distribution analysis --------------------------------------
                tabPanel(
                    "Distribution analysis", value = "distribution_analysis",
                    h4(strong("Distribution analysis")),
                    uiOutput("descDistributionUI"),

                    wellPanel(
                        fluidRow(
                            column(
                                2,
                                selectInput(
                                    "dataset.distribution", "Select data",
                                    choices = c("Main data", "Customized data"),
                                    selected = "Main data"
                                ),
                                uiOutput("selected.distribution")
                            ),
                            column(
                                2, uiOutput("var1Dist.ui")
                            ),
                            column(
                                2, uiOutput("var2Dist.ui")
                            ),
                            column(
                                2, uiOutput("percentDist.ui")
                            ),
                            column(
                                2,
                                createTextSize(
                                    "distTextSize", "Label size", 12, 100
                                )
                            ),
                            column(
                                2,
                                createPlotSize(
                                    "distWidth", "Width (px)", 600
                                )
                            )
                        )
                    ),
                    analyzeDistributionUI("distPlot")
                ),


                # * Gene age estimation ----------------------------------------
                tabPanel(
                    "Gene age estimation", value = "gene_age_estimation",
                    h4(strong("Gene age estimation")),
                    uiOutput("descGeneAgeUI"),

                    wellPanel(
                        fluidRow(
                            column(
                                2, uiOutput("var1Age.ui")
                            ),
                            column(
                                2, uiOutput("var2Age.ui")
                            ),
                            column(
                                2, uiOutput("percentAge.ui")
                            ),
                            column(
                                2,
                                strong("Appearance"),
                                actionButton("geneAgeProtConfig", "Plot config")
                            ),
                            column(
                                4,
                                checkboxInput(
                                    "addGeneAgeCustomProfile",
                                    strong(em(
                                        "Add selected genes to Customized
                                        profile",
                                        style = "color:red"
                                    )),
                                    value = FALSE,
                                    width = NULL
                                ),
                                uiOutput("addGeneAgeCustomProfileCheck.ui")
                            )
                        )
                    ),
                    plotGeneAgeUI("geneAge")
                ),

                # * Core gene identification  ----------------------------------
                tabPanel(
                    "Core gene identification", value = "core_gene_ident",
                    h4(strong("Core gene identification")),
                    uiOutput("descCoreGeneUI"),

                    wellPanel(
                        fluidRow(
                            column(
                                3, uiOutput("var1Core.ui")
                            ),
                            column(
                                3, uiOutput("var2Core.ui")
                            ),
                            column(
                                3, uiOutput("percentCore.ui")
                            ),
                            column(
                                3,
                                sliderInput(
                                    "coreCoverage",
                                    "Core taxa coverage",
                                    min = 0,
                                    max = 100,
                                    value = 90,
                                    step = 5
                                )
                            ),
                            column(
                                12,
                                uiOutput("taxaListCore.ui"),
                                actionButton("browseTaxaCore", "Browse")
                            )
                        )
                    ),
                    hr(),

                    column(
                        4,
                        downloadButton(
                            "coreGeneTableDownload", "Download gene list"
                        ),
                        checkboxInput(
                            "addCoreGeneCustomProfile",
                            strong(em("Add core genes to Customized profile",
                                      style = "color:red")),
                            value = FALSE,
                            width = NULL
                        ),
                        uiOutput("addCoreGeneCustomProfileCheck.ui")
                    ),
                    identifyCoreGeneUI("coreGene")
                ),

                # * Group Comparison  ------------------------------------------
                tabPanel(
                    "Group comparison", value = "group_comparison",
                    h4(strong("Group comparison")),
                    uiOutput("descGCUI"),
                    wellPanel(
                        fluidRow(
                            column(
                                3,
                                uiOutput("taxaListGC"),
                                checkboxInput(
                                    "useCommonAncestor",
                                    "Use common ancestor",
                                    value = TRUE,
                                    width = NULL
                                ) %>%
                                    bsplus::bs_embed_popover(
                                        title = "",
                                        content = paste(
                                            "All taxa that have the same common",
                                            "ancestor with the selected taxa above",
                                            "will be considered as the in-group"
                                        ),
                                        placement = "bottom",
                                        trigger = "hover"
                                    ),
                                h5(strong("Select in-group by supertaxon")),
                                actionButton("taxaGC", "Browse")
                            ),
                            column(
                                3,
                                uiOutput("listGenesGC"),
                                h5(strong(
                                    "Upload sequence(s) / In-group taxa"
                                )),
                                actionButton("uploadGC", "Upload")
                            ),
                            column(
                                2,
                                uiOutput("variableGC"),
                                selectInput(
                                    "compareType", "Compare using:",
                                    choices = c(
                                        "Statistical tests", "Median values"
                                    ),
                                    selected = "Statistical tests"
                                )
                            ),
                            column(
                                2,
                                sliderInput(
                                    "significance",
                                    label = "Significance level:",
                                    min = 0,
                                    max = 1,
                                    step = 0.01,
                                    value = 0.05,
                                    width = 300
                                ) %>%
                                    bsplus::bs_embed_popover(
                                        title   = "",
                                        content = "P-value cut-off of the statistic test, 
                                                OR cut-off of delta means between 2 groups",
                                        placement = "bottom",
                                        trigger = "hover"
                                    ),
                                checkboxInput(
                                    "addGCGenesCustomProfile",
                                    strong(em(
                                        "Add candidate gene(s) to Customized
                                        profile",
                                        style = "color:red"
                                    )),
                                    value = FALSE,
                                    width = NULL
                                ),
                                uiOutput("addGCCustomProfileCheck")
                            ),
                            column(
                                2,
                                actionButton("gcPlotConfig", "Plot config") %>%
                                    bsplus::bs_embed_popover(
                                        title   = "",
                                        content = "Change the appearance of the plots",
                                        placement = "bottom",
                                        trigger = "hover"
                                    ),
                                hr(),
                                actionButton(
                                    "doCompare", "COMPARE!", 
                                    class = "btn btn-danger"
                                ),
                                conditionalPanel(
                                    condition = 'input.rankSelect == "strain"',
                                    HTML('<div style="color: red; font-weight: bold;">
                                      Warning: "strain" is not a supported taxonomy rank
                                      for this function.<br>
                                      Please choose another rank.
                                    </div>')
                                ),
                                h5(),
                                actionButton(
                                    "updateGC",
                                    "Update plot",
                                    icon = icon("sync"),
                                    class = "btn btn-warning"
                                )
                            )
                        )
                    ),
                    groupComparisonUI("groupComparison")
                ),

                # * NCBI taxonomy data -----------------------------------------
                tabPanel(
                    "NCBI taxonomy data", value = "ncbi_tax_data",
                    uiOutput("descNcbiTaxDbUI"),
                    column(
                        3,
                        radioButtons(
                            inputId = "taxDB",
                            label = "Choose a task:",
                            choices = list(
                                "Update NCBI taxonomy DB" = "update",
                                "Reset NCBI taxonomy DB" = "reset",
                                "Export taxonomy DB files" = "export",
                                "Import taxonomy DB files" = "import"
                            )
                        )
                    ),
                    column(
                        9,
                        conditionalPanel(
                            condition = "input.taxDB=='update'",
                            h4(strong("Update NCBI taxonomy")),
                            actionButton(
                                "doUpdateNcbi",
                                "Do update",
                                icon = icon("wrench"),
                                class = "btn btn-warning"
                            ),
                            hr(),
                            verbatimTextOutput("updateNCBITaxStatus")
                        ),
                        conditionalPanel(
                            condition = "input.taxDB=='reset'",
                            h4(strong("Reset taxonomy data")),
                            uiOutput("taxResetWarning.ui"),
                            br(),
                            actionButton(
                                "doResetTax",
                                "Do reset",
                                icon = icon("wrench"),
                                class = "btn btn-warning"
                            ),
                            hr(),
                            verbatimTextOutput("resetTaxonomyDataStatus")
                        ),
                        conditionalPanel(
                            condition =
                                "input.taxDB=='export'",
                            h4(strong("Export current taxonomy files")),
                            uiOutput("taxExportWarning.ui"),
                            br(),
                            shinyFiles::shinyDirButton(
                                "taxDirOut",
                                "Select output directory" ,
                                title = paste(
                                    "Please select output directory"
                                ),
                                buttonType = "default", class = NULL
                            ),
                            br(),
                            uiOutput("taxDirOut.ui"),
                            br(),
                            actionButton(
                                "doExportTax",
                                "Do export",
                                icon = icon("file-export"),
                                class = "btn btn-warning",
                            ),
                            hr(),
                            verbatimTextOutput("exportTaxonomyDataStatus")
                        ),
                        conditionalPanel(
                            condition =
                                "input.taxDB=='import'",
                            h4(strong("Import your own taxonomy files")),
                            uiOutput("taxImportWarning.ui"),
                            br(),
                            shinyFiles::shinyDirButton(
                                "taxDir",
                                "Select input directory" ,
                                title = paste(
                                    "Please select directory that contains
                                    the taxonomy files"
                                ),
                                buttonType = "default", class = NULL
                            ),
                            br(),
                            uiOutput("taxDir.ui"),
                            br(),
                            actionButton(
                                "doImportTax",
                                "Do import",
                                icon = icon("file-import"),
                                class = "btn btn-warning"
                            ),
                            hr(),
                            verbatimTextOutput("importTaxonomyDataStatus")
                        )
                    )
                )
            ),

            # DATA DOWNLOAD TAB ================================================
            navbarMenu(
                "Export data",

                # * Export data ------------------------------------------------
                downloadFilteredMainUI("filteredMainDownload"),
                downloadFilteredCustomizedUI("filteredCustomizedDownload"),

                # * Export processed data --------------------------------------
                tabPanel(
                    "Processed data",
                    h4(strong("Download processed data")),
                    uiOutput("descDownloadProcessedDataUI"),
                    strong("Output dir:"),
                    br(), br(),
                    shinyFiles::shinyDirButton(
                        "procDataOutDir",
                        "Select output directory" ,
                        title = paste(
                            "Please select output directory"
                        ),
                        buttonType = "default", class = NULL
                    ),
                    br(),br(),
                    uiOutput("procDataOutDir.ui"),
                    br(),
                    actionButton(
                        "doDownloadProcData",
                        "Download",
                        icon = icon("file-export"),
                        class = "btn btn-warning"
                    ),
                    hr(),
                    verbatimTextOutput("downloadProcDataStatus")
                ),

                # * Export plot settings ---------------------------------------
                tabPanel(
                    "Export plot settings",
                    h4(strong("Export plot settings")),
                    uiOutput("descExportSettingUI"),
                    radioButtons(
                        inputId = "exportSetting",
                        label = "Export as:",
                        choices = list(
                            "a list" = "list",
                            "an Rscript" = "rscript"
                        )
                    ),
                    hr(),
                    strong("Output dir:"),
                    br(), br(),
                    shinyFiles::shinyDirButton(
                        "settingDir",
                        "Select output directory" ,
                        title = paste(
                            "Please select output directory"
                        ),
                        buttonType = "default", class = NULL
                    ),
                    br(), br(),
                    strong("File name:"),
                    uiOutput("settingFile.ui"),
                    uiOutput("settingDir.ui"),
                    br(),
                    actionButton(
                        "doExportSetting",
                        "Do export",
                        icon = icon("file-export"),
                        class = "btn btn-warning"
                    ),
                    hr(),
                    verbatimTextOutput("exportSettingStatus")
                )
            ),

            # HELP TAB =========================================================
            navbarMenu(
                "Help",
                tabPanel(
                    a(
                        "Wiki",
                        href = "https://github.com/BIONF/PhyloProfile/wiki",
                        target = "_blank"
                    )
                ),
                tabPanel(
                    a(
                        "About",
                        href = "https://BIONF.github.io/PhyloProfile/",
                        target = "_blank"
                    )
                )
            )
        ),

        # POINT INFO BOX =======================================================
        conditionalPanel(
            condition =
                "input.tabs=='Main profile' ||
                input.tabs=='Customized profile'",
            absolutePanel(
                bottom = 5, left = 30,
                fixed = TRUE,
                draggable = TRUE,
                h5("Point's info:"),
                verbatimTextOutput("pointInfo"),
                conditionalPanel(
                    condition = "output.pointInfoStatus == 0",
                    actionButton(
                        "detailedBtn",
                        "Detailed plot",
                        class = "btn btn-success"
                    ),
                    actionButton(
                        "doDomainPlotMain",
                        "Domain plot",
                        class = "btn btn-success"
                    ),
                    actionButton(
                        "highlightMain",
                        "Highlight",
                        class = "btn btn-success"
                    )
                ),
                style = "opacity: 0.80"
            )
        )
    )
)

# CREATE MANUAL COLOR SCHEME FOR A LIST

#' Create qualitative colours
#' @param n number of colors
#' @param light light colors TRUE or FALSE
#' @return list of n different colors
#' @source Modified based on https://gist.github.com/peterk87/6011397
#' @importFrom RColorBrewer brewer.pal
#' @examples
#' PhyloProfile:::qualitativeColours(5)

qualitativeColours <- function(n, light = FALSE) {
    # For more than 21 colours needed
    rich12equal <- c(
        "#000040", "#000093", "#0020E9", "#0076FF", "#00B8C2", "#04E466",
        "#49FB25", "#E7FD09", "#FEEA02", "#FFC200", "#FF8500", "#FF3300")
    # Qualitative colour schemes by Paul Tol
    ifelse (n >= 19 & n <= 21, return(grDevices::colorRampPalette(c(
        "#771155", "#AA4488", "#CC99BB", "#114477", "#4477AA", "#77AADD",
        "#117777", "#44AAAA", "#77CCCC", "#117744", "#44AA77", "#88CCAA",
        "#777711", "#AAAA44", "#DDDD77", "#774411", "#AA7744", "#DDAA77",
        "#771122", "#AA4455", "#DD7788"))(n)),
        ifelse (n >= 16 & n <= 18, return(grDevices::colorRampPalette(c(
            "#771155", "#AA4488", "#CC99BB", "#114477", "#4477AA", "#77AADD",
            "#117777", "#44AAAA", "#77CCCC", "#777711", "#AAAA44", "#DDDD77",
            "#774411", "#AA7744", "#DDAA77", "#771122", "#AA4455",
            "#DD7788"))(n)),
            ifelse (n == 15, return(grDevices::colorRampPalette(c(
                "#771155", "#AA4488", "#CC99BB", "#114477", "#4477AA",
                "#77AADD", "#117777", "#44AAAA", "#77CCCC", "#777711",
                "#AAAA44", "#DDDD77", "#774411", "#AA7744", "#DDAA77",
                "#771122", "#AA4455", "#DD7788"))(n)),
                ifelse(n > 12 & n <= 14, return(grDevices::colorRampPalette(c(
                    "#882E72", "#B178A6", "#D6C1DE", "#1965B0", "#5289C7",
                    "#7BAFDE", "#4EB265", "#90C987", "#CAE0AB", "#F7EE55",
                    "#F6C141", "#F1932D", "#E8601C", "#DC050C"))(n)),
                    ifelse(n > 9 & n <= 12,
                        ifelse(
                            light,
                            return(RColorBrewer::brewer.pal(
                                n = n, name = 'Set3')),
                            return(RColorBrewer::brewer.pal(
                                n = n, name = 'Paired'))),
                        ifelse(n <= 9,
                            ifelse(
                                light,
                                return(RColorBrewer::brewer.pal(
                                    n = n, name = 'Pastel1')),
                                return(RColorBrewer::brewer.pal(
                                    n = n, name = 'Set1'))),
                            return(grDevices::colorRampPalette(rich12equal)(n))
                        )
                    )
                )
            )
        )
    )
}

#' Get color for a list of items
#' @return list of colors for each element (same elements will have the same
#' color)
#' @param x input list
#' @export
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @seealso \code{\link{qualitativeColours}}
#' @examples
#' items <- c("a", "b", "c")
#' getQualColForVector(items)

getQualColForVector <- function(x = NULL) {
    if (is.null(x)) stop("Input list is NULL!")
    types <- unique(x)
    types <- types[!is.na(types)]
    typeColors <- qualitativeColours(length(types))
    colorsTypes <- as.vector(x)
    countTypes <- countColors <- 1
    while (countTypes <= length(types)) {
        if (countColors > length(typeColors)) countColors <- 1
        colorsTypes[colorsTypes == types[countTypes]] <- typeColors[countColors]
        countColors <- countColors + 1
        countTypes <- countTypes + 1
    }
    return(unlist(colorsTypes))
}


#' Check if a color pallete has enough colors for a list of items
#' @export
#' @param items vector contains list of items
#' @param pallete name of color palette
#' @return TRUE if color pallete has enough colors, otherwise FALSE
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom RColorBrewer brewer.pal.info
#' @examples
#' myItems <- rep("a",3)
#' checkColorPalette(myItems, "Set1")

checkColorPalette <- function(items, pallete = "Paired") {
    if (is.null(items)) stop("No item given!")
    colorDf <- data.frame(RColorBrewer::brewer.pal.info)
    if (!(pallete %in% row.names(colorDf)))
        stop("Given color pallete not found")
    colorDf$name <- row.names(colorDf)
    if (length(items) < colorDf$maxcolors[colorDf$name == pallete])
        return(TRUE)
    return(FALSE)
}

#' NCBI Taxonomy reduced data set
#'
#' @docType data
#' @name taxonNamesReduced
#' @description A list of NCBI taxonomy info (including taxon IDs, taxon names,
#' their systematic taxonomy rank and IDs of their next rank - parent IDs) for
#' 95 taxa in two experimental sets included in PhyloProfilData package.
#' @format Dataframe
#' @return A data frame with 4 columns:
#' \itemize{
#'     \item ncbiID e.g. "10090"
#'     \item fullName e.g. "Mus musculus"
#'     \item rank e.g. "species"
#'     \item parentID e.g. "862507"
#' }
#' @usage data(taxonNamesReduced)
NULL

#' NCBI ID list for experimental data sets
#'
#' @docType data
#' @name idList
#' @description Data frame, in which each row contains the complete taxonomy
#' ranks from the lowest systematic level (strain/species) upto the taxonomy
#' root and the corresponding IDs for one taxon in the experimental data sets.
#' @format Dataframe
#' @return A data frame with up to 41 columns and 95 rows corresponding to 95
#'     taxa in the 2 experimental data sets
#' @usage data(idList)
NULL

#' NCBI rank list for experimental data sets
#'
#' @docType data
#' @name rankList
#' @description Data frame, in which each row contains the complete taxonomy
#' ranks from the lowest systematic level (strain/species) upto the taxonomy
#' root for one taxon in the experimental data sets.
#' @format Dataframe
#' @return A data frame with up to 41 columns and 95 rows corresponding to 95
#'     taxa in the 2 experimental data sets
#' @usage data(rankList)
NULL

#' Taxonomy matrix for experimental data sets
#'
#' @docType data
#' @name taxonomyMatrix
#' @description Data frame containing the fully aligned taxonomy IDs of 95 taxa
#' in the experimental data sets. By talking into account both the defined
#' ranks (e.g. strain, This data is used for clustering and then creating a
#' taxon tree. It is used also for cross-linking between different taxonomy
#' ranks within a taxon.
#' @format Dataframe
#' @return A data frame with up to 149 columns and 95 rows corresponding to 95
#'     taxa in the 2 experimental data sets
#' @usage data(taxonomyMatrix)
NULL

#' An example of a taxonomy tree in newick format
#'
#' @docType data
#' @name ppTree
#' @format Dataframe
#' @return A data frame with only one entry
#' \itemize{
#'     \item V1 tree in newick format
#' }
#' @usage data(ppTree)
NULL

#' An example of a taxonomy matrix
#'
#' @docType data
#' @name ppTaxonomyMatrix
#' @format Dataframe
#' @return A data frame with 10 rows and 162 variables:
#' \itemize{
#'     \item abbrName  e.g. "ncbi10090"
#'     \item ncbiID e.g. "10090"
#'     \item fullName e.g. "Mus musculus"
#'     \item strain e.g. "10090"
#'     ...
#' }
#' @usage data(ppTaxonomyMatrix)
NULL

#' An example of a raw long input file
#'
#' @docType data
#' @name mainLongRaw
#' @format Dataframe
#' @return A data frame with 168 rows and 5 variables:
#' \itemize{
#'     \item geneID  Seed or ortholog group ID, e.g. "100136at6656"
#'     \item ncbiID  Taxon ID, e.g. "ncbi36329"
#'     \item orthoID Ortholog ID, e.g. "100136at6656|PLAF7@36329@1|Q8ILT8|1"
#'     \item FAS_F First additional variable
#'     \item FAS_B Second additional variable
#' }
#' @usage data(mainLongRaw)
NULL

#' An example of a raw long input file together with the taxonomy info
#'
#' @docType data
#' @name profileWithTaxonomy
#' @format Dataframe
#' @return A data frame with 20 rows and 12 variables:
#' \itemize{
#'     \item geneID Seed or ortholog group ID, e.g. "OG_1017"
#'     \item ncbiID Taxon ID, e.g. "ncbi176299"
#'     \item orthoID Ortholog ID, e.g. "A.fabrum@176299@1582"
#'     \item var1 First additional variable
#'     \item var2 Second additional variable
#'     \item paralog Number of co-orthologs in the current taxon
#'     \item abbrName e.g. "ncbi176299"
#'     \item taxonID Taxon ID, e.g. "176299"
#'     \item fullName Full taxon name, e.g. "Agrobacterium fabrum str. C58"
#'     \item supertaxonID Supertaxon ID (only different than ncbiID in case
#'     working with higher taxonomy rank than input's)
#'     \item supertaxon Name of the corresponding supertaxon
#'     \item rank Rank of the supertaxon
#' }
#' @usage data(profileWithTaxonomy)
NULL

#' An example of a fully processed phylogenetic profile
#'
#' @docType data
#' @name fullProcessedProfile
#' @format Dataframe
#' @return A data frame with 168 rows and 14 variables:
#' \itemize{
#'     \item supertaxon Supertaxon name together with its ordered index, e.g.
#'     "1001_Mammalia"
#'     \item ncbiID Taxon ID, e.g. "ncbi10116"
#'     \item geneID Seed or ortholog group ID, e.g. "100136at6656"
#'     \item orthoID Ortholog ID, e.g. "100136at6656|HUMAN@9606@1|Q9UNQ2|1"
#'     \item var1 First additional variable
#'     \item var2 Second additional variable
#'     \item paralog Number of co-orthologs in the current taxon
#'     \item abbrName NCBI ID of the ortholog, e.g. "ncbi9606"
#'     \item taxonID Taxon ID of the ortholog, in this case: "0"
#'     \item fullName Full taxon name of the ortholog, e.g. "Homo sapiens"
#'     \item supertaxonID Supertaxon ID (only different than ncbiID in case
#'     working with higher taxonomy rank than input's). e.g. "40674"
#'     \item rank Rank of the supertaxon, e.g. "class"
#'     \item category "cat
#'     \item numberSpec Total number of species in each supertaxon
#' }
#' @usage data(fullProcessedProfile)
NULL

#' An example of a filtered phylogenetic profile
#'
#' @docType data
#' @name filteredProfile
#' @format Dataframe
#' @return A data frame with 168 rows and 20 variables:
#' \itemize{
#'     \item geneID Seed or ortholog group ID, e.g. "100136at6656"
#'     \item supertaxon Supertaxon name together with its ordered index, e.g.
#'     "1001_Mammalia"
#'     \item ncbiID Taxon ID, e.g. "ncbi10116"
#'     \item orthoID Ortholog ID, e.g. "100136at6656|HUMAN@9606@1|Q9UNQ2|1"
#'     \item var1 First additional variable
#'     \item var2 Second additional variable
#'     \item paralog Number of co-orthologs in the current taxon
#'     \item abbrName NCBI ID of the ortholog, e.g. "ncbi9606"
#'     \item taxonID Taxon ID of the ortholog, in this case: "0"
#'     \item fullName Full taxon name of the ortholog, e.g. "Homo sapiens"
#'     \item supertaxonID Supertaxon ID (only different than ncbiID in case
#'     working with higher taxonomy rank than input's). e.g. "40674"
#'     \item rank Rank of the supertaxon, e.g. "class"
#'     \item category "cat
#'     \item numberSpec Total number of species in each supertaxon
#'     \item taxonMod Name of supersupertaxon w/o its index, e.g. "Mammalia"
#'     \item presSpec Percentage of taxa having orthologs in each supertaxon
#'     \item presentTaxa Number of taxa that have ortho in each supertaxon
#'     \item totalTaxa Total number of taxa in each supertaxon
#'     \item mVar1 Value of the 1. variable after grouping into supertaxon
#'     \item mVar2 Value of the 2. variable after grouping into supertaxon
#' }
#' @usage data(filteredProfile)
NULL

#' An example of a final processed & filtered phylogenetic profile
#'
#' @docType data
#' @name finalProcessedProfile
#' @format Dataframe
#' @return A data frame with 88 rows and 11 variables:
#' \itemize{
#'     \item geneID Seed or ortholog group ID, e.g. "100136at6656"
#'     \item supertaxon Supertaxon name together with its ordered index, e.g.
#'     "1001_Mammalia"
#'     \item supertaxonID Supertaxon ID (only different than ncbiID in case
#'     working with higher taxonomy rank than input's). e.g. "40674"
#'     \item var1 First additional variable
#'     \item presSpec The percentage of species presenting in each supertaxon
#'     \item category "cat"
#'     \item orthoID Ortholog ID, e.g. "100136at6656|RAT@10116@1|G3V7R8|1"
#'     \item var2 Second additional variable
#'     \item paralog Number of co-orthologs in the current taxon
#'     \item presentTaxa Number of taxa that have ortho in each supertaxon
#'     \item totalTaxa Total number of taxa in each supertaxon
#' }
#' @usage data(finalProcessedProfile)
NULL

#' Create protein's domain architecure plot
#' @description Create architecture plot for both seed and orthologous protein.
#' If domains of ortholog are missing, only architecture of seed protein will
#' be plotted. NOTE: seed protein ID is the one being shown in the profile plot,
#' which normally is also the orthologous group ID.
#' @export
#' @usage createArchiPlot(info, domainDf, labelArchiSize, titleArchiSize,
#'     legendArchiSize, showScore, showWeight, namePosition, firstDist, 
#'     nameType, nameSize, segmentSize, nameColor, labelPos, colorType, 
#'     ignoreInstanceNo, currentNCBIinfo, featureClassSort, featureClassOrder, 
#'     colorPalette, resolveOverlap, font)
#' @param info A list contains seed and ortholog's IDs
#' @param domainDf Dataframe contains domain info for the seed and ortholog.
#' This including the seed ID, orthologs IDs, sequence lengths, feature names,
#' start and end positions, feature weights (optional) and the status to
#' determine if that feature is important for comparison the architecture
#' between 2 proteins* (e.g. seed protein vs ortholog) (optional).
#' @param labelArchiSize Lable size (in px). Default = 12.
#' @param titleArchiSize Title size (in px). Default = 12.
#' @param legendArchiSize Title size (in px). Default = 12.
#' @param showScore Show/hide E-values and Bit-scores. Default = NULL (hide)
#' @param showWeight Show/hide feature weights. Default = NULL (hide)
#' @param namePosition list of positions for domain names, choose from "plot",
#' "legend" or "axis". Default: "plot"
#' @param firstDist Distance of the first domain to plot title. Default = 0.5
#' @param nameType Type of domain names, either "Texts" or "Labels" (default)
#' @param nameSize Size of domain names. Default = 3
#' @param segmentSize Height of domain segment. Default = 5
#' @param nameColor Color of domain names (for Texts only). Default = "black"
#' @param labelPos Position of domain names (for Labels only). Choose from
#' @param colorType Choose to color "all", "shared", "unique" features or color
#' by "Feature type". Default = "all"
#' @param ignoreInstanceNo Ignore number of feature instances while identifying
#' shared or unique features. Default = FALSE
#' @param currentNCBIinfo Dataframe of the pre-processed NCBI taxonomy
#' data. Default = NULL (will be automatically retrieved from PhyloProfile app)
#' @param featureClassSort Choose to sort features. Default = "Yes"
#' @param featureClassOrder vector of ordered feature classes
#' @param colorPalette Choose between "Paired", "Set1", "Set2", "Set3",
#' "Accent", "Dark2" for the color pallete
#' @param resolveOverlap Choose to merge non-overlapped features of a feature
#' type into one line. Default = "Yes"
#' @param font font of text. Default = Arial"
#' @return A domain plot as arrangeGrob object. Use grid::grid.draw(plot) to
#' render.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom stringr str_count
#' @seealso \code{\link{singleDomainPlotting}},\code{\link{pairDomainPlotting}},
#' \code{\link{sortDomains}}, \code{\link{parseDomainInput}}
#' @examples
#' seedID <- "101621at6656"
#' orthoID <- "101621at6656|AGRPL@224129@0|224129_0:001955|1"
#' info <- c(seedID, orthoID)
#' domainFile <- system.file(
#'     "extdata", "domainFiles/101621at6656.domains",
#'     package = "PhyloProfile", mustWork = TRUE
#' )
#' domainDf <- parseDomainInput(seedID, domainFile, "file")
#' domainDf$feature_id_mod <- domainDf$feature_id
#' domainDf$feature_id_mod <- gsub("SINGLE", "LCR", domainDf$feature_id_mod)
#' domainDf$feature_id_mod[domainDf$feature_type == "coils"] <- "Coils"
#' domainDf$feature_id_mod[domainDf$feature_type == "seg"] <- "LCR"
#' domainDf$feature_id_mod[domainDf$feature_type == "tmhmm"] <- "TM"
#' plot <- createArchiPlot(info, domainDf, font = "sans")
#' grid::grid.draw(plot)

createArchiPlot <- function(
    info = NULL, domainDf = NULL, labelArchiSize = 12, titleArchiSize = 12,
    legendArchiSize = 12, showScore = NULL, showWeight = NULL, 
    namePosition = "plot", firstDist = 0.5, nameType = "Labels", 
    nameSize = 3, segmentSize = 5,nameColor = "#000000", labelPos = "Above", 
    colorType = "Unique", ignoreInstanceNo = FALSE, currentNCBIinfo = NULL,
    featureClassSort = "Yes", featureClassOrder = NULL,
    colorPalette = "Paired", resolveOverlap = "Yes", font = "Arial"
){
    if (is.null(info) | is.null(domainDf)) return(ggplot() + theme_void())
    group <- as.character(info[1])
    ortho <- as.character(info[2])
    # get sub dataframe based on selected groupID and orthoID
    group <- gsub("\\|", ":", group)
    ortho <- gsub("\\|", ":", ortho)
    grepID <- paste(group, "#", ortho, sep = "")
    subdomainDf <- domainDf
    subdomainDf$feature <- as.character(subdomainDf$feature)
    subdomainDf <- subdomainDf[!duplicated(subdomainDf), ]
    orthoID <- NULL

    if (nrow(subdomainDf) < 1) return(paste0("No domain info available!"))
    else {
        # get minStart and maxEnd
        minStart <- min(subdomainDf$start)
        maxEnd <- max(subdomainDf$end)
        if ("length" %in% colnames(subdomainDf))
            maxEnd <- max(c(subdomainDf$end, subdomainDf$length))
        # ortho & seed domains df
        orthoDf <- subdomainDf[subdomainDf$orthoID == ortho,]
        seedDf <- subdomainDf[subdomainDf$orthoID != ortho,]
        if (nrow(seedDf) == 0) seedDf <- orthoDf
        seed <- as.character(seedDf$orthoID[1])
        if (nrow(seedDf) == 0) return(paste0("No domain info available!"))

        # simplify seed/ortho seq IDs if they are in bionf format
        if (!is.null(currentNCBIinfo)) {
            if (
                stringr::str_count(seed, ":") >= 2 &
                stringr::str_count(seed, "@") >= 2
            ) {
                seedTmp <- strsplit(as.character(seed),':', fixed = TRUE)[[1]]
                seedSpec <-
                    strsplit(as.character(seedTmp[2]),'@', fixed = TRUE)[[1]][2]
                seed <- paste0(
                    id2name(seedSpec, currentNCBIinfo)[,2], " - ", seedTmp[3]
                )
                if (ortho != seed) {
                    orthoTmp <- strsplit(
                        as.character(ortho),':', fixed = TRUE)[[1]]
                    orthoSpec <-
                        strsplit(
                            as.character(orthoTmp[2]),'@',fixed = TRUE)[[1]][2]
                    ortho <- paste0(
                        id2name(orthoSpec, currentNCBIinfo)[,2],
                        " - ", orthoTmp[3]
                    )
                }
            }
        }

        # add feature colors
        featureColorDf <- addFeatureColors(
            seedDf, orthoDf, colorType, colorPalette, ignoreInstanceNo
        )
        seedDf <- featureColorDf[[1]]
        orthoDf <- featureColorDf[[2]]

        # resolve (non)overlapped features
        if (resolveOverlap == "Yes") {
            seedDf <- resolveOverlapFeatures(seedDf)
            orthoDf <- resolveOverlapFeatures(orthoDf)
        } else {
            seedDf$featureOri <- seedDf$feature
            seedDf$featureOri <- as.character(seedDf$featureOri)
            orthoDf$featureOri <- orthoDf$feature
            orthoDf$featureOri <- as.character(orthoDf$featureOri)
        }

        if (nrow(orthoDf) > 0) {
            if (all.equal(seedDf, orthoDf)[1] == TRUE) featureClassSort <- "No"
            # sort features
            if (featureClassSort == "Yes") {
                # change order of one df's features based on order of other df's
                if (length(orthoDf$feature) < length(seedDf$feature)) {
                    orderedOrthoDf <- orthoDf[order(orthoDf$feature), ]
                    orderedSeedDf <- sortDomains(orderedOrthoDf, seedDf)
                    orderedOrthoDf <- sortDomains(orderedSeedDf, orderedOrthoDf)
                } else {
                    orderedSeedDf <- seedDf[order(seedDf$feature), ]
                    orderedOrthoDf <- sortDomains(orderedSeedDf, orthoDf)
                    orderedSeedDf <- sortDomains(orderedOrthoDf, orderedSeedDf)
                }
            } else {
                # change order based on list of feature types
                orderedSeedDf <- sortDomainsByList(seedDf, featureClassOrder)
                orderedOrthoDf <- sortDomainsByList(orthoDf, featureClassOrder)
            }
            # plotting
            g <- pairDomainPlotting(
                seed, ortho, orderedSeedDf, orderedOrthoDf, minStart, maxEnd,
                labelArchiSize, titleArchiSize, legendArchiSize, showScore, 
                showWeight, namePosition, firstDist, nameType, nameSize, 
                segmentSize, nameColor, labelPos, colorPalette, font)
        } else {
            orderedSeedDf <- sortDomainsByList(seedDf, featureClassOrder)
            # plotting
            g <- pairDomainPlotting(
                seed, seed, orderedSeedDf, orderedSeedDf, minStart, maxEnd,
                labelArchiSize, titleArchiSize, legendArchiSize, showScore, 
                showWeight, namePosition, firstDist, nameType, nameSize, 
                segmentSize, nameColor, labelPos, colorPalette, font)
        }
        return(g)
    }
}


#' Create architecure plot for a single protein
#' @usage singleDomainPlotting(df, geneID, sep, labelSize, titleSize, minStart,
#'     maxEnd, colorPalette, showScore, showWeight, namePosition, firstDist,
#'     nameType, nameSize, segmentSize, nameColor, labelPos, font)
#' @param df Domain dataframe for ploting containing the seed ID, ortholog ID,
#' ortholog sequence length, feature names, start and end positions,
#' feature weights (optional) and the status to determine if that feature is
#' important for comparison the architecture between 2 proteins* (e.g. seed
#' protein vs ortholog) (optional)
#' @param geneID ID of seed or orthologous protein
#' @param sep Separate indicator for title. Default = "|"
#' @param labelSize Lable size. Default = 12
#' @param titleSize Title size. Default = 12
#' @param minStart The smallest start position of all domains
#' @param maxEnd The highest stop position of all domains
#' @param colorPalette Color pallete. Default = Paired"
#' @param showScore Show/hide E-values and Bit-scores. Default = NULL (hide)
#' @param showWeight Show/hide feature weights. Default = NULL (hide)
#' @param namePosition List of positions for domain names, choose from "plot",
#' "legend" or "axis". Default: "plot"
#' @param firstDist Distance of the first domain to plot title. Default = 0.5
#' @param nameType Type of domain names, either "Texts" or "Labels" (default)
#' @param nameSize Size of domain names. Default = 3
#' @param segmentSize Height of domain segment. Default = 5
#' @param nameColor Color of domain names (for Texts only). Default = "black"
#' @param labelPos Position of domain names (for Labels only). Choose from
#' "Above" (default), "Below" or "Inside" the domain bar
#' @param font font of text. Default = Arial"
#' @return Domain plot of a single protein as a ggplot object.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom RColorBrewer brewer.pal
#' @importFrom stringr str_wrap
#' @seealso \code{\link{parseDomainInput}}
#' @import ggplot2
#' @examples
#' seed <- "101621at6656"
#' ortho <- "101621at6656|AGRPL@224129@0|224129_0:001955|1"
#' ortho <- gsub("\\|", ":", ortho)
#' grepID <- paste(seed, "#", ortho, sep = "")
#' domainFile <- system.file(
#'     "extdata", "domainFiles/101621at6656.domains",
#'     package = "PhyloProfile", mustWork = TRUE
#' )
#' domainDf <- parseDomainInput(seed, domainFile, "file")
#' domainDf$feature_id_mod <- domainDf$feature_id
#' subdomainDf <- domainDf[grep(grepID, domainDf$seedID), ]
#' subdomainDf$feature <- as.character(subdomainDf$feature)
#' orthoDf <- subdomainDf[subdomainDf$orthoID == ortho,]
#' seedDf <- subdomainDf[subdomainDf$orthoID != ortho,]
#' minStart <- min(subdomainDf$start)
#' maxEnd <- max(c(subdomainDf$end, subdomainDf$length))
#' # resolve overlapping domains
#' seedDf <- PhyloProfile:::resolveOverlapFeatures(seedDf)
#' orthoDf <- PhyloProfile:::resolveOverlapFeatures(orthoDf)
#' # add feature colors
#' featureColorDf <- PhyloProfile:::addFeatureColors(seedDf, orthoDf)
#' seedDf <- featureColorDf[[1]]
#' orthoDf <- featureColorDf[[2]]
#' # do plot
#' g <- PhyloProfile:::singleDomainPlotting(
#'    seedDf, seed, minStart = minStart, maxEnd = maxEnd, font = "sans"
#' )
#' grid::grid.draw(g)

singleDomainPlotting <- function(
        df = NULL, geneID = "GeneID", sep = "|", labelSize = 12, titleSize = 12,
        minStart = NULL, maxEnd = NULL, colorPalette = "Set2",
        showScore = NULL, showWeight = NULL, namePosition = "plot",
        firstDist = 0.5, nameType = "Labels", nameSize = 3, segmentSize = 5,
        nameColor = "#000000", labelPos = "Above", font = "Arial"
){
    feature <- feature_id_mod <- end <- start <- featureOri <- NULL
    evalue <- bitscore <- weight <- NULL

    # parse parameters
    if (is.null(df)) return(ggplot() + theme_void())
    if (is.null(minStart)) minStart <- min(df$start)
    if (is.null(maxEnd)) maxEnd <- max(df$end)
    if ("color" %in% colnames(df)) {
        colorScheme <- structure(
            df$color, .Names = df$featureOri
        )
    } else {
        colorScheme <- structure(
            head(
                suppressWarnings(
                    RColorBrewer::brewer.pal(
                        nlevels(as.factor(df$feature)), colorPalette
                    )
                ),
                levels(as.factor(df$feature))
            ),
            .Names = levels(as.factor(df$feature)))
    }
    # initiate ggplot object
    gg <- ggplot(df, aes(y = feature, x = end))

    # draw lines for representing sequence length
    if ("length" %in% colnames(df)) {
        gg <- gg + geom_segment(
            data = df, linewidth = 0.6, color = "#b2b2b2", alpha = 0.5,
            aes(x = 0, xend = length, y = feature, yend = feature))
    }

    # draw features
    gg <- gg + geom_segment(
        data = df, aes(
            x = start, xend = end, y = feature, yend = feature,
            color = as.factor(featureOri)
        ), linewidth = segmentSize, alpha = 1.0) +
        scale_color_manual(values = colorScheme)

    # add feature names
    if ("plot" %in% namePosition) {
        if (nameType == "Labels") {
            if (labelPos == "Above") {
                gg <- gg + geom_label(
                    aes(
                        label = stringr::str_wrap(feature_id_mod),
                        x = (start+end)/2
                    ),
                    color = "black", vjust = -0.5, size = nameSize, family=font
                )
            } else if (labelPos == "Below") {
                gg <- gg + geom_label(
                    aes(
                        label = stringr::str_wrap(feature_id_mod),
                        x = (start+end)/2
                    ),
                    color = "black", vjust = 1.5, size = nameSize, family = font
                )
            } else {
                gg <- gg + geom_label(
                    aes(
                        label = stringr::str_wrap(feature_id_mod),
                        x = (start+end)/2
                    ),
                    color = "black", size = nameSize, family = font
                )
            }
        } else {
            gg <- gg + geom_text(
                aes(
                    label = stringr::str_wrap(feature_id_mod),
                    x = (start+end)/2
                ),
                color = nameColor, check_overlap = TRUE, size = nameSize,
                family = font
            )
        }
    }
    # add scores if selected
    maxVjust <- 2
    if ("Bit-score" %in% showScore & "E-value" %in% showScore) {
        gg <- gg + geom_text(
            aes(
                label = ifelse(
                    evalue == "NA" & bitscore == "NA", "", stringr::str_wrap(
                        paste0("E: ", evalue, "; B: ", bitscore)
                    )
                ), x = (start+end)/2
            ), color = "black", vjust = maxVjust, check_overlap = TRUE
        )
    } else if ("Bit-score" %in% showScore) {
        gg <- gg + geom_text(
            aes(
                label = ifelse(bitscore == "NA", "", stringr::str_wrap(
                    paste0("B: ", bitscore)
                )), x = (start+end)/2
            ), color = "black", vjust = maxVjust, check_overlap = TRUE
        )
    } else if ("E-value" %in% showScore) {
        gg <- gg + geom_text(
            aes(
                label = ifelse(evalue == "NA", "", stringr::str_wrap(
                    paste0("E: ", evalue)
                )), x = (start+end)/2
            ), color = "black", vjust = maxVjust, check_overlap = TRUE
        )
    }
    # add weight if selected
    if ("Weight" %in% showWeight) {
        if ("E-value" %in% showScore |  "Bit-score" %in% showScore) {
            maxVjust <- 3.5
        }
        gg <- gg + geom_text(
            aes(
                label = ifelse(weight == "NA", "", stringr::str_wrap(
                    paste0("W: ", weight)
                )), x = (start+end)/2
            ), color = "black", vjust = maxVjust, check_overlap = TRUE
        )
    }
    # theme format
    gg <- gg + labs(
        title = paste0(gsub(":", sep, geneID)), color = "Feature"
    )
    gg <- gg + theme(
        axis.line = element_blank(), axis.ticks.y = element_blank(),
        panel.grid.minor.x=element_blank(), panel.grid.major.x=element_blank(),
        panel.background = element_blank()
    )
    gg <- gg + theme(
        plot.title = element_text(face = "bold", size = titleSize, hjust = 0.5)
    )
    gg <- gg + theme(
        axis.title = element_blank(),
        axis.text.x = element_text(size = labelSize))
    gg <- gg + theme(text = element_text(family = font))
    # add feature names on the axis (if required)
    if ("axis" %in% namePosition) {
        if ("plot" %in% namePosition | "legend" %in% namePosition) {
            gg <- gg + scale_y_discrete(
                expand = c(0.075, 0), breaks = df$feature,
                labels = df$feature_type)
        } else {
            gg <- gg + scale_y_discrete(
                expand = c(0.075, 0), breaks = df$feature, labels = df$feature)
        }
        gg <- gg + theme(axis.text.y = element_text(size = labelSize))
    } else {
        gg <- gg + theme(axis.text.y = element_blank())
    }
    # add legend (if required)
    if ("legend" %in% namePosition) {
        gg <- gg + theme(legend.position = "bottom")
    } else {
        gg <- gg + theme(legend.position = "none")
    }
    # add space on the top of the plot (for feature name)
    gg <- gg + coord_cartesian(
        clip = "off", ylim = c(1, nlevels(as.factor(df$feature)) + firstDist)
    )
    # scale x-axis to the length of the longest protein
    gg <- gg + xlim(0, maxEnd)
    return(gg)
}


#' Create architecure plot for a pair of seed and ortholog protein
#' @usage pairDomainPlotting(seed, ortho, seedDf, orthoDf, minStart, maxEnd,
#'     labelSize, titleSize, legendSize, showScore, showWeight, namePosition, 
#'     firstDist, nameType, nameSize, segmentSize, nameColor, labelPos, 
#'     colorPalette, font)
#' @param seed Seed ID
#' @param ortho Ortho ID
#' @param seedDf domain dataframe for seed domains containing the seed ID,
#' ortholog ID, sequence length, feature names, start and end positions,
#' feature weights (optional) and the status to determine if that feature is
#' important for comparison the architecture between 2 proteins* (e.g. seed
#' protein vs ortholog) (optional)
#' @param orthoDf domain dataframe for ortholog domains (same format as seedDf)
#' @param minStart the smallest start position of all domains
#' @param maxEnd the highest stop position of all domains
#' @param labelSize lable size. Default = 12
#' @param titleSize title size. Default = 12
#' @param legendSize legend size. Default = 12
#' @param showScore show/hide E-values and Bit-scores. Default = NULL (hide)
#' @param showWeight Show/hide feature weights. Default = NULL (hide)
#' @param namePosition list of positions for domain names, choose from "plot",
#' "legend" or "axis". Default: "plot"
#' @param firstDist distance of the first domain to plot title. Default = 0.5
#' @param nameType type of domain names, either "Texts" or "Labels" (default)
#' @param nameSize Size of domain names. Default = 3
#' @param segmentSize Height of domain segment. Default = 5
#' @param nameColor color of domain names (for Texts only). Default = "black"
#' @param labelPos position of domain names (for Labels only). Choose from
#' "Above" (default), "Below" or "Inside" the domain bar
#' @param colorPalette color pallete. Default = Paired"
#' @param font font of text. Default = Arial"
#' @return Domain plot of a pair proteins as a arrangeGrob object.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom gridExtra grid.arrange
#' @seealso \code{\link{singleDomainPlotting}}, \code{\link{sortDomains}},
#' \code{\link{parseDomainInput}}
#' @examples
#' seed <- "101621at6656"
#' ortho <- "101621at6656|AGRPL@224129@0|224129_0:001955|1"
#' ortho <- gsub("\\|", ":", ortho)
#' grepID <- paste(seed, "#", ortho, sep = "")
#' domainFile <- system.file(
#'     "extdata", "domainFiles/101621at6656.domains",
#'     package = "PhyloProfile", mustWork = TRUE
#' )
#' domainDf <- parseDomainInput(seed, domainFile, "file")
#' domainDf$feature_id_mod <- domainDf$feature_id
#' subdomainDf <- domainDf[grep(grepID, domainDf$seedID), ]
#' subdomainDf$feature <- as.character(subdomainDf$feature)
#' orthoDf <- subdomainDf[subdomainDf$orthoID == ortho,]
#' seedDf <- subdomainDf[subdomainDf$orthoID != ortho,]
#' minStart <- min(subdomainDf$start)
#' maxEnd <- max(c(subdomainDf$end, subdomainDf$length))
#' # resolve overlapping domains
#' seedDf <- PhyloProfile:::resolveOverlapFeatures(seedDf)
#' orthoDf <- PhyloProfile:::resolveOverlapFeatures(orthoDf)
#' # add feature colors
#' featureColorDf <- PhyloProfile:::addFeatureColors(seedDf, orthoDf)
#' seedDf <- featureColorDf[[1]]
#' orthoDf <- featureColorDf[[2]]
#' # do plot
#' g <- PhyloProfile:::pairDomainPlotting(
#'    seed,ortho,seedDf,orthoDf,minStart,maxEnd, font = "sans"
#' )
#' grid::grid.draw(g)

pairDomainPlotting <- function(
        seed = NULL, ortho = NULL, seedDf = NULL, orthoDf = NULL,
        minStart = 0, maxEnd = 999, labelSize = 12, titleSize = 12,
        legendSize = 12, showScore = NULL, showWeight = NULL, 
        namePosition = "plot", firstDist = 0.5, nameType = "Labels", 
        nameSize = 3, segmentSize = 5, nameColor = "#000000",  
        labelPos = "Above", colorPalette = "Paired", font = "Arial"
) {
    if(is.null(seed) | is.null(ortho) | is.null(seedDf) | is.null(orthoDf))
        stop("Seed/Ortho ID or domain dataframe is NULL!")

    sep <- "|"
    plotSeed <- singleDomainPlotting(
        seedDf, seed, sep, labelSize, titleSize, minStart, maxEnd, colorPalette,
        showScore, showWeight, namePosition, firstDist, nameType, nameSize,
        segmentSize, nameColor, labelPos, font
    )
    if (ortho == seed) {
        g <- plotSeed
    } else {
        plotOrtho <- singleDomainPlotting(
            orthoDf, ortho, sep, labelSize, titleSize, minStart, maxEnd,
            colorPalette, showScore, showWeight, namePosition, firstDist,
            nameType, nameSize, segmentSize, nameColor, labelPos, font
        )
        if ("legend" %in% namePosition) {
            g <- joinPlotMergeLegends(
                seedDf, orthoDf, plotSeed, plotOrtho, position = "bottom", font,
                legendSize
            )
        } else {
            seedHeight <- length(levels(as.factor(seedDf$feature)))
            orthoHeight <- length(levels(as.factor(orthoDf$feature)))
            g <- gridExtra::grid.arrange(
                plotSeed, plotOrtho, nrow = 2,
                heights = c(seedHeight, orthoHeight)
            )
        }
    }
    return(g)
}

#' Sort one domain dataframe based on the other domain dataframe
#' @description Sort domain dataframe of one protein (either seed or ortholog)
#' based on the dataframe of the its paired protein, in order to bring the
#' common domain feature in the same order which make it easy for comparing.
#' @param seedDf data of seed protein
#' @param orthoDf data of ortholog protein
#' @return Dataframe contains sorted domain list.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @examples
#' # get domain data
#' seedID <- "101621at6656"
#' domainFile <- system.file(
#'     "extdata", "domainFiles/101621at6656.domains",
#'     package = "PhyloProfile", mustWork = TRUE
#' )
#' domainDf <- parseDomainInput(seedID, domainFile, "file")
#' # get seedDf and orthoDf
#' subDf <- domainDf[
#'     domainDf$seedID ==
#'     "101621at6656#101621at6656:AGRPL@224129@0:224129_0:001955:1",]
#' orthoDf <- subDf[subDf$orthoID == "101621at6656:DROME@7227@1:Q9VG04",]
#' seedDf <- subDf[subDf$orthoID != "101621at6656:DROME@7227@1:Q9VG04",]
#' # sort
#' PhyloProfile:::sortDomains(seedDf, orthoDf)

sortDomains <- function(seedDf, orthoDf){
    if (is.null(seedDf) | is.null(orthoDf))
        stop("Domain data for seed & ortholog cannot be NULL!")
    orderNo <- NULL
    # get list of features in seedDf
    featureList <- as.data.frame(levels(as.factor(seedDf$feature)))
    colnames(featureList) <- c("feature")
    # and add order number to each feature
    featureList$orderNo <- seq(length(featureList$feature))

    # merge those info to orthoDf
    orderedOrthoDf <- merge(orthoDf, featureList, all.x = TRUE)

    # sort orthoDf
    index <- with(orderedOrthoDf, order(orderNo))
    orderedOrthoDf <- orderedOrthoDf[index, ]

    # convert feature column into a character vector
    orderedOrthoDf$feature <- as.character(orderedOrthoDf$feature)
    # then convert it back into an ordered factor
    # (to keep this order when plotting)
    orderedOrthoDf$feature <- factor(
        orderedOrthoDf$feature, levels = unique(orderedOrthoDf$feature)
    )
    # return sorted df
    return(orderedOrthoDf)
}

#' Sort one domain dataframe based on list of ordered feature types
#' @description Sort domain dataframe of one protein based on a given list of
#' ordered feature types
#' @param domainDf domain dataframe
#' @param featureClassOrder vector of ordered feature classes
#' @return Dataframe contains sorted domain list.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom dplyr left_join
#' @examples
#' # get domain data
#' seedID <- "101621at6656"
#' domainFile <- system.file(
#'     "extdata", "domainFiles/101621at6656.domains",
#'     package = "PhyloProfile", mustWork = TRUE
#' )
#' domainDf <- parseDomainInput(seedID, domainFile, "file")
#' # get seedDf and orthoDf
#' subDf <- domainDf[
#'     domainDf$seedID ==
#'     "101621at6656#101621at6656:AGRPL@224129@0:224129_0:001955:1",]
#' orthoDf <- subDf[subDf$orthoID == "101621at6656:DROME@7227@1:Q9VG04",]
#' featureClassOrder <- c("pfam", "smart", "tmhmm", "coils", "signalp", "seg",
#'     "flps")
#' # sort
#' PhyloProfile:::sortDomainsByList(orthoDf, featureClassOrder)

sortDomainsByList <- function(domainDf = NULL, featureClassOrder = NULL) {
    if (is.null(domainDf) | is.null(featureClassOrder))
        stop("Domain data or feature type order is NULL!")
    featureClassOrder <- rev(featureClassOrder[
        featureClassOrder %in% levels(as.factor(domainDf$feature_type))
    ])
    orderedDomainDf <- dplyr::left_join(
        data.frame(feature_type = featureClassOrder),
        domainDf, by = "feature_type"
    )
    orderedDomainDf$feature <- factor(
        orderedDomainDf$feature, levels = unique(orderedDomainDf$feature)
    )
    return(orderedDomainDf)
}

#' Modify feature names
#' @description Simplify feature names (e.g. TM for transmembrane domain,
#' LCR for low complexity regions, remove tool names from domain name) and add
#' weight to feature names (if available)
#' @param domainDf domain data as a dataframe object
#' @return Dataframe contains simlified domain names in yLabel column
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @examples
#' domainFile <- system.file(
#'     "extdata", "domainFiles/101621at6656.domains",
#'     package = "PhyloProfile", mustWork = TRUE
#' )
#' seedID <- "101621at6656"
#' domainDf <- parseDomainInput(seedID, domainFile, "file")
#' PhyloProfile:::modifyFeatureName(domainDf)

modifyFeatureName <- function(domainDf = NULL) {
    if (is.null(domainDf)) stop("Domain data cannot be NULL!")
    domainDf$yLabel <- domainDf$feature_id
    domainDf$yLabel[domainDf$yLabel == "transmembrane"] <- "TM"
    domainDf$yLabel[domainDf$yLabel == "low complexity regions"] <- "LCR"
    domainDf$yLabel[domainDf$yLabel == "low_complexity_regions"] <- "LCR"
    return(domainDf)
}


#' Join multiple plots and merge legends
#' @param df1 Data frame for plot 1
#' @param df2 Data frame for plot 2
#' @param plot1 ggplot object of plot 1
#' @param plot2 ggplot object of plot 2
#' @param position position of legend (bottom or right)
#' @param font font of text
#' @param legendSize font size
#' @return joined plots with merged legend as a grid object
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom gridExtra grid.arrange
#' @importFrom grid unit grobHeight convertHeight
#' @examples
#' seed <- "101621at6656"
#' ortho <- "101621at6656|AGRPL@224129@0|224129_0:001955|1"
#' ortho <- gsub("\\|", ":", ortho)
#' grepID <- paste(seed, "#", ortho, sep = "")
#' domainFile <- system.file(
#'     "extdata", "domainFiles/101621at6656.domains",
#'     package = "PhyloProfile", mustWork = TRUE
#' )
#' domainDf <- parseDomainInput(seed, domainFile, "file")
#' domainDf$feature_id_mod <- domainDf$feature_id
#' subdomainDf <- domainDf[grep(grepID, domainDf$seedID), ]
#' subdomainDf$feature <- as.character(subdomainDf$feature)
#' orthoDf <- subdomainDf[subdomainDf$orthoID == ortho,]
#' seedDf <- subdomainDf[subdomainDf$orthoID != ortho,]
#' minStart <- min(subdomainDf$start)
#' maxEnd <- max(c(subdomainDf$end, subdomainDf$length))
#' # resolve overlapping domains
#' seedDf <- PhyloProfile:::resolveOverlapFeatures(seedDf)
#' orthoDf <- PhyloProfile:::resolveOverlapFeatures(orthoDf)
#' # add feature colors
#' featureColorDf <- PhyloProfile:::addFeatureColors(seedDf, orthoDf)
#' seedDf <- featureColorDf[[1]]
#' orthoDf <- featureColorDf[[2]]
#' # generate plots
#' plotSeed <- PhyloProfile:::singleDomainPlotting(
#'     seedDf, seed, minStart = minStart, maxEnd = maxEnd, font = "sans"
#' )
#' plotOrtho <- PhyloProfile:::singleDomainPlotting(
#'     orthoDf, ortho, minStart = minStart, maxEnd = maxEnd, font = "sans"
#' )
#' # merge plots
#' PhyloProfile:::joinPlotMergeLegends(
#'     seedDf, orthoDf, plotSeed, plotOrtho, "bottom", font = "sans", 
#'     legendSize = 12)

joinPlotMergeLegends <- function(
    df1 = NULL, df2 = NULL, plot1 = NULL, plot2 = NULL, 
    position = c("bottom", "right"), font = "Arial", legendSize = 12)
{
    if (is.null(plot1) | is.null(df1)) stop("No plot data given!")
    if (is.null(plot2) | is.null(df2))
        return(plot1 + theme(legend.position = position))
    featureOri <- NULL
    # remove legend of the original plots
    plot1 <- plot1 + theme(legend.position = "none")
    plot2 <- plot2 + theme(legend.position = "none")
    # create a temp plot that contains all features
    mergedDf <- rbind(df1, df2)
    colorScheme <- structure(
        mergedDf$color, .Names = mergedDf$featureOri
    )
    tmpPlot <- ggplot() +
        geom_segment(
            data = mergedDf,
            aes(
                x = 0, xend = 1, y = 1, yend = 1,
                color = featureOri
            ),
            linewidth = 5, lineend = "round", alpha = 0.7
        ) +
        scale_color_manual(values = colorScheme) +
        labs(color = "Feature") +
        theme_minimal() +
        theme(
            legend.position = position, 
            text = element_text(family = font, size = legendSize)
        )
    # extract legend from the temp plot above
    getOnlyLegend <- function(plot) {
        plotTable <- ggplot_gtable(ggplot_build(plot))
        legendPlot <- which(
            vapply(
                plotTable$grobs, function(x) x$name, character(1)
            ) == "guide-box"
        )
        legend <- plotTable$grobs[[legendPlot]]
        return(legend)
    }
    legend <- getOnlyLegend(tmpPlot)
    legend_height <- grid::convertHeight(
        grid::grobHeight(legend), "cm", valueOnly = TRUE
    )
    # Combine all three vertically, using measured heights
    combined <- gridExtra::grid.arrange(
        plot1,
        plot2,
        legend,
        ncol = 1, nrow = 3,
        heights = grid::unit.c(
            grid::unit(length(unique(df1$feature)), "null"),
            grid::unit(length(unique(df2$feature)), "null"),
            grid::unit(legend_height, "cm")
        )
    )
    return(combined)
}

#' Identify feature type(s) containing overlapped domains/features
#' @param domainDf input domain dataframe
#' @return List of feature types that have overlapped domains
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom dplyr group_by add_count
#' @examples
#' # get domain data
#' seedID <- "101621at6656"
#' domainFile <- system.file(
#'     "extdata", "domainFiles/101621at6656.domains",
#'     package = "PhyloProfile", mustWork = TRUE
#' )
#' domainDf <- parseDomainInput(seedID, domainFile, "file")
#' # get seedDf and orthoDf
#' subDf <- domainDf[
#'     domainDf$seedID ==
#'     "101621at6656#101621at6656:AGRPL@224129@0:224129_0:001955:1",]
#' orthoDf <- subDf[subDf$orthoID == "101621at6656:DROME@7227@1:Q9VG04",]
#' # check overlap features
#' PhyloProfile:::checkOverlapDomains(orthoDf)

checkOverlapDomains <- function(domainDf) {
    if (is.null(domainDf)) stop("Domain data cannot be NULL!")
    feature_type <- NULL
    df <- domainDf[, c("feature", "feature_type", "start", "end")]
    df <- data.frame(
        df %>% dplyr::group_by(feature_type) %>% dplyr::add_count(feature_type)
    )
    df$tmp <- paste(df$feature_type, df$end, sep = "_")
    overlappedType <- lapply(
        df$tmp[df$n > 1],
        function (x) {
            if (length(df$feature[df$tmp == x]) > 1)
                return(df$feature_type[df$tmp == x])

            ed <- df$end[df$tmp == x][1]
            st <- df$start[df$tmp == x][1]
            type <- df$feature_type[df$tmp == x][1]

            subDf <- df[
                !(df$tmp == x) & df$feature_type == type &
                    df$start < ed & df$end > st,
            ]
            if (nrow(subDf) > 0) {
                if(!(subDf$feature[1] == df$feature[df$tmp == x]))
                    return(subDf$feature_type)
            }
        }
    )
    return(unique(unlist(overlappedType)))
}

#' Modify domain dataframe to resolve overlapped domains/features
#' @param domainDf input domain dataframe
#' @return Domain dataframe with modified feature names that join multiple
#' domains of the same type that are not overlapped
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @examples
#' # get domain data
#' seedID <- "101621at6656"
#' domainFile <- system.file(
#'     "extdata", "domainFiles/101621at6656.domains",
#'     package = "PhyloProfile", mustWork = TRUE
#' )
#' domainDf <- parseDomainInput(seedID, domainFile, "file")
#' # get seedDf and orthoDf
#' subDf <- domainDf[
#'     domainDf$seedID ==
#'     "101621at6656#101621at6656:AGRPL@224129@0:224129_0:001955:1",]
#' orthoDf <- subDf[subDf$orthoID == "101621at6656:DROME@7227@1:Q9VG04",]
#' # resolve overlapped featuers
#' PhyloProfile:::resolveOverlapFeatures(orthoDf)

resolveOverlapFeatures <- function(domainDf) {
    if (is.null(domainDf)) stop("Domain data cannot be NULL!")
    overlappedType <- checkOverlapDomains(domainDf)
    domainDf$featureOri <- domainDf$feature
    domainDf$featureOri <- as.character(domainDf$featureOri)
    if (length(overlappedType) > 0) {
        domainDf$feature <- ifelse(
            !(domainDf$feature_type %in% overlappedType),
            domainDf$feature_type,
            domainDf$feature
        )
    } else {
        domainDf$feature <- domainDf$feature_type
    }
    return(domainDf)
}

#' Linearize PFAM/SMART annotations by best e-value/bitscore
#' @export
#' @param domainDf input domain dataframe
#' @param orthoID ID of protein that needs to be linearized
#' @param value type of values that will be used for linearized, either evalue
#' (default) or bitscore
#' @return Domain dataframe of the selected protein after linearization
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom dplyr arrange
#' @examples
#' demoDomainDf <- data.frame(
#'     orthoID = rep("protID", 4),
#'     start = c(1, 5, 100, 80),
#'     end = c(30, 40, 130, 110),
#'     evalue = c(0.001, 0.0005, 0.2, 0.004),
#'     feature_type = c(rep("pfam", 2), rep("smart", 2)),
#'     feature_id = c("pf1", "pf2", "sm1", "sm2")
#' )
#' linearizeArchitecture(demoDomainDf, "protID", "evalue")

linearizeArchitecture <- function(
        domainDf = NULL, orthoID = NULL, value = "evalue"
) {
    if (is.null(domainDf) | is.null(orthoID)) stop("Input data is NULL!")
    evalue <- bitscore <- start <- end <- NULL
    # Sort the dataframe by start position and then by evalue or bitscore
    if (value == "evalue") {
        domainDf <- domainDf %>% dplyr::arrange(start, evalue)
    } else if (value == "bitscore") {
        domainDf <- domainDf %>% dplyr::arrange(start, bitscore)
    } else stop("Incorrect value specified! Either 'evalue' or 'bitscore'")

    # Get lines that need to be excluded
    pfamRows <- rownames(domainDf[domainDf$orthoID == orthoID &
                    domainDf$feature_type %in% c("pfam","smart"),])
    exclude_lines <- vapply(
        seq_len(length(pfamRows)-1),
        function(i) {
            if (domainDf[pfamRows[i],]$end >= domainDf[pfamRows[i+1],]$start) {
                # Exclude the row with the higher evalue / lower bitscore
                if (value == "evalue") {
                    if (
                        domainDf[pfamRows[i],]$evalue >
                        domainDf[pfamRows[i+1],]$evalue
                    ) {
                        return((pfamRows[i]))
                    } else {
                        return((pfamRows[i+1]))
                    }
                } else {
                    if (
                        domainDf[pfamRows[i],]$bitscore <
                        domainDf[pfamRows[i+1],]$bitscore
                    ) {
                        return((pfamRows[i]))
                    } else {
                        return((pfamRows[i+1]))
                    }
                }

            } else {
                return("0")
            }
        },
        character(1)
    )
    # return domainDf after removing overlapped features with higher e-values
    outDf <- domainDf[!(row.names(domainDf) %in% exclude_lines), ]
    return(outDf[outDf$orthoID == orthoID,])
}

#' Add colors for each feature/domain
#' @description Add colors to features/domains of 2 domain dataframes. Users can
#' choose to color only the shared features, unique features, all features
#' (default) or based on feature types. Default color pallete is "Paired", but
#' it can be changed.
#' @param seedDf Domain dataframe of seed protein (protein 1)
#' @param orthoDf Domain dataframe of orthologs protein (protein 2)
#' @param colorType Choose to color "all", "shared", "unique" features or color
#' by "Feature type". Default: "all"
#' @param colorPalette Choose between "Paired", "Set1", "Set2", "Set3",
#' "Accent", "Dark2" for the color pallete
#' @param ignoreInstanceNo Ignore number of feature instances while identifying
#' shared or unique features. Default: FALSE
#' @return 2 dataframes (seedDf and orthoDf) with an additional column for the
#' assigned color to each feature instance
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom RColorBrewer brewer.pal
#' @importFrom utils head
#' @importFrom dplyr count
#' @examples
#' # get domain data
#' seedID <- "101621at6656"
#' domainFile <- system.file(
#'     "extdata", "domainFiles/101621at6656.domains",
#'     package = "PhyloProfile", mustWork = TRUE
#' )
#' domainDf <- parseDomainInput(seedID, domainFile, "file")
#' # get seedDf and orthoDf
#' subDf <- domainDf[
#'     domainDf$seedID ==
#'     "101621at6656#101621at6656:AGRPL@224129@0:224129_0:001955:1",]
#' orthoDf <- subDf[subDf$orthoID == "101621at6656:DROME@7227@1:Q9VG04",]
#' seedDf <- subDf[subDf$orthoID != "101621at6656:DROME@7227@1:Q9VG04",]
#' # add colors to features
#' PhyloProfile:::addFeatureColors(seedDf, orthoDf)

addFeatureColors <- function(
        seedDf = NULL, orthoDf = NULL, colorType = "all",
        colorPalette = "Paired", ignoreInstanceNo = FALSE
) {
    if (is.null(seedDf) | is.null(orthoDf)) stop("Domain Df cannot be null!")
    feature <- NULL
    featureSeedCount <- seedDf %>% dplyr::count(feature)
    featureOrthoCount <- orthoDf %>% dplyr::count(feature)
    featureCount <- merge(
        featureSeedCount, featureOrthoCount, by = "feature", all = TRUE
    )
    featureCount$type <- "unique"
    featureCount$type[featureCount$n.x == featureCount$n.y] <- "shared"
    if (ignoreInstanceNo == TRUE) {
        featureCount$type[
            !is.na(featureCount$n.x) & !is.na(featureCount$n.y)] <- "shared"
    }
    featureCount$feature <- as.character(featureCount$feature)
    sharedFeatures <-unique(featureCount$feature[featureCount$type == "shared"])
    uniqueFeatures <-unique(featureCount$feature[featureCount$type == "unique"])
    allFeatures <- c(sharedFeatures, uniqueFeatures)

    if (colorType == "Unique" & length(uniqueFeatures) == 0)
        colorType <- "All"
    if (colorType == "Shared" & length(sharedFeatures) == 0)
        colorType <- "All"

    if (colorType == "Unique") {
        sharedFeaturesColors <- rep("#C9C9C9", length(sharedFeatures))
        uniqueFeaturesColors <- getQualColForVector(uniqueFeatures)
        if (checkColorPalette(uniqueFeatures, colorPalette) == TRUE) {
            uniqueFeaturesColors <-
                suppressWarnings(head(
                    RColorBrewer::brewer.pal(
                        length(uniqueFeatures), colorPalette),
                    length(uniqueFeatures)
                ))
        }
        allColors <- c(sharedFeaturesColors, uniqueFeaturesColors)
        colorScheme <- data.frame(color = allColors, feature = allFeatures)
    } else if (colorType == "Shared") {
        sharedFeaturesColors <- getQualColForVector(sharedFeatures)
        uniqueFeaturesColors <- rep("#C9C9C9", length(uniqueFeatures))
        if (checkColorPalette(sharedFeatures, colorPalette) == TRUE) {
            sharedFeaturesColors <-
                suppressWarnings(head(
                    RColorBrewer::brewer.pal(
                        length(sharedFeatures), colorPalette),
                    length(sharedFeatures)
                ))
        }
        allColors <- c(sharedFeaturesColors, uniqueFeaturesColors)
        colorScheme <- data.frame(color = allColors, feature = allFeatures)
    } else if (colorType == "All") {
        allColors <- getQualColForVector(allFeatures)
        if (checkColorPalette(allFeatures, colorPalette) == TRUE) {
            allColors <-
                suppressWarnings(head(
                    RColorBrewer::brewer.pal(length(allFeatures), colorPalette),
                    length(allFeatures)
                ))
        }
        colorScheme <- data.frame(color = allColors, feature = allFeatures)
    } else {
        tmpDf <- data.frame(str_split_fixed(allFeatures, "_", 2))
        tmpDf$name <- paste(tmpDf$X1, tmpDf$X2, sep = "_")
        tmpDf$name[tmpDf$X2 == ""] <- tmpDf$X1[tmpDf$X2 == ""]
        tmpDf$X2[tmpDf$X2 == ""] <- tmpDf$X1[tmpDf$X2 == ""]
        typeColorDf <- data.frame(
            colors = head(
                suppressWarnings(RColorBrewer::brewer.pal(
                    nlevels(as.factor(tmpDf$X1)), colorPalette)),
                nlevels(as.factor(tmpDf$X1))
            ),
            X1 = levels(as.factor(tmpDf$X1))
        )
        tmpDf <- merge(tmpDf, typeColorDf, all.x = TRUE)
        colorScheme <- data.frame(color = tmpDf$colors, feature = tmpDf$name)
    }

    # add color to seedDf and orthoDf
    seedDf <- merge(seedDf, colorScheme, by = "feature", all.x = TRUE)
    orthoDf <- merge(orthoDf, colorScheme, by = "feature", all.x = TRUE)

    return(list(seedDf, orthoDf))
}

#' Distribution plots
#' @param data data for plotting (from reactive fn "presSpecAllDt")
#' @param varID name of variable (either input$var1ID, input$var2ID or
#' "% present taxa"; from input$selectedDist)
#' @param varType type of variable (either var1, var2 or presSpec)
#' @param percent percentage cutoff (from input$percent)
#' @param distTextSize text size of distribution plot
#' @param distWidth width of distribution plot
#' (from input$distTextSize)
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}

analyzeDistributionUI <- function(id) {
    ns <- NS(id)
    tagList(
        column(
            2,
            downloadButton(ns("plotDownloadDist"), "Download plot")
        ),
        column(
            10,
            uiOutput(ns("distPlot.ui"))
        )
    )
}

analyzeDistribution <- function(input, output, session,
                                 data,
                                 varID, varType,
                                 percent,
                                 distTextSize, distWidth){

    # render distPlot.ui ------------------------------------------------------
    output$distPlot.ui <- renderUI({
        ns <- session$ns
        # shinycssloaders::withSpinner(
            plotOutput(ns("distributionPlot"),  width = distWidth())
        # )
    })

    output$distributionPlot <- renderPlot(width = distWidth(), height = 356, {
        createVarDistPlot(data(), varID(), varType(), percent(), distTextSize())
    })

    output$plotDownloadDist <- downloadHandler(
        filename = function() {
            paste0("distributionPlot.pdf")
        },
        content = function(file) {
            ggsave(
                file,
                plot = createVarDistPlot(
                    data(), varID(), varType(), percent(), distTextSize()
                ),
                width = distWidth() * 0.056458333,
                height = 356 * 0.056458333,
                unit = "cm",
                dpi = 300, device = "pdf", limitsize = FALSE)
        }
    )
}

#' Download plot seetings
#' @return a yaml file or rscript file
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}
downloadPlotSettings <- function (settings, outFile, type) {
    if (file.exists(outFile)) {
        msg <- paste(
            "Output file exists! Please choose another directory or rename",
            "the output file to continue!"
        )
        message(msg)
    } else {
        if (type == "list") {
            yaml::write_yaml(settings, outFile, indent = 4)
            message("FINISHED! The output file is ", outFile)
        } else {
            yaml::write_yaml(settings, paste0(outFile, ".yml"), indent = 4)
            lapply(
                writePlottingScript(settings),
                write, paste0(outFile, ".rscript"), append = TRUE
            )
            msg1 <- paste0(
                "FINISHED! The output files are ", outFile,
                ".yml and ", outFile, ".rscript"
            )
            msg2 <- paste0(
                "You can use this command to generate the plot:<br>",
                "Rscript ", outFile, ".rscript ", outFile, ".yml ",
                "input.phyloprofile output_plot.pdf"
            )
            message(msg1, "<br>", msg2)
        }
    }
}

writePlottingScript <- function(settingsFile) {
    out <- c(
        "library(PhyloProfile)",
        "library(ggplot2)",

        "args = commandArgs(trailingOnly = TRUE)",
        "if (is.na(args[1])) stop('Setting file missing')",
        "if (is.na(args[2])) stop('Input file missing')",
        "if (is.na(args[3])) stop('Output file missing')",

        "##### Reading setting file",
        paste(
            "if (!file.exists(args[1]))",
            "stop(paste(args[1], 'not found!'))"

        ),
        "settings <- yaml::read_yaml(args[1])",
        "print('Reading setting file...')",
        "rawInput <- args[2]",
        paste(
            "if (!file.exists(rawInput))",
            "stop(paste(rawInput, 'not found!'))"
        ),
        "rankName <- settings$rank",
        "refTaxon <- settings$refspec",
        "taxaTree <- NULL",
        "var1AggregateBy <- settings$var1AggregateBy",
        "var2AggregateBy <- settings$var2AggregateBy",
        "percentCutoff <- settings$percentCutoff",
        "coorthologCutoffMax <- 999",
        "var1Cutoff <- settings$var1Cutoff",
        "var2Cutoff <- settings$var2Cutoff",
        "var1Relation <- settings$var1Relation",
        "var2Relation <- settings$var2Relation",
        "groupByCat <- FALSE",
        "catDt <- NULL",
        "taxDB <- settings$taxDB",

        "##### Processing input",
        "print('Processing input...')",
        "inputDf <- createLongMatrix(rawInput)",
        "inputDf <- inputDf[!duplicated(inputDf),]",
        "colnames(inputDf) <- c('geneID', 'ncbiID', 'orthoID', 'var1', 'var2')",

        "sortedTaxa <- sortInputTaxa(
            taxonIDs = getInputTaxaID(inputDf),
            rankName = rankName,
            refTaxon = refTaxon,
            taxaTree = NULL,
            sortedTaxonList = NULL,
            taxDB = taxDB
        )",

        "taxaCount <- sortedTaxa %>% dplyr::count(supertaxon)",

        "fullData <- parseInfoProfile(
            inputDf = inputDf,
            sortedInputTaxa = sortedTaxa,
            taxaCount = taxaCount,
            coorthoCOMax = coorthologCutoffMax
        )",

        "filteredDf <- filterProfileData(
            fullData,
            taxaCount,
            refTaxon,
            percentCutoff,
            coorthologCutoffMax,
            var1Cutoff,
            var2Cutoff,
            var1Relation,
            var2Relation,
            groupByCat,
            catDt,
            var1AggregateBy,
            var2AggregateBy
        )",
        "dataHeat <- reduceProfile(filteredDf)",

        "##### Clustering",
        "if (settings$clusterProfile == TRUE) {",
        "print('Clustering...')",
        "clusteredDataHeat <- dataHeat",
        "clusterMethod <- settings$clusterMethod",
        "distMethod <- settings$distMethodClustering",
        paste0(
            "profiles4cluster <- getDataClustering(
            dataHeat, settings$profileTypeClustering,
            var1AggregateBy,
            var2AggregateBy
            )"
        ),
        "distanceMatrix <- getDistanceMatrix(profiles4cluster, distMethod)",
        "row.order <- hclust(distanceMatrix, method = clusterMethod)$order",
        "datNew <- profiles4cluster[row.order, ]",
        "clusteredGeneIDs <- as.factor(row.names(datNew))",
        paste0(
            "clusteredDataHeat$geneID <- factor(clusteredDataHeat$geneID,",
            "levels=clusteredGeneIDs)"
        ),
        paste0(
            "clusteredDataHeat <- clusteredDataHeat",
            "[!is.na(clusteredDataHeat$geneID),]"
        ),
        "}",

        "print('Generating plot...')",
        "plotDf <- dataMainPlot(dataHeat)",
        paste(
            "if (settings$clusterProfile == TRUE)",
            "plotDf <- dataMainPlot(clusteredDataHeat)"
        ),
        "plotParameter <- list(
            'xAxis' = settings$xAxis,
            'var1ID' = settings$var1ID,
            'var2ID' = settings$var2ID,
            'midVar1' = settings$midVar1,
            'midColorVar1' = settings$midColorVar1,
            'lowColorVar1' = settings$lowColorVar1,
            'highColorVar1' = settings$highColorVar1,
            'midVar2' = settings$midVar2,
            'midColorVar2' = settings$midColorVar2,
            'lowColorVar2' = settings$lowColorVar2,
            'highColorVar2' = settings$highColorVar2,
            'paraColor' = settings$paraColor,
            'xSize' = settings$xSize,
            'ySize' = settings$ySize,
            'legendSize' = settings$legendSize,
            'mainLegend' = settings$mainLegend,
            'dotZoom' = settings$dotZoom,
            'xAngle' = settings$xAngle,
            'guideline' = 0,
            'colorByGroup' = settings$colorByGroup
        )",
        "p <- heatmapPlotting(plotDf, plotParameter)",
        "ggsave(args[3], plot = p, width = settings$width * 0.035,
            height = settings$height * 0.035,
            units = 'cm', dpi = 300, device = 'svg', limitsize = FALSE)",
        "print('DONE! Your plot is saved in', args[3])"
    )
    return(out)
}

#' Import user defined NCBI taxonomy database
#' @return none (imported files will be saved in PhyloProfile package folder)
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}
importNcbiTax <- function(inTaxDir, taxDB) {
    taxFiles <- c(
        "newTaxa.txt",
        "taxonNamesReduced.txt",
        "idList.txt",
        "rankList.txt",
        "taxonomyMatrix.txt"
    )
    # Check required files
    message("1) Checking taxonomy files in ", inTaxDir,"...")
    flag = 1
    for (file in taxFiles) {
        if (!file.exists(paste0(inTaxDir, "/", file))) flag = 0
    }

    if (!file.exists(paste0(inTaxDir, "/preCalcTree.nw"))) {
        if (file.exists(paste0(taxDB, "/preCalcTree.nw")))
            unlink(paste0(taxDB, "/preCalcTree.nw"))
    } else taxFiles <- c(taxFiles, "preCalcTree.nw")

    if (flag == 0) {
        message(
            "<p><span style=\"color: #ff0000;\"><strong>ERROR</strong></span>: Some of the taxonomy files cannot be found! Please check <a href=\"https://github.com/BIONF/PhyloProfile/wiki/PhyloProfile-and-the-NCBI-taxonomy-database\">this link</a> for more info.</p>"
        )
    } else {
        message("2) Importing data into ", taxDB, "...")
        for (file in taxFiles) {
            system(
                paste(
                    "cp",
                    paste0(inTaxDir, "/", file),
                    paste0(taxDB, "/", file)
                )
            )
        }

        message(
            "3) FINISHED! Your NCBI taxonomy database has been imported!<br>"
        )
        message("Now you can re-upload your input or restart PhyloProfile.")
        message(
            "To get the original taxonomy data, plase use the Reset function!"
        )
    }
}

#' Update NCBI taxonomy database
#' @return none (updated files will be saved in PhyloProfile package folder)
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}
updateNcbiTax <- function() {
    # get latest NCBI taxonomy database and processing
    message("1) Getting latest NCBI taxonomy database and processing...")
    preProcessedTaxonomy <- processNcbiTaxonomy()
    # save to preProcessedTaxonomy.txt file within PhyloProfile pkg folder
    message("2) Saving new data into preProcessedTaxanomy.txt file...")
    packgePath <- find.package("PhyloProfile")
    write.table(
        preProcessedTaxonomy,
        file = paste0(packgePath,"/PhyloProfile/data/preProcessedTaxonomy.txt"),
        col.names = TRUE,
        row.names = FALSE,
        quote = FALSE,
        sep = "\t"
    )
    message("3) FINISHED! Your NCBI taxonomy database has been updated!<br>")
    message("Now you can re-upload your input or restart PhyloProfile.")
}

#' Detailed plot
#' @param data (from reactive fn "detailPlotDt")
#' @param var1ID name of variable 1 (from input$var1ID)
#' @param var2ID name of variable 2 (from input$var2ID)
#' @param detailedText text size (from input$detailedText)
#' @param detailedHeight plot height (from input$detailedHeight)
#' @return information of selected protein on detailed plot
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}

createDetailedPlotUI <- function(id) {
    ns <- NS(id)
    tagList(
        uiOutput(ns("detailPlot.ui")),
        downloadButton(ns("downloadDetailed"), "Download plot",
                       class = "butDL"),
        tags$head(
            tags$style(HTML(
                ".butDL{background-color:#476ba3;} .butDL{color: white;}"))
        ),
        hr(),
        verbatimTextOutput(ns("detailClick"))
    )
}

createDetailedPlot <- function(
    input, output, session, data, var1ID, var2ID, detailedText, detailedHeight,
    font
){
    # simplify ortho IDs if they are in BIONF format ---------------------------
    plotData <- reactive({
        plotDf <- data()
        if (
            checkBionfFormat(
                plotDf$orthoID[1], plotDf$geneID[1],
                gsub('ncbi','',plotDf$abbrName[1])
            )
        ) {
            plotDf <- within(
                plotDf,
                orthoMod <- data.frame(
                    do.call(
                        'rbind', strsplit(as.character(orthoID),'|', fixed=TRUE)
                    )
                )
            )
            plotDf$orthoID <- plotDf$orthoMod$X3
        }
        return(plotDf)
    })

    # render detailed plot -----------------------------------------------------
    output$detailPlot <- renderPlot({
        detailPlot(plotData(), detailedText(), var1ID(), var2ID(), font())
    })

    output$detailPlot.ui <- renderUI({
        ns <- session$ns
        shinycssloaders::withSpinner(
            plotOutput(
                ns("detailPlot"),
                width = 800,
                height = detailedHeight(),
                click = ns("plotClickDetail")
            )
        )
    })

    output$downloadDetailed <- downloadHandler(
        filename = function() {
            c("detailedPlot.svg")
        },
        content = function(file) {
            g <- detailPlot(data(), detailedText(), var1ID(), var2ID(), font())
            ggsave(
                file,
                plot = g,
                width = 800 * 0.035,
                height = detailedHeight() * 0.035,
                units = "cm",
                dpi = 300,
                device = "svg",
                limitsize = FALSE
            )
        }
    )

    # get info of detailed plot ------------------------------------------------
    pointInfoDetail <- reactive({
        selDf <- data()
        selDf$orthoID <- as.character(selDf$orthoID)

        # if only one ortholog, get directly from data()
        if (nrow(selDf) == 1) {
            seedID <- as.character(selDf$geneID)
            orthoID <- as.character(selDf$orthoID)
            taxID <- as.character(selDf$abbrName)
        }
        # else, get from plotClickDetail
        else {
            if (is.null(input$plotClickDetail$x)) return(NULL)
            else {
                corX <- round(input$plotClickDetail$y)
                corY <- round(input$plotClickDetail$x)
            }
            # get pair of sequence IDs
            seedID <- as.character(selDf$geneID[!is.na(selDf$geneID)][1])
            orthoID <- as.character(selDf$orthoID[corX])
            taxID <- as.character(selDf$abbrName[corX])
        }

        # get var1, var2
        var1 <- as.list(selDf$var1[selDf$orthoID == orthoID])
        var1 <- as.character(var1[!is.na(var1)])
        var2 <- as.list(selDf$var2[selDf$orthoID == orthoID])
        var2 <- as.character(var2[!is.na(var2)])
        if (length(var2) == 0) var2 = "NA"

        ncbiID <- selDf[selDf$orthoID == orthoID, ]$abbrName
        ncbiID <- as.character(ncbiID[!is.na(ncbiID)][1])

        # return info
        if (is.na(orthoID))
            return(NULL)
        else {
            if (orthoID != "NA")
                return(c(seedID, orthoID, var1, var2, ncbiID))
        }
    })

    # * show info when clicking on detailed plot -------------------------------
    output$detailClick <- renderText({
        info <- pointInfoDetail() # info = seedID, orthoID, var1

        if (is.null(info)) paste("select ortholog")
        else {
            a <- paste0("seedID = ", info[1])
            b <- paste0("hitID = ", info[2])
            c <- ""
            if (var1ID() != "") {
                c <- paste0(var1ID(), " = ", info[3])
            }
            d <- ""
            if (var2ID() != "") {
                d <- paste0(var2ID(), " = ", info[4])
            }
            paste(a, b, c, d, sep = "\n")
        }
    })
    return(pointInfoDetail)
}

#' create detailed plot
#' @param selDf data for plotting  (from reactive fn "detailPlotDt")
#' @param detailedText text size (input$detailedText)
#' @param var1ID name of variable 1 (input$var1ID)
#' @param var2ID name of variable 2 (input$var2ID)
#' @param font font of text. Default = Arial"
#' @return detailed plot (ggplot object)
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}

detailPlot <- function(selDf, detailedText, var1ID, var2ID, font = "Arial"){
    selDf$xLabel <- paste(selDf$orthoID, " (", selDf$fullName, ")", sep = "")

    # create joined DF for plotting var1 next to var2
    var1Df <- subset(selDf, select = c("xLabel", "var1"))
    var1Df$type <- var1ID
    colnames(var1Df) <- c("id", "score", "var")

    var2Df <- subset(selDf, select = c("xLabel", "var2"))
    var2Df$type <- var2ID
    colnames(var2Df) <- c("id", "score", "var")

    detailedDf <- rbind(var1Df, var2Df)
    if(any(is.na(detailedDf$score))) {
        detailedDf[
            detailedDf$id %in% detailedDf$id[is.na(detailedDf$score)],
        ]$score <- NA
    }
    
    # remove ONE missing variable
    if (nlevels(as.factor(detailedDf$var)) > 1) {
        detailedDf <- detailedDf[nchar(detailedDf$var) > 0, ]
    }

    # keep order of ID (xLabel)
    detailedDf$id <- factor(detailedDf$id, levels = unique(detailedDf$id))

    # create plot
    gp <- ggplot(detailedDf, aes(y = score, x = id, fill = var)) +
        geom_bar(stat = "identity", position = position_dodge(), na.rm = TRUE) +
        coord_flip() +
        labs(x = "") +
        labs(fill = "") +
        theme_minimal()
    gp <- gp + theme(axis.text.x = element_text(angle = 90, hjust = 1),
                     axis.text = element_text(size = detailedText),
                     axis.title = element_text(size = detailedText),
                     legend.text = element_text(size = detailedText),
                     text = element_text(family = font)
    )
    return(gp)
}

#' Run PhyloProfile app
#' @export
#' @param configFile Configuration file for specifying path to input files,
#' taxonomy rank and reference taxon, and some other settings
#' @param host IP adress (e.g. host = "127.0.0.1")
#' @param port Port (e.g. port = 8888)
#' @return A shiny application - GUI version of PhyloProfile
#' @import BiocStyle
#' @import bsplus
#' @importFrom colourpicker colourInput
#' @rawNamespace import(data.table, except = c(first, last, between))
#' @import dplyr
#' @importFrom DT dataTableOutput renderDataTable
#' @importFrom shinycssloaders withSpinner
#' @importFrom shinyjs enable disable reset html toggleState
#' @importFrom shinyFiles shinyDirButton shinyDirChoose parseDirPath
#' @importFrom yaml read_yaml write_yaml
#' @importFrom RCurl url.exists
#' @importFrom htmlwidgets saveWidget
#' @import svglite
#' @examples
#' ?runPhyloProfile
#' # runPhyloProfile()

runPhyloProfile <- function(configFile = NULL, host = NULL, port = NULL){
    appDir <- system.file("PhyloProfile", package = "PhyloProfile")
    if (appDir == "") {
        stop(
            "Could not find apps directory. Try re-installing `PhyloProfile`.",
            call = FALSE
        )
    }

    .GlobalEnv$configFile <- configFile
    on.exit(rm(configFile, envir=.GlobalEnv))

    i_host <- i_port <- NULL
    i_launchBrowser <- TRUE
    if (!is.null(configFile) && file.exists(configFile)) {
        configs <- yaml::read_yaml(configFile)
        i_host <- configs$host
        i_port <- configs$port
        i_launchBrowser <- configs$launchBrowser
    }
    if (!is.null(host)) i_host <- host
    if (!is.null(port)) i_port <- port

    if (!is.logical(i_launchBrowser)) i_launchBrowser <- TRUE
    if (!is.null(i_host) && !is.null(i_port)) {
        shiny::runApp(
            appDir,
            host = i_host, port = i_port, launch.browser = i_launchBrowser,
            display.mode = "normal"
        )
    } else {
        shiny::runApp(
            appDir,
            launch.browser = TRUE,
            display.mode = "normal"
        )
    }
}

#' Reset taxonomy data
#' @return none
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}
resetTaxData <- function() {
    packgePath <- find.package("PhyloProfile")
    data <- c(
        "idList.txt", "rankList.txt", "taxonomyMatrix.txt",
        "taxonNamesReduced.txt"
    )
    files <- paste0(packgePath, "/PhyloProfile/data/", data)
    # delete old files
    message("1) Deleting old taxonomy data...")
    for(f in files) {
        if (file.exists(f)) file.remove(f)
    }
    # rewrite precalculated data again
    message("2) Re-writing taxonomy data files...")
    data(rankList)
    write.table(
        rankList, file = paste0(packgePath, "/PhyloProfile/data/rankList.txt"),
        col.names = FALSE,
        row.names = FALSE,
        quote = FALSE,
        sep = "\t"
    )
    data(idList)
    write.table(
        idList, file = paste0(packgePath, "/PhyloProfile/data/idList.txt"),
        col.names = FALSE,
        row.names = FALSE,
        quote = FALSE,
        sep = "\t"
    )
    data(taxonNamesReduced)
    write.table(
        taxonNamesReduced,
        file = paste0(packgePath, "/PhyloProfile/data/taxonNamesReduced.txt"),
        col.names = TRUE,
        row.names = FALSE,
        quote = FALSE,
        sep = "\t"
    )
    data(taxonomyMatrix)
    write.table(
        taxonomyMatrix,
        file = paste0(packgePath, "/PhyloProfile/data/taxonomyMatrix.txt"),
        col.names = TRUE,
        row.names = FALSE,
        quote = FALSE,
        sep = "\t"
    )
    preCalcTree <- createUnrootedTree(taxonomyMatrix)
    ape::write.tree(
        preCalcTree,file=paste0(packgePath,"/PhyloProfile/data/preCalcTree.nw"))
    message("2) FINISHED!<br>")
    message("Now you can restart PhyloProfile or re-upload your input file.")
}

#' Get data for calculating distance matrix from phylogenetic profiles
#' @export
#' @usage getDataClustering(data, profileType = "binary", var1AggBy = "max",
#'     var2AggBy = "max")
#' @param data a data frame contains processed and filtered profiles (see
#' ?fullProcessedProfile and ?filterProfileData, ?fromInputToProfile)
#' @param profileType type of data used for calculating the distance matrix.
#' Either "binary" (consider only the presence/absence status of orthlogs),
#' "orthoID" (consider ortholog IDs as values for clustering),
#' "var1"/"var2" for taking values of the additional variables into account.
#' Default = "binary".
#' @param var1AggBy aggregate method for VAR1 (min, max, mean or median).
#' Default = "max".
#' @param var2AggBy aggregate method for VAR2 (min, max, mean or median).
#' Default = "max".
#' @return A wide dataframe contains values for calculating distance matrix.
#' @author Carla Mölbert (carla.moelbert@gmx.de), Vinh Tran
#' (tran@bio.uni-frankfurt.de)
#' @importFrom data.table dcast setDT
#' @seealso \code{\link{fromInputToProfile}}
#' @examples
#' data("finalProcessedProfile", package="PhyloProfile")
#' data <- finalProcessedProfile
#' profileType <- "binary"
#' var1AggregateBy <- "max"
#' var2AggregateBy <- "mean"
#' getDataClustering(data, profileType, var1AggregateBy, var2AggregateBy)

getDataClustering <- function(
    data = NULL, profileType = "binary", var1AggBy = "max", var2AggBy = "max"
) {
    if (is.null(data)) stop("Input data cannot be NULL!")
    presSpec <- geneID <- supertaxon <- orthoID <- NULL
    # Filter out rows where presSpec is 0
    subDataHeat <- data %>% dplyr::filter(presSpec > 0)
    
    # Predict orthoID format (default is "other")
    subDataHeat$orthoID <- as.character(subDataHeat$orthoID)
    idFormat <- ifelse(
        length(subDataHeat$orthoID) > 0 &&
            grepl(
                subDataHeat$supertaxonID[1],
                strsplit(subDataHeat$orthoID[1], '\\|', fixed = TRUE)[[1]][2]
            ), "bionf", "other"
    )
    # Process based on profileType
    if (profileType == "binary") {
        subDataHeat <- subDataHeat %>%
            dplyr::select(geneID, supertaxon, presSpec) %>%
            dplyr::mutate(presSpec = ifelse(presSpec > 0, 1, 0)) %>%
            dplyr::distinct()
        wideData <- data.table::dcast(
            data.table::setDT(subDataHeat),
            geneID ~ supertaxon, value.var = "presSpec"
        )
    } else if (profileType == "orthoID") {
        subDataHeat <- subDataHeat %>%
            dplyr::select(geneID, supertaxon, orthoID) %>%
            dplyr::mutate(
                orthoID = if (idFormat == "bionf") {
                    paste(
                        vapply(
                            strsplit(as.character(orthoID), "\\|"),
                            `[`, character(1), 2
                        ),
                        vapply(
                            strsplit(as.character(orthoID), "\\|"),
                            `[`, character(1), 3
                        ),
                        sep = "#"
                    )
                } else {
                    paste(supertaxon, orthoID, sep = "#")
                }
            ) %>% dplyr::distinct()
        # Assign numeric factor to orthoID
        subDataHeat <- subDataHeat %>%
            dplyr::mutate(orthoID = as.numeric(factor(orthoID)))
        wideData <- data.table::dcast(
            data.table::setDT(subDataHeat),
            geneID ~ supertaxon, value.var = "orthoID", fun.aggregate = sum
        )
    } else {
        var <- profileType
        subDataHeat <- subDataHeat %>%
            dplyr::select(geneID, supertaxon, !!sym(var)) %>%
            dplyr::distinct()
        # Aggregate values by selected method
        aggregateBy <- ifelse(var == "var1", var1AggBy, var2AggBy)
        subDataHeat <- subDataHeat %>%
            dplyr::group_by(geneID, supertaxon) %>%
            dplyr::summarise(
                !!sym(var) := match.fun(aggregateBy)(!!sym(var)),
                .groups = 'drop'
            )
        wideData <- data.table::dcast(
            data.table::setDT(subDataHeat),
            geneID ~ supertaxon, value.var = var
        )
    }
    # Convert to data frame and set geneID as row names
    wideData <- as.data.frame(wideData)
    dat <- wideData[, -1]
    rownames(dat) <- wideData$geneID
    # Replace NA with 0
    dat[is.na(dat)] <- 0
    return(dat)
}

#' Calculate the distance matrix
#' @export
#' @param profiles dataframe contains profile data for distance calculating
#' (see ?getDataClustering)
#' @param method distance calculation method ("euclidean", "maximum",
#' "manhattan", "canberra", "binary", "distanceCorrelation",
#' "mutualInformation" or "pearson" for binary data; "distanceCorrelation" or
#' "mutualInformation" for non-binary data). Default = "mutualInformation".
#' @return A calculated distance matrix for input phylogenetic profiles.
#' @importFrom bioDist mutualInfo
#' @importFrom bioDist cor.dist
#' @importFrom stats dist as.dist
#' @importFrom energy dcor
#' @importFrom Rfast Dist
#' @author Carla Mölbert (carla.moelbert@gmx.de), Vinh Tran
#' (tran@bio.uni-frankfurt.de)
#' @seealso \code{\link{getDataClustering}}
#' @examples
#' data("finalProcessedProfile", package="PhyloProfile")
#' data <- finalProcessedProfile
#' profileType <- "binary"
#' profiles <- getDataClustering(
#'     data, profileType, var1AggregateBy, var2AggregateBy)
#' method <- "mutualInformation"
#' getDistanceMatrix(profiles, method)

getDistanceMatrix <- function(profiles = NULL, method = "mutualInformation") {
    if (is.null(profiles)) stop("Profile data cannot be NULL!")
    profiles <-  profiles[, colSums(profiles != 0) > 0]
    profiles <-  profiles[rowSums(profiles != 0) > 0, ]
    distMethods <- c("euclidean", "maximum", "manhattan", "canberra", "binary")
    if (method %in% distMethods) {
        distanceMatrix <- Rfast::Dist(profiles, method = method)
        colnames(distanceMatrix)<-rownames(distanceMatrix)<-rownames(profiles)
    } else if (method == "distanceCorrelation") {
        n <- seq_len(nrow(profiles))
        matrix <- matrix(0L, nrow = nrow(profiles), ncol = nrow(profiles))
        for (i in seq_len(nrow(profiles))) { # rows
            p_i <- unlist(profiles[i,])
            for (j in seq_len(nrow(profiles))) { # columns
                if (i == j) break
                matrix[i, j] <- energy::dcor(p_i, unlist(profiles[j,]))
            }
        }
        # Swich the value so that the profiles with a high correlation
        # are clustered together
        matrix <- 1 - matrix
        matrix <- as.data.frame(matrix)
        profileNames <- rownames(profiles)
        colnames(matrix) <- profileNames[seq_len(length(profileNames)) - 1]
        rownames(matrix) <- profileNames
        distanceMatrix <- stats::as.dist(matrix)
    } else if (method == "mutualInformation") {
        distanceMatrix <- bioDist::mutualInfo(as.matrix(profiles))
        distanceMatrix <- max(distanceMatrix, na.rm = TRUE) - distanceMatrix
    } else if (method == "pearson") {
        distanceMatrix <-  bioDist::cor.dist(as.matrix(profiles))
    }
    return(as.matrix(distanceMatrix))
}

#' Create a hclust object from the distance matrix
#' @export
#' @param distanceMatrix calculated distance matrix as dist object
#' @param clusterMethod clustering method ("single", "complete",
#' "average" for UPGMA, "mcquitty" for WPGMA, "median" for WPGMC,
#' or "centroid" for UPGMC). Default = "complete".
#' @return An object class hclust generated based on input distance matrix and
#' a selected clustering method.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom fastcluster hclust
#' @seealso \code{\link{getDataClustering}},
#' \code{\link{getDistanceMatrix}}, \code{\link{hclust}}
#' @examples
#' data("finalProcessedProfile", package="PhyloProfile")
#' data <- finalProcessedProfile
#' profileType <- "binary"
#' profiles <- getDataClustering(
#'     data, profileType, var1AggregateBy, var2AggregateBy)
#' distMethod <- "mutualInformation"
#' distanceMatrix <- getDistanceMatrix(profiles, distMethod)
#' clusterMethod <- "complete"
#' clusterDataDend(as.dist(distanceMatrix), clusterMethod)

clusterDataDend <- function(distanceMatrix = NULL, clusterMethod = "complete") {
    if (is.null(distanceMatrix)) stop("Distance matrix cannot be NULL!")
    if (!inherits(distanceMatrix,"dist")) stop("Input must be a 'dist' object!")
    # Validate clustering method
    validMethods <- c(
        "complete", "ward.D", "ward.D2", "single", "average", "mcquitty",
        "median", "centroid"
    )
    if (!clusterMethod %in% validMethods) {
        stop(
            paste(
                "Invalid clustering method. Choose one of:",
                paste(validMethods, collapse = ", ")
            )
        )
    }
    # Perform clustering using fastcluster
    dendrogram <- fastcluster::hclust(distanceMatrix, method = clusterMethod)
    return(dendrogram)
}

#' Plot dendrogram tree
#' @export
#' @param dd dendrogram object (see ?clusterDataDend)
#' @return A dendrogram plot for the genes in the input phylogenetic profiles.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom ape plot.phylo
#' @seealso \code{\link{clusterDataDend}}
#' @examples
#' data("finalProcessedProfile", package="PhyloProfile")
#' data <- finalProcessedProfile
#' profileType <- "binary"
#' profiles <- getDataClustering(
#'     data, profileType, var1AggregateBy, var2AggregateBy)
#' distMethod <- "mutualInformation"
#' distanceMatrix <- getDistanceMatrix(profiles, distMethod)
#' clusterMethod <- "complete"
#' dd <- clusterDataDend(as.dist(distanceMatrix), clusterMethod)
#' getDendrogram(dd)

getDendrogram <- function(dd = NULL) {
    if (is.null(dd)) stop("Input dendrogram cannot be NULL!")
    p <- ape::plot.phylo(as.phylo(dd))
    return(p)
}

#' Function to keep user defined geneID order
#' @param data data frame contains gene ID column
#' @param orderType either "none", "alphabetically", "by profile similarity" or
#' "by a sorted list" (from input$orderGenes)
#' @param geneOrder named list of gene IDs. Either "more" (input list has more
#' genes than the main input), "missing" (some gene IDs are missing from the
#' input list) or "sortedGenes" (genes should be ordered by this list)
#' @return data either sorted or non-sorted
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}

sortGeneIDs <- function(data, orderType, geneOrder) {
    # Check if geneID is already a factor; if not, convert it
    if (!is.factor(data$geneID)) {
        data$geneID <- as.factor(data$geneID)
    }

    if (orderType == "none") {
        # Set factor levels to the current order in the data
        levels(data$geneID) <- unique(data$geneID)
    } else if (orderType == "user defined") {
        # Apply user-defined order only if `geneOrder` is valid
        if (!is.null(geneOrder) && "sortedGenes" %in% names(geneOrder)) {
            sortedGenes <- geneOrder$sortedGenes
            data$geneID <- factor(data$geneID, levels = sortedGenes)
        }
    }
    return(data)
}

#' Check installed packages
#' and install missing packages automatically
#' @param packages list of packages need to be checked
#' @return none
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}

installPackages <- function(packages){
    missingPackages <-
        packages[!(packages %in% installed.packages()[, "Package"])]
    if (length(missingPackages))
        install.packages(
            missingPackages,
            dependencies = TRUE,
            repos = "http://cran.us.r-project.org"
        )
}

installPackagesBioconductor <- function(packages){
    missingPackages <-
        packages[!(packages %in% installed.packages()[, "Package"])]
    if (length(missingPackages)) {
        if (!requireNamespace("BiocManager", quietly = TRUE))
            install.packages("BiocManager")
        BiocManager::install(missingPackages, ask = FALSE)
    }
}

#' Check internet connection
#' @return status of internet connection
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}

hasInternet <- function(){
    !is.null(curl::nslookup("r-project.org", error = FALSE))
}

#' Create link to public database
#' @return link to public database
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}

createDBlink <- function(id, source, type = "", version = ""){
    linkText <- ""
    url <- ""
    if (source == "NCBI") {
        url <- paste0("https://www.ncbi.nlm.nih.gov/protein/", id)
    } else if (source == "UniProt") {
        url <- paste0("https://www.uniprot.org/uniprot/", id)
    } else if (source == "OrthoDB") {
        if (version == "") {
            url <- paste0("https://www.orthodb.org/?query=", id)
        } else {
            version <- gsub("\\.", "-", version)
            url <- paste0("https://v", version, ".orthodb.org/?query=", id)
        }
        if (type == "gene") {
            idMod <- gsub(":", "%3A", id)
            if (version == "") {
                url <- paste0("https://www.orthodb.org/?gene=", idMod)
            } else {
                url <- paste0("https://v", version, ".orthodb.org/?gene=", idMod)
            }
        }
    } else if (source == "OMA") {
        url <- paste0("https://omabrowser.org/oma/omagroup/", id, "/members/")
        if (type == "gene") {
            url <- paste0("https://omabrowser.org/oma/info/", id)
        }
    }

    if (length(url > 0)) {
        if (RCurl::url.exists(url)) {
            linkText <- paste0(
                "<p><a href='", url, "' target='_blank'>",
                source, " entry for <strong>", id, "</strong></a></p>"
            )
        }
    }
    return(linkText)
}

#' Get the most specific taxonomy rank from input taxa
#' @return the most specific taxonomy rank
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}
getLowestRank <- function(longInput, taxDB) {
    inputTaxa <- gsub("ncbi","",levels(as.factor(longInput$ncbiID)))
    allDB <- getNameList(taxDB)
    allRanks <- getTaxonomyRanks()
    inputRanks <- levels(as.factor(allDB[allDB$ncbiID %in% inputTaxa,]$rank))
    if ("subspecies" %in% inputRanks) inputRanks <- c(inputRanks, "strain")
    inputRanks <- inputRanks[inputRanks %in% allRanks]
    return(allRanks[min(match(inputRanks, allRanks))][1])
}

#' Get colors for gene categoies
#' @param geneCategoryFile gene category file
#' @param type Either from manually uploaded file or predefined in config file
#' @return named vector of gene categories and their colors
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}

getCatColors <- function(geneCategoryFile, type = "file"){
    catColors <- NULL
    if (type == "file") {
        inputCatDt <- read.table(
            file = geneCategoryFile$datapath,
            sep = "\t",
            header = FALSE,
            check.names = FALSE,
            comment.char = "",
            fill = TRUE
        )

    } else if (type == "config"){
        inputCatDt <- read.table(
            file = geneCategoryFile,
            sep = "\t",
            header = FALSE,
            check.names = FALSE,
            comment.char = "",
            fill = TRUE
        )
    }
    if (ncol(inputCatDt) == 3) {
        catColorDf <- unique(inputCatDt[,c(2,3)])
        catColors <- as.character(catColorDf$V3)
        names(catColors) <- catColorDf$V2
    }
    return(catColors)
}

#' Get if ortho ID is in BIONF format (e.g. Q6PCB6|SACCE@4932@qfo|PROTID|1)
#' @param orthoID one ortholog ID
#' @param seedID seed ID of that ortholog
#' @param ncbiID ncbi ID of that ortholog
#' @return TRUE (if it is in BIONF format) or FALSE
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}

checkBionfFormat <- function(orthoID, seedID, ncbiID) {
    ortho <- strsplit(as.character(orthoID),'|',fixed = TRUE)[[1]]
    if (length(ortho) >= 3 && ortho[1] == seedID && grepl(ncbiID, ortho[2]))
        return(TRUE)
    return(FALSE)
}

#' Adapt plot size based on number of genes and taxa
#' @param nrTaxa number of taxa
#' @param nrGene number of genes
#' @param xAxis type of x-axis, either "taxa" or "genes"
#' @param dotZoom zoom factor for dots
#' @return a vector contains number of itemes in y-axis, plot height and width
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}

adaptPlotSize <- function(nrTaxa = 0, nrGene = 0, xAxis = "taxa", dotZoom = 0) {
    if (nrTaxa < 10000 && nrGene < 10000) {
        # adapte to axis type
        if (xAxis == "taxa") {
            h <- nrGene
            w <- nrTaxa
        } else {
            w <- nrGene
            h <- nrTaxa
        }
        # adapt to dot zoom factor
        if (dotZoom < -0.5){
            hv <- (200 + 12 * h) * (1 + dotZoom) + 500
            wv <- (200 + 12 * w) * (1 + dotZoom) + 500
        }  else if ((dotZoom < 0)) {
            hv <- (200 + 12 * h) * (1 + dotZoom) + 200
            wv <- (200 + 12 * w) * (1 + dotZoom) + 200
        } else {
            hv <- (200 + 12 * h) * (1 + dotZoom)
            wv <- (200 + 12 * w) * (1 + dotZoom)
        }
        # minimum size
        if (hv < 300) hv <- 300
        if (wv < 300) wv <- 300
        # update plot size based on number of genes/taxa
        hv <- hv + 300
        wv <- wv + 300
        return(c(h, hv, wv))
    } else return(c())
}

#' Scale a list of values into 0-1
#' @param x a numeric vector
#' @return a vector with values scaled from 0 to 1

scale01 <- function(x){(x-min(x))/(max(x)-min(x))}

#' Helper function to read single-column files
#' @param fileInput fileInput object
#' @return vector of unique values in input file

readSingleColFile <- function(fileInput) {
    if (!is.null(fileInput)) {
        labels <- readLines(fileInput$datapath)
        return(unique(labels))
    }
    return(NULL)
}

idWithSuffix <- function(base, suffix) {
    if (suffix == "") base else paste0(base, suffix)
}

# FUNCTIONS FOR RENDER UI ELEMENTS ============================================
createSliderCutoff <- function(id, title, start, stop, varID){
    if (is.null(varID)) return()
    if (varID == "") {
        sliderInput(id, title,
                    min = 1,
                    max = 1,
                    step = 0.025,
                    value = 1,
                    width = 200)
    } else {
        sliderInput(id, title,
                    min = 0,
                    max = 1,
                    step = 0.025,
                    value = c(start, stop),
                    width = 200)
    }
}

updateSliderCutoff <- function(session, id, title, newVar, varID){
    if (is.null(varID) || varID == "") return()
    updateSliderInput(session, id, title,
                      value = newVar,
                      min = 0,
                      max = 1,
                      step = 0.025)
}

createPlotSize <- function(id, title, value, width = 100) {
    numericInput(id,
                 title,
                 min = 100,
                 max = 3200,
                 step = 50,
                 value = value,
                 width = width)
}

createTextSize <- function(id, title, value, width = 100) {
    numericInput(id,
                 title,
                 min = 3,
                 max = 99,
                 step = 1,
                 value = value,
                 width = width)
}

#' Replace ~ symbol by the name of home folder
#' @export
#' @param fullPath complete path to input file or folder
#' @return complete path of input file or folder without ~ symbol
#' @author Vinh Tran {tran@bio.uni-frankfurt.de}
#' @import stringr
#' @examples
#' replaceHomeCharacter("~/path/to/something")
replaceHomeCharacter <- function (fullPath = NULL) {
    if (!(Sys.info()['sysname'] == "Windows")){
        homeName <- system("echo $HOME", intern = TRUE)
        return(stringr::str_replace(fullPath, "~", homeName))
    } else return(fullPath)
}

substrRight <- function(x, n) {
    substr(x, nchar(x)-n+1, nchar(x))
}

substrLeft <- function(x, n) {
    substr(x, 1, n)
}

# Functions for parsing and pre-processing PhyloProfile input

#' Create a list containing all main taxanomy ranks
#' @export
#' @return A list of all main ranks (from strain to superkingdom)
#' @author Carla Mölbert (carla.moelbert@gmx.de)
#' @examples
#' getTaxonomyRanks()

getTaxonomyRanks <- function(){
    allRanks <- list(
        "Strain " = "strain",
        "Species" = "species",
        "Genus" = "genus",
        "Family" = "family",
        "Order" = "order",
        "Class" = "class",
        "Phylum" = "phylum",
        "Kingdom" = "kingdom",
        "Superkingdom" = "superkingdom",
        "unselected" = ""
    )
    return(allRanks)
}

#' Get list of pre-installed NCBI taxon names
#' @description Get all NCBI taxon names from
#' "PhyloProfile/data/taxonNamesReduced.txt"
#' @export
#' @param taxDB Path to the taxonomy DB files
#' @return List of taxon IDs, their full names, taxonomy ranks and parent IDs
#' obtained from "PhyloProfile/data/taxonNamesReduced.txt"
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @examples
#' getNameList()

getNameList <- function(taxDB = NULL) {
    if (is.null(taxDB)) {
        nameReducedFile <- paste(
            system.file(package = "PhyloProfile"),
            "PhyloProfile/data/taxonNamesReduced.txt", sep = "/"
        )
    } else nameReducedFile <- paste(taxDB, "taxonNamesReduced.txt", sep = "/")

    if (!file.exists(nameReducedFile)) {
        utils::data(taxonNamesReduced)
    } else {
        taxonNamesReduced <- data.table::fread(
            nameReducedFile, sep = "\t", header = TRUE, fill = TRUE,
            showProgress = FALSE
        )
    }

    taxonNamesReduced$fullName <- as.character(taxonNamesReduced$fullName)
    taxonNamesReduced$rank <- as.character(taxonNamesReduced$rank)
    taxonNamesReduced <- unique(taxonNamesReduced)

    return(taxonNamesReduced)
}

#' Get taxonomy matrix
#' @description Get the (full or subset) taxonomy matrix from
#' "data/taxonomyMatrix.txt" based on an input taxon list
#' @export
#' @param taxDB Path to the taxonomy DB files
#' @param subsetTaxaCheck TRUE/FALSE subset taxonomy matrix based on input taxon
#' IDs. Default = FALSE
#' @param taxonIDs list of input taxon IDs (e.g. ncbi1234). Default = NULL
#' @return Data frame contains the (subset of) taxonomy matrix for list of
#' input taxa.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @examples
#' # get full pre-installed taxonomy matrix
#' getTaxonomyMatrix()
#' # get taxonomy matrix for a list of taxon IDs
#' taxonIDs <- c("ncbi9606", "ncbi10116")
#' getTaxonomyMatrix(NULL, TRUE, taxonIDs)

getTaxonomyMatrix <- function(
        taxDB = NULL, subsetTaxaCheck = FALSE, taxonIDs = NULL
){
    taxonomyMatrixFile <- if (is.null(taxDB)) {
        file.path(
            system.file(package = "PhyloProfile"),
            "PhyloProfile/data/taxonomyMatrix.txt"
        )
    } else {
        file.path(taxDB, "taxonomyMatrix.txt")
    }
    if (!file.exists(taxonomyMatrixFile)) {
        utils::data(taxonomyMatrix)
    } else {
        taxonomyMatrix <- data.table::fread(
            taxonomyMatrixFile, sep = "\t", header = TRUE,
            stringsAsFactors = TRUE, showProgress = FALSE
        )
    }
    if (subsetTaxaCheck) {
        if (!missing(taxonIDs)) {
            taxonomyMatrix <- taxonomyMatrix[
                taxonomyMatrix$abbrName %in% taxonIDs,]
        }
    }
    return(data.frame(taxonomyMatrix))
}

#' Get ID list of input taxa from the main input
#' @param rawProfile A dataframe of input phylogenetic profile in long format
#' @return List of all input taxon IDs (e.g. ncbi1234). Default = NULL.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @export
#' @seealso \code{\link{createLongMatrix}}, \code{\link{mainLongRaw}}
#' @examples
#' data("mainLongRaw", package="PhyloProfile")
#' getInputTaxaID(mainLongRaw)

getInputTaxaID <- function(rawProfile = NULL){
    if (is.null(rawProfile)) stop("Input profile data cannot be NULL!")
    inputTaxa <- unique(rawProfile$ncbiID)
    inputTaxa <- inputTaxa[inputTaxa != "geneID"]
    # return input taxa
    return(inputTaxa)
}

#' Get NCBI taxon names for a selected list of taxa
#' @description Get NCBI taxon names from
#' "PhyloProfile/data/taxonNamesReduced.txt" for a list of input taxa
#' @param rankName taxonomy rank (e.g. "species","phylum",...)
#' @param taxonIDs list of taxon IDs (e.g. ncbi1234). Default = NULL
#' @param taxDB Path to the taxonomy DB files
#' @return Data frame contains a list of full names, taxonomy ranks and parent
#' IDs for the input taxa.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @export
#' @seealso \code{\link{getInputTaxaID}} for getting input taxon IDs,
#' \code{\link{getNameList}} for getting the full taxon name list
#' @examples
#' taxonIDs <- c("ncbi9606", "ncbi10116")
#' getInputTaxaName("species", taxonIDs)

getInputTaxaName <- function(rankName, taxonIDs = NULL, taxDB = NULL){
    # check input parameters
    if (missing(rankName)) stop("No taxonomy rank name given!")
    allMainRanks <- getTaxonomyRanks()
    if (!(rankName[1] %in% allMainRanks)) stop("Invalid taxonomy rank given!")
    # load list of unsorted taxa
    taxMatrix <- getTaxonomyMatrix(
        taxDB, subsetTaxaCheck = TRUE, taxonIDs = taxonIDs
    )
    # load list of taxon name
    nameList <- getNameList(taxDB)
    # return
    choice <- data.frame(
        "ncbiID" = unlist(taxMatrix[[rankName]]), stringsAsFactors = FALSE
    )
    choice <- merge(choice, nameList, by = "ncbiID", all = FALSE)
    return(choice)
}

#' Get a subset of input taxa based on a selected taxonomy rank
#' @description Get a subset of taxon ncbi IDs and names from an input list of
#' taxa based on a selected supertaxon (identified by its taxonomy rank and
#' supertaxon name or supertaxon ID).
#' @usage getSelectedTaxonNames(inputTaxonIDs = NULL, rank = NULL,
#'     higherRank = NULL, higherID = NULL, higherName = NULL, taxDB = NULL)
#' @param inputTaxonIDs list of input taxon IDs (e.g. c("10116", "122586"))
#' @param rank taxonomy rank of input taxa (e.g. "species")
#' @param higherRank selected taxonomy rank (e.g. "phylum")
#' @param higherID supertaxon ID (e.g. 7711). NOTE: either supertaxon ID or
#' name is required, not neccessary to give both
#' @param higherName supertaxon name (e.g. "Chordata"). NOTE: either
#' supertaxon ID or name is required, not neccessary to give both
#' @param taxDB Path to the taxonomy DB files
#' @export
#' @return A data frame contains ncbi IDs and names of taxa from the input taxon
#' list that belong to the selected supertaxon.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @examples
#' inputTaxonIDs <- c("10116", "122586", "123851", "13616", "188937", "189518",
#' "208964", "224129", "224324", "237631", "243230")
#' rank <- "species"
#' higherRank <- "phylum"
#' higherID <- 7711
#' getSelectedTaxonNames(inputTaxonIDs, rank, higherRank, higherID, NULL)
#' higherName <- "Chordata"
#' getSelectedTaxonNames(inputTaxonIDs, rank, higherRank, NULL, higherName,NULL)

getSelectedTaxonNames <- function(
    inputTaxonIDs = NULL, rank = NULL,
    higherRank = NULL, higherID = NULL, higherName = NULL, taxDB = NULL
) {
    rankName <- NULL
    if (is.null(inputTaxonIDs) | is.null(rank))
        stop("Input taxa and taxonomy rank cannot be NULL!")
    taxDf <- getTaxonomyMatrix(taxDB, TRUE, paste0("ncbi", inputTaxonIDs))
    if (is.null(higherID) & is.null(higherName))
        return(
            data.frame(
                ncbiID = taxDf$ncbiID[
                    taxDf$abbrName %in% paste0("ncbi", inputTaxonIDs)],
                name = taxDf$fullName[
                    taxDf$abbrName %in% paste0("ncbi", inputTaxonIDs)],
                stringsAsFactors = FALSE))
    if (is.null(higherRank)) {
        return(
            data.frame(
                ncbiID = taxDf$ncbiID[
                    taxDf$abbrName %in% paste0("ncbi", inputTaxonIDs)],
                name = taxDf$fullName[
                    taxDf$abbrName %in% paste0("ncbi", inputTaxonIDs)],
                stringsAsFactors = FALSE))
    } else {
        if (!is.null(higherName) & is.null(higherID)) {
            taxaList <- getNameList(taxDB)
            superID <- taxaList$ncbiID[
                taxaList$fullName == higherName
                & taxaList$rank %in% c(higherRank, "norank")]
            customizedtaxaID <- levels(
                as.factor(taxDf[rank][taxDf[higherRank] == superID, ]))
            return(
                data.frame(
                    ncbiID = taxaList$ncbiID[
                        taxaList$rank %in% c(rank, "norank")
                        & taxaList$ncbiID %in% customizedtaxaID],
                    name = taxaList$fullName[
                        taxaList$rank %in% c(rank, "norank")
                        & taxaList$ncbiID %in% customizedtaxaID],
                    stringsAsFactors = FALSE))
        } else if (!is.null(higherID)) {
            return(
                data.frame(
                    ncbiID = taxDf$ncbiID[taxDf[,higherRank] == higherID],
                    name = taxDf$fullName[taxDf[,higherRank] == higherID],
                    stringsAsFactors = FALSE))
        }
    }
}

#' Sort list of (super)taxa based on a selected reference (super)taxon
#' @usage sortInputTaxa(taxonIDs = NULL, rankName, refTaxon = NULL,
#'     taxaTree = NULL, sortedTaxonList = NULL, taxDB = NULL)
#' @param taxonIDs list of taxon IDs (e.g.: ncbi1234, ncbi9999, ...). Default =
#' NULL
#' @param rankName working taxonomy rank (e.g. "species", "phylum",...)
#' @param refTaxon selected reference taxon. Default = NULL
#' @param taxaTree taxonomy tree for the input taxa (optional). Default = NULL
#' @param sortedTaxonList list of sorted taxa (optional). Default = NULL
#' @param taxDB Path to the taxonomy DB files
#' @return A taxonomy matrix for the input taxa ordered by the selected
#' reference taxon. This matrix is sorted either based on the NCBI taxonomy
#' info, or based on an user-defined taxonomy tree (if provided).
#' @importFrom ape read.tree root
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @export
#' @seealso \code{\link{getNameList}}, \code{\link{getTaxonomyMatrix}},
#' \code{\link{createUnrootedTree}}, \code{\link{sortTaxaFromTree}},
#' \code{\link{getInputTaxaName}}, \code{\link{getInputTaxaID}},
#' \code{\link{createLongMatrix}}
#' @examples
#' taxonIDs <- c(
#'     "ncbi10116", "ncbi123851", "ncbi3702", "ncbi13616", "ncbi9606"
#' )
#' sortInputTaxa(taxonIDs, "species", "Homo sapiens", NULL, NULL)

sortInputTaxa <- function(
    taxonIDs = NULL, rankName, refTaxon = NULL, taxaTree = NULL,
    sortedTaxonList = NULL, taxDB = NULL
){
    ncbiID <- fullName <- abbrName <- NULL
    if (missing(rankName)) return("No taxonomy rank name given!")
    allMainRanks <- getTaxonomyRanks()
    if (!(rankName[1] %in% allMainRanks)) return("Invalid taxonomy rank given!")
    # get list of taxon names
    fullnameList <- getNameList(taxDB)
    taxonNames <- getInputTaxaName(rankName, taxonIDs, taxDB)
    if (is.null(refTaxon))  refTaxon <- taxonNames$fullName[1]
    # get selected supertaxon ID(s)
    rankNameTMP <- taxonNames$rank[taxonNames$fullName == refTaxon]
    if (rankName == "strain") {
        superID <- fullnameList$ncbiID[fullnameList$fullName == refTaxon]
    } else
        superID <- fullnameList$ncbiID[
            fullnameList$fullName == refTaxon
            & fullnameList$rank == rankNameTMP[1]]
    # get full taxonomy data & representative taxon
    Dt <- getTaxonomyMatrix(taxDB)
    repTaxon <- Dt[Dt[, rankName] == superID, ][1, ]
    # THEN, SORT TAXON LIST BASED ON TAXONOMY TREE or SORTED TAXON LIST
    if (is.null(sortedTaxonList)) {
        if (is.null(taxaTree)) {
            if (is.null(taxDB)) {
                preCalcTreeFile <- paste(
                    system.file(package = "PhyloProfile"),
                    "PhyloProfile/data/preCalcTree.nw", sep = "/"
                )
            } else preCalcTreeFile <- paste(taxDB, "preCalcTree.nw", sep = "/")

            if (file.exists(preCalcTreeFile)) {
                preTree <- ape::read.tree(preCalcTreeFile)
            } else preTree <- createUnrootedTree(Dt)

            if (!(repTaxon$abbrName %in% preTree$tip.label))
                message(c(repTaxon$abbrName, " not found in ", preCalcTreeFile))
            taxaTree <- ape::root(
                preTree, outgroup=as.character(repTaxon$abbrName),
                resolve.root=TRUE
            )
        } else
            taxaTree <- ape::root(
                taxaTree, outgroup=as.character(repTaxon$abbrName),
                resolve.root=TRUE
            )
        taxonList <- sortTaxaFromTree(taxaTree)
    } else {
        taxonList <- sortedTaxonList
    }
    sortedDt <- Dt[match(taxonList, Dt$abbrName), ]
    # subset to get list of input taxa only
    sortedDt <- subset(sortedDt, abbrName %in% taxonIDs)
    # get only taxonIDs list of selected rank and rename columns
    sortedOut <- subset(
        sortedDt,
        select = c("abbrName", "ncbiID", "fullName", as.character(rankName)))
    colnames(sortedOut) <- c("abbrName", "species", "fullName", "ncbiID")
    # add name of supertaxa into sortedOut list
    sortedOut <- merge(
        sortedOut, fullnameList, by = "ncbiID", all.x = TRUE, sort = FALSE)
    sortedOut$species <- paste0("ncbi", sortedOut$species)
    ## create new column for sorted supertaxon
    indexSpec <- unlist(lapply(
        seq_len(nlevels(as.factor(sortedOut$fullName.y))),function (x) 100000+x)
    )
    indexSpecDf <- data.frame(
        fullName.y = unique(as.character(sortedOut$fullName.y)),
        sortedSupertaxon = paste0(
            indexSpec, "_", unique(as.character(sortedOut$fullName.y))
        ), stringsAsFactors = FALSE)
    sortedOut <- merge(indexSpecDf, sortedOut, by = "fullName.y")
    # final sorted supertaxa list
    sortedOut$taxonID <- 0
    sortedOut$category <- "cat"
    sortedOut <- sortedOut[, c(
        "abbrName", "taxonID", "fullName.x", "species", "ncbiID",
        "sortedSupertaxon", "rank", "category")]
    colnames(sortedOut) <- c(
        "abbrName", "taxonID", "fullName", "ncbiID", "supertaxonID",
        "supertaxon", "rank", "category")
    sortedOut$ncbiID <- as.factor(sortedOut$ncbiID)
    sortedOut$supertaxon <- as.factor(sortedOut$supertaxon)
    sortedOut$category <- as.factor(sortedOut$category)
    return(sortedOut)
}

#' Calculate percentage of present species in each super taxon
#' @export
#' @param profileWithTax data frame of main PhyloProfile input together
#' with their taxonomy info (see ?profileWithTaxonomy)
#' @param taxaCount number of species occur in each supertaxon (e.g. phylum
#' or kingdom)
#' @return A data frame with % of present species in each supertaxon
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom dplyr count select distinct summarise group_by inner_join mutate
#' filter
#' @seealso \code{\link{profileWithTaxonomy}} for a demo input data
#' @examples
#' # NOTE: for internal testing only
#' library(dplyr)
#' data("profileWithTaxonomy", package="PhyloProfile")
#' taxaCount <- profileWithTaxonomy %>% dplyr::count(supertaxon)
#' taxaCount$n <- 1
#' calcPresSpec(profileWithTaxonomy, taxaCount)

calcPresSpec <- function(profileWithTax, taxaCount) {
    if (missing(profileWithTax)) stop("No input data given")
    if (missing(taxaCount)) stop("No supertaxon count given")
    geneID <- supertaxon <- paralog <- abbrName <- NULL
    presSpec <- presentTaxa <- totalTaxa <- NULL
    # Check if  working with the lowest taxonomy rank
    if (all(profileWithTax$numberSpec == 1)) {
        profileWithTax <- profileWithTax %>% select(geneID, supertaxon) %>%
            mutate(presentTaxa = 1, presSpec = as.double(1), totalTaxa = 1) %>%
            arrange(geneID, supertaxon) %>% distinct()
        return(profileWithTax)
    }
    # Filter out rows with missing orthoID to reduce subsequent data size
    profileWithTax <- profileWithTax[
        !is.na(profileWithTax$orthoID) & profileWithTax$orthoID != "NA",
    ]
    # Pre-calculate geneID and supertaxon combinations for merging later
    geneIDSupertaxon <- profileWithTax %>%
        select(geneID, supertaxon, paralog, abbrName) %>% distinct()
    # Use summarise directly instead of count for better control
    geneSupertaxonCount <- geneIDSupertaxon %>% group_by(geneID, supertaxon) %>%
        summarise(presentTaxa = n(), .groups = "drop")
    # Perform the join with taxaCount and calculate presSpec efficiently
    presSpecDt <- geneSupertaxonCount %>%
        inner_join(taxaCount, by = "supertaxon") %>%
        mutate(presSpec = presentTaxa / n, totalTaxa = n) %>%
        select(-n) %>% filter(presSpec <= 1)
    # Merge with geneIDSupertaxon for missing combinations and fill defaults
    finalPresSpecDt <- geneIDSupertaxon %>%
        left_join(presSpecDt, by = c("geneID", "supertaxon")) %>%
        mutate(
            presSpec = ifelse(is.na(presSpec), 0, presSpec),
            presentTaxa = ifelse(is.na(presentTaxa), 0, presentTaxa),
            totalTaxa = ifelse(is.na(totalTaxa), 0, totalTaxa)
        ) %>% select(-c(abbrName, paralog))
    # Remove duplicates and sort results
    finalPresSpecDt <- finalPresSpecDt %>% arrange(geneID, supertaxon) %>%
        distinct()
    return(finalPresSpecDt)
}

#' Parsing info for phylogenetic profiles
#' @description Creating main dataframe for the input phylogenetic profiles
#' based on selected input taxonomy level (e.g. strain, species) and reference
#' taxon. The output contains the number of paralogs, the max/min/mean/median
#' of VAR1 and VAR2.
#' @usage parseInfoProfile(inputDf, sortedInputTaxa, taxaCount, coorthoCOMax)
#' @param inputDf input profiles in long format
#' @param sortedInputTaxa sorted taxonomy data for the input taxa
#' (check sortInputTaxa())
#' @param taxaCount dataframe counting present taxa in each supertaxon
#' @param coorthoCOMax maximum number of co-orthologs allowed
#' @return A dataframe contains all info for the input phylogenetic profiles.
#' This full processed profile that is required for several profiling analyses
#' e.g. estimation of gene age (?estimateGeneAge) or identification of core gene
#' (?getCoreGene).
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom dplyr count
#' @export
#' @seealso \code{\link{createLongMatrix}}, \code{\link{sortInputTaxa}},
#' \code{\link{calcPresSpec}}, \code{\link{mainLongRaw}}
#' @examples
#' library(dplyr)
#' data("mainLongRaw", package="PhyloProfile")
#' taxonIDs <- getInputTaxaID(mainLongRaw)
#' sortedInputTaxa <- sortInputTaxa(
#'     taxonIDs, "class", "Mammalia", NULL, NULL
#' )
#' taxaCount <- sortedInputTaxa %>% dplyr::group_by(supertaxon) %>%
#'     summarise(n = n(), .groups = "drop")
#' coorthoCOMax <- 999
#' parseInfoProfile(
#'     mainLongRaw, sortedInputTaxa, taxaCount, coorthoCOMax
#' )

parseInfoProfile <- function(
        inputDf, sortedInputTaxa, taxaCount, coorthoCOMax = 9999
) {
    if (is.null(inputDf) | is.null(sortedInputTaxa))
        stop("Input profiles and sorted taxonomy data cannot be NULL!")
    geneID <- ncbiID <- paralog <- geneName <- NULL
    if (ncol(inputDf) > 3) {
        if (ncol(inputDf) < 5) {colnames(inputDf)[4] <- "var1"}
        else if (ncol(inputDf) < 6) {
            colnames(inputDf)[c(4,5)] <- c("var1", "var2")
        }
    }
    if (coorthoCOMax < 1) coorthoCOMax <- 1
    # count number of inparalogs & calculate frequency of all supertaxa
    inputDf <- inputDf %>%
        dplyr::group_by(geneID, ncbiID) %>%
        dplyr::mutate(paralog = dplyr::n()) %>%
        dplyr::ungroup()
    # filter by number of coorthologs
    inputDf <- inputDf %>% filter(!(paralog > coorthoCOMax))
    # add tax info from sortedInputTaxa and number of species from taxaCount
    fullMdData <- inputDf %>%
        left_join(sortedInputTaxa, by = "ncbiID") %>%
        left_join(taxaCount, by = "supertaxon") %>%
        distinct()
    fullMdData$fullName <- as.character(fullMdData$fullName)
    # add geneName column if not yet exist
    if (!("geneName" %in% colnames(fullMdData)))
        fullMdData$geneName <- fullMdData$geneID
    # Sort `geneName` based on the order of `geneID`
    # Extract unique gene names for each geneID
    orderedName <- fullMdData %>%
        dplyr::group_by(geneID) %>%
        dplyr::reframe(geneName = unique(as.character(geneName))) %>%
        dplyr::pull(geneName)
    # Update `geneName` as a factor with the ordered levels
    fullMdData$geneName <- factor(fullMdData$geneName, levels = orderedName)
    # Rename column "n" to "numberSpec"
    fullMdData <- fullMdData %>% dplyr::rename(numberSpec = n)
    return(fullMdData)
}


#' Filter phylogentic profiles
#' @description Create a filtered data needed for plotting or clustering
#' phylogenetic profiles. NOTE: this function require some intermediate steps
#' using the results from other functions. If you would like to get a full
#' processed data from the raw input, please use the function
#' fromInputToProfile() instead!
#' @usage filterProfileData(DF, taxaCount, refTaxon = NULL,
#'     percentCO = c(0, 1), coorthoCOMax = 9999,
#'     var1CO  = c(0, 1), var2CO = c(0, 1), var1Rel = "protein",
#'     var2Rel = "protein", groupByCat = FALSE, catDt = NULL,
#'     var1AggregateBy = "max", var2AggregateBy = "max")
#' @param DF a reduced dataframe contains info for all phylogenetic
#' profiles in the selected taxonomy rank.
#' @param taxaCount dataframe counting present taxa in each supertaxon
#' @param refTaxon selected reference taxon. NOTE: This taxon will not be
#' affected by the filtering. If you want to filter all, set refTaxon <- NULL.
#' Default = NULL.
#' @param percentCO min and max cutoffs for percentage of species present
#' in a supertaxon. Default = c(0, 1).
#' @param coorthoCOMax maximum number of co-orthologs allowed. Default =
#' 9999.
#' @param var1CO min and max cutoffs for var1. Default = c(0, 1).
#' @param var2CO min anc max cutoffs for var2. Default = c(0, 1).
#' @param var1Rel relation of var1 ("protein" for protein-protein or
#' "species" for protein-species). Default = "protein".
#' @param var2Rel relation of var2 ("protein" for protein-protein or
#' "species" for protein-species). Default = "protein".
#' @param groupByCat group genes by their categories (TRUE or FALSE). Default =
#' FALSE.
#' @param catDt dataframe contains gene categories
#' (optional, NULL if groupByCat = FALSE or no info provided). Default = NULL.
#' @param var1AggregateBy aggregate method for VAR1 (max, min, mean
#' or median), applied for calculating var1 of supertaxa. Default = "max".
#' @param var2AggregateBy aggregate method for VAR2 (max, min, mean
#' or median), applied for calculating var2 of supertaxa. Default = "max".
#' @return A filtered dataframe for generating profile plot including seed gene
#' IDs (or orthologous group IDs), their ortholog IDs and the corresponding
#' (super)taxa, (super)taxon IDs, number of co-orthologs in each (super)taxon,
#' values for two additional variables var1, var2, % of species present in each
#' supertaxon, and the categories of seed genes (or ortholog groups).
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom stats complete.cases aggregate
#' @export
#' @seealso \code{\link{parseInfoProfile}} and \code{\link{reduceProfile}}
#' for generating input dataframe, \code{\link{fullProcessedProfile}} for a
#' demo full processed profile dataframe, \code{\link{fromInputToProfile}} for
#' generating fully processed data from raw input.
#' @examples
#' # NOTE: this function require some intermediate steps using the results from
#' # other functions. If you would like to get a full processed data from the
#' # raw input, please use the function fromInputToProfile() instead!
#' library(dplyr)
#' data("fullProcessedProfile", package="PhyloProfile")
#' rankName <- "class"
#' refTaxon <- "Mammalia"
#' percentCutoff <- c(0.0, 1.0)
#' coorthologCutoffMax <- 10
#' var1Cutoff <- c(0.75, 1.0)
#' var2Cutoff <- c(0.5, 1.0)
#' var1Relation <- "protein"
#' var2Relation <- "species"
#' groupByCat <- FALSE
#' catDt <- NULL
#' var1AggregateBy <- "max"
#' var2AggregateBy <- "max"
#' taxonIDs <- levels(as.factor(fullProcessedProfile$ncbiID))
#' sortedInputTaxa <- sortInputTaxa(
#'     taxonIDs, rankName, refTaxon, NULL, NULL
#' )
#' taxaCount <- sortedInputTaxa %>% dplyr::group_by(supertaxon) %>%
#'     summarise(n = n(), .groups = "drop")
#' filterProfileData(
#'     fullProcessedProfile,
#'     taxaCount,
#'     refTaxon,
#'     percentCutoff,
#'     coorthologCutoffMax,
#'     var1Cutoff,
#'     var2Cutoff,
#'     var1Relation,
#'     var2Relation,
#'     groupByCat,
#'     catDt,
#'     var1AggregateBy,
#'     var2AggregateBy
#' )

filterProfileData <- function(
    DF, taxaCount, refTaxon = NULL, percentCO = c(0, 1), coorthoCOMax = 9999,
    var1CO = c(0, 1), var2CO = c(0, 1), var1Rel = "protein",
    var2Rel = "protein", groupByCat = FALSE, catDt = NULL,
    var1AggregateBy = "max", var2AggregateBy = "max"
) {
    if (is.null(DF)) stop("Profile data cannot be NULL!")
    if (is.null(refTaxon)) refTaxon <- "NA"
    geneID <- supertaxon <- value <- NULL
    flag <- ifelse(all(DF$numberSpec == 1), 0, 1)

    DF$taxonMod <- sub("^[[:digit:]]*_", "", DF$supertaxon)
    filter_var <- function(var, varCO, rel, refTaxon, taxonMod) {
        if (rel == "protein") {
            var[taxonMod != refTaxon & (var < varCO[1] | var > varCO[2])] <- NA
        } else {
            var[taxonMod != refTaxon & (var < varCO[1] | var > varCO[2])] <- 0
        }
        return(var)
    }
    DF$var1 <- filter_var(DF$var1, var1CO, var1Rel, refTaxon, DF$taxonMod)
    DF$var2 <- filter_var(DF$var2, var2CO, var2Rel, refTaxon, DF$taxonMod)
    
    DFtmp <- DF[stats::complete.cases(DF), ]
    finalPresSpecDt <- calcPresSpec(DFtmp, taxaCount)
    DF <- DF %>% left_join(finalPresSpecDt, by = c("geneID", "supertaxon"))
    DF$presSpec <- ifelse(
        DF$presSpec < percentCO[1] | DF$presSpec > percentCO[2], 0, DF$presSpec
    )

    # Convert orthoID to character and set to NA where presSpec == 0
    DF$orthoID <- ifelse(DF$presSpec == 0, NA, as.character(DF$orthoID))
    # Conditionally update var1 and var2 based on orthoID being NA
    if (var1Rel == "protein") DF$var1[is.na(DF$orthoID)] <- NA
    if (var2Rel == "protein") DF$var2[is.na(DF$orthoID)] <- NA
    # Set presSpec to 0 where orthoID is NA
    DF$presSpec[is.na(DF$orthoID)] <- 0
    # Set paralog to 1 if flag == 1
    if (flag == 1) DF$paralog <- 1
    # Dropping unused levels and ensuring geneID and supertaxon are factors
    DF <- DF %>% droplevels() %>% mutate(
        geneID = factor(geneID), supertaxon = factor(supertaxon)
    )

    aggregate_scores <- function(df, var, aggBy) {
        if (nrow(df) == 0) {
            # Return an empty data frame
            return(data.frame(
                supertaxon=character(0), geneID=character(0), value=numeric(0)
            ))
        }
        aggregated <- stats::aggregate(
            df[[var]], by = list(df$supertaxon, df$geneID), FUN = aggBy
        )
        colnames(aggregated) <- c("supertaxon", "geneID", "value")
        return(aggregated)
    }

    mVar1Dt <- aggregate_scores(
        DF[!is.na(DF$var1), ], "var1", var1AggregateBy
    ) %>% dplyr::rename(mVar1 = value)

    mVar2Dt <- aggregate_scores(DF[!is.na(DF$var2), ], "var2", var2AggregateBy)
    if (nrow(mVar2Dt) == 0) {
        mVar2Dt <- unique(DF[, c("supertaxon", "geneID")]) %>%
            dplyr::mutate(mVar2 = 0)
    } else {
        mVar2Dt <- mVar2Dt %>% dplyr::rename(mVar2 = value)
    }

    scoreDf <- dplyr::full_join(mVar1Dt, mVar2Dt, by = c("supertaxon","geneID"))
    DF <- dplyr::left_join(DF, scoreDf, by = c("geneID", "supertaxon"))

    if (groupByCat) {
        if (is.null(catDt)) {
            catDt <- data.frame(geneID = levels(DF$geneID), group="noCategory")
        }
        dfCat <- merge(
            expand.grid(
                supertaxon = levels(DF$supertaxon), geneID = levels(DF$geneID)
            ), catDt, by = "geneID", all.x = TRUE
        )
        DF <- dplyr::left_join(dfCat, DF, by = c("geneID", "supertaxon"))
        DF$category <- DF$group
    }

    return(DF)
}

#' Reduce the filtered profile data into supertaxon level
#' @description Reduce data of the processed phylogenetic profiles from input
#' taxonomy rank into supertaxon level (e.g. from species to phylum)
#' @param filteredProfile dataframe contains the filtered profiles (see
#' ?parseInfoProfile, ?filterProfileData and ?filteredProfile)
#' @return A reduced dataframe contains only profile data for the selected
#' supertaxon rank. This dataframe contains only supertaxa and their value
#' (mVar1 & mVar2) for each gene.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @export
#' @importFrom dplyr tibble
#' @seealso \code{\link{parseInfoProfile}} for creating a full processed
#' profile dataframe, \code{\link{filterProfileData}} for filter processed
#' profile and \code{\link{filteredProfile}} for a demo filtered
#' profile dataframe
#' @examples
#' data("filteredProfile", package="PhyloProfile")
#' reduceProfile(filteredProfile)

reduceProfile <- function(filteredProfile) {
    if (is.null(filteredProfile)) stop("Profile data cannot be NULL!")
    # check if working with the lowest taxonomy rank; 1 for NO; 0 for YES
    flag <- 1
    if (length(unique(levels(as.factor(filteredProfile$numberSpec)))) == 1) {
        if (unique(levels(as.factor(filteredProfile$numberSpec))) == 1) {
            superDfExt <- filteredProfile[, c(
                "geneID", "supertaxon", "supertaxonID", "orthoID", "category",
                "presSpec", "paralog", "mVar1", "mVar2", "geneName"
            )]
            superDfExt$presentTaxa <- 1
            superDfExt$totalTaxa <- 1
            # rename mVar to var
            names(superDfExt)[names(superDfExt) == "mVar1"] <- "var1"
            names(superDfExt)[names(superDfExt) == "mVar2"] <- "var2"
            flag <- 0
        }
    }
    if (flag == 1) {
        # get representative orthoID that has m VAR1 for each supertaxon
        mOrthoID <- filteredProfile[, c(
            "geneID", "supertaxon", "var1", "mVar1", "orthoID", "presSpec",
            "geneName"
        )]
        mOrthoID$sVar1 <- mOrthoID$mVar1 - mOrthoID$var1
        # mOrthoID <- subset(mOrthoID, mOrthoID$var1 == mOrthoID$mVar1)
        mOrthoID <- subset(mOrthoID, mOrthoID$sVar1 <= 0)
        colnames(mOrthoID) <- c(
            "geneID","supertaxon","var1","mVar1","orthoID","presSpec","sVar1",
            "geneName"
        )
        mOrthoID <- mOrthoID[order(mOrthoID$sVar1, decreasing = TRUE), ]
        mOrthoID <- mOrthoID[!is.na(mOrthoID$orthoID), ]
        mOrthoID <- mOrthoID[, c("geneID", "supertaxon", "orthoID", "presSpec")]
        mOrthoID <- mOrthoID[!duplicated(mOrthoID[, seq_len(2)]), ]
        # get data set for PhyloProfile plotting (contains only supertaxa info)
        superDf <- subset(filteredProfile, select = c(
            "geneID", "supertaxon", "supertaxonID", "mVar1", "category",
            "mVar2", "paralog", "presentTaxa", "totalTaxa", "geneName"
        ))
        superDf <- superDf[!duplicated(superDf), ]
        superDfExt <- merge(
            superDf, mOrthoID, by = c("geneID", "supertaxon"), all.x = TRUE
        )
        superDfExt <- superDfExt[, c(
            "geneID", "supertaxon", "supertaxonID",
            "mVar1", "presSpec", "category", "orthoID", "mVar2", "paralog",
            "presentTaxa", "totalTaxa", "geneName"
        )]
        # rename mVar to var
        names(superDfExt)[names(superDfExt) == "mVar1"] <- "var1"
        names(superDfExt)[names(superDfExt) == "mVar2"] <- "var2"
    }
    superDfExt$orthoID <- as.character(superDfExt$orthoID)
    return(dplyr::tibble(superDfExt))
}

#' Complete processing of raw input phylogenetic profiles
#' @description Create a processed and filtered data for plotting or analysing
#' phylogenetic profiles from raw input file (from raw input to final filtered
#' dataframe)
#' @usage fromInputToProfile(rawInput, rankName, refTaxon = NULL,
#'     taxaTree = NULL, sortedTaxonList = NULL, var1AggregateBy = "max",
#'     var2AggregateBy = "max", percentCutoff = c(0, 1),
#'     coorthologCutoffMax = 9999, var1Cutoff = c(0, 1), var2Cutoff = c(0, 1),
#'     var1Relation = "protein", var2Relation = "protein", groupByCat = FALSE,
#'     catDt = NULL, taxDB = NULL)
#' @param rawInput input file (in long, wide, multi-fasta or orthoxml format)
#' @param rankName taxonomy rank (e.g. "species","phylum",...)
#' @param refTaxon selected reference taxon name (used for sorting and will be
#' protected from filtering). Default = NULL.
#' @param taxaTree input taxonomy tree for taxa in input profiles (optional).
#' Default = NULL.
#' @param sortedTaxonList list of sorted taxa (optional). Default = NULL.
#' @param var1AggregateBy aggregate method for var1 (min, max, mean or median).
#' Default = "max".
#' @param var2AggregateBy aggregate method for VAR2 (min, max, mean or median).
#' Default = "max".
#' @param percentCutoff min and max cutoffs for percentage of species present
#' in a supertaxon. Default = c(0, 1).
#' @param coorthologCutoffMax maximum number of co-orthologs allowed. Default =
#' 9999.
#' @param var1Cutoff min and max cutoffs for var1. Default = c(0, 1).
#' @param var2Cutoff min and max cutoffs for var2. Default = c(0, 1).
#' @param var1Relation relation of var1 ("protein" for protein-protein or
#' "species" for protein-species). Default = "protein".
#' @param var2Relation relation of var2 ("protein" for protein-protein or
#' "species" for protein-species). Default = "protein".
#' @param groupByCat group genes by their categories (TRUE or FALSE). Default =
#' FALSE.
#' @param catDt dataframe contains gene categories. Default = NULL
#' @param taxDB Path to the taxonomy DB files
#' @return Dataframe required for generating phylogenetic profile plot or
#' clustering analysis. It contains seed gene IDs (or orthologous group IDs),
#' their ortholog IDs and the corresponding (super)taxa, (super)taxon IDs,
#' number of co-orthologs in each (super)taxon, values for two additional
#' variables var1, var2, % of species present in each supertaxon, and the
#' categories of seed genes (or ortholog groups).
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom dplyr count
#' @export
#' @seealso \code{\link{createLongMatrix}}, \code{\link{getInputTaxaID}},
#' \code{\link{getInputTaxaName}}, \code{\link{sortInputTaxa}},
#' \code{\link{parseInfoProfile}}, \code{\link{reduceProfile}},
#' \code{\link{filterProfileData}}
#' @examples
#' rawInput <- system.file(
#'     "extdata", "test.main.long", package = "PhyloProfile", mustWork = TRUE
#' )
#' rankName <- "class"
#' refTaxon <- "Mammalia"
#' taxaTree <- NULL
#' sortedTaxonList <- NULL
#' var1AggregateBy <- "max"
#' var2AggregateBy <- "mean"
#' percentCutoff <- c(0.0, 1.0)
#' coorthologCutoffMax <- 10
#' var1Cutoff <- c(0.75, 1.0)
#' var2Cutoff <- c(0.5, 1.0)
#' var1Relation <- "protein"
#' var2Relation <- "species"
#' groupByCat <- FALSE
#' catDt <- NULL
#' fromInputToProfile(
#'     rawInput,
#'     rankName,
#'     refTaxon,
#'     taxaTree,
#'     sortedTaxonList,
#'     var1AggregateBy,
#'     var2AggregateBy,
#'     percentCutoff,
#'     coorthologCutoffMax,
#'     var1Cutoff,
#'     var2Cutoff,
#'     var1Relation,
#'     var2Relation,
#'     groupByCat,
#'     catDt
#' )

fromInputToProfile <- function(
    rawInput,
    rankName,
    refTaxon = NULL,
    taxaTree = NULL,
    sortedTaxonList = NULL,
    var1AggregateBy = "max",
    var2AggregateBy = "max",
    percentCutoff = c(0, 1),
    coorthologCutoffMax = 9999,
    var1Cutoff = c(0, 1),
    var2Cutoff = c(0, 1),
    var1Relation = "protein",
    var2Relation = "protein",
    groupByCat = FALSE,
    catDt = NULL,
    taxDB = NULL
) {
    if (missing(rawInput) | missing(rankName)) return("Missing input")
    if (is.null(rawInput) | is.null(rankName)) return("Missing input")
    supertaxon <- NULL
    # convert raw input into long format
    inputDf <- createLongMatrix(rawInput)
    # get input taxon IDs and names
    inputTaxonID <- getInputTaxaID(inputDf)
    # sort input taxa based on selected reference taxon or input taxonomy tree
    sortedInputTaxa <- sortInputTaxa(
        inputTaxonID, rankName, refTaxon, taxaTree, sortedTaxonList, taxDB
    )
    # count present taxa in each supertaxon
    taxaCount <- sortedInputTaxa %>% dplyr::group_by(supertaxon) %>%
        summarise(n = n(), .groups = "drop")
    # parse info (additional values...) into profile df
    fullMdData <- parseInfoProfile(inputDf, sortedInputTaxa, taxaCount)
    # filter profile
    filteredDf <- filterProfileData(
        fullMdData,
        taxaCount,
        refTaxon,
        percentCutoff,
        coorthologCutoffMax,
        var1Cutoff,
        var2Cutoff,
        var1Relation,
        var2Relation,
        groupByCat = FALSE,
        catDt = NULL,
        var1AggregateBy, var2AggregateBy
    )
    # reduce profile df into supertaxon level
    dataHeat <- reduceProfile(filteredDf)
    return(dataHeat)
}

context("test parsing and pre-processing PhyloProfile input")

test_that("test connection to taxonomy files and getting input taxa", {
    rankName <- "species"
    inputDf <- createLongMatrix("mainWideTest.txt")

    inputTaxonID <- getInputTaxaID(inputDf)
    expect_true(length(inputTaxonID) == 5)

    inputTaxonName <- getInputTaxaName(rankName, inputTaxonID)
    expect_true(nrow(inputTaxonName) == 5)
})

context("test identification of core genes for a selected set of taxa")

test_that("test core gene estimation", {
    library(dplyr)
    data("fullProcessedProfile", package="PhyloProfile")
    rankName <- "class"
    refTaxon <- "Mammalia"
    taxaCore <- c("Mammalia", "Saccharomycetes", "Insecta")
    var1Cutoff <- c(0,1)
    var2Cutoff <- c(0,1)
    percentCutoff <- c(0,1)
    coreCoverage <- 100
    taxonIDs <- levels(as.factor(fullProcessedProfile$ncbiID))
    sortedInputTaxa <- sortInputTaxa(
        taxonIDs, rankName, refTaxon, NULL
    )
    taxaCount <- sortedInputTaxa %>% dplyr::count(supertaxon)
    coreGene <- getCoreGene(
        rankName,
        taxaCore,
        fullProcessedProfile,
        taxaCount,
        var1Cutoff, var2Cutoff,
        percentCutoff, coreCoverage
    )
    expect_true(length(coreGene) == 3)
})

#' Identify core genes for a list of selected taxa
#' @description Identify core genes for a list of selected (super)taxa. The
#' identified core genes must be present in at least a certain proportion of
#' species in each selected (super)taxon (identified via percentCutoff) and
#' that criteria must be fullfilled for a certain percentage of selected taxa
#' or all of them (determined via coreCoverage).
#' @export
#' @usage getCoreGene(rankName, taxaCore, profileDt, taxaCount,
#'     var1Cutoff = c(0, 1), var2Cutoff = c(0, 1), percentCutoff = c(0, 1),
#'     coreCoverage = 100, taxDB = NULL)
#' @param rankName working taxonomy rank (e.g. "species", "genus", "family")
#' @param taxaCore list of selected taxon names
#' @param profileDt dataframe contains the full processed
#' phylogenetic profiles (see ?fullProcessedProfile or ?parseInfoProfile)
#' @param taxaCount dataframe counting present taxa in each supertaxon
#' @param var1Cutoff cutoff for var1. Default = c(0, 1).
#' @param var2Cutoff cutoff for var2. Default = c(0, 1).
#' @param percentCutoff cutoff for percentage of species present in each
#' supertaxon. Default = c(0, 1).
#' @param coreCoverage the least percentage of selected taxa should be
#' considered. Default = 1.
#' @param taxDB Path to the taxonomy DB files
#' @return A list of identified core genes.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom dplyr count
#' @seealso \code{\link{parseInfoProfile}} for creating a full processed
#' profile dataframe
#' @examples
#' library(dplyr)
#' data("fullProcessedProfile", package="PhyloProfile")
#' rankName <- "class"
#' refTaxon <- "Mammalia"
#' taxaCore <- c("Mammalia", "Saccharomycetes", "Insecta")
#' profileDt <- fullProcessedProfile
#' taxonIDs <- levels(as.factor(fullProcessedProfile$ncbiID))
#' sortedInputTaxa <- sortInputTaxa(
#'     taxonIDs, rankName, refTaxon, NULL, NULL
#' )
#' taxaCount <- sortedInputTaxa %>% dplyr::count(supertaxon)
#' var1Cutoff <- c(0.75, 1.0)
#' var2Cutoff <- c(0.75, 1.0)
#' percentCutoff <- c(0.0, 1.0)
#' coreCoverage <- 100
#' getCoreGene(
#'     rankName,
#'     taxaCore,
#'     profileDt,
#'     taxaCount,
#'     var1Cutoff, var2Cutoff,
#'     percentCutoff, coreCoverage
#' )

getCoreGene <- function(
    rankName = NULL, taxaCore = c(), profileDt = NULL, taxaCount = NULL,
    var1Cutoff = c(0, 1), var2Cutoff = c(0, 1),
    percentCutoff = c(0, 1), coreCoverage = 100, taxDB = NULL
) {
    if (is.null(profileDt)) stop("Processed profile cannot be NULL!")
    if (is.null(rankName)) stop("Rank name cannot be NULL!")
    var1 <- var2 <- 0
    supertaxonID <- mVar1 <- mVar2 <- presSpec <- geneID <- Freq <- NULL
    # get ID list of chosen taxa & main input profile
    taxaList <- getNameList(taxDB)
    if (length(taxaCore) == 0) {
        stop("WARNING: Please select taxa of interest!")
    }
    superID <- taxaList$ncbiID[
        taxaList$fullName%in%taxaCore & taxaList$rank %in% c(rankName,"norank")]
    # filter by var1 and var2 cutoffs
    if (!is.null(var1Cutoff[2])) {
        if (!is.na(var1Cutoff[2])) {
            profileDt <- subset(
                profileDt, supertaxonID %in% superID & var1 >= var1Cutoff[1]
                & var1 <= var1Cutoff[2])
        }
    }
    if (!is.null(var2Cutoff[2])) {
        if (!is.na(var2Cutoff[2])) {
            profileDt <- subset(
                profileDt, supertaxonID %in% superID & var2 >= var2Cutoff[1]
                & var2 <= var2Cutoff[2])
        }
    }
    # calculate % present taxa
    finalPresSpecDt <- calcPresSpec(profileDt, taxaCount)
    profileDt <- profileDt[, c(
        "geneID", "ncbiID", "fullName", "supertaxon", "supertaxonID", "rank",
        "var1", "var2")]
    profileDt <- Reduce(
        function(x, y) merge(x, y, by = c("geneID", "supertaxon"), all.x=TRUE),
        list(profileDt, finalPresSpecDt))
    # filter by selecting taxa
    if (is.na(superID[1])) stop("No core gene found!")
    else {
        data <- subset(
            profileDt, supertaxonID %in% superID & presSpec >= percentCutoff[1]
            & presSpec <= percentCutoff[2])
        # get supertaxa present in each geneID
        supertaxonCount <- data %>% dplyr::count(geneID, supertaxonID)
        # count no. supertaxa for each gene & min no. supertaxa muss be present
        count <- as.data.frame(table(supertaxonCount$geneID))
        requireCoverage <- length(superID) * (coreCoverage / 100)
        # get only gene that contains orthologs in that coverage # of taxa
        coreGene <- subset(count, Freq >= requireCoverage)
        return(levels(factor(coreGene$Var1)))
    }
}

#' Get all NCBI taxonomy rank names
#' @export
#' @return A list of all available NCBI taxonomy rank names.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @examples
#' mainTaxonomyRank()

mainTaxonomyRank <- function() {
    return(
        c(
            "isolate","strain","pathogroup","serotype","serogroup",
            "forma","formaspecialis","varietas",
            "genotype","morph","subvariety","biotype",
            "subspecies","species","speciessubgroup",
            "speciesgroup","series","section","subgenus","genus",
            "subtribe","tribe", "subfamily","family","superfamily",
            "parvorder","infraorder","suborder","order","superorder",
            "subcohort","cohort","infraclass","subclass","class","superclass",
            "subphylum","phylum","superphylum",
            "subkingdom","kingdom","superkingdom"
        )
    )
}

#' Pre-processing NCBI taxonomy data
#' @description Download NCBI taxonomy database and parse information that are
#' needed for PhyloProfile, including taxon IDs, their scientific names,
#' systematic ranks, and parent (next higher) rank IDs.
#' @return A dataframe contains NCBI taxon IDs, taxon names, taxon ranks and the
#' next higher taxon IDs (parent's IDs) of all taxa in the NCBI taxonomy
#' database.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @export
#' @examples
#' \dontrun{
#' ?processNcbiTaxonomy
#' preProcessedTaxonomy <- PhyloProfile:::processNcbiTaxonomy()
#' # save to text (tab-delimited) file
#' write.table(
#'     preProcessedTaxonomy,
#'     file = "preProcessedTaxonomy.txt",
#'     col.names = TRUE,
#'     row.names = FALSE,
#'     quote = FALSE,
#'     sep = "\t"
#' )
#' # save to rdata file
#' save(
#'     preProcessedTaxonomy, file = "preProcessedTaxonomy.RData", compress='xz'
#' )
#' }

processNcbiTaxonomy <- function() {
    temp <- tempfile()
    utils::download.file(
        "https://ftp.ncbi.nlm.nih.gov/pub/taxonomy/taxdmp.zip", temp
    )
    names <- utils::read.table(
        unz(temp, "names.dmp"), header = FALSE, fill = TRUE, sep = "\t",
        quote = "", comment.char = "", stringsAsFactors = FALSE
    )
    nodes <- utils::read.table(
        unz(temp, "nodes.dmp"), header = FALSE, fill = TRUE, sep = "\t",
        quote = "", comment.char = "", stringsAsFactors = FALSE
    )
    unlink(temp)
    message("Download NCBI taxonomy done!")
    # Create data frame containing taxon ID, the scientific name, its taxonomy
    # rank and the taxon ID of the higer rank (parent's ID)
    outTax <- merge(
        names[names$V7 == "scientific name", c("V1", "V3")],
        nodes[,c("V1", "V5", "V3")],
        by = "V1"
    )
    colnames(outTax) <- c("ncbiID", "fullName", "rank","parentID")
    # Remove "'" from taxon names and ranks, remove space from taxon ranks
    outTax$fullName <- gsub("'", "",outTax$fullName)
    outTax$rank <- gsub("'", "", outTax$rank)
    outTax$rank <- gsub(" ", "", outTax$rank)
    # Add Holozoa and Holomycota ranks
    opiMod <- data.frame(
        ncbiID = c(999999001, 999999002, 999999003, 999999004, 999999005),
        fullName = c(
            "Choanozoa", "Filozoa", "Pluriformea", "Holozoa", "Holomycota"),
        rank = rep("norank", 5),
        parentID = c(999999002, 999999003, 999999004, 33154, 33154),
        stringsAsFactors = FALSE
    )
    outTax[outTax$ncbiID %in% c(28009, 33208),]$parentID <- 999999001
    outTax[outTax$ncbiID == 2687318,]$parentID <- 999999002
    outTax[outTax$ncbiID == 42461,]$parentID <- 999999003
    outTax[outTax$ncbiID == 127916,]$parentID <- 999999004
    outTax[outTax$ncbiID %in% c(4751, 2686024),]$parentID <- 999999005
    outTax <- do.call(rbind, list(outTax, opiMod))
    # Return
    message("Parsing NCBI taxonomy done!")
    return(outTax)
}

#' Get taxonomy info for a list of input taxa
#' @param inputTaxa NCBI taxonomy IDs of input taxa.
#' @param currentNCBIinfo table/dataframe of the pre-processed NCBI taxonomy
#' data (/PhyloProfile/data/preProcessedTaxonomy.txt)
#' @return A list of NCBI taxonomy info for input taxa, including the taxonomy
#' IDs, full scientific names, taxonomy ranks and the parent IDs.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom data.table fread
#' @export
#' @examples
#' inputTaxa <- c("272557", "176299")
#' ncbiFilein <- system.file(
#'     "extdata", "data/preProcessedTaxonomy.txt",
#'     package = "PhyloProfile", mustWork = TRUE
#' )
#' currentNCBIinfo <- as.data.frame(data.table::fread(ncbiFilein))
#' getTaxonomyInfo(inputTaxa, currentNCBIinfo)

getTaxonomyInfo <- function(inputTaxa = NULL, currentNCBIinfo = NULL) {
    tmp <- list()
    outList <- list()
    k <- 1
    missingTaxa <- setdiff(inputTaxa, currentNCBIinfo$ncbiID)
    if (length(missingTaxa) > 0) {
        warning(
            sprintf(
                "%d id(s) missing in currentNCBIinfo: %s",
                length(missingTaxa), paste(missingTaxa, collapse = ", ")
            )
        )
    }
    for (refID in inputTaxa) {
        # get info for this taxon
        if (refID %in% missingTaxa) next
        refEntry <- currentNCBIinfo[currentNCBIinfo$ncbiID == refID, ]
        lastID <- refEntry$parentID
        inputTaxaInfo <- refEntry
        if(length(lastID) == 0) {
            msg <- paste("PROBLEM with", refID)
            stop(msg)
        }
        while (lastID != 1) {
            if (lastID %in% names(tmp)) {
                inputTaxaInfo <- rbindlist(
                    list(inputTaxaInfo, tmp[[toString(lastID)]]),
                    use.names = TRUE, fill = TRUE, idcol = NULL)
                lastID <- 1
            } else {
                nextEntry <- currentNCBIinfo[currentNCBIinfo$ncbiID == lastID, ]
                inputTaxaInfo <- rbindlist(
                    list(inputTaxaInfo, nextEntry), use.names = TRUE,
                    fill = TRUE, idcol = NULL)
                lastID <- nextEntry$parentID
            }
        }
        for (i in seq(inputTaxaInfo$ncbiID)) {
            if (!(inputTaxaInfo$ncbiID[i] %in% names(tmp))) {
                tmp[[toString(inputTaxaInfo$ncbiID[i])]] <-
                    inputTaxaInfo[i:length(inputTaxaInfo$ncbiID)]
            }
        }
        outList[[k]] <- inputTaxaInfo
        k <- k + 1
    }
    return(outList)
}

#' Get taxonomy info for a list of taxa
#' @description Get NCBI taxonomy IDs, ranks and names for an input taxon list.
#' @param inputTaxa NCBI ID list of input taxa.
#' @param currentNCBIinfo table/dataframe of the pre-processed NCBI taxonomy
#' data (/PhyloProfile/data/preProcessedTaxonomy.txt)
#' @return A list of 3 dataframes: idList, rankList and reducedInfoList. The
#' "rankList" contains taxon names and all taxonomy ranks of the input taxa
#' including also the noranks from the input rank to the taxonomy root. The
#' "idList" contains input taxon IDs, taxon names, all the ranks from current
#' rank to the taxonomy root together with their IDs (with the format
#' "id#rank"). The reducedInfoList is a subset of preProcessedTaxonomy.txt file,
#' containing the NCBI IDs, taxon fullnames, their current rank and their
#' direct parent ID.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom data.table fread rbindlist
#' @export
#' @examples
#' inputTaxa <- c("272557", "176299")
#' ncbiFilein <- system.file(
#'     "extdata", "data/preProcessedTaxonomy.txt",
#'     package = "PhyloProfile", mustWork = TRUE
#' )
#' currentNCBIinfo <- as.data.frame(data.table::fread(ncbiFilein))
#' getIDsRank(inputTaxa, currentNCBIinfo)

getIDsRank <- function(inputTaxa = NULL, currentNCBIinfo = NULL){
    if (is.null(currentNCBIinfo)) stop("Pre-processed NCBI tax data is NULL!")
    inputTaxaInfo <- getTaxonomyInfo(inputTaxa, currentNCBIinfo)
    allMainRank <- mainTaxonomyRank()
    ## get reduced taxonomy info (subset of preProcessedTaxonomy.txt)
    reducedDf <- unique(rbindlist(inputTaxaInfo))
    ## get list of all ranks and rank#IDs
    rankMod <- ncbiID <- NULL
    inputRankIDDf <- lapply(
        seq_len(length(inputTaxaInfo)),
        function (x) {
            inputTaxaInfo[[x]]$rank[
                !(inputTaxaInfo[[x]]$rank %in% allMainRank)] <- "norank"
            inputTaxaInfo[[x]]$rankMod <- inputTaxaInfo[[x]]$rank
            if (inputTaxaInfo[[x]]$rank[1] == "norank")
                inputTaxaInfo[[x]]$rankMod[1] <-
                    paste0("strain_", inputTaxaInfo[[x]]$ncbiID[1])
            inputTaxaInfo[[x]]$rankMod <- with(
                inputTaxaInfo[[x]],
                ifelse(rankMod == "norank", paste0("norank_", ncbiID), rankMod))
            inputTaxaInfo[[x]]$id <- paste0(
                inputTaxaInfo[[x]]$ncbiID, "#", inputTaxaInfo[[x]]$rankMod)
            return(inputTaxaInfo[[x]][ , c("rankMod", "id")])
        }
    )
    inputRankList <- lapply(
        seq_len(length(inputRankIDDf)),
        function (x) {
            ll <- c(
                paste0("ncbi", inputTaxa[x]),
                reducedDf$fullName[reducedDf$ncbiID == inputTaxa[x]],
                inputRankIDDf[[x]]$rank, "norank_1")
            ll <- gsub("strain_[[:digit:]]+", "strain", ll)
            return(data.frame(
                matrix(ll, nrow = 1, byrow = TRUE), stringsAsFactors = FALSE))
        }
    )
    inputRankDf <- data.frame(data.table::rbindlist(inputRankList, fill = TRUE))
    inputIDList <- lapply(
        seq_len(length(inputRankIDDf)),
        function (x) {
            ll <- c(paste0(
                reducedDf$fullName[reducedDf$ncbiID==inputTaxa[x]],"#name"),
                inputRankIDDf[[x]]$id, "1#norank_1")
            return(data.frame(
                matrix(ll, nrow = 1, byrow = TRUE), stringsAsFactors = FALSE))
        }
    )
    inputIDDf <- data.frame(data.table::rbindlist(inputIDList, fill = TRUE))
    newCol <- seq(ncol(inputIDDf) + 1, ncol(inputRankDf))
    inputIDDf[paste0("X", newCol)] <- NA
    reducedDf$rank[!(reducedDf$rank %in% allMainRank)] <- "norank"
    return(list(inputIDDf, inputRankDf, as.data.frame(reducedDf)))
}

#' Indexing all available ranks (including norank)
#' @param rankListFile Input file, where each row is a rank list of a taxon
#' (see rankListFile in example)
#' @return A dataframe containing a list of all possible ranks and their indexed
#' values.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @examples
#' rankListFile <- system.file(
#'     "extdata", "data/rankList.txt", package = "PhyloProfile", mustWork = TRUE
#' )
#' PhyloProfile:::rankIndexing(rankListFile)

rankIndexing <- function (rankListFile = NULL) {
    if (is.null(rankListFile)) stop("Rank list file is NULL!")
    . <- NULL
    rankList <- utils::read.table(
        rankListFile, sep = '\t', header = FALSE,fill = TRUE,
        stringsAsFactors = TRUE, na.strings = c("", "NA")
    )
    ### remove duplicated and "normal" taxa
    rankList <- rankList[,-(seq_len(2))]
    rankList <- rankList[!duplicated(rankList), ]
    mainRank <- mainTaxonomyRank()
    filterRanks <- c(mainRank, "norank_1", NA)
    rankList <- rankList %>% filter_all(any_vars(!(. %in% filterRanks)))
    ### get list of all inpu ranks
    allInputRank <- as.character(unique(unlist(rankList)))
    allInputRank <- allInputRank[!is.na(allInputRank)]
    ### initial index for main ranks
    mainRank <- mainTaxonomyRank()
    rank2index <- new.env(hash = TRUE)
    getHash <- Vectorize(get, vectorize.args = "x")
    assignHash <- Vectorize(assign, vectorize.args = c("x", "value"))
    for (i in seq_len(length(mainRank))) rank2index[[mainRank[i]]] <- i
    ### the magic happens here
    for (k in seq_len(nrow(rankList))) {
        subList <- rankList[k,][!is.na(rankList[k,])]
        ## indexing
        tmpEnv <- new.env(hash = TRUE)
        flag <- 0
        for (i in seq_len(length(subList))) {
            iRank <- subList[i]
            if (is.null(rank2index[[iRank]])) {
                for (j in seq_len(length(subList))) {
                    if (j < i) {
                        if (!is.null(tmpEnv[[subList[i - j]]])) {
                            tmpEnv[[iRank]] <- tmpEnv[[subList[i - j]]] + 1
                            break
                        }
                    } else j <- j - 1
                }
            } else {
                # for old rank
                if (i > 1) {
                    if (flag == 0) {
                        if (!(iRank %in% ls(rank2index)))
                            stop(iRank," not found!")
                        currentIndex <- rank2index[[iRank]]
                    } else {
                        if (!(iRank %in% ls(tmpEnv))) {
                            tmpEnv[[iRank]] <- tmpEnv[[subList[i-1]]]
                        }
                        currentIndex <- tmpEnv[[iRank]]
                    }

                    if (currentIndex <= tmpEnv[[subList[i-1]]]) {
                        if (flag == 0) {
                            tmpEnv[[iRank]] <- tmpEnv[[subList[i-1]]] + 1
                            # list of curr ranks whose index should be increased
                            candidateList <- unlist(
                                mget(
                                    ls(rank2index)[
                                        !(ls(rank2index) %in% ls(tmpEnv))
                                    ],
                                    rank2index
                                )
                            )
                            candidateList <- candidateList[
                                order(unlist(candidateList))
                            ]
                            for (c in seq_len(length(candidateList))) {
                                r <- names(candidateList)[c]
                                fromIndex <- rank2index[[iRank]]
                                if(subList[i-1] %in% ls(rank2index)) {
                                    fromIndex <- rank2index[[subList[i-1]]]
                                }

                                if (rank2index[[r]] > fromIndex) {
                                    tmpEnv[[r]] <-
                                        rank2index[[r]] +
                                        (tmpEnv[[iRank]] - rank2index[[iRank]])
                                    flag <- 1
                                }
                            }
                        } else {
                            step <- tmpEnv[[subList[i-1]]] -
                                rank2index[[iRank]] + 1
                            tmpEnv[[iRank]] <- tmpEnv[[subList[i-1]]] + 1
                            for (t in ls(tmpEnv)) {
                                if (tmpEnv[[t]] >= tmpEnv[[subList[i-1]]]){
                                    if (!(t == iRank) & !(t == subList[i-1])){
                                        tmpEnv[[t]] <- tmpEnv[[t]] + step
                                    }
                                }
                            }
                        }
                        assignHash(
                            ls(tmpEnv), getHash(ls(tmpEnv), tmpEnv), rank2index
                        )
                    } else {
                        if (is.null(tmpEnv[[iRank]])) {
                            tmpEnv[[iRank]] <- rank2index[[iRank]]
                        }
                    }
                } else {
                    tmpEnv[[iRank]] <- rank2index[[iRank]]
                }
            }
        }
        assignHash(ls(tmpEnv), getHash(ls(tmpEnv), tmpEnv), rank2index)
    }
    # convert env into dataframe and return
    index2RankList <- lapply(
        seq_len(length(allInputRank)), function (x) {
            data.frame(
                index = rank2index[[allInputRank[x]]],
                rank = allInputRank[x], stringsAsFactors = FALSE
            )
        }
    )
    index2RankDf <- do.call(rbind, index2RankList)
    index2RankDf <- index2RankDf[with(index2RankDf, order(index2RankDf$index)),]
    return(index2RankDf)
}

#' Align NCBI taxonomy IDs of list of taxa into a sorted rank list.
#' @export
#' @param idListFile a text file whose each row is a rank+ID list of a taxon
#' (see idListFile in example)
#' @param rankListFile a text file whose each row is a rank list of a taxon
#' (see rankListFile in example)
#' @return An aligned taxonomy dataframe which contains all the available
#' taxonomy ranks from the id and rank list file. This dataframe can be used for
#' creating a well resolved taxonomy tree (see ?createUnrootedTree) and sorting
#' taxa based on a selected reference taxon (see ?sortInputTaxa).
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom data.table melt transpose setDT
#' @importFrom pbapply pblapply
#' @importFrom zoo na.locf
#' @seealso \code{\link{rankIndexing}}, \code{\link{createUnrootedTree}},
#' \code{\link{sortInputTaxa}}
#' @examples
#' idListFile <- system.file(
#'     "extdata", "data/idList.txt", package = "PhyloProfile", mustWork = TRUE
#' )
#' rankListFile <- system.file(
#'     "extdata", "data/rankList.txt", package = "PhyloProfile", mustWork = TRUE
#' )
#' taxonomyTableCreator(idListFile, rankListFile)

taxonomyTableCreator <- function(idListFile = NULL, rankListFile = NULL) {
    if (is.null(idListFile) | is.null(rankListFile))
        stop("Taxonomy ID list or rank list file is NULL!")
    index <- NULL
    index2RankDf <- rankIndexing(rankListFile)
    ncol <- max(utils::count.fields(rankListFile, sep = '\t'))
    idList <- utils::read.table(
        idListFile, sep = '\t', header = FALSE, check.names = FALSE,
        comment.char = "", fill = TRUE, stringsAsFactors = TRUE,
        na.strings = c("","NA"), col.names = paste0('X', seq_len(ncol)))
    colnames(idList)[1] <- "tip"
    orderedRank <- factor(index2RankDf$rank, levels = index2RankDf$rank)
    ### create a dataframe containing ordered ranks
    fullRankIDdf <- data.frame(
        rank = matrix(
            unlist(orderedRank), nrow = length(orderedRank), byrow = TRUE
        ), stringsAsFactors = FALSE)
    fullRankIDdf$index <- as.numeric(rownames(fullRankIDdf))
    fullRankIDdf <- data.table(fullRankIDdf)
    setkey(fullRankIDdf, rank)
    mTaxonDf <- pbapply::pblapply(
        seq_len(nrow(idList)),
        function (x) {
            ### get list of all IDs for this taxon & convert into long format
            taxonDf <- data.frame(idList[x,])
            taxonName <- unlist(
                strsplit(as.character(idList[x,]$tip), "#", fixed = TRUE))
            mTaxonDf <- suppressWarnings(
                data.table::melt(setDT(taxonDf), id = "tip")
            )
            ### get rank names and corresponding IDs, then remove NA cases
            splitCol <- data.frame(
                do.call(
                    'rbind',
                    strsplit(as.character(mTaxonDf$value), '#', fixed = TRUE)
                )
            )
            mTaxonDf <- cbind(mTaxonDf, splitCol)
            mTaxonDf <- mTaxonDf[stats::complete.cases(mTaxonDf),]
            ### subselect mTaxonDf to keep only 2 column rank id and rank name
            mTaxonDf <- mTaxonDf[, c("X1","X2")]
            if (mTaxonDf$X2[1] != index2RankDf$rank[1])
                mTaxonDf <- rbind(
                    data.frame(
                        "X1" = mTaxonDf$X1[1], "X2" = index2RankDf$rank[1]
                    ), mTaxonDf)
            ### rename columns & return
            colnames(mTaxonDf) <- c(taxonName[1], "rank")
            mTaxonDf <- data.table(mTaxonDf)
            setkey(mTaxonDf, rank)
            return(mTaxonDf)
        }
    )
    ### merge into data frame contains all available ranks from input
    mTaxonDfFull <- c(list(fullRankIDdf), mTaxonDf)
    fullRankIDdf <- Reduce(
        function (x, y) merge(x , y, all.x = TRUE, allow.cartesian = TRUE),
        mTaxonDfFull
    )
    ### reorder ranks & replace NA id by id of previous rank
    fullRankIDdf <- fullRankIDdf[order(fullRankIDdf$index),]
    fullRankIDdf <- zoo::na.locf(fullRankIDdf)
    ### remove index column & transpose into wide format
    fullRankIDdf <- subset(fullRankIDdf, select = -c(index))
    tFullRankIDdf <- data.table::transpose(fullRankIDdf)
    ### set first row to column names
    colnames(tFullRankIDdf) <- as.character(unlist(tFullRankIDdf[1,]))
    tFullRankIDdf <- tFullRankIDdf[-1,]
    ### add "abbrName  ncbiID  fullName" columns
    fullRankIDdfOut <- data.frame(
        abbrName = paste0("ncbi", unlist(tFullRankIDdf[,1])),
        ncbiID = unlist(tFullRankIDdf[,1]),
        fullName = colnames(fullRankIDdf)[-c(1)], stringsAsFactors = FALSE)
    fullRankIDdfOut <- cbind(fullRankIDdfOut, tFullRankIDdf)
    ### rename last column to "root"
    names(fullRankIDdfOut)[ncol(fullRankIDdfOut)] <- "root"
    numericCol <- c(seq(4, ncol(fullRankIDdfOut)))
    fullRankIDdfOut[,numericCol] <-
        as.numeric(as.character(unlist(fullRankIDdfOut[,numericCol])))
    return(fullRankIDdfOut)
}

#' Get taxonomy hierarchy for a list of taxon IDs
#' @description Get NCBI taxonomy hierarchy and URLs for an input taxon list.
#' @param inputTaxa NCBI ID list of input taxa.
#' @param currentNCBIinfo table/dataframe of the pre-processed NCBI taxonomy
#' data (/PhyloProfile/data/preProcessedTaxonomy.txt)
#' @return A list of dataframs containing taxonomy hierarchy and its URL to
#' NCBI database for input taxon IDs
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom data.table fread
#' @examples
#' inputTaxa <- c("272557", "176299")
#' ncbiFilein <- system.file(
#'     "extdata", "data/preProcessedTaxonomy.txt",
#'     package = "PhyloProfile", mustWork = TRUE
#' )
#' currentNCBIinfo <- as.data.frame(data.table::fread(ncbiFilein))
#' PhyloProfile:::getTaxHierarchy(inputTaxa, currentNCBIinfo)

getTaxHierarchy <- function(inputTaxa = NULL, currentNCBIinfo = NULL){
    if (is.null(currentNCBIinfo)) {
        dataPath <- system.file(
            "PhyloProfile", "data/",
            package = "PhyloProfile", mustWork = TRUE
        )
        nameFullFile <- paste0(dataPath, "/preProcessedTaxonomy.txt")
        if (file.exists(nameFullFile)) {
            currentNCBIinfo <- as.data.frame(data.table::fread(nameFullFile))
        } else stop("Pre-processed NCBI tax data is NULL!")
    }
    inputTaxaInfo <- PhyloProfile::getTaxonomyInfo(inputTaxa, currentNCBIinfo)
    ## get reduced taxonomy info (subset of preProcessedTaxonomy.txt)
    reducedDf <- unique(rbindlist(inputTaxaInfo))
    ## get list of all rank#IDs
    rankMod <- ncbiID <- NULL
    inputRankIDDf <- lapply(
        seq_len(length(inputTaxaInfo)),
        function (x) {
            inputTaxaInfo[[x]]$url <- paste0(
                "https://www.ncbi.nlm.nih.gov/Taxonomy/Browser/wwwtax.cgi?id=",
                inputTaxaInfo[[x]]$ncbiID
            )
            inputTaxaInfo[[x]]$link <- paste0(
                "<p><a href='", inputTaxaInfo[[x]]$url, "' target='_blank'>",
                inputTaxaInfo[[x]]$fullName, "</a>",
                " (", inputTaxaInfo[[x]]$rank, ")", "</p>"
            )
            return(inputTaxaInfo[[x]][ , c("fullName", "ncbiID", "link")])
        }
    )
    return(inputRankIDDf)
}

#' Get taxon names for a list of taxon IDs
#' @export
#' @param idList list of taxonomy IDs
#' @param currentNCBIinfo table/dataframe of the pre-processed NCBI taxonomy
#' data (/PhyloProfile/data/preProcessedTaxonomy.txt)
#' @return A dataframe contains input taxon Ids and their full names.
#' @author Vinh Tran tran@bio.uni-frankfurt.de
#' @importFrom data.table fread
#' @examples
#' ncbiFilein <- system.file(
#'     "extdata", "data/preProcessedTaxonomy.txt",
#'     package = "PhyloProfile", mustWork = TRUE
#' )
#' currentNCBIinfo <- as.data.frame(data.table::fread(ncbiFilein))
#' idList <- c("9606", "5207", "40674", "4751")
#' id2name(idList, currentNCBIinfo)

id2name <- function(idList = NULL, currentNCBIinfo = NULL) {
    if (is.null(idList)) stop("No list of taxon IDs given!")
    nameList <- data.frame(
        ncbiID = idList,
        fullName = paste0("NCBI taxID ", idList)
    )
    if (is.null(currentNCBIinfo)) {
        ncbiFilein <- system.file(
            "PhyloProfile", "data/preProcessedTaxonomy.txt",
            package = "PhyloProfile", mustWork = FALSE
        )
        if (file.exists(ncbiFilein)) {
            currentNCBIinfo <- as.data.frame(data.table::fread(ncbiFilein))
            # get taxon names
            nameList <- currentNCBIinfo[
                currentNCBIinfo$ncbiID %in% idList, c("ncbiID","fullName")
            ]
        }
    } else {
        nameListTmp <- currentNCBIinfo[
            currentNCBIinfo$ncbiID %in% idList, c("ncbiID","fullName")
        ]
        nameList <- merge(nameList, nameListTmp, by = "ncbiID", all.x = TRUE)
        nameList$fullName.y[is.na(nameList$fullName.y)] <-
            nameList$fullName.x[is.na(nameList$fullName.y)]
        nameList <- nameList[, c("ncbiID","fullName.y")]
        colnames(nameList) <- c("ncbiID", "fullName")
    }
    return(nameList)
}

context("test creating taxonomy matrix function")

test_that("get taxonomy IDs for list of taxa", {
    allTaxonInfo <- data.table::fread("taxonNamesTest.txt")
    inputTaxa <- c("272557")

    taxonomyInfo <- getIDsRank(inputTaxa, allTaxonInfo)
    reducedInfoList <- as.data.frame(taxonomyInfo[3])
    expect_that(nrow(reducedInfoList), equals(10))
})

test_that("get taxonomy Info for list of taxa", {
    inputTaxa <- c("272557", "176299", "123456789")
    ncbiFilein <- system.file(
        "extdata", "data/preProcessedTaxonomy.txt",
        package = "PhyloProfile", mustWork = TRUE
    )
    currentNCBIinfo <- as.data.frame(data.table::fread(ncbiFilein))
    expect_warning(getTaxonomyInfo(inputTaxa, currentNCBIinfo))
    
    out <- suppressWarnings(getTaxonomyInfo(inputTaxa, currentNCBIinfo))
    expect_that(length(out), equals(2))
})

test_that("create taxonomy matrix file", {
    taxMatrix <- taxonomyTableCreator("idListTest.txt", "rankListTest.txt")
    expect_that(taxMatrix, is_a("data.frame"))
})

---
title: "PhyloProfile"
author:
-   name: Vinh Tran
    affiliation: Goethe University Frankfurt, Frankfurt am Main, Germany
    email: tran@bio.uni-frankfurt.de
date: "`r Sys.Date()`"
output:
    BiocStyle::html_document:
        toc_float: true
        number_sections: false
package: PhyloProfile
abstract: |
    PhyloProfile: dynamic visualization and exploration of multi-layered
    phylogenetic profiles
vignette: >
    %\VignetteIndexEntry{PhyloProfile}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
    library(PhyloProfile),
    collapse = TRUE,
    comment = "#>",
    dev = 'png',
    crop = NULL
)
```

# Introduction
Phylogenetic profiles capture the presence - absence pattern of genes across
species (Pellegrini et al., 1999). The presence of an ortholog in a given
species is often taken as evidence that also the corresponding function is
represented (Lee et al., 2007). Moreover, if two genes agree in their
phylogenetic profile, it can suggest that they functionally interact
(Pellegrini et al., 1999). Phylogenetic profiles are therefore commonly used
for tracing functional protein clusters or metabolic networks across species
and through time. However, orthology inference is not error-free (Altenhoff
et al., 2016), and orthology does not guarantee functional equivalence for
two genes (Studer and Robinson-Rechavi, 2009). Therefore, phylogenetic
profiles are often integrated with accessory information layers, such as
sequence similarity, domain architecture similarity, or semantic similarity
of Gene Ontology-term descriptions.

Various approaches exist to visualize such profiles. However, there is still a
shortage of tools that provide a comprehensive set of functions for the display,
filtering and analysis of multi-layered phylogenetic profiles comprising
hundreds of genes and taxa. To close this methodological gap, we present here
**PhyloProfile**, an *R-based tool to visualize, explore and analyze
multi-layered phylogenetic profiles*.

# How to install PhyloProfile

To install the PhyloProfile package via
[Bioconductor](https://bioconductor.org/packages/PhyloProfile)
using BiocManager:

```
if (!requireNamespace("BiocManager"))
    install.packages("BiocManager")
BiocManager::install("PhyloProfile")
```

To install the dev version from Bioconductor:
```
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install(version='devel')
BiocManager::install("PhyloProfile")
```

To install the dev version from [github](https://github.com/BIONF/PhyloProfile):

```
if (!requireNamespace("remotes"))
    install.packages("remotes", repos = "http://cran.us.r-project.org")
remotes::install_github(
    "BIONF/PhyloProfile",
    INSTALL_opts = c('--no-lock'),
    build_vignettes = TRUE
)
```

Or use directly the online version at
http://applbio.biologie.uni-frankfurt.de/phyloprofile/.

# Input
PhyloProfile expects as a main input the phylogenetic distribution of orthologs,
or more generally of homologs. This information can be complemented with domain
architecture annotation and data for up to two additional annotation layers.

Beside tab delimited text and sequences in FASTA format, the tool also accepts
orthoXML (Schmitt et al., 2011), or a list of OMA IDs (Altenhoff et al., 2015)
as input.

Here is an example of a tab delimited input with two additional annotation
layers:
```{r example_input, echo = FALSE, results = 'asis'}
data("mainLongRaw", package="PhyloProfile")
knitr::kable(head(mainLongRaw, 10), row.names = FALSE)
```

*The [WIKI](https://github.com/BIONF/PhyloProfile/wiki/Input-Data) accompanying
PhyloProfile gives a comprehensive guide of how to format input data.*

# Features and capabilities
## Interactive visualization and dynamic exploration of phylogenetic profiles
Together with several functions for exploring phylogenetic profiles, we provide
an interactive visualization application implemented with
[Shiny](https://shiny.rstudio.com) (https://CRAN.R-project.org/package=shiny).

```{r poster, echo=FALSE, fig.cap="PhyloProfile's GUI", out.width = '100%'}
knitr::include_graphics("./posterSub.png")
```

For both command-based and visualization analyses, users can:

- dynamically **change the resolution of the analysis** from invidual species
to phyla or entire kingdoms by collapsing the input taxa into higher systematic
rank. Species are automatically linked to the NCBI taxonomy, and are ordered
in increasing taxonomic distance from a user-specified reference taxon.
*PhyloProfile is able to represent co-orthologs (in-paralogs), if the working
taxonomic rank is the deepest one (e.g. strain or species) that can be
found in the input taxa.*
- dynamically **filter data** by applying different thresholds to the integrated
information (e.g. increasing the fraction of species in a systematic group that
must harbor an ortholog before the gene is considered present in this group
reduces the impact of spurious ortholog identification on evolutionary
interpretations).
- dynamically **modify the apperance of profile** with diverse plot
configuration options.

PhyloProfile is able to represent the entire data matrix (`Main profile`) or to
visualize only a subset of genes and taxa for a detailed inspection
(`Customized profile`), without the need of modifying the input data.

Besides, PhyloProfile's interface will be automatically varied according to
user's input files, such as the names of two additional information layers or
list of input taxa.

## Analysis functions
PhyloProfile provides several functions for dynamically analyzing phylogenetic
profiles.

### Profile clustering
The identification of proteins with similar phylogenetic profiles is a crucial
step in the identification and characterization of novel functional protein
interaction networks (Pellegrini, 2012). PhyloProfile offers the option to
cluster genes according to the distance of their phylogenetic profiles.

```{r profile-clustering, fig.show='hold', dev='png'}
### An example for plotting the clustered profiles tree.
### See ?getDendrogram for more details.

#' Load built-in data
data("finalProcessedProfile", package="PhyloProfile")
data <- finalProcessedProfile

#' Calculate distance matrix
#' Check ?getDistanceMatrix
profileType <- "binary"
profiles <- getDataClustering(
    data, profileType, var1AggregateBy, var2AggregateBy)
method <- "mutualInformation"
distanceMatrix <- getDistanceMatrix(profiles, method)

#' Create clustered profile tree
clusterMethod <- "complete"
dd <- clusterDataDend(as.dist(distanceMatrix), clusterMethod)
getDendrogram(dd)
```

### Gene age estimation
PhyloProfile can estimate the evolutionary age of a gene from the phylogenetic
profiles using an LCA algorithm (Capra et al., 2013). Specifically, the last
common ancestor of the two most distantly related species displaying a given
gene serves as the minimal gene age. Age estimates are dynamically updated
upon filtering of the data.

```{r gene-age}
### An example for calculating gene age for the built-in data set.
### See ?estimateGeneAge for more details.
#' Load built-in data
data("fullProcessedProfile", package="PhyloProfile")

#' Choose the working rank and the reference taxon
rankName <- "class"
refTaxon <- "Mammalia"

#' Count taxa within each supertaxon
taxonIDs <- levels(as.factor(fullProcessedProfile$ncbiID))
sortedInputTaxa <- sortInputTaxa(
    taxonIDs, rankName, refTaxon, NULL, NULL, NULL
)
library(dplyr)
taxaCount <- sortedInputTaxa %>% dplyr::count(supertaxon)

#' Set cutoff for 2 additional variables and the percentage of present species
#' in each supertaxon
var1Cutoff <- c(0,1)
var2Cutoff <- c(0,1)
percentCutoff <- c(0,1)

#' Estimate gene age
estimateGeneAge(
    fullProcessedProfile, taxaCount, rankName, refTaxon,
    var1Cutoff, var2Cutoff, percentCutoff
)
```

### Core gene identification
Phylogenomic reconstructions are typically based on a collection of core genes
(Daubin et al., 2002), i.e. genes that are shared among all genomes in a taxon
collection. PhyloProfile enables users to select a set of taxa and returns
their core genes.

```{r core-gene}
### An example for calculating core gene set for the built-in data set.
### See ?getCoreGene for more details.

#' Load built-in data
data("fullProcessedProfile", package="PhyloProfile")

#' Choose the working rank and a set of taxa of interest
rankName <- "class"
refTaxon <- "Mammalia"
taxaCore <- c("Mammalia", "Saccharomycetes", "Insecta")

#' Set cutoff for 2 additional variables and the percentage of present species
#' in each supertaxon
var1Cutoff <- c(0.75, 1.0)
var2Cutoff <- c(0.75, 1.0)
percentCutoff <- c(0.0, 1.0)

#' Set core coverage, the % of taxa that must be present in the selected set
coreCoverage <- 1

#' Count taxa within each supertaxon
taxonIDs <- levels(as.factor(fullProcessedProfile$ncbiID))
sortedInputTaxa <- sortInputTaxa(
    taxonIDs, rankName, refTaxon, NULL, NULL, NULL
)
taxaCount <- sortedInputTaxa %>% dplyr::count(supertaxon)

#' Identify core genes
getCoreGene(
    rankName,
    taxaCore,
    fullProcessedProfile,
    taxaCount,
    var1Cutoff, var2Cutoff,
    percentCutoff, coreCoverage
)
```

### Group comparison
This function is used to compare the distribution of the additional variables
between two taxon groups, an in- and an out-group. Users can define the in-group
and all taxa not included in this are used as the out-group. The value
distributions of the variables are then compared using statistical tests
(Kolmogorov-Smirnov and Wilcoxon-Mann-Whitney) using the specified significant
level (*0.05 by default*). Genes that have a significantly different
distribution will be shown in the candidate gene list for further analysis.

```{r group-comparison}
#' Load built-in data
data("mainLongRaw", package="PhyloProfile")
data <- mainLongRaw
#' choose the in-group taxa
inGroup <- c("ncbi9606", "ncbi10116")
#' choose variable to be compared
variable <- colnames(data)[4]
#' compare the selected variable between the in-group and out-group taxa
compareTaxonGroups(data, inGroup, TRUE, variable, 0.05)
```

### Distribution analysis
The interpretation of phylogenetic profiles, and the result of downstream
analyses can change substantially upon filtering the data. To help users to
decide on reasonable filtering thresholds, PhyloProfile provides a function to
plot the distributions of the values incurred by the integrated information
layers.

```{r distribution-analysis, fig.show='hold', dev='png'}
### An example for plotting the distribution of the 1st additional variable.
### See ?createVarDistPlot for more details.

#' Load built-in data
data("mainLongRaw", package="PhyloProfile")

#' Process data for distribution analysis
#' See ?createVariableDistributionData
data <- createVariableDistributionData(
    mainLongRaw, c(0, 1), c(0.5, 1)
)
head(data, 6)

#' Choose a variable for plotting and set the variable name
varType <- "var1"
varName <- "Variable 1"

#' Set cutoff for the percentage of present species in each supertaxon
percentCutoff <- c(0,1)

#' Set text size
distTextSize <- 12

#' Create distribution plot
createVarDistPlot(
    data,
    varName,
    varType,
    percentCutoff,
    distTextSize
)
```

# Examples
## Process raw input
Process raw input (in different format) into a dataframe that contains all
required information for the phylogenetic profile analysis.

```{r process-input}
#' Load built-in data
#' If input data is in other format (e.g. fasta, OrthoXML, or wide matrix),
#' see ?createLongMatrix
rawInput <- system.file(
    "extdata", "test.main.long", package = "PhyloProfile", mustWork = TRUE
)

#' Set working rank and the reference taxon
rankName <- "class"
refTaxon <- "Mammalia"

#' Input a user-defined taxonomy tree to replace NCBI taxonomy tree (optional)
taxaTree <- NULL
#' Or a sorted taxon list (optional)
sortedTaxonList <- NULL

#' Choose how to aggregate the additional variables when pocessing the data
#' into supertaxon
var1AggregateBy <- "max"
var2AggregateBy <- "mean"

#' Set cutoffs for for percentage of species present in a supertaxon,
#' allowed number of co orthologs, and cutoffs for the additional variables
percentCutoff <- c(0.0, 1.0)
coorthologCutoffMax <- 10
var1Cutoff <- c(0.75, 1.0)
var2Cutoff <- c(0.5, 1.0)

#' Choose the relationship of the additional variables, if they are related to
#' the orthologous proteins or to the species
var1Relation <- "protein"
var2Relation <- "species"

#' Identify categories for input genes (by a mapping tab-delimited file)
groupByCat <- FALSE
catDt <- NULL

#' Process the input file into a phylogenetic profile data that contains
#' taxonomy information and the aggregated values for the 2 additional variables
profileData <- fromInputToProfile(
    rawInput,
    rankName,
    refTaxon,
    taxaTree,
    sortedTaxonList,
    var1AggregateBy,
    var2AggregateBy,
    percentCutoff,
    coorthologCutoffMax,
    var1Cutoff,
    var2Cutoff,
    var1Relation,
    var2Relation,
    groupByCat,
    catDt
)

head(profileData)
```

## Create profile plot
Generate phylogenetic profile heatmap after processing the raw input file.
```{r profile-plot, fig.show='hold', dev='png'}
#' Load built-in processed data
data("finalProcessedProfile", package="PhyloProfile")

#' Create data for plotting
plotDf <- dataMainPlot(finalProcessedProfile)

#' You can also choose a subset of genes and/or taxa for plotting with:
#' selectedTaxa <- c("Mammalia", "Echinoidea", "Gunneridae")
#' selectedSeq <- "all"
#' plotDf <- dataCustomizedPlot(
#'     finalProcessedProfile, selectedTaxa, selectedSeq
#' )

#' Identify plot's parameters
plotParameter <- list(
    "xAxis" = "taxa",
    "var1ID" = "FAS_FW",
    "var2ID"  = "FAS_BW",
    "midVar1" = 0.5,
    "midColorVar1" =  "#FFFFFF",
    "lowColorVar1" =  "#FF8C00",
    "highColorVar1" = "#4682B4",
    "midVar2" = 1,
    "midColorVar2" =  "#FFFFFF",
    "lowColorVar2" = "#CB4C4E",
    "highColorVar2" = "#3E436F",
    "paraColor" = "#07D000",
    "xSize" = 8,
    "ySize" = 8,
    "legendSize" = 8,
    "mainLegend" = "top",
    "dotZoom" = 0,
    "xAngle" = 60,
    "guideline" = 0,
    "colorByGroup" = FALSE,
    "colorByOrthoID" = FALSE
)

#' Generate profile plot
p <- heatmapPlotting(plotDf, plotParameter)
p
#' To highlight a gene and/or taxon of interest
taxonHighlight <- "Mammalia"
workingRank <- "class"
geneHighlight <- "none"
#' Then use ?highlightProfilePlot function
# highlightProfilePlot(
#    p, plotDf, taxonHighlight, workingRank, geneHighlight, plotParameter$xAxis
# )
```

## Create protein domain architecture plot
Generate the domain architecture plot for a gene of interest and its orthologs.
```{r domain-plot, fig.show='hold', dev='png'}
#' Load protein domain architecture file
domainFile <- system.file(
    "extdata", "domainFiles/101621at6656.domains",
    package = "PhyloProfile", mustWork = TRUE
)

#' Identify IDs of gene of interest and its ortholog partner
seedID <- "101621at6656"
orthoID <- "101621at6656|AGRPL@224129@0|224129_0:001955|1"
info <- c(seedID, orthoID)

#' Get data for 2 selected genes from input file
domainDf <- parseDomainInput(seedID, domainFile, "file")

#' Modify feature IDs
domainDf$feature_id_mod <- domainDf$feature_id
domainDf$feature_id_mod <- gsub("SINGLE", "LCR", domainDf$feature_id_mod)
domainDf$feature_id_mod[domainDf$feature_type == "coils"] <- "Coils"
domainDf$feature_id_mod[domainDf$feature_type == "seg"] <- "LCR"
domainDf$feature_id_mod[domainDf$feature_type == "tmhmm"] <- "TM"

#' Generate plot
plot <- createArchiPlot(info, domainDf, 9, 9)
grid::grid.draw(plot)
```

## Other use cases

**Get taxonomy IDs and names in a specified rank for taxa in the input file.**
```{r get-tax-id-name}
#' Load raw input
data("mainLongRaw", package="PhyloProfile")
inputDf <- mainLongRaw

#' Set working rank and the reference taxon
rankName <- "phylum"

#' Get taxonomy IDs and names for the input data
inputTaxonID <- getInputTaxaID(inputDf)
inputTaxonName <- getInputTaxaName(rankName, inputTaxonID)

head(inputTaxonName)
```

**Get taxonomy info for list of input taxa and sort it based on the taxonomy**
**distance to a reference taxon**
```{r sort-taxa}
#' Get list of taxon IDs and names for input profile
data("mainLongRaw", package="PhyloProfile")
inputDf <- mainLongRaw
rankName <- "phylum"
inputTaxonID <- getInputTaxaID(inputDf)

#' Input a user-defined taxonomy tree to replace NCBI taxonomy tree (optional)
inputTaxaTree <- NULL

#' Sort taxonomy list based on a selected refTaxon
refTaxon <- "Chordata"
sortedTaxonomy <- sortInputTaxa(
    taxonIDs = inputTaxonID,
    rankName = rankName,
    refTaxon = refTaxon,
    taxaTree = inputTaxaTree,
    NULL,
    NULL
)

head(
    sortedTaxonomy[
        , c("ncbiID", "fullName", "supertaxon", "supertaxonID", "rank")
    ]
)
```

**More examples? Please tell us what you want to see ;-)**

# How to cite
>Ngoc-Vinh Tran, Bastian Greshake Tzovaras, Ingo Ebersberger, PhyloProfile:
dynamic visualization and exploration of multi-layered phylogenetic profiles,
Bioinformatics, Volume 34, Issue 17, 01 September 2018, Pages 3041–3043,
https://doi.org/10.1093/bioinformatics/bty225

Or use the citation function in R CMD to have the citation in BibTex or
LaTeX format

```{r citation}
citation("PhyloProfile")
```

# How to contribute

Thanks so much for your interest in contributing to *PhyloProfile* 🎉👍🍾

Contributions to *PhyloProfile* can take many forms. If you are

**biologist**, you can

- [report bugs](https://github.com/BIONF/PhyloProfile/issues/new) for both
online & standalone version,
- [tell us about what features you would love to see](https://goo.gl/SBU4EG),
- improve our documentation, both
[in the Wiki](https://github.com/BIONF/PhyloProfile/wiki) and in our
[README](https://github.com/BIONF/PhyloProfile/blob/master/README.md),
- discuss about [non-coding issues](https://goo.gl/bDS9Cb)

**biologist and love coding**, you can

- [fix existing bugs](https://github.com/BIONF/PhyloProfile/issues/),
- or add new features. Some things we'd love to see are: add scripts to add
out-of-the-box support for further orthology prediction tools, increasing the
test coverage from 0% to something above that, or basically whatever great
idea you have!
- and all points for non-coding contributors as well :)

**not biologist but can code**, it would be great if you can

- test the tool in different environments (Windows, Linux, Mac - Firefox,
Chrome, IE, Safari,...),
- suggest a better user interface,
- improve the code quality

Don't hesitate to get in touch with us if you have any questions. You can
contact us at [tran@bio.uni-frankfurt.de](mailto:tran@bio.uni-frankfurt.de)

## Contributors {.unnumbered}
- [Vinh Tran](https://github.com/trvinh)
- [Bastian Greshake Tzovaras](https://github.com/gedankenstuecke)
- [Carla Moelbert](https://github.com/CarlaMoelbert)

# SessionInfo()

Here is the output of `sessionInfo()` on the system on which this document was
compiles:

```{r sessionInfo, echo = FALSE}
sessionInfo(package = "PhyloProfile")
```

# References

1. Adebali, O. and Zhulin, I.B. (2017) Aquerium: A web application for
comparative exploration of domain-based protein occurrences on the taxonomically
clustered genome tree. Proteins, 85, 72-77.

2. Altenhoff, A.M. et al. (2016) Standardized benchmarking in the quest for
orthologs. Nat Methods, 13, 425-430.

3. Altenhoff, A.M. et al. (2015) The OMA orthology database in 2015: function
predictions, better plant support, synteny view and other improvements.
Nucleic Acids Res, 43, D240-249.

4. Capra, J.A. et al. (2013) How old is my gene? Trends Genet, 29, 659-668.
Daubin, V., Gouy, M. and Perriere, G. (2002) A phylogenomic approach to
bacterial phylogeny: evidence of a core of genes sharing a common history.
Genome Res, 12, 1080-1090.

5. Huerta-Cepas, J., Serra, F. and Bork, P. (2016) ETE 3: Reconstruction,
Analysis, and Visualization of Phylogenomic Data. Mol Biol Evol, 33, 1635-1638.

6. Koestler, T., Haeseler, A.v. and Ebersberger, I. (2010) FACT: Functional
annotation transfer between proteins with similar feature architectures.
BMC Bioinformatics, 11, 417.

7. Lee, D., Redfern, O. and Orengo, C. (2007) Predicting protein function
from sequence and structure. Nat Rev Mol Cell Biol, 8, 995-1005.

8. Moore, A.D. et al. (2014) DoMosaics: software for domain arrangement
visualization and domain-centric analysis of proteins. Bioinformatics,
30, 282-283.

9. Pellegrini, M. (2012) Using phylogenetic profiles to predict functional
relationships. Methods Mol Biol, 804, 167-177.

10. Pellegrini, M. et al. (1999) Assigning protein functions by comparative
genome analysis: protein phylogenetic profiles. Proc Natl Acad Sci U S A, 96,
4285-4288.

11. Schmitt, T. et al. (2011) Letter to the editor: SeqXML and OrthoXML:
standards for sequence and orthology information. Brief. Bioinform., 12,
485-488.

12. Studer, R.A. and Robinson-Rechavi, M. (2009) How confident can we be that
orthologs are similar, but paralogs differ? Trends Genet, 25, 210-216.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parsePhylotree.R
\name{sortTaxaFromTree}
\alias{sortTaxaFromTree}
\title{Get sorted supertaxon list based on a rooted taxonomy tree}
\usage{
sortTaxaFromTree(tree)
}
\arguments{
\item{tree}{an "phylo" object for a rooted taxonomy tree}
}
\value{
A list of sorted taxa obtained the input taxonomy tree.
}
\description{
Get sorted supertaxon list based on a rooted taxonomy tree
}
\examples{
data("ppTaxonomyMatrix", package = "PhyloProfile")
# create taxonomy tree rooted by ncbi10090
tree <- createUnrootedTree(ppTaxonomyMatrix)
rootedTree <- ape::root(tree, outgroup = "ncbi10090", resolve.root = TRUE)
# get taxon list sorted from tree
sortTaxaFromTree(rootedTree)
}
\seealso{
\code{\link{ppTaxonomyMatrix}} for a demo taxonomy matrix data
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getOmaBrowser.R
\name{getAllFastaOma}
\alias{getAllFastaOma}
\title{Get all fasta sequences from a raw OMA dataframe}
\usage{
getAllFastaOma(finalOmaDf = NULL)
}
\arguments{
\item{finalOmaDf}{raw OMA data for a list of proteins (see ?getDataForOneOma)}
}
\value{
A list contains all protein sequences in fasta format.
}
\description{
Get all fasta sequences from a raw OMA dataframe
}
\examples{
### Uncomment the following line to run the function
# omaData <- getDataForOneOma("HUMAN29397", "OG")
# getAllFastaOma(omaData)
}
\seealso{
\code{\link{getDataForOneOma}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getOmaBrowser.R
\name{getOmaDataForOneOrtholog}
\alias{getOmaDataForOneOrtholog}
\title{Get taxonomy ID, sequence and annotation for one OMA protein}
\usage{
getOmaDataForOneOrtholog(id = NULL)
}
\arguments{
\item{id}{oma ID of one protein}
}
\value{
Data frame contains the input protein ID with its taxonomy ID,
sequence, length and domain annotations (tab delimited) for input OMA protein
}
\description{
Get taxonomy ID, sequence and annotation for one OMA protein
}
\examples{
### Uncomment the following line to run the function
# getOmaDataForOneOrtholog("HUMAN29397")
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parsePhyloProfile.R
\name{reduceProfile}
\alias{reduceProfile}
\title{Reduce the filtered profile data into supertaxon level}
\usage{
reduceProfile(filteredProfile)
}
\arguments{
\item{filteredProfile}{dataframe contains the filtered profiles (see
?parseInfoProfile, ?filterProfileData and ?filteredProfile)}
}
\value{
A reduced dataframe contains only profile data for the selected
supertaxon rank. This dataframe contains only supertaxa and their value
(mVar1 & mVar2) for each gene.
}
\description{
Reduce data of the processed phylogenetic profiles from input
taxonomy rank into supertaxon level (e.g. from species to phylum)
}
\examples{
data("filteredProfile", package="PhyloProfile")
reduceProfile(filteredProfile)
}
\seealso{
\code{\link{parseInfoProfile}} for creating a full processed
profile dataframe, \code{\link{filterProfileData}} for filter processed
profile and \code{\link{filteredProfile}} for a demo filtered
profile dataframe
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/umapClustering.R
\name{performPCA}
\alias{performPCA}
\title{Helper function to perform PCA}
\usage{
performPCA(pcaDt)
}
\arguments{
\item{pcaDt}{data matrix for PCA}
}
\value{
A table contains coordinates of the first 3 PCs
}
\description{
Helper function to perform PCA
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createProfilePlot.R
\name{dataMainPlot}
\alias{dataMainPlot}
\title{Create data for main profile plot}
\usage{
dataMainPlot(dataHeat = NULL)
}
\arguments{
\item{dataHeat}{a data frame contains processed profiles (see
?fullProcessedProfile, ?filterProfileData)}
}
\value{
A dataframe for plotting the phylogenetic profile, containing seed
protein IDs (geneID), ortholog IDs (orthoID) together with their ncbi
taxonomy IDs (ncbiID and abbrName), full names (fullName), indexed supertaxa
(supertaxon), values for additional variables (var1, var2) and the aggregated
values of those additional variables for each supertaxon (mVar1, mVar2),
number of original and filtered co-orthologs in each supertaxon (paralog and
paralogNew), number of species in each supertaxon (numberSpec) and the % of
species that have orthologs in each supertaxon (presSpec).
}
\description{
Create data for main profile plot
}
\examples{
data("finalProcessedProfile", package="PhyloProfile")
dataMainPlot(finalProcessedProfile)
}
\seealso{
\code{\link{filterProfileData}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runPhyloprofile.R
\name{runPhyloProfile}
\alias{runPhyloProfile}
\title{Run PhyloProfile app}
\usage{
runPhyloProfile(configFile = NULL, host = NULL, port = NULL)
}
\arguments{
\item{configFile}{Configuration file for specifying path to input files,
taxonomy rank and reference taxon, and some other settings}

\item{host}{IP adress (e.g. host = "127.0.0.1")}

\item{port}{Port (e.g. port = 8888)}
}
\value{
A shiny application - GUI version of PhyloProfile
}
\description{
Run PhyloProfile app
}
\examples{
?runPhyloProfile
# runPhyloProfile()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compareTaxaGroups.R
\name{getCommonAncestor}
\alias{getCommonAncestor}
\title{Get all taxa that share a common ancestor}
\usage{
getCommonAncestor(inputTaxa = NULL, inGroup = NULL, taxDB = NULL)
}
\arguments{
\item{inputTaxa}{ID list of all input taxa (e.g. "ncbi12345")}

\item{inGroup}{ID list of selected taxa used for identify the common ancestor
(e.g.: "ncbi55555")}

\item{taxDB}{Path to the taxonomy DB files}
}
\value{
A list containing the taxonomy rank and name of the common ancestor,
together with a dataframe storing the full taxonomy info of all taxa that
share that corresponding common ancestor.
}
\description{
Identify the common ancestor for a selected taxa and return a
list of all taxa that have that common ancestor from an large input taxa set.
}
\examples{
inputTaxa <- c("ncbi34740", "ncbi9606", "ncbi374847", "ncbi123851",
    "ncbi5664", "ncbi189518", "ncbi418459", "ncbi10116", "ncbi284812",
    "ncbi35128", "ncbi7070")
inGroup <-  c("ncbi9606", "ncbi10116")
getCommonAncestor(inputTaxa, inGroup)
}
\author{
Vinh Tran (tran@bio.uni-frankfurt.de)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compareTaxaGroups.R
\name{dataFeatureTaxGroup}
\alias{dataFeatureTaxGroup}
\title{Create data for feature distribution comparison plot}
\usage{
dataFeatureTaxGroup(mainDf, domainDf, inGroup, gene)
}
\arguments{
\item{mainDf}{input phylogenetic profile in long format (see ?mainLongRaw
and ?createLongMatrix)}

\item{domainDf}{dataframe contains domain info for the seed and ortholog.
This including the seed ID, orthologs IDs, sequence lengths, feature names,
start and end positions, feature weights (optional) and the status to
determine if that feature is important for comparison the architecture
between 2 proteins* (e.g. seed protein vs ortholog) (optional). (see
?parseDomainInput)}

\item{inGroup}{ID list of in-group taxa (e.g. "ncbi1234")}

\item{gene}{ID of gene that need to be plotted the feature distribution
comparison between in- and out-group taxa.}
}
\value{
Dataframe containing all feature names, their frequencies (absolute
count and the average instances per protein - IPP) in each taxon group and
the corresponding taxa group type (in- or out-group).
}
\description{
Create data for plotting the distribution of the protein domain
features between 2 group of taxa for a selected gene (average number of
feature occurrency per protein/ortholog).
}
\examples{
data("mainLongRaw", package="PhyloProfile")
mainDf <- mainLongRaw
gene <- "101621at6656"
inputFile <- system.file(
    "extdata", "domainFiles/101621at6656.domains",
    package = "PhyloProfile", mustWork = TRUE
)
type <- "file"
domainDf <- parseDomainInput(gene, inputFile, type)
inGroup <- c("ncbi9606", "ncbi10116")
dataFeatureTaxGroup(mainDf, domainDf, inGroup, gene)
}
\seealso{
\code{\link{createLongMatrix}}, \code{\link{parseDomainInput}}
}
\author{
Vinh Tran (tran@bio.uni-frankfurt.de)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parsePhyloProfile.R
\name{getTaxonomyRanks}
\alias{getTaxonomyRanks}
\title{Create a list containing all main taxanomy ranks}
\usage{
getTaxonomyRanks()
}
\value{
A list of all main ranks (from strain to superkingdom)
}
\description{
Create a list containing all main taxanomy ranks
}
\examples{
getTaxonomyRanks()
}
\author{
Carla Mölbert (carla.moelbert@gmx.de)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimateGeneAge.R
\name{estimateGeneAge}
\alias{estimateGeneAge}
\title{Calculate the phylogenetic gene age from the phylogenetic profiles}
\usage{
estimateGeneAge(processedProfileData, taxaCount, rankName, refTaxon,
    var1CO, var2CO, percentCO, taxDB = NULL)
}
\arguments{
\item{processedProfileData}{dataframe contains the full processed
phylogenetic profiles (see ?fullProcessedProfile or ?parseInfoProfile)}

\item{taxaCount}{dataframe counting present taxa in each supertaxon}

\item{rankName}{working taxonomy rank (e.g. "species", "genus", "family")}

\item{refTaxon}{reference taxon name (e.g. "Homo sapiens", "Homo" or
"Hominidae")}

\item{var1CO}{cutoff for var1. Default: c(0, 1)}

\item{var2CO}{cutoff for var2. Default: c(0, 1)}

\item{percentCO}{cutoff for percentage of species present in each
supertaxon. Default: c(0, 1)}

\item{taxDB}{Path to the taxonomy DB files}
}
\value{
A dataframe contains estimated gene ages for the seed proteins.
}
\description{
Calculate the phylogenetic gene age from the phylogenetic profiles
}
\examples{
library(dplyr)
data("fullProcessedProfile", package="PhyloProfile")
rankName <- "class"
refTaxon <- "Mammalia"
processedProfileData <- fullProcessedProfile
taxonIDs <- levels(as.factor(processedProfileData$ncbiID))
sortedInputTaxa <- sortInputTaxa(
    taxonIDs, rankName, refTaxon, NULL, NULL
)
taxaCount <- sortedInputTaxa \%>\% dplyr::count(supertaxon)
var1Cutoff <- c(0, 1)
var2Cutoff <- c(0, 1)
percentCutoff <- c(0, 1)
estimateGeneAge(
    processedProfileData,
    taxaCount,
    rankName,
    refTaxon,
    var1Cutoff, var2Cutoff, percentCutoff
)
}
\seealso{
\code{\link{parseInfoProfile}} for creating a full processed
profile dataframe; \code{\link{getNameList}} and
\code{\link{getTaxonomyMatrix}} for getting taxonomy info,
\code{\link{fullProcessedProfile}} for a demo input dataframe
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getOmaBrowser.R
\name{getDataForOneOma}
\alias{getDataForOneOma}
\title{Get OMA info for a query protein and its orthologs}
\usage{
getDataForOneOma(seedID = NULL, orthoType = "OG")
}
\arguments{
\item{seedID}{OMA protein ID}

\item{orthoType}{type of OMA orthologs ("OG" or "HOG"). Default = "OG".}
}
\value{
Data frame contains info for all sequences of the input OMA group
(or HOG). That info contains the protein IDs, taxonomy IDs, sequences,
lengths, domain annotations (tab delimited) and the corresponding seed ID.
}
\description{
Get taxonomy IDs, sequences, length and annotations for an OMA
orthologous group (or OMA HOG).
}
\examples{
### Uncomment the following line to run the function
# getDataForOneOma("HUMAN29397", "OG")
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/umapClustering.R
\name{createDimRedPlotData}
\alias{createDimRedPlotData}
\title{Generate data for dimension reduction plot}
\usage{
createDimRedPlotData(dimRedCoord = NULL, data4dimRed = NULL,
    freqCutoff = c(0,200), excludeTaxa = "None", currentNCBIinfo = NULL)
}
\arguments{
\item{dimRedCoord}{data contains DIM reduction coordinates (from
dimReduction)}

\item{data4dimRed}{data for dimension reduction (from prepareDimRedData())}

\item{freqCutoff}{gene/taxon frequency cutoff range. Any labels that are
outside of this range will be assigned as [Other]}

\item{excludeTaxa}{hide taxa from plot. Default: "None"}

\item{currentNCBIinfo}{table/dataframe of the pre-processed NCBI taxonomy
data (/PhyloProfile/data/preProcessedTaxonomy.txt)}
}
\value{
A plot as ggplot object
}
\description{
Generate data for dimension reduction plot
}
\examples{
rawInput <- system.file(
   "extdata", "test.main.long", package = "PhyloProfile", mustWork = TRUE
)
longDf <- createLongMatrix(rawInput)
data4dimRed <- prepareDimRedData(longDf, "phylum")
dimRedCoord <- dimReduction(data4dimRed)
createDimRedPlotData(dimRedCoord, data4dimRed)
}
\seealso{
\code{\link{prepareDimRedData}}, \code{\link{dimReduction}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parsePhyloProfile.R
\name{getTaxonomyMatrix}
\alias{getTaxonomyMatrix}
\title{Get taxonomy matrix}
\usage{
getTaxonomyMatrix(taxDB = NULL, subsetTaxaCheck = FALSE, taxonIDs = NULL)
}
\arguments{
\item{taxDB}{Path to the taxonomy DB files}

\item{subsetTaxaCheck}{TRUE/FALSE subset taxonomy matrix based on input taxon
IDs. Default = FALSE}

\item{taxonIDs}{list of input taxon IDs (e.g. ncbi1234). Default = NULL}
}
\value{
Data frame contains the (subset of) taxonomy matrix for list of
input taxa.
}
\description{
Get the (full or subset) taxonomy matrix from
"data/taxonomyMatrix.txt" based on an input taxon list
}
\examples{
# get full pre-installed taxonomy matrix
getTaxonomyMatrix()
# get taxonomy matrix for a list of taxon IDs
taxonIDs <- c("ncbi9606", "ncbi10116")
getTaxonomyMatrix(NULL, TRUE, taxonIDs)
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createTaxonomyMatrix.R
\name{getIDsRank}
\alias{getIDsRank}
\title{Get taxonomy info for a list of taxa}
\usage{
getIDsRank(inputTaxa = NULL, currentNCBIinfo = NULL)
}
\arguments{
\item{inputTaxa}{NCBI ID list of input taxa.}

\item{currentNCBIinfo}{table/dataframe of the pre-processed NCBI taxonomy
data (/PhyloProfile/data/preProcessedTaxonomy.txt)}
}
\value{
A list of 3 dataframes: idList, rankList and reducedInfoList. The
"rankList" contains taxon names and all taxonomy ranks of the input taxa
including also the noranks from the input rank to the taxonomy root. The
"idList" contains input taxon IDs, taxon names, all the ranks from current
rank to the taxonomy root together with their IDs (with the format
"id#rank"). The reducedInfoList is a subset of preProcessedTaxonomy.txt file,
containing the NCBI IDs, taxon fullnames, their current rank and their
direct parent ID.
}
\description{
Get NCBI taxonomy IDs, ranks and names for an input taxon list.
}
\examples{
inputTaxa <- c("272557", "176299")
ncbiFilein <- system.file(
    "extdata", "data/preProcessedTaxonomy.txt",
    package = "PhyloProfile", mustWork = TRUE
)
currentNCBIinfo <- as.data.frame(data.table::fread(ncbiFilein))
getIDsRank(inputTaxa, currentNCBIinfo)
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyzeDistribution.R
\name{createVariableDistributionData}
\alias{createVariableDistributionData}
\title{Create data for additional variable distribution}
\usage{
createVariableDistributionData(inputData, var1Cutoff = c(0 ,1),
    var2Cutoff = c(0, 1))
}
\arguments{
\item{inputData}{dataframe contains raw input data in long format
(see ?mainLongRaw)}

\item{var1Cutoff}{min and max cutoff for var1. Default = c(0, 1).}

\item{var2Cutoff}{min and max cutoff for var2. Default = c(0, 1).}
}
\value{
A dataframe for analysing the distribution of the additional
variable(s) containing the protein (ortholog) IDs and the values of their
variables (var1 and var2).
}
\description{
Create data for additional variable distribution
}
\examples{
data("mainLongRaw", package="PhyloProfile")
createVariableDistributionData(
    mainLongRaw, c(0, 1), c(0.5, 1)
)
}
\seealso{
\code{\link{mainLongRaw}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{finalProcessedProfile}
\alias{finalProcessedProfile}
\title{An example of a final processed & filtered phylogenetic profile}
\format{
Dataframe
}
\usage{
data(finalProcessedProfile)
}
\value{
A data frame with 88 rows and 11 variables:
\itemize{
    \item geneID Seed or ortholog group ID, e.g. "100136at6656"
    \item supertaxon Supertaxon name together with its ordered index, e.g.
    "1001_Mammalia"
    \item supertaxonID Supertaxon ID (only different than ncbiID in case
    working with higher taxonomy rank than input's). e.g. "40674"
    \item var1 First additional variable
    \item presSpec The percentage of species presenting in each supertaxon
    \item category "cat"
    \item orthoID Ortholog ID, e.g. "100136at6656|RAT@10116@1|G3V7R8|1"
    \item var2 Second additional variable
    \item paralog Number of co-orthologs in the current taxon
    \item presentTaxa Number of taxa that have ortho in each supertaxon
    \item totalTaxa Total number of taxa in each supertaxon
}
}
\description{
An example of a final processed & filtered phylogenetic profile
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parsePhylotree.R
\name{taxa2dist}
\alias{taxa2dist}
\title{taxa2dist}
\usage{
taxa2dist(x, varstep = FALSE, check = TRUE, labels)
}
\arguments{
\item{x}{taxa matrix}

\item{varstep}{var-step}

\item{check}{check}

\item{labels}{labels}
}
\value{
a distance matrix
}
\description{
taxa2dist
}
\author{
function from taxize library
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parsePhyloProfile.R
\name{parseInfoProfile}
\alias{parseInfoProfile}
\title{Parsing info for phylogenetic profiles}
\usage{
parseInfoProfile(inputDf, sortedInputTaxa, taxaCount, coorthoCOMax)
}
\arguments{
\item{inputDf}{input profiles in long format}

\item{sortedInputTaxa}{sorted taxonomy data for the input taxa
(check sortInputTaxa())}

\item{taxaCount}{dataframe counting present taxa in each supertaxon}

\item{coorthoCOMax}{maximum number of co-orthologs allowed}
}
\value{
A dataframe contains all info for the input phylogenetic profiles.
This full processed profile that is required for several profiling analyses
e.g. estimation of gene age (?estimateGeneAge) or identification of core gene
(?getCoreGene).
}
\description{
Creating main dataframe for the input phylogenetic profiles
based on selected input taxonomy level (e.g. strain, species) and reference
taxon. The output contains the number of paralogs, the max/min/mean/median
of VAR1 and VAR2.
}
\examples{
library(dplyr)
data("mainLongRaw", package="PhyloProfile")
taxonIDs <- getInputTaxaID(mainLongRaw)
sortedInputTaxa <- sortInputTaxa(
    taxonIDs, "class", "Mammalia", NULL, NULL
)
taxaCount <- sortedInputTaxa \%>\% dplyr::group_by(supertaxon) \%>\%
    summarise(n = n(), .groups = "drop")
coorthoCOMax <- 999
parseInfoProfile(
    mainLongRaw, sortedInputTaxa, taxaCount, coorthoCOMax
)
}
\seealso{
\code{\link{createLongMatrix}}, \code{\link{sortInputTaxa}},
\code{\link{calcPresSpec}}, \code{\link{mainLongRaw}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterProfile.R
\name{getDendrogram}
\alias{getDendrogram}
\title{Plot dendrogram tree}
\usage{
getDendrogram(dd = NULL)
}
\arguments{
\item{dd}{dendrogram object (see ?clusterDataDend)}
}
\value{
A dendrogram plot for the genes in the input phylogenetic profiles.
}
\description{
Plot dendrogram tree
}
\examples{
data("finalProcessedProfile", package="PhyloProfile")
data <- finalProcessedProfile
profileType <- "binary"
profiles <- getDataClustering(
    data, profileType, var1AggregateBy, var2AggregateBy)
distMethod <- "mutualInformation"
distanceMatrix <- getDistanceMatrix(profiles, distMethod)
clusterMethod <- "complete"
dd <- clusterDataDend(as.dist(distanceMatrix), clusterMethod)
getDendrogram(dd)
}
\seealso{
\code{\link{clusterDataDend}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createDomainPlot.R
\name{pairDomainPlotting}
\alias{pairDomainPlotting}
\title{Create architecure plot for a pair of seed and ortholog protein}
\usage{
pairDomainPlotting(seed, ortho, seedDf, orthoDf, minStart, maxEnd,
    labelSize, titleSize, legendSize, showScore, showWeight, namePosition, 
    firstDist, nameType, nameSize, segmentSize, nameColor, labelPos, 
    colorPalette, font)
}
\arguments{
\item{seed}{Seed ID}

\item{ortho}{Ortho ID}

\item{seedDf}{domain dataframe for seed domains containing the seed ID,
ortholog ID, sequence length, feature names, start and end positions,
feature weights (optional) and the status to determine if that feature is
important for comparison the architecture between 2 proteins* (e.g. seed
protein vs ortholog) (optional)}

\item{orthoDf}{domain dataframe for ortholog domains (same format as seedDf)}

\item{minStart}{the smallest start position of all domains}

\item{maxEnd}{the highest stop position of all domains}

\item{labelSize}{lable size. Default = 12}

\item{titleSize}{title size. Default = 12}

\item{legendSize}{legend size. Default = 12}

\item{showScore}{show/hide E-values and Bit-scores. Default = NULL (hide)}

\item{showWeight}{Show/hide feature weights. Default = NULL (hide)}

\item{namePosition}{list of positions for domain names, choose from "plot",
"legend" or "axis". Default: "plot"}

\item{firstDist}{distance of the first domain to plot title. Default = 0.5}

\item{nameType}{type of domain names, either "Texts" or "Labels" (default)}

\item{nameSize}{Size of domain names. Default = 3}

\item{segmentSize}{Height of domain segment. Default = 5}

\item{nameColor}{color of domain names (for Texts only). Default = "black"}

\item{labelPos}{position of domain names (for Labels only). Choose from
"Above" (default), "Below" or "Inside" the domain bar}

\item{colorPalette}{color pallete. Default = Paired"}

\item{font}{font of text. Default = Arial"}
}
\value{
Domain plot of a pair proteins as a arrangeGrob object.
}
\description{
Create architecure plot for a pair of seed and ortholog protein
}
\examples{
seed <- "101621at6656"
ortho <- "101621at6656|AGRPL@224129@0|224129_0:001955|1"
ortho <- gsub("\\\\|", ":", ortho)
grepID <- paste(seed, "#", ortho, sep = "")
domainFile <- system.file(
    "extdata", "domainFiles/101621at6656.domains",
    package = "PhyloProfile", mustWork = TRUE
)
domainDf <- parseDomainInput(seed, domainFile, "file")
domainDf$feature_id_mod <- domainDf$feature_id
subdomainDf <- domainDf[grep(grepID, domainDf$seedID), ]
subdomainDf$feature <- as.character(subdomainDf$feature)
orthoDf <- subdomainDf[subdomainDf$orthoID == ortho,]
seedDf <- subdomainDf[subdomainDf$orthoID != ortho,]
minStart <- min(subdomainDf$start)
maxEnd <- max(c(subdomainDf$end, subdomainDf$length))
# resolve overlapping domains
seedDf <- PhyloProfile:::resolveOverlapFeatures(seedDf)
orthoDf <- PhyloProfile:::resolveOverlapFeatures(orthoDf)
# add feature colors
featureColorDf <- PhyloProfile:::addFeatureColors(seedDf, orthoDf)
seedDf <- featureColorDf[[1]]
orthoDf <- featureColorDf[[2]]
# do plot
g <- PhyloProfile:::pairDomainPlotting(
   seed,ortho,seedDf,orthoDf,minStart,maxEnd, font = "sans"
)
grid::grid.draw(g)
}
\seealso{
\code{\link{singleDomainPlotting}}, \code{\link{sortDomains}},
\code{\link{parseDomainInput}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/identifyCoreGene.R
\name{getCoreGene}
\alias{getCoreGene}
\title{Identify core genes for a list of selected taxa}
\usage{
getCoreGene(rankName, taxaCore, profileDt, taxaCount,
    var1Cutoff = c(0, 1), var2Cutoff = c(0, 1), percentCutoff = c(0, 1),
    coreCoverage = 100, taxDB = NULL)
}
\arguments{
\item{rankName}{working taxonomy rank (e.g. "species", "genus", "family")}

\item{taxaCore}{list of selected taxon names}

\item{profileDt}{dataframe contains the full processed
phylogenetic profiles (see ?fullProcessedProfile or ?parseInfoProfile)}

\item{taxaCount}{dataframe counting present taxa in each supertaxon}

\item{var1Cutoff}{cutoff for var1. Default = c(0, 1).}

\item{var2Cutoff}{cutoff for var2. Default = c(0, 1).}

\item{percentCutoff}{cutoff for percentage of species present in each
supertaxon. Default = c(0, 1).}

\item{coreCoverage}{the least percentage of selected taxa should be
considered. Default = 1.}

\item{taxDB}{Path to the taxonomy DB files}
}
\value{
A list of identified core genes.
}
\description{
Identify core genes for a list of selected (super)taxa. The
identified core genes must be present in at least a certain proportion of
species in each selected (super)taxon (identified via percentCutoff) and
that criteria must be fullfilled for a certain percentage of selected taxa
or all of them (determined via coreCoverage).
}
\examples{
library(dplyr)
data("fullProcessedProfile", package="PhyloProfile")
rankName <- "class"
refTaxon <- "Mammalia"
taxaCore <- c("Mammalia", "Saccharomycetes", "Insecta")
profileDt <- fullProcessedProfile
taxonIDs <- levels(as.factor(fullProcessedProfile$ncbiID))
sortedInputTaxa <- sortInputTaxa(
    taxonIDs, rankName, refTaxon, NULL, NULL
)
taxaCount <- sortedInputTaxa \%>\% dplyr::count(supertaxon)
var1Cutoff <- c(0.75, 1.0)
var2Cutoff <- c(0.75, 1.0)
percentCutoff <- c(0.0, 1.0)
coreCoverage <- 100
getCoreGene(
    rankName,
    taxaCore,
    profileDt,
    taxaCount,
    var1Cutoff, var2Cutoff,
    percentCutoff, coreCoverage
)
}
\seealso{
\code{\link{parseInfoProfile}} for creating a full processed
profile dataframe
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createDomainPlot.R
\name{singleDomainPlotting}
\alias{singleDomainPlotting}
\title{Create architecure plot for a single protein}
\usage{
singleDomainPlotting(df, geneID, sep, labelSize, titleSize, minStart,
    maxEnd, colorPalette, showScore, showWeight, namePosition, firstDist,
    nameType, nameSize, segmentSize, nameColor, labelPos, font)
}
\arguments{
\item{df}{Domain dataframe for ploting containing the seed ID, ortholog ID,
ortholog sequence length, feature names, start and end positions,
feature weights (optional) and the status to determine if that feature is
important for comparison the architecture between 2 proteins* (e.g. seed
protein vs ortholog) (optional)}

\item{geneID}{ID of seed or orthologous protein}

\item{sep}{Separate indicator for title. Default = "|"}

\item{labelSize}{Lable size. Default = 12}

\item{titleSize}{Title size. Default = 12}

\item{minStart}{The smallest start position of all domains}

\item{maxEnd}{The highest stop position of all domains}

\item{colorPalette}{Color pallete. Default = Paired"}

\item{showScore}{Show/hide E-values and Bit-scores. Default = NULL (hide)}

\item{showWeight}{Show/hide feature weights. Default = NULL (hide)}

\item{namePosition}{List of positions for domain names, choose from "plot",
"legend" or "axis". Default: "plot"}

\item{firstDist}{Distance of the first domain to plot title. Default = 0.5}

\item{nameType}{Type of domain names, either "Texts" or "Labels" (default)}

\item{nameSize}{Size of domain names. Default = 3}

\item{segmentSize}{Height of domain segment. Default = 5}

\item{nameColor}{Color of domain names (for Texts only). Default = "black"}

\item{labelPos}{Position of domain names (for Labels only). Choose from
"Above" (default), "Below" or "Inside" the domain bar}

\item{font}{font of text. Default = Arial"}
}
\value{
Domain plot of a single protein as a ggplot object.
}
\description{
Create architecure plot for a single protein
}
\examples{
seed <- "101621at6656"
ortho <- "101621at6656|AGRPL@224129@0|224129_0:001955|1"
ortho <- gsub("\\\\|", ":", ortho)
grepID <- paste(seed, "#", ortho, sep = "")
domainFile <- system.file(
    "extdata", "domainFiles/101621at6656.domains",
    package = "PhyloProfile", mustWork = TRUE
)
domainDf <- parseDomainInput(seed, domainFile, "file")
domainDf$feature_id_mod <- domainDf$feature_id
subdomainDf <- domainDf[grep(grepID, domainDf$seedID), ]
subdomainDf$feature <- as.character(subdomainDf$feature)
orthoDf <- subdomainDf[subdomainDf$orthoID == ortho,]
seedDf <- subdomainDf[subdomainDf$orthoID != ortho,]
minStart <- min(subdomainDf$start)
maxEnd <- max(c(subdomainDf$end, subdomainDf$length))
# resolve overlapping domains
seedDf <- PhyloProfile:::resolveOverlapFeatures(seedDf)
orthoDf <- PhyloProfile:::resolveOverlapFeatures(orthoDf)
# add feature colors
featureColorDf <- PhyloProfile:::addFeatureColors(seedDf, orthoDf)
seedDf <- featureColorDf[[1]]
orthoDf <- featureColorDf[[2]]
# do plot
g <- PhyloProfile:::singleDomainPlotting(
   seedDf, seed, minStart = minStart, maxEnd = maxEnd, font = "sans"
)
grid::grid.draw(g)
}
\seealso{
\code{\link{parseDomainInput}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createProfilePlot.R
\name{addRankDivisionPlot}
\alias{addRankDivisionPlot}
\title{Add taxonomy rank division lines to the heatmap plot}
\usage{
addRankDivisionPlot(profilePlot = NULL, plotDf = NULL,
    taxDB = NULL, workingRank = NULL, superRank = NULL, xAxis = "taxa",
    font = "Arial", groupLabelSize = 14, groupLabelDist = 2,
    groupLabelAngle = 90, refLine = TRUE)
}
\arguments{
\item{profilePlot}{initial (highlighted) profile plot}

\item{plotDf}{dataframe for plotting the heatmap phylogentic profile}

\item{taxDB}{path to taxonomy database (taxonomyMatrix.txt file required!)}

\item{workingRank}{working taxonomy rank (e.g. species)}

\item{superRank}{taxonomy rank for division lines (e.g. superkingdom)}

\item{xAxis}{type of x-axis (either "genes" or "taxa")}

\item{font}{font of text. Default = Arial"}

\item{groupLabelSize}{size of rank labels}

\item{groupLabelDist}{size of the plot area for rank labels}

\item{groupLabelAngle}{angle of rank labels}

\item{refLine}{add vertical line to separate reference taxon}
}
\value{
A profile heatmap plot with highlighted gene and/or taxon of interest
as ggplot object.
}
\description{
Add taxonomy rank division lines to the heatmap plot
}
\examples{
data("finalProcessedProfile", package="PhyloProfile")
plotDf <- dataMainPlot(finalProcessedProfile)
plotParameter <- list(
    "xAxis" = "taxa",
    "geneIdType" = "geneID",
    "var1ID" = "FAS_FW",
    "var2ID"  = "FAS_BW",
    "midVar1" = 0.5,
    "midColorVar1" =  "#FFFFFF",
    "lowColorVar1" =  "#FF8C00",
    "highColorVar1" = "#4682B4",
    "midVar2" = 1,
    "midColorVar2" =  "#FFFFFF",
    "lowColorVar2" = "#CB4C4E",
    "highColorVar2" = "#3E436F",
    "paraColor" = "#07D000",
    "xSize" = 8,
    "ySize" = 8,
    "legendSize" = 8,
    "mainLegend" = "top",
    "dotZoom" = 0,
    "xAngle" = 60,
    "guideline" = 0,
    "colorByGroup" = FALSE,
    "colorByOrthoID" = FALSE
)
profilePlot <- heatmapPlotting(plotDf, plotParameter)
workingRank <- "class"
superRank <- "superkingdom"
addRankDivisionPlot(
    profilePlot, plotDf, NULL, workingRank, superRank, "taxa", font = "sans"
)
}
\seealso{
\code{\link{heatmapPlotting}}, \code{\link{highlightProfilePlot}},
\code{\link{getTaxonomyMatrix}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getOmaBrowser.R
\name{getOmaMembers}
\alias{getOmaMembers}
\title{Get OMA members}
\usage{
getOmaMembers(id = NULL, orthoType = "OG")
}
\arguments{
\item{id}{ID of the seed protein (OMA or UniProt ID)}

\item{orthoType}{type of OMA orthologs: either "HOG", "OG"
(orthologous group) or "PAIR" (orthologous pair - CURRENTLY NOT WORKING).
Default = "OG".}
}
\value{
List of OMA orthologs for an input seed protein.
}
\description{
Get OMA ortholog group, OMA HOG or OMA pair's members for a
seed protein from OMA Browser.
}
\examples{
### Uncomment the following line to run the function
# getOmaMembers("HUMAN29397", "OG")
}
\author{
Carla Mölbert carla.moelbert@gmx.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterProfile.R
\name{getDistanceMatrix}
\alias{getDistanceMatrix}
\title{Calculate the distance matrix}
\usage{
getDistanceMatrix(profiles = NULL, method = "mutualInformation")
}
\arguments{
\item{profiles}{dataframe contains profile data for distance calculating
(see ?getDataClustering)}

\item{method}{distance calculation method ("euclidean", "maximum",
"manhattan", "canberra", "binary", "distanceCorrelation",
"mutualInformation" or "pearson" for binary data; "distanceCorrelation" or
"mutualInformation" for non-binary data). Default = "mutualInformation".}
}
\value{
A calculated distance matrix for input phylogenetic profiles.
}
\description{
Calculate the distance matrix
}
\examples{
data("finalProcessedProfile", package="PhyloProfile")
data <- finalProcessedProfile
profileType <- "binary"
profiles <- getDataClustering(
    data, profileType, var1AggregateBy, var2AggregateBy)
method <- "mutualInformation"
getDistanceMatrix(profiles, method)
}
\seealso{
\code{\link{getDataClustering}}
}
\author{
Carla Mölbert (carla.moelbert@gmx.de), Vinh Tran
(tran@bio.uni-frankfurt.de)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createTaxonomyMatrix.R
\name{rankIndexing}
\alias{rankIndexing}
\title{Indexing all available ranks (including norank)}
\usage{
rankIndexing(rankListFile = NULL)
}
\arguments{
\item{rankListFile}{Input file, where each row is a rank list of a taxon
(see rankListFile in example)}
}
\value{
A dataframe containing a list of all possible ranks and their indexed
values.
}
\description{
Indexing all available ranks (including norank)
}
\examples{
rankListFile <- system.file(
    "extdata", "data/rankList.txt", package = "PhyloProfile", mustWork = TRUE
)
PhyloProfile:::rankIndexing(rankListFile)
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createDomainPlot.R
\name{createArchiPlot}
\alias{createArchiPlot}
\title{Create protein's domain architecure plot}
\usage{
createArchiPlot(info, domainDf, labelArchiSize, titleArchiSize,
    legendArchiSize, showScore, showWeight, namePosition, firstDist, 
    nameType, nameSize, segmentSize, nameColor, labelPos, colorType, 
    ignoreInstanceNo, currentNCBIinfo, featureClassSort, featureClassOrder, 
    colorPalette, resolveOverlap, font)
}
\arguments{
\item{info}{A list contains seed and ortholog's IDs}

\item{domainDf}{Dataframe contains domain info for the seed and ortholog.
This including the seed ID, orthologs IDs, sequence lengths, feature names,
start and end positions, feature weights (optional) and the status to
determine if that feature is important for comparison the architecture
between 2 proteins* (e.g. seed protein vs ortholog) (optional).}

\item{labelArchiSize}{Lable size (in px). Default = 12.}

\item{titleArchiSize}{Title size (in px). Default = 12.}

\item{legendArchiSize}{Title size (in px). Default = 12.}

\item{showScore}{Show/hide E-values and Bit-scores. Default = NULL (hide)}

\item{showWeight}{Show/hide feature weights. Default = NULL (hide)}

\item{namePosition}{list of positions for domain names, choose from "plot",
"legend" or "axis". Default: "plot"}

\item{firstDist}{Distance of the first domain to plot title. Default = 0.5}

\item{nameType}{Type of domain names, either "Texts" or "Labels" (default)}

\item{nameSize}{Size of domain names. Default = 3}

\item{segmentSize}{Height of domain segment. Default = 5}

\item{nameColor}{Color of domain names (for Texts only). Default = "black"}

\item{labelPos}{Position of domain names (for Labels only). Choose from}

\item{colorType}{Choose to color "all", "shared", "unique" features or color
by "Feature type". Default = "all"}

\item{ignoreInstanceNo}{Ignore number of feature instances while identifying
shared or unique features. Default = FALSE}

\item{currentNCBIinfo}{Dataframe of the pre-processed NCBI taxonomy
data. Default = NULL (will be automatically retrieved from PhyloProfile app)}

\item{featureClassSort}{Choose to sort features. Default = "Yes"}

\item{featureClassOrder}{vector of ordered feature classes}

\item{colorPalette}{Choose between "Paired", "Set1", "Set2", "Set3",
"Accent", "Dark2" for the color pallete}

\item{resolveOverlap}{Choose to merge non-overlapped features of a feature
type into one line. Default = "Yes"}

\item{font}{font of text. Default = Arial"}
}
\value{
A domain plot as arrangeGrob object. Use grid::grid.draw(plot) to
render.
}
\description{
Create architecture plot for both seed and orthologous protein.
If domains of ortholog are missing, only architecture of seed protein will
be plotted. NOTE: seed protein ID is the one being shown in the profile plot,
which normally is also the orthologous group ID.
}
\examples{
seedID <- "101621at6656"
orthoID <- "101621at6656|AGRPL@224129@0|224129_0:001955|1"
info <- c(seedID, orthoID)
domainFile <- system.file(
    "extdata", "domainFiles/101621at6656.domains",
    package = "PhyloProfile", mustWork = TRUE
)
domainDf <- parseDomainInput(seedID, domainFile, "file")
domainDf$feature_id_mod <- domainDf$feature_id
domainDf$feature_id_mod <- gsub("SINGLE", "LCR", domainDf$feature_id_mod)
domainDf$feature_id_mod[domainDf$feature_type == "coils"] <- "Coils"
domainDf$feature_id_mod[domainDf$feature_type == "seg"] <- "LCR"
domainDf$feature_id_mod[domainDf$feature_type == "tmhmm"] <- "TM"
plot <- createArchiPlot(info, domainDf, font = "sans")
grid::grid.draw(plot)
}
\seealso{
\code{\link{singleDomainPlotting}},\code{\link{pairDomainPlotting}},
\code{\link{sortDomains}}, \code{\link{parseDomainInput}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compareTaxaGroups.R
\name{distributionTest}
\alias{distributionTest}
\title{Compare the distribution of 2 numeric vectors}
\usage{
distributionTest(varIn, varOut, significanceLevel)
}
\arguments{
\item{varIn}{first numeric vector}

\item{varOut}{second numeric vector}

\item{significanceLevel}{significant cutoff of the Kolmogorov-Smirnov test.
Default = 0.05.}
}
\value{
p-value of the comparison test.
}
\description{
This function tests the difference between the distributions of
two input numeric samples using the statistical tess. First the
Kolmogorov-Smirnov is used to check if 2 samples have the same distribution.
If yes, Wilcoxon-Mann-Whitney will be used to compare the distribution
difference.
}
\author{
Carla Mölbert (carla.moelbert@gmx.de)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createColorScheme.R
\name{qualitativeColours}
\alias{qualitativeColours}
\title{Create qualitative colours}
\source{
Modified based on https://gist.github.com/peterk87/6011397
}
\usage{
qualitativeColours(n, light = FALSE)
}
\arguments{
\item{n}{number of colors}

\item{light}{light colors TRUE or FALSE}
}
\value{
list of n different colors
}
\description{
Create qualitative colours
}
\examples{
PhyloProfile:::qualitativeColours(5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compareTaxaGroups.R
\name{featureDistTaxPlot}
\alias{featureDistTaxPlot}
\title{Create feature distribution comparison plot}
\usage{
featureDistTaxPlot(data, plotParameters)
}
\arguments{
\item{data}{dataframe for plotting (see ?dataFeatureTaxGroup)}

\item{plotParameters}{plot parameters, including size of x-axis, y-axis,
legend and title; position of legend ("right", "bottom" or "none"); names of
in-group and out-group; flip the plot coordinate ("Yes" or "No").
NOTE: Leave blank or NULL to use default values.}
}
\value{
Distribution plots as a ggplot2 object.
}
\description{
Create protein feature distribution plots between 2 groups of
taxa for a selected gene.
}
\examples{
data("mainLongRaw", package="PhyloProfile")
data <- mainLongRaw
gene <- "101621at6656"
inputFile <- system.file(
    "extdata", "domainFiles/101621at6656.domains",
    package = "PhyloProfile", mustWork = TRUE
)
type <- "file"
domainDf <- parseDomainInput(gene, inputFile, type)
inGroup <- c("ncbi9606", "ncbi10116")
plotDf <- dataFeatureTaxGroup(data, domainDf, inGroup, gene)
plotParameters <- list(
    "xSize" = 12,
    "ySize" = 12,
    "angle" = 15,
    "legendSize" = 12,
    "inGroupName" = "In-group",
    "outGroupName" = "Out-group",
    "flipPlot" = "No"
)
featureDistTaxPlot(plotDf, plotParameters)
}
\seealso{
\code{\link{dataFeatureTaxGroup}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimateGeneAge.R
\name{geneAgePlotDf}
\alias{geneAgePlotDf}
\title{Create data for plotting gene ages}
\usage{
geneAgePlotDf(geneAgeDf)
}
\arguments{
\item{geneAgeDf}{data frame containing estimated gene ages for seed proteins}
}
\value{
A dataframe for plotting gene age plot containing the absolute number
and percentage of genes for each calculated evolutionary ages and the
corresponding position for writting those number on the plot.
}
\description{
Create data for plotting gene ages
}
\examples{
geneAgeDf <- data.frame(
geneID = c("100136at6656", "100265at6656", "101621at6656", "103479at6656"),
cath = c("0000001", "0000011", "0000001", "0000011"),
age = c("07_LUCA", "06_Eukaryota", "07_LUCA", "06_Eukaryota")
)
geneAgePlotDf(geneAgeDf)
}
\seealso{
\code{\link{estimateGeneAge}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyzeDistribution.R
\name{createVariableDistributionDataSubset}
\alias{createVariableDistributionDataSubset}
\title{Create data for additional variable distribution (for a subset data)}
\usage{
createVariableDistributionDataSubset(fullProfileData,
    distributionData, selectedGenes, selectedTaxa)
}
\arguments{
\item{fullProfileData}{dataframe contains the full processed profiles (see
?fullProcessedProfile, ?filterProfileData or ?fromInputToProfile)}

\item{distributionData}{dataframe contains the full distribution data (see
?createVariableDistributionData)}

\item{selectedGenes}{list of genes of interest. Default = "all".}

\item{selectedTaxa}{list of taxa of interest Default = "all".}
}
\value{
A dataframe for analysing the distribution of the additional
variable(s) for a subset of genes and/or taxa containing the protein
(ortholog) IDs and the values of their variables (var1 and var2).
}
\description{
Create data for additional variable distribution (for a subset data)
}
\examples{
data("fullProcessedProfile", package="PhyloProfile")
data("mainLongRaw", package="PhyloProfile")
distributionData <- createVariableDistributionData(
    mainLongRaw, c(0, 1), c(0.5, 1)
)
selectedGenes <- "100136at6656"
selectedTaxa <- c("Mammalia", "Saccharomycetes", "Insecta")
createVariableDistributionDataSubset(
    fullProcessedProfile,
    distributionData,
    selectedGenes,
    selectedTaxa
)
}
\seealso{
\code{\link{parseInfoProfile}},
\code{\link{createVariableDistributionData}},
\code{\link{fullProcessedProfile}}, \code{\link{mainLongRaw}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/umapClustering.R
\name{addDimRedTaxaColors}
\alias{addDimRedTaxaColors}
\title{Add colors for taxa in dimension reduction plot}
\usage{
addDimRedTaxaColors(plotDf = NULL, colorPalette = "Set2",
    highlightTaxa = NULL)
}
\arguments{
\item{plotDf}{data for dimension reduction plot}

\item{colorPalette}{color palette. Default: "Set2"}

\item{highlightTaxa}{list of taxa to be highlighted}
}
\value{
A dataframe for dimension reduction plot with an additional column
for the assigned color to each taxon
}
\description{
Add colors for taxa in dimension reduction plot
}
\examples{
rawInput <- system.file(
   "extdata", "test.main.long", package = "PhyloProfile", mustWork = TRUE
)
longDf <- createLongMatrix(rawInput)
data4dimRed <- prepareDimRedData(longDf, "phylum")
dimRedCoord <- dimReduction(data4dimRed)
plotDf <- createDimRedPlotData(dimRedCoord, data4dimRed)
PhyloProfile:::addDimRedTaxaColors(plotDf, colorPalette = "Set2")
}
\seealso{
\code{\link{prepareDimRedData}}, \code{\link{dimReduction}},
\code{\link{createDimRedPlotData}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ppTaxonomyMatrix}
\alias{ppTaxonomyMatrix}
\title{An example of a taxonomy matrix}
\format{
Dataframe
}
\usage{
data(ppTaxonomyMatrix)
}
\value{
A data frame with 10 rows and 162 variables:
\itemize{
    \item abbrName  e.g. "ncbi10090"
    \item ncbiID e.g. "10090"
    \item fullName e.g. "Mus musculus"
    \item strain e.g. "10090"
    ...
}
}
\description{
An example of a taxonomy matrix
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parsePhyloProfile.R
\name{filterProfileData}
\alias{filterProfileData}
\title{Filter phylogentic profiles}
\usage{
filterProfileData(DF, taxaCount, refTaxon = NULL,
    percentCO = c(0, 1), coorthoCOMax = 9999,
    var1CO  = c(0, 1), var2CO = c(0, 1), var1Rel = "protein",
    var2Rel = "protein", groupByCat = FALSE, catDt = NULL,
    var1AggregateBy = "max", var2AggregateBy = "max")
}
\arguments{
\item{DF}{a reduced dataframe contains info for all phylogenetic
profiles in the selected taxonomy rank.}

\item{taxaCount}{dataframe counting present taxa in each supertaxon}

\item{refTaxon}{selected reference taxon. NOTE: This taxon will not be
affected by the filtering. If you want to filter all, set refTaxon <- NULL.
Default = NULL.}

\item{percentCO}{min and max cutoffs for percentage of species present
in a supertaxon. Default = c(0, 1).}

\item{coorthoCOMax}{maximum number of co-orthologs allowed. Default =
9999.}

\item{var1CO}{min and max cutoffs for var1. Default = c(0, 1).}

\item{var2CO}{min anc max cutoffs for var2. Default = c(0, 1).}

\item{var1Rel}{relation of var1 ("protein" for protein-protein or
"species" for protein-species). Default = "protein".}

\item{var2Rel}{relation of var2 ("protein" for protein-protein or
"species" for protein-species). Default = "protein".}

\item{groupByCat}{group genes by their categories (TRUE or FALSE). Default =
FALSE.}

\item{catDt}{dataframe contains gene categories
(optional, NULL if groupByCat = FALSE or no info provided). Default = NULL.}

\item{var1AggregateBy}{aggregate method for VAR1 (max, min, mean
or median), applied for calculating var1 of supertaxa. Default = "max".}

\item{var2AggregateBy}{aggregate method for VAR2 (max, min, mean
or median), applied for calculating var2 of supertaxa. Default = "max".}
}
\value{
A filtered dataframe for generating profile plot including seed gene
IDs (or orthologous group IDs), their ortholog IDs and the corresponding
(super)taxa, (super)taxon IDs, number of co-orthologs in each (super)taxon,
values for two additional variables var1, var2, % of species present in each
supertaxon, and the categories of seed genes (or ortholog groups).
}
\description{
Create a filtered data needed for plotting or clustering
phylogenetic profiles. NOTE: this function require some intermediate steps
using the results from other functions. If you would like to get a full
processed data from the raw input, please use the function
fromInputToProfile() instead!
}
\examples{
# NOTE: this function require some intermediate steps using the results from
# other functions. If you would like to get a full processed data from the
# raw input, please use the function fromInputToProfile() instead!
library(dplyr)
data("fullProcessedProfile", package="PhyloProfile")
rankName <- "class"
refTaxon <- "Mammalia"
percentCutoff <- c(0.0, 1.0)
coorthologCutoffMax <- 10
var1Cutoff <- c(0.75, 1.0)
var2Cutoff <- c(0.5, 1.0)
var1Relation <- "protein"
var2Relation <- "species"
groupByCat <- FALSE
catDt <- NULL
var1AggregateBy <- "max"
var2AggregateBy <- "max"
taxonIDs <- levels(as.factor(fullProcessedProfile$ncbiID))
sortedInputTaxa <- sortInputTaxa(
    taxonIDs, rankName, refTaxon, NULL, NULL
)
taxaCount <- sortedInputTaxa \%>\% dplyr::group_by(supertaxon) \%>\%
    summarise(n = n(), .groups = "drop")
filterProfileData(
    fullProcessedProfile,
    taxaCount,
    refTaxon,
    percentCutoff,
    coorthologCutoffMax,
    var1Cutoff,
    var2Cutoff,
    var1Relation,
    var2Relation,
    groupByCat,
    catDt,
    var1AggregateBy,
    var2AggregateBy
)
}
\seealso{
\code{\link{parseInfoProfile}} and \code{\link{reduceProfile}}
for generating input dataframe, \code{\link{fullProcessedProfile}} for a
demo full processed profile dataframe, \code{\link{fromInputToProfile}} for
generating fully processed data from raw input.
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/umapClustering.R
\name{performUmap}
\alias{performUmap}
\title{Helper function to handle UMAP logic}
\usage{
performUmap(umapDt, randomSeed = 123, dim = 2)
}
\arguments{
\item{umapDt}{data matrix for UMAP}

\item{randomSeed}{random seed. Default: 123}

\item{dim}{dimension, either 2 for 2D (default) or 3 for 3D}
}
\value{
A table contains coordinates UMAP reduction
}
\description{
Helper function to handle UMAP logic
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compareTaxaGroups.R
\name{generateSinglePlot}
\alias{generateSinglePlot}
\title{Create a single violin distribution plot}
\usage{
generateSinglePlot(plotDf, parameters, variable)
}
\arguments{
\item{plotDf}{dataframe for plotting containing values for each variable in
in-group and out-group.}

\item{parameters}{plot parameters, including size of x-axis, y-axis,
legend and title; position of legend ("right", "bottom" or "none");
mean/median point; names of in-group and out-group; and plot title.
NOTE: Leave blank or NULL to use default values.}

\item{variable}{name of variable that need to be plotted (one of the column
names of input dataframe plotDf).}
}
\value{
A violin plot as a ggplot object.
}
\description{
Create a single violin distribution plot
}
\examples{
data("mainLongRaw", package="PhyloProfile")
data <- mainLongRaw
inGroup <- c("ncbi9606", "ncbi10116")
varNames <- colnames(data)[c(4, 5)]
plotDf <- dataVarDistTaxGroup(data, inGroup, "101621at6656", varNames)
plotParameters <- list(
    "xSize" = 12,
    "ySize" = 12,
    "titleSize" = 15,
    "legendSize" = 12,
    "legendPosition" = "right",
    "mValue" = "mean",
    "inGroupName" = "In-group",
    "outGroupName" = "Out-group",
    "title" = "101621at6656"
)
generateSinglePlot(plotDf, plotParameters, colnames(plotDf)[1])
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/umapClustering.R
\name{plotDimRed3D}
\alias{plotDimRed3D}
\title{Create dimension reduction 3D plot}
\usage{
plotDimRed3D(plotDf = NULL, legendPos = "bottom",
    colorPalette = "Set2", transparent = 0,highlightTaxa = NULL,
    dotZoom = 0)
}
\arguments{
\item{plotDf}{data for dimension reduction 3D plot}

\item{legendPos}{position of legend. Default: "right"}

\item{colorPalette}{color palette. Default: "Set2"}

\item{transparent}{transparent level (from 0 to 1). Default: 0}

\item{highlightTaxa}{list of taxa to be highlighted}

\item{dotZoom}{dot size zooming factor. Default: 0}
}
\value{
A plot as ggplot object
}
\description{
Create dimension reduction 3D plot
}
\examples{
rawInput <- system.file(
   "extdata", "test.main.long", package = "PhyloProfile", mustWork = TRUE
)
longDf <- createLongMatrix(rawInput)
data4dimRed <- prepareDimRedData(longDf, "phylum")
dimRedCoord3d <- dimReduction(data4dimRed, dimension = "3d")
plotDf <- createDimRedPlotData(dimRedCoord3d, data4dimRed)
plotDimRed3D(plotDf)
}
\seealso{
\code{\link{prepareDimRedData}}, \code{\link{dimReduction}},
\code{\link{createDimRedPlotData}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{rankList}
\alias{rankList}
\title{NCBI rank list for experimental data sets}
\format{
Dataframe
}
\usage{
data(rankList)
}
\value{
A data frame with up to 41 columns and 95 rows corresponding to 95
    taxa in the 2 experimental data sets
}
\description{
Data frame, in which each row contains the complete taxonomy
ranks from the lowest systematic level (strain/species) upto the taxonomy
root for one taxon in the experimental data sets.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/umapClustering.R
\name{fallbackUmap}
\alias{fallbackUmap}
\title{Fallback for UMAP in case of insufficient samples}
\usage{
fallbackUmap(umapDt, randomSeed, dim)
}
\arguments{
\item{umapDt}{data matrix for UMAP}

\item{randomSeed}{random seed. Default: 123}

\item{dim}{dimension, either 2 for 2D (default) or 3 for 3D}
}
\value{
A table contains coordinates UMAP reduction
}
\description{
Fallback for UMAP in case of insufficient samples
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parseMainInput.R
\name{xmlParser}
\alias{xmlParser}
\title{Parse orthoXML input file}
\usage{
xmlParser(inputFile = NULL)
}
\arguments{
\item{inputFile}{input file in xml format}
}
\value{
A data frame of input data in long-format containing seed gene IDs (
or orthologous group IDs), their orthologous proteins together with the
corresponding taxonomy IDs and values of (up to) two additional variables.
}
\description{
Parse orthoXML input file
}
\examples{
inputFile <- system.file(
    "extdata", "test.main.xml", package = "PhyloProfile", mustWork = TRUE
)
xmlParser(inputFile)
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createDomainPlot.R
\name{modifyFeatureName}
\alias{modifyFeatureName}
\title{Modify feature names}
\usage{
modifyFeatureName(domainDf = NULL)
}
\arguments{
\item{domainDf}{domain data as a dataframe object}
}
\value{
Dataframe contains simlified domain names in yLabel column
}
\description{
Simplify feature names (e.g. TM for transmembrane domain,
LCR for low complexity regions, remove tool names from domain name) and add
weight to feature names (if available)
}
\examples{
domainFile <- system.file(
    "extdata", "domainFiles/101621at6656.domains",
    package = "PhyloProfile", mustWork = TRUE
)
seedID <- "101621at6656"
domainDf <- parseDomainInput(seedID, domainFile, "file")
PhyloProfile:::modifyFeatureName(domainDf)
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getFastaSeqs.R
\name{getFastaFromFolder}
\alias{getFastaFromFolder}
\title{Get fasta sequences}
\usage{
getFastaFromFolder(seqIDs = NULL, path = NULL, dirFormat = NULL,
    fileExt = NULL, idFormat = NULL)
}
\arguments{
\item{seqIDs}{list of sequences IDs.}

\item{path}{path to fasta folder.}

\item{dirFormat}{directory format (either 1 for "path/speciesID.fa*" or 2 for
"path/speciesID/speciesID.fa*")}

\item{fileExt}{fasta file extension ("fa", "fasta", "fas" or "txt")}

\item{idFormat}{fasta header format (1 for ">speciesID:seqID", 2 for
">speciesID@seqID", 3 for ">speciesID|seqID" or 4 for "seqID")}
}
\value{
A dataframe with one column contains sequences in fasta format.
}
\description{
Get fasta sequences for the input phylogenetic profiles.
}
\examples{
seqIDs <- "RAT@10116@1|D3ZUE4"
path <- system.file(
    "extdata", "fastaFiles", package = "PhyloProfile", mustWork = TRUE
)
dirFormat <- 1
fileExt <- "fa"
idFormat <- 3
getFastaFromFolder(seqIDs, path, dirFormat, fileExt, idFormat)
}
\seealso{
\code{\link{mainLongRaw}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterProfile.R
\name{clusterDataDend}
\alias{clusterDataDend}
\title{Create a hclust object from the distance matrix}
\usage{
clusterDataDend(distanceMatrix = NULL, clusterMethod = "complete")
}
\arguments{
\item{distanceMatrix}{calculated distance matrix as dist object}

\item{clusterMethod}{clustering method ("single", "complete",
"average" for UPGMA, "mcquitty" for WPGMA, "median" for WPGMC,
or "centroid" for UPGMC). Default = "complete".}
}
\value{
An object class hclust generated based on input distance matrix and
a selected clustering method.
}
\description{
Create a hclust object from the distance matrix
}
\examples{
data("finalProcessedProfile", package="PhyloProfile")
data <- finalProcessedProfile
profileType <- "binary"
profiles <- getDataClustering(
    data, profileType, var1AggregateBy, var2AggregateBy)
distMethod <- "mutualInformation"
distanceMatrix <- getDistanceMatrix(profiles, distMethod)
clusterMethod <- "complete"
clusterDataDend(as.dist(distanceMatrix), clusterMethod)
}
\seealso{
\code{\link{getDataClustering}},
\code{\link{getDistanceMatrix}}, \code{\link{hclust}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createDomainPlot.R
\name{sortDomainsByList}
\alias{sortDomainsByList}
\title{Sort one domain dataframe based on list of ordered feature types}
\usage{
sortDomainsByList(domainDf = NULL, featureClassOrder = NULL)
}
\arguments{
\item{domainDf}{domain dataframe}

\item{featureClassOrder}{vector of ordered feature classes}
}
\value{
Dataframe contains sorted domain list.
}
\description{
Sort domain dataframe of one protein based on a given list of
ordered feature types
}
\examples{
# get domain data
seedID <- "101621at6656"
domainFile <- system.file(
    "extdata", "domainFiles/101621at6656.domains",
    package = "PhyloProfile", mustWork = TRUE
)
domainDf <- parseDomainInput(seedID, domainFile, "file")
# get seedDf and orthoDf
subDf <- domainDf[
    domainDf$seedID ==
    "101621at6656#101621at6656:AGRPL@224129@0:224129_0:001955:1",]
orthoDf <- subDf[subDf$orthoID == "101621at6656:DROME@7227@1:Q9VG04",]
featureClassOrder <- c("pfam", "smart", "tmhmm", "coils", "signalp", "seg",
    "flps")
# sort
PhyloProfile:::sortDomainsByList(orthoDf, featureClassOrder)
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterProfile.R
\name{getDataClustering}
\alias{getDataClustering}
\title{Get data for calculating distance matrix from phylogenetic profiles}
\usage{
getDataClustering(data, profileType = "binary", var1AggBy = "max",
    var2AggBy = "max")
}
\arguments{
\item{data}{a data frame contains processed and filtered profiles (see
?fullProcessedProfile and ?filterProfileData, ?fromInputToProfile)}

\item{profileType}{type of data used for calculating the distance matrix.
Either "binary" (consider only the presence/absence status of orthlogs),
"orthoID" (consider ortholog IDs as values for clustering),
"var1"/"var2" for taking values of the additional variables into account.
Default = "binary".}

\item{var1AggBy}{aggregate method for VAR1 (min, max, mean or median).
Default = "max".}

\item{var2AggBy}{aggregate method for VAR2 (min, max, mean or median).
Default = "max".}
}
\value{
A wide dataframe contains values for calculating distance matrix.
}
\description{
Get data for calculating distance matrix from phylogenetic profiles
}
\examples{
data("finalProcessedProfile", package="PhyloProfile")
data <- finalProcessedProfile
profileType <- "binary"
var1AggregateBy <- "max"
var2AggregateBy <- "mean"
getDataClustering(data, profileType, var1AggregateBy, var2AggregateBy)
}
\seealso{
\code{\link{fromInputToProfile}}
}
\author{
Carla Mölbert (carla.moelbert@gmx.de), Vinh Tran
(tran@bio.uni-frankfurt.de)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getOmaBrowser.R
\name{getAllDomainsOma}
\alias{getAllDomainsOma}
\title{Create domain annotation dataframe from a raw OMA dataframe}
\usage{
getAllDomainsOma(finalOmaDf = NULL)
}
\arguments{
\item{finalOmaDf}{raw OMA data for a list of proteins (see ?getDataForOneOma)}
}
\value{
Dataframe of the domain annotation used for PhyloProfile, which
contains seed IDs, ortholog IDs, ortholog lengths, annotated features, start
and end positions of those features.
}
\description{
Create domain annotation dataframe from a raw OMA dataframe
}
\examples{
### Uncomment the following line to run the function
# omaData <- getDataForOneOma("HUMAN29397", "OG")
# getAllDomainsOma(omaData)
}
\seealso{
\code{\link{getDataForOneOma}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createTaxonomyMatrix.R
\name{processNcbiTaxonomy}
\alias{processNcbiTaxonomy}
\title{Pre-processing NCBI taxonomy data}
\usage{
processNcbiTaxonomy()
}
\value{
A dataframe contains NCBI taxon IDs, taxon names, taxon ranks and the
next higher taxon IDs (parent's IDs) of all taxa in the NCBI taxonomy
database.
}
\description{
Download NCBI taxonomy database and parse information that are
needed for PhyloProfile, including taxon IDs, their scientific names,
systematic ranks, and parent (next higher) rank IDs.
}
\examples{
\dontrun{
?processNcbiTaxonomy
preProcessedTaxonomy <- PhyloProfile:::processNcbiTaxonomy()
# save to text (tab-delimited) file
write.table(
    preProcessedTaxonomy,
    file = "preProcessedTaxonomy.txt",
    col.names = TRUE,
    row.names = FALSE,
    quote = FALSE,
    sep = "\t"
)
# save to rdata file
save(
    preProcessedTaxonomy, file = "preProcessedTaxonomy.RData", compress='xz'
)
}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createTaxonomyMatrix.R
\name{id2name}
\alias{id2name}
\title{Get taxon names for a list of taxon IDs}
\usage{
id2name(idList = NULL, currentNCBIinfo = NULL)
}
\arguments{
\item{idList}{list of taxonomy IDs}

\item{currentNCBIinfo}{table/dataframe of the pre-processed NCBI taxonomy
data (/PhyloProfile/data/preProcessedTaxonomy.txt)}
}
\value{
A dataframe contains input taxon Ids and their full names.
}
\description{
Get taxon names for a list of taxon IDs
}
\examples{
ncbiFilein <- system.file(
    "extdata", "data/preProcessedTaxonomy.txt",
    package = "PhyloProfile", mustWork = TRUE
)
currentNCBIinfo <- as.data.frame(data.table::fread(ncbiFilein))
idList <- c("9606", "5207", "40674", "4751")
id2name(idList, currentNCBIinfo)
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compareTaxaGroups.R
\name{compareTaxonGroups}
\alias{compareTaxonGroups}
\title{Compare the score distributions between 2 taxon groups}
\usage{
compareTaxonGroups(data, inGroup, useCommonAncestor, variable,
    significanceLevel, taxDB)
}
\arguments{
\item{data}{input phylogenetic profile in long format (see ?mainLongRaw and
?createLongMatrix)}

\item{inGroup}{ID list of in-group taxa (e.g. "ncbi1234")}

\item{useCommonAncestor}{TRUE/FALSE if using all taxa that share the same
common ancestor with the pre-selected in-group as the in-group taxa.
Default = TRUE.}

\item{variable}{name of the variable that need to be compared}

\item{significanceLevel}{significant cutoff for the statistic test (between
0 and 1). Default = 0.05.}

\item{taxDB}{Path to the taxonomy DB files}
}
\value{
list of genes that have a significant difference in the variable
distributions between the in-group and out-group taxa and their corresponding
p-values.
}
\description{
Given the phylogenetic profiles that contains up to 2 additional
variables besides the presence/absence information of the orthologous
proteins. This function will compare the distribution of those variables
between 2 different taxon groups (e.g. parasitic species vs non-parasitic
species), which are defined as in-group and out-group. In-group is identified
by the user. Out-group contains all taxa in the input phylogenetic profiles
that are not part of the in-group.
}
\examples{
data("mainLongRaw", package="PhyloProfile")
data <- mainLongRaw
inGroup <- c("ncbi9606", "ncbi10116")
variable <- colnames(data)[4]
compareTaxonGroups(data, inGroup, TRUE, variable, 0.05)
}
\author{
Vinh Tran (tran@bio.uni-frankfurt.de)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ppTree}
\alias{ppTree}
\title{An example of a taxonomy tree in newick format}
\format{
Dataframe
}
\usage{
data(ppTree)
}
\value{
A data frame with only one entry
\itemize{
    \item V1 tree in newick format
}
}
\description{
An example of a taxonomy tree in newick format
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createProfilePlot.R
\name{processOrthoID}
\alias{processOrthoID}
\title{Process ortholog IDs}
\usage{
processOrthoID(dataHeat = NULL)
}
\arguments{
\item{dataHeat}{a data frame contains processed profiles (see
?fullProcessedProfile, ?filterProfileData)}
}
\value{
the same dataframe as input, but the ortholog IDs are changed into
<taxID:orthoID>. New column "orthoFreq" specifies if the ortholog IDs are
single or duplicated
}
\description{
Process ortholog IDs to identify duplicated IDs
}
\examples{
?processOrthoID
data("finalProcessedProfile", package="PhyloProfile")
processOrthoID(finalProcessedProfile)
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compareTaxaGroups.R
\name{dataVarDistTaxGroup}
\alias{dataVarDistTaxGroup}
\title{Create data for variable distribution comparison plot}
\usage{
dataVarDistTaxGroup(data, inGroup, gene, variable)
}
\arguments{
\item{data}{input phylogenetic profile in long format (see ?mainLongRaw and
?createLongMatrix)}

\item{inGroup}{ID list of in-group taxa (e.g. "ncbi1234")}

\item{gene}{ID of gene that need to be plotted the distribution comparison
between in- and out-group taxa.}

\item{variable}{var1 or c(var1, var2)}
}
\value{
Dataframe containing list of values for all available variables for
the selected genes in in-group and out-group taxa (max. 3 columns).
}
\description{
Create data for plotting the distribution comparison between 2
groups of taxa for a selected gene.
}
\examples{
data("mainLongRaw", package="PhyloProfile")
data <- mainLongRaw
inGroup <- c("ncbi9606", "ncbi10116")
variable <- colnames(data)[c(4, 5)]
dataVarDistTaxGroup(data, inGroup, "101621at6656", variable)
}
\seealso{
\code{\link{createLongMatrix}}
}
\author{
Vinh Tran (tran@bio.uni-frankfurt.de)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parsePhyloProfile.R
\name{getInputTaxaID}
\alias{getInputTaxaID}
\title{Get ID list of input taxa from the main input}
\usage{
getInputTaxaID(rawProfile = NULL)
}
\arguments{
\item{rawProfile}{A dataframe of input phylogenetic profile in long format}
}
\value{
List of all input taxon IDs (e.g. ncbi1234). Default = NULL.
}
\description{
Get ID list of input taxa from the main input
}
\examples{
data("mainLongRaw", package="PhyloProfile")
getInputTaxaID(mainLongRaw)
}
\seealso{
\code{\link{createLongMatrix}}, \code{\link{mainLongRaw}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/umapClustering.R
\name{dimReduction}
\alias{dimReduction}
\title{Perform dimension reduction 2D}
\usage{
dimReduction(data4dimRed = NULL, by = "taxa", type = "binary",
    randomSeed = 123, reductionTechnique = "umap", dimension = "2d",
    tsneIter = 1000)
}
\arguments{
\item{data4dimRed}{data for dimension reduction (from prepareDimRedData)}

\item{by}{cluster data by "taxa" (default) or "genes"}

\item{type}{type of data, either "binary" (default) or "non-binary"}

\item{randomSeed}{random seed. Default: 123}

\item{reductionTechnique}{dimensionality reduction technique, either "umap"
(default) or "tsne"}

\item{dimension}{either "2d" (default) or "3d"}

\item{tsneIter}{number of iterations for t-SNE. Default: 1000}
}
\value{
A table contains coordinates of the 2D dimension reduction
}
\description{
Perform dimension reduction 2D
}
\examples{
rawInput <- system.file(
   "extdata", "test.main.long", package = "PhyloProfile", mustWork = TRUE
)
longDf <- createLongMatrix(rawInput)
data4dimRed <- prepareDimRedData(longDf, "phylum")
dimReduction(data4dimRed)
}
\seealso{
\code{\link{prepareDimRedData}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parseDomainInput.R
\name{parseDomainInput}
\alias{parseDomainInput}
\title{Parse domain input file}
\usage{
parseDomainInput(seed = NULL, inputFile = NULL, type = "file")
}
\arguments{
\item{seed}{seed protein ID}

\item{inputFile}{name of input file (file name or path to folder contains
individual domain files)}

\item{type}{type of data (file" or "folder"). Default = "file".}
}
\value{
A dataframe for protein domains including seed ID, its orthologs IDs,
sequence lengths, feature names, start and end positions, feature weights
(optional) and the status to determine if that feature is important for
comparison the architecture between 2 proteins* (e.g. seed protein vs
ortholog) (optional).
}
\description{
Get all domain annotations for one seed protein IDs.
}
\examples{
seed <- "101621at6656"
inputFile <- system.file(
    "extdata", "domainFiles/101621at6656.domains",
    package = "PhyloProfile", mustWork = TRUE
)
type <- "file"
parseDomainInput(seed, inputFile, type)
}
\seealso{
\code{\link{getDomainFolder}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getOmaBrowser.R
\name{getOmaDomainFromURL}
\alias{getOmaDomainFromURL}
\title{Get domain annotation from OMA Browser}
\usage{
getOmaDomainFromURL(domainURL = NULL)
}
\arguments{
\item{domainURL}{URL address for domain annotation of ONE OMA id or a
raw data frame contains annotation info from OMA}
}
\value{
Data frame contains feature names with their start and end positions
}
\description{
Get domain annotation from OMA Browser based on a URL or a
raw data frame contains annotation info from OMA
}
\examples{
### Uncomment the following line to run the function
# getOmaDomainFromURL("https://omabrowser.org/api/protein/7916808/domains/")
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createDomainPlot.R
\name{joinPlotMergeLegends}
\alias{joinPlotMergeLegends}
\title{Join multiple plots and merge legends}
\usage{
joinPlotMergeLegends(
  df1 = NULL,
  df2 = NULL,
  plot1 = NULL,
  plot2 = NULL,
  position = c("bottom", "right"),
  font = "Arial",
  legendSize = 12
)
}
\arguments{
\item{df1}{Data frame for plot 1}

\item{df2}{Data frame for plot 2}

\item{plot1}{ggplot object of plot 1}

\item{plot2}{ggplot object of plot 2}

\item{position}{position of legend (bottom or right)}

\item{font}{font of text}

\item{legendSize}{font size}
}
\value{
joined plots with merged legend as a grid object
}
\description{
Join multiple plots and merge legends
}
\examples{
seed <- "101621at6656"
ortho <- "101621at6656|AGRPL@224129@0|224129_0:001955|1"
ortho <- gsub("\\\\|", ":", ortho)
grepID <- paste(seed, "#", ortho, sep = "")
domainFile <- system.file(
    "extdata", "domainFiles/101621at6656.domains",
    package = "PhyloProfile", mustWork = TRUE
)
domainDf <- parseDomainInput(seed, domainFile, "file")
domainDf$feature_id_mod <- domainDf$feature_id
subdomainDf <- domainDf[grep(grepID, domainDf$seedID), ]
subdomainDf$feature <- as.character(subdomainDf$feature)
orthoDf <- subdomainDf[subdomainDf$orthoID == ortho,]
seedDf <- subdomainDf[subdomainDf$orthoID != ortho,]
minStart <- min(subdomainDf$start)
maxEnd <- max(c(subdomainDf$end, subdomainDf$length))
# resolve overlapping domains
seedDf <- PhyloProfile:::resolveOverlapFeatures(seedDf)
orthoDf <- PhyloProfile:::resolveOverlapFeatures(orthoDf)
# add feature colors
featureColorDf <- PhyloProfile:::addFeatureColors(seedDf, orthoDf)
seedDf <- featureColorDf[[1]]
orthoDf <- featureColorDf[[2]]
# generate plots
plotSeed <- PhyloProfile:::singleDomainPlotting(
    seedDf, seed, minStart = minStart, maxEnd = maxEnd, font = "sans"
)
plotOrtho <- PhyloProfile:::singleDomainPlotting(
    orthoDf, ortho, minStart = minStart, maxEnd = maxEnd, font = "sans"
)
# merge plots
PhyloProfile:::joinPlotMergeLegends(
    seedDf, orthoDf, plotSeed, plotOrtho, "bottom", font = "sans", 
    legendSize = 12)
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parseMainInput.R
\name{fastaParser}
\alias{fastaParser}
\title{Parse multi-fasta input file}
\usage{
fastaParser(inputFile = NULL)
}
\arguments{
\item{inputFile}{input multiple fasta file. Check extdata/test.main.fasta or
https://github.com/BIONF/PhyloProfile/wiki/Input-Data#multi-fasta-format for
the supported FASTA header.}
}
\value{
A data frame of input data in long-format containing seed gene IDs (
or orthologous group IDs), their orthologous proteins together with the
corresponding taxonomy IDs and values of (up to) two additional variables.
}
\description{
Parse multi-fasta input file
}
\examples{
inputFile <- system.file(
    "extdata", "test.main.fasta", package = "PhyloProfile", mustWork = TRUE
)
fastaParser(inputFile)
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createProfilePlot.R
\name{dataCustomizedPlot}
\alias{dataCustomizedPlot}
\title{Create data for customized profile plot}
\usage{
dataCustomizedPlot(dataHeat = NULL, selectedTaxa = "all",
    selectedSeq = "all")
}
\arguments{
\item{dataHeat}{a data frame contains processed profiles (see
?fullProcessedProfile, ?filterProfileData)}

\item{selectedTaxa}{selected subset of taxa. Default = "all".}

\item{selectedSeq}{selected subset of genes. Default = "all".}
}
\value{
A dataframe contains data for plotting the customized profile.
}
\description{
Create data for customized profile plot based on a selected
list of genes and/or taxa, containing seed protein IDs (geneID), ortholog IDs
(orthoID) together with their ncbi taxonomy IDs (ncbiID and abbrName), full
names (fullName), indexed supertaxa (supertaxon), values for additional
variables (var1, var2) and the aggregated values of those additional
variables for each supertaxon (mVar1, mVar2), number of original and filtered
co-orthologs in each supertaxon (paralog and paralogNew), number of species
in each supertaxon (numberSpec) and the % of species that have orthologs in
each supertaxon (presSpec).
}
\examples{
data("finalProcessedProfile", package="PhyloProfile")
selectedTaxa <- c("Mammalia", "Saccharomycetes", "Insecta")
selectedSeq <- "all"
dataCustomizedPlot(finalProcessedProfile, selectedTaxa, selectedSeq)
}
\seealso{
\code{\link{filterProfileData}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/umapClustering.R
\name{plotDimRed}
\alias{plotDimRed}
\title{Create dimension reduction plot}
\usage{
plotDimRed(plotDf = NULL, legendPos = "bottom",
    colorPalette = "Set2", transparent = 0, textSize = 12, font = "Arial",
    highlightTaxa = NULL, dotZoom = 0)
}
\arguments{
\item{plotDf}{data for dimension reduction 2D plot}

\item{legendPos}{position of legend. Default: "right"}

\item{colorPalette}{color palette. Default: "Set2"}

\item{transparent}{transparent level (from 0 to 1). Default: 0}

\item{textSize}{size of axis and legend text. Default: 12}

\item{font}{font of text. Default = Arial"}

\item{highlightTaxa}{list of taxa to be highlighted}

\item{dotZoom}{dot size zooming factor. Default: 0}
}
\value{
A plot as ggplot object
}
\description{
Create dimension reduction plot
}
\examples{
rawInput <- system.file(
   "extdata", "test.main.long", package = "PhyloProfile", mustWork = TRUE
)
longDf <- createLongMatrix(rawInput)
data4dimRed <- prepareDimRedData(longDf, "phylum")
dimRedCoord <- dimReduction(data4dimRed)
plotDf <- createDimRedPlotData(dimRedCoord, data4dimRed)
plotDimRed(plotDf, font = "sans")
}
\seealso{
\code{\link{prepareDimRedData}}, \code{\link{dimReduction}},
\code{\link{createDimRedPlotData}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parsePhyloProfile.R
\name{sortInputTaxa}
\alias{sortInputTaxa}
\title{Sort list of (super)taxa based on a selected reference (super)taxon}
\usage{
sortInputTaxa(taxonIDs = NULL, rankName, refTaxon = NULL,
    taxaTree = NULL, sortedTaxonList = NULL, taxDB = NULL)
}
\arguments{
\item{taxonIDs}{list of taxon IDs (e.g.: ncbi1234, ncbi9999, ...). Default =
NULL}

\item{rankName}{working taxonomy rank (e.g. "species", "phylum",...)}

\item{refTaxon}{selected reference taxon. Default = NULL}

\item{taxaTree}{taxonomy tree for the input taxa (optional). Default = NULL}

\item{sortedTaxonList}{list of sorted taxa (optional). Default = NULL}

\item{taxDB}{Path to the taxonomy DB files}
}
\value{
A taxonomy matrix for the input taxa ordered by the selected
reference taxon. This matrix is sorted either based on the NCBI taxonomy
info, or based on an user-defined taxonomy tree (if provided).
}
\description{
Sort list of (super)taxa based on a selected reference (super)taxon
}
\examples{
taxonIDs <- c(
    "ncbi10116", "ncbi123851", "ncbi3702", "ncbi13616", "ncbi9606"
)
sortInputTaxa(taxonIDs, "species", "Homo sapiens", NULL, NULL)
}
\seealso{
\code{\link{getNameList}}, \code{\link{getTaxonomyMatrix}},
\code{\link{createUnrootedTree}}, \code{\link{sortTaxaFromTree}},
\code{\link{getInputTaxaName}}, \code{\link{getInputTaxaID}},
\code{\link{createLongMatrix}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{profileWithTaxonomy}
\alias{profileWithTaxonomy}
\title{An example of a raw long input file together with the taxonomy info}
\format{
Dataframe
}
\usage{
data(profileWithTaxonomy)
}
\value{
A data frame with 20 rows and 12 variables:
\itemize{
    \item geneID Seed or ortholog group ID, e.g. "OG_1017"
    \item ncbiID Taxon ID, e.g. "ncbi176299"
    \item orthoID Ortholog ID, e.g. "A.fabrum@176299@1582"
    \item var1 First additional variable
    \item var2 Second additional variable
    \item paralog Number of co-orthologs in the current taxon
    \item abbrName e.g. "ncbi176299"
    \item taxonID Taxon ID, e.g. "176299"
    \item fullName Full taxon name, e.g. "Agrobacterium fabrum str. C58"
    \item supertaxonID Supertaxon ID (only different than ncbiID in case
    working with higher taxonomy rank than input's)
    \item supertaxon Name of the corresponding supertaxon
    \item rank Rank of the supertaxon
}
}
\description{
An example of a raw long input file together with the taxonomy info
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parsePhyloProfile.R
\name{getNameList}
\alias{getNameList}
\title{Get list of pre-installed NCBI taxon names}
\usage{
getNameList(taxDB = NULL)
}
\arguments{
\item{taxDB}{Path to the taxonomy DB files}
}
\value{
List of taxon IDs, their full names, taxonomy ranks and parent IDs
obtained from "PhyloProfile/data/taxonNamesReduced.txt"
}
\description{
Get all NCBI taxon names from
"PhyloProfile/data/taxonNamesReduced.txt"
}
\examples{
getNameList()
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getOmaBrowser.R
\name{checkOmaID}
\alias{checkOmaID}
\title{Check the validity of input OMA IDs}
\usage{
checkOmaID(ids)
}
\arguments{
\item{ids}{list of ids needs to be checked}
}
\value{
List of invalid IDs (not readable for OMA)
}
\description{
Check if input IDs are valid OMA IDs for OMA Browser
}
\examples{
### Uncomment the following line to run the function
# checkOmaID("HUMAN29398")
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parsePhyloProfile.R
\name{fromInputToProfile}
\alias{fromInputToProfile}
\title{Complete processing of raw input phylogenetic profiles}
\usage{
fromInputToProfile(rawInput, rankName, refTaxon = NULL,
    taxaTree = NULL, sortedTaxonList = NULL, var1AggregateBy = "max",
    var2AggregateBy = "max", percentCutoff = c(0, 1),
    coorthologCutoffMax = 9999, var1Cutoff = c(0, 1), var2Cutoff = c(0, 1),
    var1Relation = "protein", var2Relation = "protein", groupByCat = FALSE,
    catDt = NULL, taxDB = NULL)
}
\arguments{
\item{rawInput}{input file (in long, wide, multi-fasta or orthoxml format)}

\item{rankName}{taxonomy rank (e.g. "species","phylum",...)}

\item{refTaxon}{selected reference taxon name (used for sorting and will be
protected from filtering). Default = NULL.}

\item{taxaTree}{input taxonomy tree for taxa in input profiles (optional).
Default = NULL.}

\item{sortedTaxonList}{list of sorted taxa (optional). Default = NULL.}

\item{var1AggregateBy}{aggregate method for var1 (min, max, mean or median).
Default = "max".}

\item{var2AggregateBy}{aggregate method for VAR2 (min, max, mean or median).
Default = "max".}

\item{percentCutoff}{min and max cutoffs for percentage of species present
in a supertaxon. Default = c(0, 1).}

\item{coorthologCutoffMax}{maximum number of co-orthologs allowed. Default =
9999.}

\item{var1Cutoff}{min and max cutoffs for var1. Default = c(0, 1).}

\item{var2Cutoff}{min and max cutoffs for var2. Default = c(0, 1).}

\item{var1Relation}{relation of var1 ("protein" for protein-protein or
"species" for protein-species). Default = "protein".}

\item{var2Relation}{relation of var2 ("protein" for protein-protein or
"species" for protein-species). Default = "protein".}

\item{groupByCat}{group genes by their categories (TRUE or FALSE). Default =
FALSE.}

\item{catDt}{dataframe contains gene categories. Default = NULL}

\item{taxDB}{Path to the taxonomy DB files}
}
\value{
Dataframe required for generating phylogenetic profile plot or
clustering analysis. It contains seed gene IDs (or orthologous group IDs),
their ortholog IDs and the corresponding (super)taxa, (super)taxon IDs,
number of co-orthologs in each (super)taxon, values for two additional
variables var1, var2, % of species present in each supertaxon, and the
categories of seed genes (or ortholog groups).
}
\description{
Create a processed and filtered data for plotting or analysing
phylogenetic profiles from raw input file (from raw input to final filtered
dataframe)
}
\examples{
rawInput <- system.file(
    "extdata", "test.main.long", package = "PhyloProfile", mustWork = TRUE
)
rankName <- "class"
refTaxon <- "Mammalia"
taxaTree <- NULL
sortedTaxonList <- NULL
var1AggregateBy <- "max"
var2AggregateBy <- "mean"
percentCutoff <- c(0.0, 1.0)
coorthologCutoffMax <- 10
var1Cutoff <- c(0.75, 1.0)
var2Cutoff <- c(0.5, 1.0)
var1Relation <- "protein"
var2Relation <- "species"
groupByCat <- FALSE
catDt <- NULL
fromInputToProfile(
    rawInput,
    rankName,
    refTaxon,
    taxaTree,
    sortedTaxonList,
    var1AggregateBy,
    var2AggregateBy,
    percentCutoff,
    coorthologCutoffMax,
    var1Cutoff,
    var2Cutoff,
    var1Relation,
    var2Relation,
    groupByCat,
    catDt
)
}
\seealso{
\code{\link{createLongMatrix}}, \code{\link{getInputTaxaID}},
\code{\link{getInputTaxaName}}, \code{\link{sortInputTaxa}},
\code{\link{parseInfoProfile}}, \code{\link{reduceProfile}},
\code{\link{filterProfileData}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parseDomainInput.R
\name{getDomainFolder}
\alias{getDomainFolder}
\title{Get domain file from a folder for a seed protein}
\usage{
getDomainFolder(seed, domainPath)
}
\arguments{
\item{seed}{seed protein ID}

\item{domainPath}{path to domain folder}
}
\value{
Domain file and its complete directory path for the selected protein.
}
\description{
Get domain file from a folder for a seed protein
}
\examples{
domainPath <- paste0(
    path.package("PhyloProfile", quiet = FALSE), "/extdata/domainFiles"
)
PhyloProfile:::getDomainFolder("101621at6656", domainPath)
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{taxonNamesReduced}
\alias{taxonNamesReduced}
\title{NCBI Taxonomy reduced data set}
\format{
Dataframe
}
\usage{
data(taxonNamesReduced)
}
\value{
A data frame with 4 columns:
\itemize{
    \item ncbiID e.g. "10090"
    \item fullName e.g. "Mus musculus"
    \item rank e.g. "species"
    \item parentID e.g. "862507"
}
}
\description{
A list of NCBI taxonomy info (including taxon IDs, taxon names,
their systematic taxonomy rank and IDs of their next rank - parent IDs) for
95 taxa in two experimental sets included in PhyloProfilData package.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/umapClustering.R
\name{groupLabelDimRedData}
\alias{groupLabelDimRedData}
\title{Reduce the number of labels for DIM reduction plot based on the
gene/taxon frequency}
\usage{
groupLabelDimRedData(data4dimRed = NULL, freqCutoff = c(0,200))
}
\arguments{
\item{data4dimRed}{data for dimension reduction (from prepareDimRedData)}

\item{freqCutoff}{gene/taxon frequency cutoff range. Any labels that are
outside of this range will be assigned as [Other]}
}
\value{
A dataframe similar to input data4dimRed, but with modified Label
column, where less frequent labels are grouped together as "Other"
}
\description{
Reduce the number of labels for DIM reduction plot based on the
gene/taxon frequency
}
\examples{
rawInput <- system.file(
   "extdata", "test.main.long", package = "PhyloProfile", mustWork = TRUE
)
longDf <- createLongMatrix(rawInput)
data4dimRed <- prepareDimRedData(longDf, "phylum")
groupLabelDimRedData(data4dimRed, freqCutoff = c(3,5))
}
\seealso{
\code{\link{prepareDimRedData}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compareTaxaGroups.R
\name{gridArrangeSharedLegend}
\alias{gridArrangeSharedLegend}
\title{Plot Multiple Graphs with Shared Legend in a Grid}
\usage{
gridArrangeSharedLegend(...,  ncol = length(list(...)), nrow = 1,
    position = c("bottom", "right"), title = NA, titleSize = 12)
}
\arguments{
\item{...}{Plots to be arranged in grid}

\item{ncol}{Number of columns in grid}

\item{nrow}{Number of rows in grid}

\item{position}{Gird position (bottom or right)}

\item{title}{Title of grid}

\item{titleSize}{Size of grid title}
}
\value{
Grid of plots with common legend
}
\description{
Plot Multiple Graphs with Shared Legend in a Grid
}
\note{
adapted from https://rdrr.io/github/PhilBoileau/CLSAR/src/R/
gridArrangeSharedLegend.R
}
\examples{
\dontrun{
data("mainLongRaw", package="PhyloProfile")
data <- mainLongRaw
inGroup <- c("ncbi9606", "ncbi10116")
varNames <- colnames(data)[c(4, 5)]
plotDf <- dataVarDistTaxGroup(data, inGroup, "101621at6656", varNames)
plotParameters <- list(
    "xSize" = 12,
    "ySize" = 12,
    "titleSize" = 15,
    "legendSize" = 12,
    "legendPosition" = "right",
    "mValue" = "mean",
    "inGroupName" = "In-group",
    "outGroupName" = "Out-group",
    "title" = "101621at6656"
)
plotVar1 <- generateSinglePlot(plotDf, plotParameters, colnames(plotDf)[1])
plotVar2 <- generateSinglePlot(plotDf, plotParameters, colnames(plotDf)[2])
g <- gridArrangeSharedLegend(
    plotVar1, plotVar2,
    position = plotParameters$legendPosition,
    title = plotParameters$title,
    size = plotParameters$titleSize
)
}
}
\author{
Phil Boileau, \email{philippe.boileau@rimuhc.ca}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getOmaBrowser.R
\name{createProfileFromOma}
\alias{createProfileFromOma}
\title{Create a phylogenetic profile from a raw OMA dataframe}
\usage{
createProfileFromOma(finalOmaDf = NULL)
}
\arguments{
\item{finalOmaDf}{raw OMA data for a list of proteins (see ?getDataForOneOma)}
}
\value{
Dataframe of the phylogenetic profiles in long format, which
contains the seed protein IDs, their orthologous proteins and the
corresponding taxononmy IDs of the orthologs.
}
\description{
Create a phylogenetic profile from a raw OMA dataframe
}
\examples{
### Uncomment the following lines to run the function
# omaData <- getDataForOneOma("HUMAN29397", "OG")
# createProfileFromOma(omaData)
}
\seealso{
\code{\link{getDataForOneOma}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parseMainInput.R
\name{createLongMatrix}
\alias{createLongMatrix}
\title{Create a long matrix format for all kinds of input phylogenetic profiles}
\usage{
createLongMatrix(inputFile = NULL)
}
\arguments{
\item{inputFile}{input profile file in orthoXML, multiple FASTA,
tab-delimited matrix format (wide or long).}
}
\value{
A data frame of input data in long-format containing seed gene IDs (
or orthologous group IDs), their orthologous proteins together with the
corresponding taxonomy IDs and values of (up to) two additional variables.
}
\description{
Create a long matrix format for all kinds of input phylogenetic profiles
}
\examples{
inputFile <- system.file(
    "extdata", "test.main.wide", package = "PhyloProfile", mustWork = TRUE
)
createLongMatrix(inputFile)
}
\seealso{
\code{\link{xmlParser}}, \code{\link{fastaParser}},
\code{\link{wideToLong}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyzeDistribution.R
\name{createPercentageDistributionData}
\alias{createPercentageDistributionData}
\title{Create data for percentage present taxa distribution}
\usage{
createPercentageDistributionData(inputData = NULL, rankName = NULL,
    taxDB = NULL)
}
\arguments{
\item{inputData}{dataframe contains raw input data in long format
(see ?mainLongRaw)}

\item{rankName}{name of the working taxonomy rank (e.g. "species", "family")}

\item{taxDB}{Path to the taxonomy DB files}
}
\value{
A dataframe for analysing the distribution of the percentage of
species in the selected supertaxa, containing the seed protein IDs,
percentage of their orthologs in each supertaxon and the corresponding
supertaxon names.
}
\description{
Create data for percentage present taxa distribution
}
\examples{
data("mainLongRaw", package="PhyloProfile")
createPercentageDistributionData(mainLongRaw, "class")
}
\seealso{
\code{\link{mainLongRaw}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createDomainPlot.R
\name{resolveOverlapFeatures}
\alias{resolveOverlapFeatures}
\title{Modify domain dataframe to resolve overlapped domains/features}
\usage{
resolveOverlapFeatures(domainDf)
}
\arguments{
\item{domainDf}{input domain dataframe}
}
\value{
Domain dataframe with modified feature names that join multiple
domains of the same type that are not overlapped
}
\description{
Modify domain dataframe to resolve overlapped domains/features
}
\examples{
# get domain data
seedID <- "101621at6656"
domainFile <- system.file(
    "extdata", "domainFiles/101621at6656.domains",
    package = "PhyloProfile", mustWork = TRUE
)
domainDf <- parseDomainInput(seedID, domainFile, "file")
# get seedDf and orthoDf
subDf <- domainDf[
    domainDf$seedID ==
    "101621at6656#101621at6656:AGRPL@224129@0:224129_0:001955:1",]
orthoDf <- subDf[subDf$orthoID == "101621at6656:DROME@7227@1:Q9VG04",]
# resolve overlapped featuers
PhyloProfile:::resolveOverlapFeatures(orthoDf)
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parsePhylotree.R
\name{createUnrootedTree}
\alias{createUnrootedTree}
\title{Create unrooted tree from a taxonomy matrix}
\usage{
createUnrootedTree(df)
}
\arguments{
\item{df}{data frame contains taxonomy matrix used for generating tree}
}
\value{
A unrooted taxonomy tree as an object of class "phylo".
}
\description{
Create unrooted tree from a taxonomy matrix
}
\examples{
data("ppTaxonomyMatrix", package = "PhyloProfile")
createUnrootedTree(ppTaxonomyMatrix)
}
\seealso{
\code{\link{taxa2dist}} for distance matrix generation from a
taxonomy matrix, \code{\link{getTaxonomyMatrix}} for getting taxonomy
matrix, \code{\link{ppTaxonomyMatrix}} for a demo taxonomy matrix data
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createProfilePlot.R
\name{highlightProfilePlot}
\alias{highlightProfilePlot}
\title{Highlight gene and/or taxon of interest on the phylogenetic profile plot}
\usage{
highlightProfilePlot(profilePlot = NULL, plotDf = NULL,
    taxonHighlight = "none", workingRank = "none", geneHighlight = NULL,
    taxDB = NULL, xAxis = "taxa")
}
\arguments{
\item{profilePlot}{initial (highlighted) profile plot}

\item{plotDf}{dataframe for plotting the heatmap phylogentic profile}

\item{taxonHighlight}{taxon of interst. Default = "none".}

\item{workingRank}{working taxonomy rank (needed only for highlight taxon).}

\item{geneHighlight}{gene of interest. Default = NULL.}

\item{taxDB}{Path to the taxonomy DB files}

\item{xAxis}{type of x-axis (either "genes" or "taxa")}
}
\value{
A profile heatmap plot with highlighted gene and/or taxon of interest
as ggplot object.
}
\description{
Highlight gene and/or taxon of interest on the phylogenetic profile plot
}
\examples{
data("finalProcessedProfile", package="PhyloProfile")
plotDf <- dataMainPlot(finalProcessedProfile)
plotParameter <- list(
    "xAxis" = "taxa",
    "geneIdType" = "geneID",
    "var1ID" = "FAS_FW",
    "var2ID"  = "FAS_BW",
    "midVar1" = 0.5,
    "midColorVar1" =  "#FFFFFF",
    "lowColorVar1" =  "#FF8C00",
    "highColorVar1" = "#4682B4",
    "midVar2" = 1,
    "midColorVar2" =  "#FFFFFF",
    "lowColorVar2" = "#CB4C4E",
    "highColorVar2" = "#3E436F",
    "paraColor" = "#07D000",
    "xSize" = 8,
    "ySize" = 8,
    "legendSize" = 8,
    "mainLegend" = "top",
    "dotZoom" = 0,
    "xAngle" = 60,
    "guideline" = 0,
    "colorByGroup" = FALSE,
    "colorByOrthoID" = FALSE
)
profilePlot <- heatmapPlotting(plotDf, plotParameter)
taxonHighlight <- "none"
workingRank <- "class"
geneHighlight <- "100265at6656"
highlightProfilePlot(
    profilePlot, plotDf, taxonHighlight, workingRank, geneHighlight,
    NULL, plotParameter$xAxis
)
}
\seealso{
\code{\link{dataMainPlot}}, \code{\link{dataCustomizedPlot}},
\code{\link{heatmapPlotting}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createColorScheme.R
\name{getQualColForVector}
\alias{getQualColForVector}
\title{Get color for a list of items}
\usage{
getQualColForVector(x = NULL)
}
\arguments{
\item{x}{input list}
}
\value{
list of colors for each element (same elements will have the same
color)
}
\description{
Get color for a list of items
}
\examples{
items <- c("a", "b", "c")
getQualColForVector(items)
}
\seealso{
\code{\link{qualitativeColours}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parsePhyloProfile.R
\name{getSelectedTaxonNames}
\alias{getSelectedTaxonNames}
\title{Get a subset of input taxa based on a selected taxonomy rank}
\usage{
getSelectedTaxonNames(inputTaxonIDs = NULL, rank = NULL,
    higherRank = NULL, higherID = NULL, higherName = NULL, taxDB = NULL)
}
\arguments{
\item{inputTaxonIDs}{list of input taxon IDs (e.g. c("10116", "122586"))}

\item{rank}{taxonomy rank of input taxa (e.g. "species")}

\item{higherRank}{selected taxonomy rank (e.g. "phylum")}

\item{higherID}{supertaxon ID (e.g. 7711). NOTE: either supertaxon ID or
name is required, not neccessary to give both}

\item{higherName}{supertaxon name (e.g. "Chordata"). NOTE: either
supertaxon ID or name is required, not neccessary to give both}

\item{taxDB}{Path to the taxonomy DB files}
}
\value{
A data frame contains ncbi IDs and names of taxa from the input taxon
list that belong to the selected supertaxon.
}
\description{
Get a subset of taxon ncbi IDs and names from an input list of
taxa based on a selected supertaxon (identified by its taxonomy rank and
supertaxon name or supertaxon ID).
}
\examples{
inputTaxonIDs <- c("10116", "122586", "123851", "13616", "188937", "189518",
"208964", "224129", "224324", "237631", "243230")
rank <- "species"
higherRank <- "phylum"
higherID <- 7711
getSelectedTaxonNames(inputTaxonIDs, rank, higherRank, higherID, NULL)
higherName <- "Chordata"
getSelectedTaxonNames(inputTaxonIDs, rank, higherRank, NULL, higherName,NULL)
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parsePhyloProfile.R
\name{getInputTaxaName}
\alias{getInputTaxaName}
\title{Get NCBI taxon names for a selected list of taxa}
\usage{
getInputTaxaName(rankName, taxonIDs = NULL, taxDB = NULL)
}
\arguments{
\item{rankName}{taxonomy rank (e.g. "species","phylum",...)}

\item{taxonIDs}{list of taxon IDs (e.g. ncbi1234). Default = NULL}

\item{taxDB}{Path to the taxonomy DB files}
}
\value{
Data frame contains a list of full names, taxonomy ranks and parent
IDs for the input taxa.
}
\description{
Get NCBI taxon names from
"PhyloProfile/data/taxonNamesReduced.txt" for a list of input taxa
}
\examples{
taxonIDs <- c("ncbi9606", "ncbi10116")
getInputTaxaName("species", taxonIDs)
}
\seealso{
\code{\link{getInputTaxaID}} for getting input taxon IDs,
\code{\link{getNameList}} for getting the full taxon name list
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parsePhylotree.R
\name{checkNewick}
\alias{checkNewick}
\title{Check the validity of input newick tree}
\usage{
checkNewick(tree, inputTaxonID = NULL)
}
\arguments{
\item{tree}{input newick tree}

\item{inputTaxonID}{list of all input taxon IDs for the phylogenetic profiles}
}
\value{
Possible formatting error of input tree. 0 = suitable tree for using
with PhyloProfile, 1 = missing parenthesis; 2 = missing comma;
3 = tree has singleton; or a list of taxa that do not exist in the input
phylogenetic profile.
}
\description{
Check the validity of input newick tree
}
\examples{
data("ppTree", package="PhyloProfile")
checkNewick(ppTree, c("ncbi3702", "ncbi3711", "ncbi7029"))
}
\seealso{
\code{\link{getInputTaxaID}} for getting input taxon IDs,
\code{\link{ppTree}} for an example of input tree
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parseMainInput.R
\name{wideToLong}
\alias{wideToLong}
\title{Transform input file in wide matrix into long matrix format}
\usage{
wideToLong(inputFile = NULL)
}
\arguments{
\item{inputFile}{input file in wide matrix format}
}
\value{
A data frame of input data in long-format containing seed gene IDs (
or orthologous group IDs), their orthologous proteins together with the
corresponding taxonomy IDs and values of (up to) two additional variables.
}
\description{
Transform input file in wide matrix into long matrix format
}
\examples{
inputFile <- system.file(
    "extdata", "test.main.wide", package = "PhyloProfile", mustWork = TRUE
)
wideToLong(inputFile)
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createProfilePlot.R
\name{heatmapPlotting}
\alias{heatmapPlotting}
\title{Create profile heatmap plot}
\usage{
heatmapPlotting(data = NULL, parm = NULL)
}
\arguments{
\item{data}{dataframe for plotting the heatmap phylogentic profile (either
full or subset profiles)}

\item{parm}{plot parameters, including (1) type of x-axis "taxa" or
"genes" - default = "taxa"; (2) display gene IDs (default) or gene names;
(3+4) names of 2 variables var1ID and var2ID - default = "var1" & "var2";
(5+6) mid value and color for mid value of var1 -
default is 0.5 and #FFFFFF; (7) color for lowest var1 - default = "#FF8C00";
(8) color for highest var1 - default = "#4682B4"; (9+10) mid value and color
for mid value of var2 - default is 1 and #FFFFFF;(11) color for lowest var2 -
default = "#FFFFFF", (12) color for highest var2 - default = "#F0E68C", (13)
color of co-orthologs - default = "#07D000"; (14+15+16) text sizes for x, y
axis and legend - default = 9 for each; (17) legend position "top", "bottom",
"right", "left" or "none" - default = "top"; (18) zoom ratio of the
co-ortholog dots from -1 to 3 - default = 0; (19) angle of x-axis from 0 to
90 - default = 60; (20) show/hide separate line for reference taxon 1/0 -
default = 0; (21) enable/disable coloring gene categories TRUE/FALSE -
default = FALSE; (22) enable/disable coloring duplicated ortholog IDs
TRUE/FALSE - default=FALSE). NOTE: Leave blank or NULL to use default values.}
}
\value{
A profile heatmap plot as a ggplot object.
}
\description{
Create profile heatmap plot
}
\examples{
data("finalProcessedProfile", package="PhyloProfile")
plotDf <- dataMainPlot(finalProcessedProfile)
plotParameter <- list(
    "xAxis" = "taxa",
    "geneIdType" = "geneID",
    "var1ID" = "FAS_FW",
    "var2ID"  = "FAS_BW",
    "midVar1" = 0.5,
    "midColorVar1" =  "#FFFFFF",
    "lowColorVar1" =  "#FF8C00",
    "highColorVar1" = "#4682B4",
    "midVar2" = 1,
    "midColorVar2" =  "#FFFFFF",
    "lowColorVar2" = "#CB4C4E",
    "highColorVar2" = "#3E436F",
    "paraColor" = "#07D000",
    "xSize" = 8,
    "ySize" = 8,
    "legendSize" = 8,
    "mainLegend" = "top",
    "dotZoom" = 0,
    "xAngle" = 60,
    "guideline" = 0,
    "colorByGroup" = FALSE,
    "catColors" = NULL,
    "colorByOrthoID" = FALSE
)

heatmapPlotting(plotDf, plotParameter)
}
\seealso{
\code{\link{dataMainPlot}}, \code{\link{dataCustomizedPlot}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getFastaSeqs.R
\name{getFastaFromFile}
\alias{getFastaFromFile}
\title{Get fasta sequences from main input file in multi-fasta format}
\usage{
getFastaFromFile(seqIDs = NULL, concatFasta = NULL)
}
\arguments{
\item{seqIDs}{list of sequences IDs. Set seqIDs = "all" if you want to get
all fasta sequences from the concatenated input fasta file.}

\item{concatFasta}{input concatenated fasta file.}
}
\value{
A dataframe with one column contains sequences in fasta format.
}
\description{
Get fasta sequences from main input file in multi-fasta format
}
\examples{
concatFasta <- system.file(
    "extdata", "fastaFiles/concatenatedFile.fa",
    package = "PhyloProfile", mustWork = TRUE
)
getFastaFromFasInput("all", concatFasta)
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createDomainPlot.R
\name{linearizeArchitecture}
\alias{linearizeArchitecture}
\title{Linearize PFAM/SMART annotations by best e-value/bitscore}
\usage{
linearizeArchitecture(domainDf = NULL, orthoID = NULL, value = "evalue")
}
\arguments{
\item{domainDf}{input domain dataframe}

\item{orthoID}{ID of protein that needs to be linearized}

\item{value}{type of values that will be used for linearized, either evalue
(default) or bitscore}
}
\value{
Domain dataframe of the selected protein after linearization
}
\description{
Linearize PFAM/SMART annotations by best e-value/bitscore
}
\examples{
demoDomainDf <- data.frame(
    orthoID = rep("protID", 4),
    start = c(1, 5, 100, 80),
    end = c(30, 40, 130, 110),
    evalue = c(0.001, 0.0005, 0.2, 0.004),
    feature_type = c(rep("pfam", 2), rep("smart", 2)),
    feature_id = c("pf1", "pf2", "sm1", "sm2")
)
linearizeArchitecture(demoDomainDf, "protID", "evalue")
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compareTaxaGroups.R
\name{varDistTaxPlot}
\alias{varDistTaxPlot}
\title{Create variable distribution comparison plot}
\usage{
varDistTaxPlot(data, plotParameters)
}
\arguments{
\item{data}{dataframe for plotting. Last column
indicates what type of taxon group (in- or out-group). The first (or first 2)
column contains values of the variables. See ?dataVarDistTaxGroup}

\item{plotParameters}{plot parameters, including size of x-axis, y-axis,
legend and title; position of legend ("right", "bottom" or "none");
mean/median point; names of in-group and out-group; and plot title.
NOTE: Leave blank or NULL to use default values.}
}
\value{
Distribution plots as a grob (gtable) object. Use grid.draw to plot.
}
\description{
Create variable distribution plots between 2 groups of taxa for
a selected gene.
}
\examples{
data("mainLongRaw", package="PhyloProfile")
data <- mainLongRaw
inGroup <- c("ncbi9606", "ncbi10116")
variable <- colnames(data)[c(4, 5)]
plotDf <- dataVarDistTaxGroup(data, inGroup, "101621at6656", variable)
plotParameters <- list(
    "xSize" = 12,
    "ySize" = 12,
    "titleSize" = 15,
    "legendSize" = 12,
    "legendPosition" = "right",
    "mValue" = "mean",
    "inGroupName" = "In-group",
    "outGroupName" = "Out-group",
    "title" = "101621at6656"
)
g <- varDistTaxPlot(plotDf, plotParameters)
grid::grid.draw(g)
}
\seealso{
\code{\link{dataVarDistTaxGroup}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createDomainPlot.R
\name{sortDomains}
\alias{sortDomains}
\title{Sort one domain dataframe based on the other domain dataframe}
\usage{
sortDomains(seedDf, orthoDf)
}
\arguments{
\item{seedDf}{data of seed protein}

\item{orthoDf}{data of ortholog protein}
}
\value{
Dataframe contains sorted domain list.
}
\description{
Sort domain dataframe of one protein (either seed or ortholog)
based on the dataframe of the its paired protein, in order to bring the
common domain feature in the same order which make it easy for comparing.
}
\examples{
# get domain data
seedID <- "101621at6656"
domainFile <- system.file(
    "extdata", "domainFiles/101621at6656.domains",
    package = "PhyloProfile", mustWork = TRUE
)
domainDf <- parseDomainInput(seedID, domainFile, "file")
# get seedDf and orthoDf
subDf <- domainDf[
    domainDf$seedID ==
    "101621at6656#101621at6656:AGRPL@224129@0:224129_0:001955:1",]
orthoDf <- subDf[subDf$orthoID == "101621at6656:DROME@7227@1:Q9VG04",]
seedDf <- subDf[subDf$orthoID != "101621at6656:DROME@7227@1:Q9VG04",]
# sort
PhyloProfile:::sortDomains(seedDf, orthoDf)
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createDomainPlot.R
\name{addFeatureColors}
\alias{addFeatureColors}
\title{Add colors for each feature/domain}
\usage{
addFeatureColors(
  seedDf = NULL,
  orthoDf = NULL,
  colorType = "all",
  colorPalette = "Paired",
  ignoreInstanceNo = FALSE
)
}
\arguments{
\item{seedDf}{Domain dataframe of seed protein (protein 1)}

\item{orthoDf}{Domain dataframe of orthologs protein (protein 2)}

\item{colorType}{Choose to color "all", "shared", "unique" features or color
by "Feature type". Default: "all"}

\item{colorPalette}{Choose between "Paired", "Set1", "Set2", "Set3",
"Accent", "Dark2" for the color pallete}

\item{ignoreInstanceNo}{Ignore number of feature instances while identifying
shared or unique features. Default: FALSE}
}
\value{
2 dataframes (seedDf and orthoDf) with an additional column for the
assigned color to each feature instance
}
\description{
Add colors to features/domains of 2 domain dataframes. Users can
choose to color only the shared features, unique features, all features
(default) or based on feature types. Default color pallete is "Paired", but
it can be changed.
}
\examples{
# get domain data
seedID <- "101621at6656"
domainFile <- system.file(
    "extdata", "domainFiles/101621at6656.domains",
    package = "PhyloProfile", mustWork = TRUE
)
domainDf <- parseDomainInput(seedID, domainFile, "file")
# get seedDf and orthoDf
subDf <- domainDf[
    domainDf$seedID ==
    "101621at6656#101621at6656:AGRPL@224129@0:224129_0:001955:1",]
orthoDf <- subDf[subDf$orthoID == "101621at6656:DROME@7227@1:Q9VG04",]
seedDf <- subDf[subDf$orthoID != "101621at6656:DROME@7227@1:Q9VG04",]
# add colors to features
PhyloProfile:::addFeatureColors(seedDf, orthoDf)
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimateGeneAge.R
\name{createGeneAgePlot}
\alias{createGeneAgePlot}
\title{Create gene age plot}
\usage{
createGeneAgePlot(geneAgePlotDf, textFactor = 1, font = "Arial")
}
\arguments{
\item{geneAgePlotDf}{data frame required for plotting gene age (see
?geneAgePlotDf)}

\item{textFactor}{increase factor of text size}

\item{font}{font of text. Default = Arial"}
}
\value{
A gene age distribution plot as a ggplot2 object
}
\description{
Create gene age plot
}
\examples{
geneAgePlotDf <- data.frame(
    name = c("Streptophyta (Phylum)", "Bikonta", "Eukaryota (Superkingdom)"),
    count = c(7, 1, 30),
    percentage = c(18, 3, 79)
)
createGeneAgePlot(geneAgePlotDf, 1, "sans")
}
\seealso{
\code{\link{estimateGeneAge}} and \code{\link{geneAgePlotDf}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{mainLongRaw}
\alias{mainLongRaw}
\title{An example of a raw long input file}
\format{
Dataframe
}
\usage{
data(mainLongRaw)
}
\value{
A data frame with 168 rows and 5 variables:
\itemize{
    \item geneID  Seed or ortholog group ID, e.g. "100136at6656"
    \item ncbiID  Taxon ID, e.g. "ncbi36329"
    \item orthoID Ortholog ID, e.g. "100136at6656|PLAF7@36329@1|Q8ILT8|1"
    \item FAS_F First additional variable
    \item FAS_B Second additional variable
}
}
\description{
An example of a raw long input file
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createTaxonomyMatrix.R
\name{mainTaxonomyRank}
\alias{mainTaxonomyRank}
\title{Get all NCBI taxonomy rank names}
\usage{
mainTaxonomyRank()
}
\value{
A list of all available NCBI taxonomy rank names.
}
\description{
Get all NCBI taxonomy rank names
}
\examples{
mainTaxonomyRank()
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createTaxonomyMatrix.R
\name{getTaxHierarchy}
\alias{getTaxHierarchy}
\title{Get taxonomy hierarchy for a list of taxon IDs}
\usage{
getTaxHierarchy(inputTaxa = NULL, currentNCBIinfo = NULL)
}
\arguments{
\item{inputTaxa}{NCBI ID list of input taxa.}

\item{currentNCBIinfo}{table/dataframe of the pre-processed NCBI taxonomy
data (/PhyloProfile/data/preProcessedTaxonomy.txt)}
}
\value{
A list of dataframs containing taxonomy hierarchy and its URL to
NCBI database for input taxon IDs
}
\description{
Get NCBI taxonomy hierarchy and URLs for an input taxon list.
}
\examples{
inputTaxa <- c("272557", "176299")
ncbiFilein <- system.file(
    "extdata", "data/preProcessedTaxonomy.txt",
    package = "PhyloProfile", mustWork = TRUE
)
currentNCBIinfo <- as.data.frame(data.table::fread(ncbiFilein))
PhyloProfile:::getTaxHierarchy(inputTaxa, currentNCBIinfo)
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{fullProcessedProfile}
\alias{fullProcessedProfile}
\title{An example of a fully processed phylogenetic profile}
\format{
Dataframe
}
\usage{
data(fullProcessedProfile)
}
\value{
A data frame with 168 rows and 14 variables:
\itemize{
    \item supertaxon Supertaxon name together with its ordered index, e.g.
    "1001_Mammalia"
    \item ncbiID Taxon ID, e.g. "ncbi10116"
    \item geneID Seed or ortholog group ID, e.g. "100136at6656"
    \item orthoID Ortholog ID, e.g. "100136at6656|HUMAN@9606@1|Q9UNQ2|1"
    \item var1 First additional variable
    \item var2 Second additional variable
    \item paralog Number of co-orthologs in the current taxon
    \item abbrName NCBI ID of the ortholog, e.g. "ncbi9606"
    \item taxonID Taxon ID of the ortholog, in this case: "0"
    \item fullName Full taxon name of the ortholog, e.g. "Homo sapiens"
    \item supertaxonID Supertaxon ID (only different than ncbiID in case
    working with higher taxonomy rank than input's). e.g. "40674"
    \item rank Rank of the supertaxon, e.g. "class"
    \item category "cat
    \item numberSpec Total number of species in each supertaxon
}
}
\description{
An example of a fully processed phylogenetic profile
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyzeDistribution.R
\name{createVarDistPlot}
\alias{createVarDistPlot}
\title{Create distribution plot}
\usage{
createVarDistPlot(data, varName = "var", varType = "var1",
    percent = c(0, 1), textSize = 12)
}
\arguments{
\item{data}{dataframe contains data for plotting (see
?createVariableDistributionData, ?createVariableDistributionDataSubset or
?createPercentageDistributionData)}

\item{varName}{name of the variable that need to be analyzed (either name of
variable 1 or variable 2 or "percentage of present taxa"). Default = "var".}

\item{varType}{type of variable (either "var1", "var2" or "presSpec").
Default = "var1".}

\item{percent}{range of percentage cutoff (between 0 and 1). Default = c(0,1)}

\item{textSize}{text size of the distribution plot (in px). Default = 12.}
}
\value{
A distribution plot for the selected variable as a ggplot object
}
\description{
Create distribution plot for one of the additional variable or
the percentage of the species present in the supertaxa.
}
\examples{
data("mainLongRaw", package="PhyloProfile")
data <- createVariableDistributionData(
    mainLongRaw, c(0, 1), c(0.5, 1)
)
varName <- "Variable abc"
varType <- "var1"
percent <- c(0,1)
textSize <- 12
createVarDistPlot(
    data,
    varName,
    varType,
    percent,
    textSize
)
}
\seealso{
\code{\link{mainLongRaw}},
\code{\link{createVariableDistributionData}},
\code{\link{createVariableDistributionDataSubset}},
\code{\link{createPercentageDistributionData}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parsePhyloProfile.R
\name{calcPresSpec}
\alias{calcPresSpec}
\title{Calculate percentage of present species in each super taxon}
\usage{
calcPresSpec(profileWithTax, taxaCount)
}
\arguments{
\item{profileWithTax}{data frame of main PhyloProfile input together
with their taxonomy info (see ?profileWithTaxonomy)}

\item{taxaCount}{number of species occur in each supertaxon (e.g. phylum
or kingdom)}
}
\value{
A data frame with % of present species in each supertaxon
}
\description{
Calculate percentage of present species in each super taxon
}
\examples{
# NOTE: for internal testing only
library(dplyr)
data("profileWithTaxonomy", package="PhyloProfile")
taxaCount <- profileWithTaxonomy \%>\% dplyr::count(supertaxon)
taxaCount$n <- 1
calcPresSpec(profileWithTaxonomy, taxaCount)
}
\seealso{
\code{\link{profileWithTaxonomy}} for a demo input data
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/umapClustering.R
\name{prepareDimRedData}
\alias{prepareDimRedData}
\title{Prepare data for dimension reduction}
\usage{
prepareDimRedData(longDf = NULL, taxonRank = NULL, type = "taxa",
    taxDB = NULL, filterVar = "both", cutoff = 0, groupLabelsBy = "taxa")
}
\arguments{
\item{longDf}{input phyloprofile file in long format}

\item{taxonRank}{taxonomy rank for labels (e.g. "phylum")}

\item{type}{type of clustering, either "taxa" (default) or "genes"}

\item{taxDB}{path to taxonomy database}

\item{filterVar}{choose variable (either "var1", "var2" or "both") to filter
the data. Default: "both"}

\item{cutoff}{cutoff to filter data values. Default: 0}

\item{groupLabelsBy}{group labels by the number of "taxa" (default) or
"genes"}
}
\value{
A dataframe in wide format
}
\description{
Prepare data for dimension reduction
}
\examples{
rawInput <- system.file(
   "extdata", "test.main.long", package = "PhyloProfile", mustWork = TRUE
)
longDf <- createLongMatrix(rawInput)
prepareDimRedData(longDf, "phylum")
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createColorScheme.R
\name{checkColorPalette}
\alias{checkColorPalette}
\title{Check if a color pallete has enough colors for a list of items}
\usage{
checkColorPalette(items, pallete = "Paired")
}
\arguments{
\item{items}{vector contains list of items}

\item{pallete}{name of color palette}
}
\value{
TRUE if color pallete has enough colors, otherwise FALSE
}
\description{
Check if a color pallete has enough colors for a list of items
}
\examples{
myItems <- rep("a",3)
checkColorPalette(myItems, "Set1")
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parseMainInput.R
\name{checkInputValidity}
\alias{checkInputValidity}
\title{Check the validity of the input phylogenetic profile file}
\usage{
checkInputValidity(filein)
}
\arguments{
\item{filein}{input file}
}
\value{
The format of the input file format, or type of error
}
\description{
Check if input file has one of the following format: orthoXML,
multiple FASTA, tab-delimited matrix (wide or long), or list of OMA IDs.
}
\examples{
filein <- system.file(
    "extdata", "test.main.wide", package = "PhyloProfile", mustWork = TRUE
)
checkInputValidity(filein)
}
\seealso{
\code{\link{checkOmaID}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{taxonomyMatrix}
\alias{taxonomyMatrix}
\title{Taxonomy matrix for experimental data sets}
\format{
Dataframe
}
\usage{
data(taxonomyMatrix)
}
\value{
A data frame with up to 149 columns and 95 rows corresponding to 95
    taxa in the 2 experimental data sets
}
\description{
Data frame containing the fully aligned taxonomy IDs of 95 taxa
in the experimental data sets. By talking into account both the defined
ranks (e.g. strain, This data is used for clustering and then creating a
taxon tree. It is used also for cross-linking between different taxonomy
ranks within a taxon.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createTaxonomyMatrix.R
\name{getTaxonomyInfo}
\alias{getTaxonomyInfo}
\title{Get taxonomy info for a list of input taxa}
\usage{
getTaxonomyInfo(inputTaxa = NULL, currentNCBIinfo = NULL)
}
\arguments{
\item{inputTaxa}{NCBI taxonomy IDs of input taxa.}

\item{currentNCBIinfo}{table/dataframe of the pre-processed NCBI taxonomy
data (/PhyloProfile/data/preProcessedTaxonomy.txt)}
}
\value{
A list of NCBI taxonomy info for input taxa, including the taxonomy
IDs, full scientific names, taxonomy ranks and the parent IDs.
}
\description{
Get taxonomy info for a list of input taxa
}
\examples{
inputTaxa <- c("272557", "176299")
ncbiFilein <- system.file(
    "extdata", "data/preProcessedTaxonomy.txt",
    package = "PhyloProfile", mustWork = TRUE
)
currentNCBIinfo <- as.data.frame(data.table::fread(ncbiFilein))
getTaxonomyInfo(inputTaxa, currentNCBIinfo)
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createTaxonomyMatrix.R
\name{taxonomyTableCreator}
\alias{taxonomyTableCreator}
\title{Align NCBI taxonomy IDs of list of taxa into a sorted rank list.}
\usage{
taxonomyTableCreator(idListFile = NULL, rankListFile = NULL)
}
\arguments{
\item{idListFile}{a text file whose each row is a rank+ID list of a taxon
(see idListFile in example)}

\item{rankListFile}{a text file whose each row is a rank list of a taxon
(see rankListFile in example)}
}
\value{
An aligned taxonomy dataframe which contains all the available
taxonomy ranks from the id and rank list file. This dataframe can be used for
creating a well resolved taxonomy tree (see ?createUnrootedTree) and sorting
taxa based on a selected reference taxon (see ?sortInputTaxa).
}
\description{
Align NCBI taxonomy IDs of list of taxa into a sorted rank list.
}
\examples{
idListFile <- system.file(
    "extdata", "data/idList.txt", package = "PhyloProfile", mustWork = TRUE
)
rankListFile <- system.file(
    "extdata", "data/rankList.txt", package = "PhyloProfile", mustWork = TRUE
)
taxonomyTableCreator(idListFile, rankListFile)
}
\seealso{
\code{\link{rankIndexing}}, \code{\link{createUnrootedTree}},
\code{\link{sortInputTaxa}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}
